@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: debug.h 14121 2021-02-04 18:25:27Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2016  The Bochs Project
@@ -73,7 +73,15 @@ bool bx_dbg_register_debug_info(const char *devname, void *dev);
 
 #define EMPTY_ARG (-1)
 
+void bx_dbg_addlyt(const char* new_layoutpath);
+void bx_dbg_remlyt(void);
+void bx_dbg_lyt(void);
+void bx_dbg_set_magic_bp_mask(Bit8u new_mask);
+void bx_dbg_clr_magic_bp_mask(Bit8u mask);
+Bit8u bx_dbg_get_magic_bp_mask_from_str(const char *str);
+void bx_dbg_print_magic_bp_mask_from_str(Bit8u mask);
 bool bx_dbg_read_linear(unsigned which_cpu, bx_address laddr, unsigned len, Bit8u *buf);
+bool bx_dbg_write_linear(unsigned which_cpu, bx_address laddr, unsigned len, Bit8u *buf);
 Bit16u bx_dbg_get_selector_value(unsigned int seg_no);
 Bit16u bx_dbg_get_ip (void);
 Bit32u bx_dbg_get_eip(void);
@@ -111,6 +119,8 @@ void bx_dbg_xlate_address(bx_lin_address laddr);
 void bx_dbg_tlb_lookup(bx_lin_address laddr);
 void bx_dbg_show_command(const char*);
 void bx_dbg_print_stack_command(unsigned nwords);
+void bx_dbg_deref_command(bx_address addr, unsigned deep);
+bx_address bx_dbg_deref(bx_address addr, unsigned deep, unsigned* error_deep, bx_address* last_data_found);
 void bx_dbg_bt_command(unsigned dist);
 void bx_dbg_print_watchpoints(void);
 void bx_dbg_watchpoint_continue(bool watch_continue);
@@ -141,7 +151,9 @@ void bx_dbg_quit_command(void);
 #define BX_INFO_SSE_REGS 0x08
 #define BX_INFO_YMM_REGS 0x10
 #define BX_INFO_ZMM_REGS 0x20
+#define BX_INFO_AMX_REGS 0x40
 void bx_dbg_info_registers_command(int);
+void bx_dbg_print_amx_tile_command(int tile);
 void bx_dbg_info_ivt_command(unsigned from, unsigned to);
 void bx_dbg_info_idt_command(unsigned from, unsigned to);
 void bx_dbg_info_gdt_command(unsigned from, unsigned to);
@@ -157,7 +169,8 @@ void bx_dbg_examine_command(const char *command, const char *format, bool format
 Bit32u bx_dbg_lin_indirect(bx_address addr);
 Bit32u bx_dbg_phy_indirect(bx_phy_address addr);
 void bx_dbg_writemem_command(const char *filename, bx_address laddr, unsigned len);
-void bx_dbg_setpmem_command(bx_phy_address addr, unsigned len, Bit32u val);
+void bx_dbg_loadmem_command(const char *filename, bx_address laddr);
+void bx_dbg_setpmem_command(bx_phy_address addr, unsigned len, Bit64u val);
 void bx_dbg_query_command(const char *);
 void bx_dbg_take_command(const char *, unsigned n);
 void bx_dbg_disassemble_current(const char *);
@@ -188,7 +201,7 @@ void bx_dbg_check_memory_watchpoints(unsigned cpu, bx_phy_address phy, unsigned
 void bx_dbg_restore_command(const char *param_name, const char *path);
 void bx_dbg_show_param_command(const char *param, bool xml);
 
-int bx_dbg_show_symbolic(void);
+void bx_dbg_show_symbolic(void);
 void bx_dbg_set_symbol_command(const char *symbol, bx_address val);
 const char* bx_dbg_symbolic_address(bx_address context, bx_address eip, bx_address base);
 int bx_dbg_symbol_command(const char* filename, bool global, bx_address offset);
