@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: access2.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2008-2019 Stanislav Shwartsman
@@ -57,7 +57,7 @@ BX_CPU_C::write_linear_word(unsigned s, bx_address laddr, Bit16u data)
   bx_address lpf = AlignedAccessLPFOf(laddr, (1 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -84,7 +84,7 @@ BX_CPU_C::write_linear_dword(unsigned s, bx_address laddr, Bit32u data)
   bx_address lpf = AlignedAccessLPFOf(laddr, (3 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -111,7 +111,7 @@ BX_CPU_C::write_linear_qword(unsigned s, bx_address laddr, Bit64u data)
   bx_address lpf = AlignedAccessLPFOf(laddr, (7 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -304,11 +304,18 @@ BX_CPU_C::tickle_read_linear(unsigned s, bx_address laddr)
   bx_TLB_entry *tlbEntry = BX_DTLB_ENTRY_OF(laddr, 0);
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us read access from this CPL
-    if (isReadOK(tlbEntry, USER_PL)) return;
+    if (isReadOK(tlbEntry, USER_PL)) {
+      BX_CPU_THIS_PTR address_xlation.paddress1 = tlbEntry->ppf | PAGE_OFFSET(laddr);
+      BX_CPU_THIS_PTR address_xlation.pages     = 1;
+#if BX_SUPPORT_MEMTYPE
+      BX_CPU_THIS_PTR address_xlation.memtype1  = tlbEntry->get_memtype();
+#endif
+      return;
+    }
   }
 
 #if BX_SUPPORT_X86_64
-  if (! IsCanonical(laddr)) {
+  if (! IsCanonicalAccess(laddr, USER_PL)) {
     BX_ERROR(("tickle_read_linear(): canonical failure"));
     exception(int_number(s), 0);
   }
@@ -361,7 +368,7 @@ BX_CPU_C::read_linear_word(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (1 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us read access from this CPL
     if (isReadOK(tlbEntry, USER_PL)) {
@@ -390,7 +397,7 @@ BX_CPU_C::read_linear_dword(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (3 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us read access from this CPL
     if (isReadOK(tlbEntry, USER_PL)) {
@@ -419,7 +426,7 @@ BX_CPU_C::read_linear_qword(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (7 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us read access from this CPL
     if (isReadOK(tlbEntry, USER_PL)) {
@@ -638,7 +645,7 @@ BX_CPU_C::read_RMW_linear_word(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (1 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -673,7 +680,7 @@ BX_CPU_C::read_RMW_linear_dword(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (3 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -708,7 +715,7 @@ BX_CPU_C::read_RMW_linear_qword(unsigned s, bx_address laddr)
   bx_address lpf = AlignedAccessLPFOf(laddr, (7 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, USER_PL)) {
@@ -943,7 +950,7 @@ void BX_CPU_C::write_RMW_linear_dqword(Bit64u hi, Bit64u lo)
   else {
     BX_ASSERT(BX_CPU_THIS_PTR address_xlation.pages == 1);
   }
-  
+
   write_RMW_linear_qword(hi);
 }
 
@@ -962,7 +969,7 @@ void BX_CPU_C::write_new_stack_word(bx_address laddr, unsigned curr_pl, Bit16u d
   bx_address lpf = AlignedAccessLPFOf(laddr, (1 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, user)) {
@@ -989,7 +996,7 @@ void BX_CPU_C::write_new_stack_dword(bx_address laddr, unsigned curr_pl, Bit32u
   bx_address lpf = AlignedAccessLPFOf(laddr, (3 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, user)) {
@@ -1016,7 +1023,7 @@ void BX_CPU_C::write_new_stack_qword(bx_address laddr, unsigned curr_pl, Bit64u
   bx_address lpf = AlignedAccessLPFOf(laddr, (7 & BX_CPU_THIS_PTR alignment_check_mask));
 #else
   bx_address lpf = LPFOf(laddr);
-#endif    
+#endif
   if (tlbEntry->lpf == lpf) {
     // See if the TLB entry privilege level allows us write access from this CPL
     if (isWriteOK(tlbEntry, user)) {
@@ -1056,7 +1063,7 @@ accessOK:
   // add error code when segment violation occurs when pushing into new stack
   if (!write_virtual_checks(seg, offset, 2)) {
     BX_ERROR(("write_new_stack_word(): segment limit violation"));
-    exception(BX_SS_EXCEPTION, 
+    exception(BX_SS_EXCEPTION,
          seg->selector.rpl != CPL ? (seg->selector.value & 0xfffc) : 0);
   }
   goto accessOK;
@@ -1083,7 +1090,7 @@ accessOK:
   // add error code when segment violation occurs when pushing into new stack
   if (!write_virtual_checks(seg, offset, 4)) {
     BX_ERROR(("write_new_stack_dword(): segment limit violation"));
-    exception(BX_SS_EXCEPTION, 
+    exception(BX_SS_EXCEPTION,
          seg->selector.rpl != CPL ? (seg->selector.value & 0xfffc) : 0);
   }
   goto accessOK;
@@ -1110,7 +1117,7 @@ accessOK:
   // add error code when segment violation occurs when pushing into new stack
   if (!write_virtual_checks(seg, offset, 8)) {
     BX_ERROR(("write_new_stack_qword(): segment limit violation"));
-    exception(BX_SS_EXCEPTION, 
+    exception(BX_SS_EXCEPTION,
         seg->selector.rpl != CPL ? (seg->selector.value & 0xfffc) : 0);
   }
   goto accessOK;
@@ -1215,7 +1222,7 @@ void BX_CPP_AttrRegparmN(3) BX_CPU_C::shadow_stack_write_qword(bx_address offset
 
 bool BX_CPP_AttrRegparmN(4) BX_CPU_C::shadow_stack_lock_cmpxchg8b(bx_address offset, unsigned curr_pl, Bit64u data, Bit64u expected_data)
 {
-  Bit64u val64 = shadow_stack_read_qword(offset, curr_pl);
+  Bit64u val64 = shadow_stack_read_qword(offset, curr_pl); // should be locked and RMW
   if (val64 == expected_data) {
     shadow_stack_write_qword(offset, curr_pl, data);
     return true;
