@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc 14321 2021-07-25 18:02:36Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (c) 2002-2019 Zwane Mwaikambo, Stanislav Shwartsman
+//  Copyright (c) 2002-2023 Zwane Mwaikambo, Stanislav Shwartsman
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -23,30 +23,32 @@
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
-#include "scalar_arith.h"
-#include "iodev/iodev.h"
 
 #if BX_SUPPORT_APIC
 
+#include "apic.h"
+#include "scalar_arith.h"
+#include "iodev/iodev.h"
+
 extern bool simulate_xapic;
 
 #define LOG_THIS this->
 
-#define BX_CPU_APIC(i) (&(BX_CPU(i)->lapic))
+#define BX_CPU_APIC(i) (BX_CPU(i)->lapic)
 
 const unsigned BX_LAPIC_FIRST_VECTOR = 0x10;
 const unsigned BX_LAPIC_LAST_VECTOR  = 0xff;
 
 ///////////// APIC BUS /////////////
 
-int apic_bus_deliver_interrupt(Bit8u vector, apic_dest_t dest, Bit8u delivery_mode, bool logical_dest, bool level, bool trig_mode)
+bool apic_bus_deliver_interrupt(Bit8u vector, apic_dest_t dest, Bit8u delivery_mode, bool logical_dest, bool level, bool trig_mode)
 {
   if(delivery_mode == APIC_DM_LOWPRI)
   {
      if(! logical_dest) {
        // I/O subsytem initiated interrupt with lowest priority delivery
        // which is not supported in physical destination mode
-       return 0;
+       return false;
      }
      else {
        return apic_bus_deliver_lowest_priority(vector, dest, trig_mode, 0);
@@ -65,16 +67,16 @@ int apic_bus_deliver_interrupt(Bit8u vector, apic_dest_t dest, Bit8u delivery_mo
        {
          if(BX_CPU_APIC(i)->get_id() == dest) {
            BX_CPU_APIC(i)->deliver(vector, delivery_mode, trig_mode);
-           return 1;
+           return true;
          }
        }
 
-       return 0;
+       return false;
     }
   }
   else {
     // logical destination mode
-    if(dest == 0) return 0;
+    if(dest == 0) return false;
 
     bool interrupt_delivered = false;
 
@@ -85,11 +87,11 @@ int apic_bus_deliver_interrupt(Bit8u vector, apic_dest_t dest, Bit8u delivery_mo
       }
     }
 
-    return (int) interrupt_delivered;
+    return interrupt_delivered;
   }
 }
 
-int apic_bus_deliver_lowest_priority(Bit8u vector, apic_dest_t dest, bool trig_mode, bool broadcast)
+bool apic_bus_deliver_lowest_priority(Bit8u vector, apic_dest_t dest, bool trig_mode, bool broadcast)
 {
   int i;
 
@@ -98,7 +100,7 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, apic_dest_t dest, bool trig_m
     for (i=0; i<BX_NUM_LOCAL_APICS; i++) {
       if(BX_CPU_APIC(i)->is_focus(vector)) {
         BX_CPU_APIC(i)->deliver(vector, APIC_DM_LOWPRI, trig_mode);
-        return 1;
+        return true;
       }
     }
   }
@@ -122,13 +124,13 @@ int apic_bus_deliver_lowest_priority(Bit8u vector, apic_dest_t dest, bool trig_m
   if(lowest_priority_agent >= 0)
   {
     BX_CPU_APIC(lowest_priority_agent)->deliver(vector, APIC_DM_LOWPRI, trig_mode);
-    return 1;
+    return true;
   }
 
-  return 0;
+  return false;
 }
 
-int apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bool trig_mode, int exclude_cpu)
+bool apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bool trig_mode, int exclude_cpu)
 {
   if(delivery_mode == APIC_DM_LOWPRI)
   {
@@ -141,7 +143,7 @@ int apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bool trig_mo
     BX_CPU_APIC(i)->deliver(vector, delivery_mode, trig_mode);
   }
 
-  return 1;
+  return true;
 }
 
 static void apic_bus_broadcast_eoi(Bit8u vector)
@@ -187,22 +189,22 @@ bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu, unsigned id)
   // Register a non-active timer for use when the timer is started.
   timer_handle = bx_pc_system.register_timer_ticks(this,
             bx_local_apic_c::periodic_smf, 0, 0, 0, "lapic");
-  timer_active = 0;
-  
+  timer_active = false;
+
 #if BX_SUPPORT_VMX >= 2
   // Register a non-active timer for VMX preemption timer.
   vmx_timer_handle = bx_pc_system.register_timer_ticks(this,
             bx_local_apic_c::vmx_preemption_timer_expired, 0, 0, 0, "vmx_preemption");
   BX_DEBUG(("vmx_timer is = %d", vmx_timer_handle));
   vmx_preemption_timer_rate = VMX_MISC_PREEMPTION_TIMER_RATE;
-  vmx_timer_active = 0;
+  vmx_timer_active = false;
 #endif
 
 #if BX_SUPPORT_MONITOR_MWAIT
   mwaitx_timer_handle = bx_pc_system.register_timer_ticks(this,
             bx_local_apic_c::mwaitx_timer_expired, 0, 0, 0, "mwaitx_timer");
   BX_DEBUG(("mwaitx_timer is = %d", mwaitx_timer_handle));
-  mwaitx_timer_active = 0;
+  mwaitx_timer_active = false;
 #endif
 
   xapic = simulate_xapic; // xAPIC or legacy APIC
@@ -238,20 +240,20 @@ void bx_local_apic_c::reset(unsigned type)
 
   if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
-    timer_active = 0;
+    timer_active = false;
   }
 
 #if BX_SUPPORT_VMX >= 2
   if(vmx_timer_active) {
     bx_pc_system.deactivate_timer(vmx_timer_handle);
-    vmx_timer_active = 0;
+    vmx_timer_active = false;
   }
 #endif
 
 #if BX_SUPPORT_MONITOR_MWAIT
   if(mwaitx_timer_active) {
     bx_pc_system.deactivate_timer(mwaitx_timer_handle);
-    mwaitx_timer_active = 0;
+    mwaitx_timer_active = false;
   }
 #endif
 
@@ -268,7 +270,7 @@ void bx_local_apic_c::reset(unsigned type)
 
   if (xapic)
     apic_version_id = 0x00050014; // P4 has 6 LVT entries
-  else 
+  else
     apic_version_id = 0x00030010; // P6 has 4 LVT entries
 
 #if BX_CPU_LEVEL >= 6
@@ -286,16 +288,17 @@ void bx_local_apic_c::enable_xapic_extensions(void)
 
 void bx_local_apic_c::set_base(bx_phy_address newbase)
 {
-#if BX_CPU_LEVEL >= 6
-  if (mode == BX_APIC_X2APIC_MODE)
-    ldr = ((apic_id & 0xfffffff0) << 16) | (1 << (apic_id & 0xf));
-#endif
   mode = (newbase >> 10) & 3;
   newbase &= ~((bx_phy_address) 0xfff);
   base_addr = newbase;
   BX_INFO(("allocate APIC id=%d (MMIO %s) to 0x" FMT_PHY_ADDRX,
     apic_id, (mode == BX_APIC_XAPIC_MODE) ? "enabled" : "disabled", newbase));
 
+#if BX_CPU_LEVEL >= 6
+  if (mode == BX_APIC_X2APIC_MODE)
+    ldr = ((apic_id & 0xfffffff0) << 16) | (1 << (apic_id & 0xf));
+#endif
+
   if (mode == BX_APIC_GLOBALLY_DISABLED) {
     // if local apic becomes globally disabled reset some fields back to defaults
     write_spurious_interrupt_register(0xff);
@@ -309,9 +312,9 @@ bool bx_local_apic_c::is_selected(bx_phy_address addr)
   if((addr & ~0xfff) == base_addr) {
     if((addr & 0xf) != 0)
       BX_INFO(("warning: misaligned APIC access. addr=0x" FMT_PHY_ADDRX, addr));
-    return 1;
+    return true;
   }
-  return 0;
+  return false;
 }
 
 void bx_local_apic_c::read(bx_phy_address addr, void *data, unsigned len)
@@ -500,7 +503,7 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
   if (apic_reg >= 0x400 && !cpu->is_cpu_extension_supported(BX_ISA_XAPIC_EXT))
     apic_reg = 0xffffffff; // choose some obviosly invalid register if extended xapic is not supported
 #endif
-  
+
   switch(apic_reg) {
     case BX_LAPIC_TPR: // task priority
       set_tpr(value & 0xff);
@@ -634,7 +637,7 @@ void bx_local_apic_c::set_lvt_entry(unsigned apic_reg, Bit32u value)
          // Transition between TSC-Deadline and other timer modes disarm the timer
          if(timer_active) {
             bx_pc_system.deactivate_timer(timer_handle);
-            timer_active = 0;
+            timer_active = false;
          }
       }
     }
@@ -655,7 +658,7 @@ void bx_local_apic_c::send_ipi(apic_dest_t dest, Bit32u lo_cmd)
   int logical_dest = (lo_cmd >> 11) & 1;
   int delivery_mode = (lo_cmd >> 8) & 7;
   int vector = (lo_cmd & 0xff);
-  int accepted = 0;
+  bool accepted = false;
 
   if(delivery_mode == APIC_DM_INIT)
   {
@@ -675,7 +678,7 @@ void bx_local_apic_c::send_ipi(apic_dest_t dest, Bit32u lo_cmd)
     break;
   case 1:  // self
     trigger_irq(vector, trig_mode);
-    accepted = 1;
+    accepted = true;
     break;
   case 2:  // all including self
     accepted = apic_bus_broadcast_interrupt(vector, delivery_mode, trig_mode, apic_id_mask);
@@ -762,7 +765,7 @@ void bx_local_apic_c::startup_msg(Bit8u vector)
   cpu->deliver_SIPI(vector);
 }
 
-bool bx_local_apic_c::get_vector(Bit32u *reg, unsigned vector)
+bool bx_local_apic_c::get_vector(const Bit32u *reg, unsigned vector)
 {
   return (reg[vector / 32] >> (vector % 32)) & 0x1;
 }
@@ -777,7 +780,7 @@ void bx_local_apic_c::clear_vector(Bit32u *reg, unsigned vector)
   reg[vector / 32] &= ~(1 << (vector % 32));
 }
 
-int bx_local_apic_c::highest_priority_int(Bit32u *array)
+int bx_local_apic_c::highest_priority_int(const Bit32u *array) const
 {
   for (int reg=7; reg>=0; reg--) {
     Bit32u tmp = array[reg];
@@ -787,7 +790,7 @@ int bx_local_apic_c::highest_priority_int(Bit32u *array)
 // ignore of interrupt vectors < 16 happen naturally as there is no way to ISR to it
 //  if (reg == 0) tmp &= 0xffff0000;
     if (tmp) {
-      int vector = (reg * 32) + (31 - lzcntd(tmp));
+      int vector = (reg * 32) + most_significant_bitd(tmp);
       return vector;
     }
   }
@@ -834,11 +837,11 @@ bool bx_local_apic_c::deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_mode
   case APIC_DM_SMI:
     BX_INFO(("Deliver SMI"));
     cpu->deliver_SMI();
-    return 1;
+    break;
   case APIC_DM_NMI:
     BX_INFO(("Deliver NMI"));
     cpu->deliver_NMI();
-    return 1;
+    break;
   case APIC_DM_INIT:
     BX_INFO(("Deliver INIT IPI"));
     cpu->deliver_INIT();
@@ -852,10 +855,10 @@ bool bx_local_apic_c::deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_mode
     trigger_irq(vector, trig_mode, 1);
     break;
   default:
-    return 0;
+    return false;
   }
 
-  return 1;
+  return true;
 }
 
 void bx_local_apic_c::trigger_irq(Bit8u vector, unsigned trigger_mode, bool bypass_irr_isr)
@@ -969,7 +972,7 @@ bool bx_local_apic_c::match_logical_addr(apic_dest_t address)
   return match;
 }
 
-Bit8u bx_local_apic_c::get_ppr(void)
+Bit8u bx_local_apic_c::get_ppr(void) const
 {
   int ppr = highest_priority_int(isr);
 
@@ -1015,7 +1018,7 @@ Bit8u bx_local_apic_c::get_apr(void)
   return(Bit8u) apr;
 }
 
-bool bx_local_apic_c::is_focus(Bit8u vector)
+bool bx_local_apic_c::is_focus(Bit8u vector) const
 {
   if(focus_disable) return 0;
   return get_vector(irr, vector) || get_vector(isr, vector);
@@ -1048,7 +1051,7 @@ void bx_local_apic_c::periodic(void)
   if(timervec & 0x20000) {
     // Periodic mode - reload timer values
     timer_current = timer_initial;
-    timer_active = 1;
+    timer_active = true;
     ticksInitial = bx_pc_system.time_ticks(); // timer value when it started to count
     BX_DEBUG(("local apic timer(periodic) triggered int, reset counter to 0x%08x", timer_current));
     bx_pc_system.activate_timer_ticks(timer_handle,
@@ -1057,7 +1060,7 @@ void bx_local_apic_c::periodic(void)
   else {
     // one-shot mode
     timer_current = 0;
-    timer_active = 0;
+    timer_active = false;
     BX_DEBUG(("local apic timer(one-shot) triggered int"));
     bx_pc_system.deactivate_timer(timer_handle);
   }
@@ -1085,7 +1088,7 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
   // If active before, deactivate the current timer before changing it.
   if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
-    timer_active = 0;
+    timer_active = false;
   }
 
   timer_initial = value;
@@ -1097,7 +1100,7 @@ void bx_local_apic_c::set_initial_timer_count(Bit32u value)
     // restart from the new start value.
     BX_DEBUG(("APIC: Initial Timer Count Register = %u", value));
     timer_current = timer_initial;
-    timer_active = 1;
+    timer_active = true;
     ticksInitial = bx_pc_system.time_ticks(); // timer value when it started to count
     bx_pc_system.activate_timer_ticks(timer_handle,
             Bit64u(timer_initial) * Bit64u(timer_divide_factor), 0);
@@ -1113,7 +1116,7 @@ Bit32u bx_local_apic_c::get_current_timer_count(void)
   if (timervec & 0x40000) return 0;
 #endif
 
-  if(timer_active==0) {
+  if(! timer_active) {
     return timer_current;
   } else {
     Bit64u delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
@@ -1138,14 +1141,14 @@ void bx_local_apic_c::set_tsc_deadline(Bit64u deadline)
   // If active before, deactivate the current timer before changing it.
   if(timer_active) {
     bx_pc_system.deactivate_timer(timer_handle);
-    timer_active = 0;
+    timer_active = false;
   }
 
   ticksInitial = deadline;
   if (deadline != 0) {
     BX_DEBUG(("APIC: TSC-Deadline is set to " FMT_LL "d", deadline));
     Bit64u currtime = bx_pc_system.time_ticks();
-    timer_active = 1;
+    timer_active = true;
     bx_pc_system.activate_timer_ticks(timer_handle, (deadline > currtime) ? (deadline - currtime) : 1 , 0);
   }
 }
@@ -1176,16 +1179,16 @@ void bx_local_apic_c::set_vmx_preemption_timer(Bit32u value)
   vmx_preemption_timer_value = value;
   vmx_preemption_timer_initial = bx_pc_system.time_ticks();
   vmx_preemption_timer_fire = ((vmx_preemption_timer_initial >> vmx_preemption_timer_rate) + value) << vmx_preemption_timer_rate;
-  BX_DEBUG(("VMX Preemption timer: value = %u, rate = %u, init = %u, fire = %u", value, vmx_preemption_timer_rate, vmx_preemption_timer_initial, vmx_preemption_timer_fire));
+  BX_DEBUG(("VMX Preemption timer: value = %u, rate = %u, init = " FMT_LL "u, fire = " FMT_LL "u", value, vmx_preemption_timer_rate, vmx_preemption_timer_initial, vmx_preemption_timer_fire));
   bx_pc_system.activate_timer_ticks(vmx_timer_handle, vmx_preemption_timer_fire - vmx_preemption_timer_initial, 0);
-  vmx_timer_active = 1;
+  vmx_timer_active = true;
 }
 
 void bx_local_apic_c::deactivate_vmx_preemption_timer(void)
 {
   if (! vmx_timer_active) return;
   bx_pc_system.deactivate_timer(vmx_timer_handle);
-  vmx_timer_active = 0;
+  vmx_timer_active = false;
 }
 
 // Invoked when VMX preemption timer expired
@@ -1199,18 +1202,21 @@ void bx_local_apic_c::vmx_preemption_timer_expired(void *this_ptr)
 
 #if BX_SUPPORT_MONITOR_MWAIT
 
-void bx_local_apic_c::set_mwaitx_timer(Bit32u value)
+void bx_local_apic_c::set_mwaitx_timer(Bit64u value)
 {
-  BX_DEBUG(("MWAITX timer: value = %u", value));
-  bx_pc_system.activate_timer_ticks(mwaitx_timer_active, value, 0);
-  mwaitx_timer_active = 1;
+  if (mwaitx_timer_active) deactivate_mwaitx_timer();
+  if (value) {
+    BX_DEBUG(("MWAITX timer: delay value = " FMT_LL "u", value));
+    bx_pc_system.activate_timer_ticks(mwaitx_timer_handle, value, 0);
+    mwaitx_timer_active = true;
+  }
 }
 
 void bx_local_apic_c::deactivate_mwaitx_timer(void)
 {
   if (! mwaitx_timer_active) return;
   bx_pc_system.deactivate_timer(mwaitx_timer_handle);
-  mwaitx_timer_active = 0;
+  mwaitx_timer_active = false;
 }
 
 // Invoked when MWAITX timer expired
@@ -1222,7 +1228,7 @@ void bx_local_apic_c::mwaitx_timer_expired(void *this_ptr)
 }
 
 #endif
-  
+
 #if BX_CPU_LEVEL >= 6
 // return false when x2apic is not supported/not readable
 bool bx_local_apic_c::read_x2apic(unsigned index, Bit64u *val_64)
@@ -1239,7 +1245,7 @@ bool bx_local_apic_c::read_x2apic(unsigned index, Bit64u *val_64)
     break;
   // full 64-bit access to ICR
   case BX_LAPIC_ICR_LO:
-    *val_64 = ((Bit64u) icr_lo) | (((Bit64u) icr_hi) << 32);
+    *val_64 = GET64_FROM_HI32_LO32(icr_hi, icr_lo);
     break;
   // not supported/not readable in x2apic mode
   case BX_LAPIC_ARBITRATION_PRIORITY:
@@ -1247,7 +1253,7 @@ bool bx_local_apic_c::read_x2apic(unsigned index, Bit64u *val_64)
   case BX_LAPIC_ICR_HI:
   case BX_LAPIC_EOI: // write only
   case BX_LAPIC_SELF_IPI: // write only
-    return 0;
+    return false;
   // compatible to legacy lapic mode
   case BX_LAPIC_VERSION:
   case BX_LAPIC_TPR:
@@ -1292,10 +1298,10 @@ bool bx_local_apic_c::read_x2apic(unsigned index, Bit64u *val_64)
     break;
   default:
     BX_ERROR(("read_x2apic: not supported apic register 0x%08x", index));
-    return 0;
+    return false;
   }
 
-  return 1;
+  return true;
 }
 
 // return false when x2apic is not supported/not writeable
@@ -1306,7 +1312,7 @@ bool bx_local_apic_c::write_x2apic(unsigned index, Bit32u val32_hi, Bit32u val32
   if (index != BX_LAPIC_ICR_LO) {
     // upper 32-bit are reserved for all x2apic MSRs except for the ICR
     if (val32_hi != 0)
-      return 0;
+      return false;
   }
 
   switch(index) {
@@ -1343,29 +1349,29 @@ bool bx_local_apic_c::write_x2apic(unsigned index, Bit32u val32_hi, Bit32u val32
   case BX_LAPIC_IRR8:
   case BX_LAPIC_ICR_HI:
   case BX_LAPIC_TIMER_CURRENT_COUNT:
-    return 0;
+    return false;
   // send self ipi
   case BX_LAPIC_SELF_IPI:
     trigger_irq(val32_lo & 0xff, APIC_EDGE_TRIGGERED);
-    return 1;
+    return true;
   case BX_LAPIC_ICR_LO:
     // handle full 64-bit write
     send_ipi(val32_hi, val32_lo);
-    return 1;
+    return true;
   case BX_LAPIC_TPR:
     // handle reserved bits, only bits 0-7 are writeable
     if ((val32_lo & 0xffffff00) != 0)
-      return 0;
+      return false;
     break; // use legacy write
   case BX_LAPIC_SPURIOUS_VECTOR:
     // handle reserved bits, only bits 0-8, 12 are writeable
     // we do not support directed EOI capability, so reserve bit 12 as well
     if ((val32_lo & 0xfffffe00) != 0)
-      return 0;
+      return false;
     break; // use legacy write
   case BX_LAPIC_EOI:
   case BX_LAPIC_ESR:
-    if (val32_lo != 0) return 0;
+    if (val32_lo != 0) return false;
     break; // use legacy write
   case BX_LAPIC_LVT_TIMER:
   case BX_LAPIC_LVT_THERMAL:
@@ -1379,11 +1385,11 @@ bool bx_local_apic_c::write_x2apic(unsigned index, Bit32u val32_hi, Bit32u val32
     break; // use legacy write
   default:
     BX_ERROR(("write_x2apic: not supported apic register 0x%08x", index));
-    return 0;
+    return false;
   }
 
   write_aligned(index, val32_lo);
-  return 1;
+  return true;
 }
 #endif
 
@@ -1439,12 +1445,10 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   BXRS_HEX_PARAM_SIMPLE(lapic, timer_current);
   BXRS_HEX_PARAM_SIMPLE(lapic, timer_divconf);
   BXRS_DEC_PARAM_SIMPLE(lapic, timer_divide_factor);
-  BXRS_DEC_PARAM_SIMPLE(lapic, timer_handle);
   BXRS_PARAM_BOOL(lapic, timer_active, timer_active);
   BXRS_HEX_PARAM_SIMPLE(lapic, ticksInitial);
 
 #if BX_SUPPORT_VMX >= 2
-  BXRS_DEC_PARAM_SIMPLE(lapic, vmx_timer_handle);
   BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_initial);
   BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_fire);
   BXRS_HEX_PARAM_SIMPLE(lapic, vmx_preemption_timer_value);
@@ -1453,7 +1457,6 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
 #endif
 
 #if BX_SUPPORT_MONITOR_MWAIT
-  BXRS_DEC_PARAM_SIMPLE(lapic, mwaitx_timer_handle);
   BXRS_PARAM_BOOL(lapic, mwaitx_timer_active, mwaitx_timer_active);
 #endif
 }
