@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: avx512.cc 13853 2020-05-19 16:01:23Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2013-2019 Stanislav Shwartsman
+//   Copyright (c) 2013-2023 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
 
 #include "simd_int.h"
 #include "simd_compare.h"
-#include "wide_int.h"
+#include "scalar_arith.h"
 
 // compare
 
@@ -280,7 +280,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPCMPUQ_MASK_KGbHdqWdqIbR(bxInstruction_c
 
 #define AVX512_COMPARE_BYTE_EL(HANDLER, func)                                               \
   /* AVX-512 compare instruction with two src operands working on BYTE elements */          \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                      \
   {                                                                                         \
     BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
     unsigned len = i->getVL();                                                              \
@@ -305,7 +305,7 @@ AVX512_COMPARE_BYTE_EL(VPTESTNMB_MASK_KGqHdqWdqR, xmm_ptestnmb_mask)
 
 #define AVX512_COMPARE_WORD_EL(HANDLER, func)                                               \
   /* AVX-512 compare instruction with two src operands working on WORD elements */          \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                      \
   {                                                                                         \
     BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
     unsigned len = i->getVL();                                                              \
@@ -330,7 +330,7 @@ AVX512_COMPARE_WORD_EL(VPTESTNMW_MASK_KGdHdqWdqR, xmm_ptestnmw_mask)
 
 #define AVX512_COMPARE_DWORD_EL(HANDLER, func)                                              \
   /* AVX-512 compare instruction with two src operands working on DWORD elements */         \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                      \
   {                                                                                         \
     BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
     unsigned len = i->getVL();                                                              \
@@ -355,7 +355,7 @@ AVX512_COMPARE_DWORD_EL(VPTESTNMD_MASK_KGwHdqWdqR, xmm_ptestnmd_mask)
 
 #define AVX512_COMPARE_QWORD_EL(HANDLER, func)                                              \
   /* AVX-512 compare instruction with two src operands working on QWORD elements */         \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                      \
   {                                                                                         \
     BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
     unsigned len = i->getVL();                                                              \
@@ -378,267 +378,6 @@ AVX512_COMPARE_QWORD_EL(VPCMPEQQ_MASK_KGbHdqWdqR, xmm_pcmpeqq_mask)
 AVX512_COMPARE_QWORD_EL(VPTESTMQ_MASK_KGbHdqWdqR, xmm_ptestmq_mask)
 AVX512_COMPARE_QWORD_EL(VPTESTNMQ_MASK_KGbHdqWdqR, xmm_ptestnmq_mask)
 
-// compute, shift and rotate
-
-#define AVX512_2OP_QWORD_EL(HANDLER, func)                                                  \
-  /* AVX-512 instruction with two src operands working on QWORD elements */                 \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    for (unsigned n=0; n < len; n++)                                                        \
-      (func)(&op1.vmm128(n), &op2.vmm128(n));                                               \
-                                                                                            \
-    avx512_write_regq_masked(i, &op1, len, BX_READ_8BIT_OPMASK(i->opmask()));               \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-AVX512_2OP_QWORD_EL(VPMULLQ_MASK_VdqHdqWdqR, xmm_pmullq)
-AVX512_2OP_QWORD_EL(VPADDQ_MASK_VdqHdqWdqR, xmm_paddq)
-AVX512_2OP_QWORD_EL(VPSUBQ_MASK_VdqHdqWdqR, xmm_psubq)
-AVX512_2OP_QWORD_EL(VPANDQ_MASK_VdqHdqWdqR, xmm_andps)
-AVX512_2OP_QWORD_EL(VPANDNQ_MASK_VdqHdqWdqR, xmm_andnps)
-AVX512_2OP_QWORD_EL(VPORQ_MASK_VdqHdqWdqR, xmm_orps)
-AVX512_2OP_QWORD_EL(VPXORQ_MASK_VdqHdqWdqR, xmm_xorps)
-AVX512_2OP_QWORD_EL(VPMAXSQ_MASK_VdqHdqWdqR, xmm_pmaxsq)
-AVX512_2OP_QWORD_EL(VPMAXUQ_MASK_VdqHdqWdqR, xmm_pmaxuq)
-AVX512_2OP_QWORD_EL(VPMINSQ_MASK_VdqHdqWdqR, xmm_pminsq)
-AVX512_2OP_QWORD_EL(VPMINUQ_MASK_VdqHdqWdqR, xmm_pminuq)
-AVX512_2OP_QWORD_EL(VUNPCKLPD_MASK_VpdHpdWpdR, xmm_unpcklpd)
-AVX512_2OP_QWORD_EL(VUNPCKHPD_MASK_VpdHpdWpdR, xmm_unpckhpd)
-AVX512_2OP_QWORD_EL(VPMULDQ_MASK_VdqHdqWdqR, xmm_pmuldq)
-AVX512_2OP_QWORD_EL(VPMULUDQ_MASK_VdqHdqWdqR, xmm_pmuludq)
-AVX512_2OP_QWORD_EL(VPSRAVQ_MASK_VdqHdqWdqR, xmm_psravq)
-AVX512_2OP_QWORD_EL(VPSRLVQ_MASK_VdqHdqWdqR, xmm_psrlvq)
-AVX512_2OP_QWORD_EL(VPSLLVQ_MASK_VdqHdqWdqR, xmm_psllvq)
-AVX512_2OP_QWORD_EL(VPRORVQ_MASK_VdqHdqWdqR, xmm_prorvq)
-AVX512_2OP_QWORD_EL(VPROLVQ_MASK_VdqHdqWdqR, xmm_prolvq)
-
-#define AVX512_2OP_DWORD_EL(HANDLER, func)                                                  \
-  /* AVX-512 instruction with two src operands working on DWORD elements */                 \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    for (unsigned n=0; n < len; n++)                                                        \
-      (func)(&op1.vmm128(n), &op2.vmm128(n));                                               \
-                                                                                            \
-    avx512_write_regd_masked(i, &op1, len, BX_READ_16BIT_OPMASK(i->opmask()));              \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-AVX512_2OP_DWORD_EL(VPADDD_MASK_VdqHdqWdqR, xmm_paddd)
-AVX512_2OP_DWORD_EL(VPSUBD_MASK_VdqHdqWdqR, xmm_psubd)
-AVX512_2OP_DWORD_EL(VPANDD_MASK_VdqHdqWdqR, xmm_andps)
-AVX512_2OP_DWORD_EL(VPANDND_MASK_VdqHdqWdqR, xmm_andnps)
-AVX512_2OP_DWORD_EL(VPORD_MASK_VdqHdqWdqR, xmm_orps)
-AVX512_2OP_DWORD_EL(VPXORD_MASK_VdqHdqWdqR, xmm_xorps)
-AVX512_2OP_DWORD_EL(VPMAXSD_MASK_VdqHdqWdqR, xmm_pmaxsd)
-AVX512_2OP_DWORD_EL(VPMAXUD_MASK_VdqHdqWdqR, xmm_pmaxud)
-AVX512_2OP_DWORD_EL(VPMINSD_MASK_VdqHdqWdqR, xmm_pminsd)
-AVX512_2OP_DWORD_EL(VPMINUD_MASK_VdqHdqWdqR, xmm_pminud)
-AVX512_2OP_DWORD_EL(VUNPCKLPS_MASK_VpsHpsWpsR, xmm_unpcklps)
-AVX512_2OP_DWORD_EL(VUNPCKHPS_MASK_VpsHpsWpsR, xmm_unpckhps)
-AVX512_2OP_DWORD_EL(VPMULLD_MASK_VdqHdqWdqR, xmm_pmulld)
-AVX512_2OP_DWORD_EL(VPSRAVD_MASK_VdqHdqWdqR, xmm_psravd)
-AVX512_2OP_DWORD_EL(VPSRLVD_MASK_VdqHdqWdqR, xmm_psrlvd)
-AVX512_2OP_DWORD_EL(VPSLLVD_MASK_VdqHdqWdqR, xmm_psllvd)
-AVX512_2OP_DWORD_EL(VPRORVD_MASK_VdqHdqWdqR, xmm_prorvd)
-AVX512_2OP_DWORD_EL(VPROLVD_MASK_VdqHdqWdqR, xmm_prolvd)
-AVX512_2OP_DWORD_EL(VPMADDWD_MASK_VdqHdqWdqR, xmm_pmaddwd)
-
-#define AVX512_2OP_WORD_EL(HANDLER, func)                                                   \
-  /* AVX-512 instruction with two src operands working on WORD elements */                  \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    for (unsigned n=0; n < len; n++)                                                        \
-      (func)(&op1.vmm128(n), &op2.vmm128(n));                                               \
-                                                                                            \
-    avx512_write_regw_masked(i, &op1, len, BX_READ_32BIT_OPMASK(i->opmask()));              \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-AVX512_2OP_WORD_EL(VPADDW_MASK_VdqHdqWdqR, xmm_paddw)
-AVX512_2OP_WORD_EL(VPADDSW_MASK_VdqHdqWdqR, xmm_paddsw)
-AVX512_2OP_WORD_EL(VPADDUSW_MASK_VdqHdqWdqR, xmm_paddusw)
-AVX512_2OP_WORD_EL(VPSUBW_MASK_VdqHdqWdqR, xmm_psubw)
-AVX512_2OP_WORD_EL(VPSUBSW_MASK_VdqHdqWdqR, xmm_psubsw)
-AVX512_2OP_WORD_EL(VPSUBUSW_MASK_VdqHdqWdqR, xmm_psubusw)
-AVX512_2OP_WORD_EL(VPMINSW_MASK_VdqHdqWdqR, xmm_pminsw)
-AVX512_2OP_WORD_EL(VPMINUW_MASK_VdqHdqWdqR, xmm_pminuw)
-AVX512_2OP_WORD_EL(VPMAXSW_MASK_VdqHdqWdqR, xmm_pmaxsw)
-AVX512_2OP_WORD_EL(VPMAXUW_MASK_VdqHdqWdqR, xmm_pmaxuw)
-AVX512_2OP_WORD_EL(VPMADDUBSW_MASK_VdqHdqWdqR, xmm_pmaddubsw)
-AVX512_2OP_WORD_EL(VPAVGW_MASK_VdqHdqWdqR, xmm_pavgw)
-AVX512_2OP_WORD_EL(VPMULLW_MASK_VdqHdqWdqR, xmm_pmullw)
-AVX512_2OP_WORD_EL(VPMULHW_MASK_VdqHdqWdqR, xmm_pmulhw)
-AVX512_2OP_WORD_EL(VPMULHUW_MASK_VdqHdqWdqR, xmm_pmulhuw)
-AVX512_2OP_WORD_EL(VPMULHRSW_MASK_VdqHdqWdqR, xmm_pmulhrsw)
-AVX512_2OP_WORD_EL(VPACKSSDW_MASK_VdqHdqWdqR, xmm_packssdw)
-AVX512_2OP_WORD_EL(VPACKUSDW_MASK_VdqHdqWdqR, xmm_packusdw)
-AVX512_2OP_WORD_EL(VPUNPCKLWD_MASK_VdqHdqWdqR, xmm_punpcklwd)
-AVX512_2OP_WORD_EL(VPUNPCKHWD_MASK_VdqHdqWdqR, xmm_punpckhwd)
-AVX512_2OP_WORD_EL(VPSRAVW_MASK_VdqHdqWdqR, xmm_psravw)
-AVX512_2OP_WORD_EL(VPSRLVW_MASK_VdqHdqWdqR, xmm_psrlvw)
-AVX512_2OP_WORD_EL(VPSLLVW_MASK_VdqHdqWdqR, xmm_psllvw)
-
-#define AVX512_2OP_BYTE_EL(HANDLER, func)                                                   \
-  /* AVX-512 instruction with two src operands working on BYTE elements */                  \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)              \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    for (unsigned n=0; n < len; n++)                                                        \
-      (func)(&op1.vmm128(n), &op2.vmm128(n));                                               \
-                                                                                            \
-    avx512_write_regb_masked(i, &op1, len, BX_READ_OPMASK(i->opmask()));                    \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-AVX512_2OP_BYTE_EL(VPADDB_MASK_VdqHdqWdqR, xmm_paddb)
-AVX512_2OP_BYTE_EL(VPADDSB_MASK_VdqHdqWdqR, xmm_paddsb)
-AVX512_2OP_BYTE_EL(VPADDUSB_MASK_VdqHdqWdqR, xmm_paddusb)
-AVX512_2OP_BYTE_EL(VPSUBB_MASK_VdqHdqWdqR, xmm_psubb)
-AVX512_2OP_BYTE_EL(VPSUBSB_MASK_VdqHdqWdqR, xmm_psubsb)
-AVX512_2OP_BYTE_EL(VPSUBUSB_MASK_VdqHdqWdqR, xmm_psubusb)
-AVX512_2OP_BYTE_EL(VPMINSB_MASK_VdqHdqWdqR, xmm_pminsb)
-AVX512_2OP_BYTE_EL(VPMINUB_MASK_VdqHdqWdqR, xmm_pminub)
-AVX512_2OP_BYTE_EL(VPMAXUB_MASK_VdqHdqWdqR, xmm_pmaxub)
-AVX512_2OP_BYTE_EL(VPMAXSB_MASK_VdqHdqWdqR, xmm_pmaxsb)
-AVX512_2OP_BYTE_EL(VPAVGB_MASK_VdqHdqWdqR, xmm_pavgb)
-AVX512_2OP_BYTE_EL(VPACKSSWB_MASK_VdqHdqWdqR, xmm_packsswb)
-AVX512_2OP_BYTE_EL(VPACKUSWB_MASK_VdqHdqWdqR, xmm_packuswb)
-AVX512_2OP_BYTE_EL(VPUNPCKLBW_MASK_VdqHdqWdqR, xmm_punpcklbw)
-AVX512_2OP_BYTE_EL(VPUNPCKHBW_MASK_VdqHdqWdqR, xmm_punpckhbw)
-
-#define AVX512_PSHIFT_WORD_EL(HANDLER, func)                                  \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src1());                     \
-    Bit64u count = BX_READ_XMM_REG_LO_QWORD(i->src2());                       \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), count);                                           \
-                                                                              \
-    avx512_write_regw_masked(i, &op, len, BX_READ_32BIT_OPMASK(i->opmask())); \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_WORD_EL(VPSRLW_MASK_VdqHdqWdqR, xmm_psrlw);
-AVX512_PSHIFT_WORD_EL(VPSRAW_MASK_VdqHdqWdqR, xmm_psraw);
-AVX512_PSHIFT_WORD_EL(VPSLLW_MASK_VdqHdqWdqR, xmm_psllw);
-
-#define AVX512_PSHIFT_DWORD_EL(HANDLER, func)                                 \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src1());                     \
-    Bit64u count = BX_READ_XMM_REG_LO_QWORD(i->src2());                       \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), count);                                           \
-                                                                              \
-    avx512_write_regd_masked(i, &op, len, BX_READ_16BIT_OPMASK(i->opmask())); \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_DWORD_EL(VPSRLD_MASK_VdqHdqWdqR, xmm_psrld);
-AVX512_PSHIFT_DWORD_EL(VPSRAD_MASK_VdqHdqWdqR, xmm_psrad);
-AVX512_PSHIFT_DWORD_EL(VPSLLD_MASK_VdqHdqWdqR, xmm_pslld);
-
-#define AVX512_PSHIFT_QWORD_EL(HANDLER, func)                                 \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src1());                     \
-    Bit64u count = BX_READ_XMM_REG_LO_QWORD(i->src2());                       \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), count);                                           \
-                                                                              \
-    avx512_write_regq_masked(i, &op, len, BX_READ_8BIT_OPMASK(i->opmask()));  \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_QWORD_EL(VPSRLQ_MASK_VdqHdqWdqR, xmm_psrlq);
-AVX512_PSHIFT_QWORD_EL(VPSRAQ_MASK_VdqHdqWdqR, xmm_psraq);
-AVX512_PSHIFT_QWORD_EL(VPSLLQ_MASK_VdqHdqWdqR, xmm_psllq);
-
-#define AVX512_PSHIFT_IMM_WORD_EL(HANDLER, func)                              \
-  /* AVX packed shift with imm8 instruction */                                \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src());                      \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), i->Ib());                                         \
-                                                                              \
-    avx512_write_regw_masked(i, &op, len, BX_READ_32BIT_OPMASK(i->opmask())); \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_IMM_WORD_EL(VPSRLW_MASK_UdqIb, xmm_psrlw);
-AVX512_PSHIFT_IMM_WORD_EL(VPSRAW_MASK_UdqIb, xmm_psraw);
-AVX512_PSHIFT_IMM_WORD_EL(VPSLLW_MASK_UdqIb, xmm_psllw);
-
-#define AVX512_PSHIFT_IMM_DWORD_EL(HANDLER, func)                             \
-  /* AVX packed shift with imm8 instruction */                                \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src());                      \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), i->Ib());                                         \
-                                                                              \
-    avx512_write_regd_masked(i, &op, len, BX_READ_16BIT_OPMASK(i->opmask())); \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_IMM_DWORD_EL(VPSRLD_MASK_UdqIb, xmm_psrld);
-AVX512_PSHIFT_IMM_DWORD_EL(VPSRAD_MASK_UdqIb, xmm_psrad);
-AVX512_PSHIFT_IMM_DWORD_EL(VPSLLD_MASK_UdqIb, xmm_pslld);
-AVX512_PSHIFT_IMM_DWORD_EL(VPRORD_MASK_UdqIb, xmm_prord);
-AVX512_PSHIFT_IMM_DWORD_EL(VPROLD_MASK_UdqIb, xmm_prold);
-
-#define AVX512_PSHIFT_IMM_QWORD_EL(HANDLER, func)                             \
-  /* AVX packed shift with imm8 instruction */                                \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i) \
-  {                                                                           \
-    BxPackedAvxRegister op  = BX_READ_AVX_REG(i->src());                      \
-    unsigned len = i->getVL();                                                \
-                                                                              \
-    for (unsigned n=0; n < len; n++)                                          \
-      (func)(&op.vmm128(n), i->Ib());                                         \
-                                                                              \
-    avx512_write_regq_masked(i, &op, len, BX_READ_8BIT_OPMASK(i->opmask()));  \
-                                                                              \
-    BX_NEXT_INSTR(i);                                                         \
-  }
-
-AVX512_PSHIFT_IMM_QWORD_EL(VPSRLQ_MASK_UdqIb, xmm_psrlq);
-AVX512_PSHIFT_IMM_QWORD_EL(VPSRAQ_MASK_UdqIb, xmm_psraq);
-AVX512_PSHIFT_IMM_QWORD_EL(VPSLLQ_MASK_UdqIb, xmm_psllq);
-AVX512_PSHIFT_IMM_QWORD_EL(VPRORQ_MASK_UdqIb, xmm_prorq);
-AVX512_PSHIFT_IMM_QWORD_EL(VPROLQ_MASK_UdqIb, xmm_prolq);
-
 // concatenate and shift
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPSHLDW_MASK_VdqHdqWdqIbR(bxInstruction_c *i)
@@ -1003,32 +742,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSHUFPD_MASK_VpdHpdWpdIbR(bxInstruction_c
   BX_NEXT_INSTR(i);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPERMILPS_MASK_VpsHpsWpsR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1());
-  BxPackedAvxRegister op2 = BX_READ_AVX_REG(i->src2()), result;
-  unsigned len = i->getVL();
-
-  for (unsigned n=0; n < len; n++)
-    xmm_permilps(&result.vmm128(n), &op1.vmm128(n), &op2.vmm128(n));
-
-  avx512_write_regd_masked(i, &result, len, BX_READ_16BIT_OPMASK(i->opmask()));
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPERMILPD_MASK_VpdHpdWpdR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1());
-  BxPackedAvxRegister op2 = BX_READ_AVX_REG(i->src2()), result;
-  unsigned len = i->getVL();
-
-  for (unsigned n=0; n < len; n++)
-    xmm_permilpd(&result.vmm128(n), &op1.vmm128(n), &op2.vmm128(n));
-
-  avx512_write_regq_masked(i, &result, len, BX_READ_8BIT_OPMASK(i->opmask()));
-  BX_NEXT_INSTR(i);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPERMILPS_MASK_VpsWpsIbR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src()), result;
@@ -1454,7 +1167,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VINSERTF32x4_MASK_VpsHpsWpsIbR(bxInstructi
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src1());
   unsigned len = i->getVL();
   unsigned offset = i->Ib() & (len-1);
-  
+
   op.vmm128(offset) = BX_READ_XMM_REG(i->src2());
   avx512_write_regd_masked(i, &op, len, BX_READ_16BIT_OPMASK(i->opmask()));
   BX_NEXT_INSTR(i);
@@ -1465,7 +1178,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VINSERTF64x2_MASK_VpdHpdWpdIbR(bxInstructi
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src1());
   unsigned len = i->getVL();
   unsigned offset = i->Ib() & (len-1);
-  
+
   op.vmm128(offset) = BX_READ_XMM_REG(i->src2());
   avx512_write_regq_masked(i, &op, len, BX_READ_8BIT_OPMASK(i->opmask()));
 
@@ -1611,9 +1324,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VEXTRACTF32x8_MASK_WpsVpsIbM(bxInstruction
   if (i->Ib() & 0x1)
     op.vmm256(0) = op.vmm256(1);
 
-  Bit32u opmask = BX_READ_8BIT_OPMASK(i->opmask());
   bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
-  avx_masked_store32(i, eaddr, &op, opmask);
+  avx_masked_store32(i, eaddr, &op, BX_READ_8BIT_OPMASK(i->opmask()));
   BX_NEXT_INSTR(i);
 }
 
@@ -1697,7 +1409,7 @@ BX_CPP_INLINE Bit64u ternlogq_scalar(Bit64u op1, Bit64u op2, Bit64u op3, unsigne
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGD_VdqHdqWdqIbR(bxInstruction_c *i)
 {
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()), 
+  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()),
                       op2 = BX_READ_AVX_REG(i->src1()),
                       op3 = BX_READ_AVX_REG(i->src2());
 
@@ -1714,7 +1426,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGD_VdqHdqWdqIbR(bxInstruction_c *i
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGD_MASK_VdqHdqWdqIbR(bxInstruction_c *i)
 {
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()), 
+  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()),
                       op2 = BX_READ_AVX_REG(i->src1()),
                       op3 = BX_READ_AVX_REG(i->src2());
 
@@ -1736,7 +1448,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGD_MASK_VdqHdqWdqIbR(bxInstruction
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGQ_VdqHdqWdqIbR(bxInstruction_c *i)
 {
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()), 
+  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()),
                       op2 = BX_READ_AVX_REG(i->src1()),
                       op3 = BX_READ_AVX_REG(i->src2());
 
@@ -1753,7 +1465,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGQ_VdqHdqWdqIbR(bxInstruction_c *i
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPTERNLOGQ_MASK_VdqHdqWdqIbR(bxInstruction_c *i)
 {
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()), 
+  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->dst()),
                       op2 = BX_READ_AVX_REG(i->src1()),
                       op3 = BX_READ_AVX_REG(i->src2());
 
@@ -1860,136 +1572,179 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VBLENDMPD_MASK_VpdHpdWpdR(bxInstruction_c
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPEXPANDB_MASK_VdqWdqR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
 
-  Bit64u opmask = BX_READ_OPMASK(i->opmask()), mask = opmask;
-  unsigned len = i->getVL(), n = 0, k = 0;
+  Bit64u opmask = BX_READ_OPMASK(i->opmask());
+  unsigned len = i->getVL(), k = 0;
 
-  for (; n < len*16; n++, mask >>= 1) {
-    if (mask & 0x1) {
+  for (unsigned n = 0; n < len*16; n++, opmask >>= 1) {
+    if (! opmask) break;
+
+    if (opmask & 0x1) {
       result.vmmubyte(n) = op.vmmubyte(k);
       k++;
     }
-    else {
-      result.vmmubyte(n) = 0;
-    }
-  }
-
-  if (i->isZeroMasking()) {
-    BX_WRITE_AVX_REGZ(i->dst(), result, len);
-  }
-  else {
-    for (unsigned n=0; n < len; n++, opmask >>= 16)
-      xmm_pblendb(&BX_READ_AVX_REG_LANE(i->dst(), n), &result.vmm128(n), opmask);
-
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPEXPANDW_MASK_VdqWdqR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
 
-  Bit32u opmask = BX_READ_32BIT_OPMASK(i->opmask()), mask = opmask;
-  unsigned len = i->getVL(), n = 0, k = 0;
+  Bit32u opmask = BX_READ_32BIT_OPMASK(i->opmask());
+  unsigned len = i->getVL(), k = 0;
 
-  for (; n < len*8; n++, mask >>= 1) {
-    if (mask & 0x1) {
+  for (unsigned n = 0; n < len*8; n++, opmask >>= 1) {
+    if (! opmask) break;
+
+    if (opmask & 0x1) {
       result.vmm16u(n) = op.vmm16u(k);
       k++;
     }
-    else {
-      result.vmm16u(n) = 0;
+  }
+
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VEXPANDPS_MASK_VpsWpsM(bxInstruction_c *i)
+{
+  BxPackedAvxRegister result, op;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
+
+  Bit32u opmask = BX_READ_16BIT_OPMASK(i->opmask());
+  unsigned len = i->getVL();
+  opmask &= CUT_OPMASK_TO(DWORD_ELEMENTS(len));
+
+  if (opmask) {
+    // the EXPAND is going to read an element for each bit set to '1 in the opmask
+    // and place it into the element corresponding to the opmask bit in the result
+    // so it will read popcntw(opmask) bits from the source
+    Bit32u load_mask = (1 << popcntw(opmask)) - 1;
+    avx_masked_load32(i, BX_CPU_RESOLVE_ADDR(i), &op, load_mask); // read only popcntw(opmask) elements from the memory
+
+    for (unsigned n = 0, k = 0; n < len*4; n++, opmask >>= 1) {
+      if (! opmask) break;
+
+      if (opmask & 0x1) {
+        result.vmm32u(n) = op.vmm32u(k);
+        k++;
+      }
     }
   }
 
-  if (i->isZeroMasking()) {
-    BX_WRITE_AVX_REGZ(i->dst(), result, len);
-  }
-  else {
-    for (unsigned n=0; n < len; n++, opmask >>= 8)
-      xmm_pblendw(&BX_READ_AVX_REG_LANE(i->dst(), n), &result.vmm128(n), opmask);
-
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VEXPANDPS_MASK_VpsWpsR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
 
-  Bit32u opmask = BX_READ_16BIT_OPMASK(i->opmask()), mask = opmask;
-  unsigned len = i->getVL(), n = 0, k = 0;
+  Bit32u opmask = BX_READ_16BIT_OPMASK(i->opmask());
+  unsigned len = i->getVL(), k = 0;
 
-  for (; n < len*4; n++, mask >>= 1) {
-    if (mask & 0x1) {
+  for (unsigned n = 0; n < len*4; n++, opmask >>= 1) {
+    if (! opmask) break;
+
+    if (opmask & 0x1) {
       result.vmm32u(n) = op.vmm32u(k);
       k++;
     }
-    else {
-      result.vmm32u(n) = 0;
+  }
+
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
+  BX_NEXT_INSTR(i);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VEXPANDPD_MASK_VpdWpdM(bxInstruction_c *i)
+{
+  BxPackedAvxRegister result, op;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
+
+  Bit32u opmask = BX_READ_8BIT_OPMASK(i->opmask());
+  unsigned len = i->getVL();
+  opmask &= CUT_OPMASK_TO(QWORD_ELEMENTS(len));
+
+  if (opmask) {
+    // the EXPAND is going to read an element for each bit set to '1 in the opmask
+    // and place it into the element corresponding to the opmask bit in the result
+    // so it will read popcntb(opmask) bits from the source
+    Bit32u load_mask = (1 << popcntb(opmask)) - 1;
+    avx_masked_load64(i, BX_CPU_RESOLVE_ADDR(i), &op, load_mask); // read only popcntb(opmask) elements from the memory
+
+    for (unsigned n = 0, k = 0; n < len*2; n++, opmask >>= 1) {
+      if (! opmask) break;
+
+      if (opmask & 0x1) {
+        result.vmm64u(n) = op.vmm64u(k);
+        k++;
+      }
     }
   }
 
-  if (i->isZeroMasking()) {
-    BX_WRITE_AVX_REGZ(i->dst(), result, len);
-  }
-  else {
-    for (unsigned n=0; n < len; n++, opmask >>= 4)
-      xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &result.vmm128(n), opmask);
-
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VEXPANDPD_MASK_VpdWpdR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
+  if (i->isZeroMasking())
+    result.clear();
+  else
+    result = BX_READ_AVX_REG(i->dst());
 
-  Bit32u opmask = BX_READ_8BIT_OPMASK(i->opmask()), mask = opmask;
-  unsigned len = i->getVL(), n = 0, k = 0;
+  Bit32u opmask = BX_READ_8BIT_OPMASK(i->opmask());
+  unsigned len = i->getVL();
 
-  for (; n < len*2; n++, mask >>= 1) {
-    if (mask & 0x1) {
+  for (unsigned n = 0, k = 0; n < len*2; n++, opmask >>= 1) {
+    if (! opmask) break;
+
+    if (opmask & 0x1) {
       result.vmm64u(n) = op.vmm64u(k);
       k++;
     }
-    else {
-      result.vmm64u(n) = 0;
-    }
-  }
-
-  if (i->isZeroMasking()) {
-    BX_WRITE_AVX_REGZ(i->dst(), result, len);
-  }
-  else {
-    for (unsigned n=0; n < len; n++, opmask >>= 2)
-      xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &result.vmm128(n), opmask);
-
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
   }
 
+  BX_WRITE_AVX_REGZ(i->dst(), result, len);
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPCOMPRESSB_MASK_WdqVdq(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
-  
+
   Bit64u opmask = BX_READ_OPMASK(i->opmask());
   unsigned len = i->getVL(), n = 0, k = 0;
 
   for (; n < len*16; n++, opmask >>= 1) {
+    if (! opmask) break;
+
     if (opmask & 0x1) {
       result.vmmubyte(k) = op.vmmubyte(n);
       k++;
     }
-    if (! opmask) break;
   }
 
   Bit64u writemask = (BX_CONST64(1) << k) - 1;
@@ -2008,16 +1763,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPCOMPRESSB_MASK_WdqVdq(bxInstruction_c *i
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPCOMPRESSW_MASK_WdqVdq(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
-  
+
   Bit32u opmask = BX_READ_32BIT_OPMASK(i->opmask());
   unsigned len = i->getVL(), n = 0, k = 0;
 
   for (; n < len*8; n++, opmask >>= 1) {
+    if (! opmask) break;
+
     if (opmask & 0x1) {
       result.vmm16u(k) = op.vmm16u(n);
       k++;
     }
-    if (! opmask) break;
   }
 
   Bit32u writemask = (1 << k) - 1;
@@ -2036,16 +1792,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPCOMPRESSW_MASK_WdqVdq(bxInstruction_c *i
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCOMPRESSPS_MASK_WpsVps(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
-  
+
   Bit32u opmask = BX_READ_16BIT_OPMASK(i->opmask());
   unsigned len = i->getVL(), n = 0, k = 0;
 
   for (; n < len*4; n++, opmask >>= 1) {
+    if (! opmask) break;
+
     if (opmask & 0x1) {
       result.vmm32u(k) = op.vmm32u(n);
       k++;
     }
-    if (! opmask) break;
   }
 
   Bit32u writemask = (1 << k) - 1;
@@ -2064,16 +1821,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCOMPRESSPS_MASK_WpsVps(bxInstruction_c *i
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VCOMPRESSPD_MASK_WpdVpd(bxInstruction_c *i)
 {
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src()), result;
-  
+
   Bit32u opmask = BX_READ_8BIT_OPMASK(i->opmask());
   unsigned len = i->getVL(), n = 0, k = 0;
 
   for (; n < len*2; n++, opmask >>= 1) {
+    if (! opmask) break;
+
     if (opmask & 0x1) {
       result.vmm64u(k) = op.vmm64u(n);
       k++;
     }
-    if (! opmask) break;
   }
 
   Bit32u writemask = (1 << k) - 1;
@@ -2271,89 +2029,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMULTISHIFTQB_MASK_VdqHdqWdqR(bxInstructi
   BX_NEXT_INSTR(i);
 }
 
-// 52-bit integer FMA
-
-BX_CPP_INLINE Bit64u pmadd52luq_scalar(Bit64u dst, Bit64u op1, Bit64u op2)
-{
-  op1 &= BX_CONST64(0x000fffffffffffff);
-  op2 &= BX_CONST64(0x000fffffffffffff);
-
-  return dst + ((op1 * op2) & BX_CONST64(0x000fffffffffffff));
-}
-
-BX_CPP_INLINE Bit64u pmadd52huq_scalar(Bit64u dst, Bit64u op1, Bit64u op2)
-{
-  op1 &= BX_CONST64(0x000fffffffffffff);
-  op2 &= BX_CONST64(0x000fffffffffffff);
-
-  Bit128u product_128;
-  long_mul(&product_128, op1, op2);
-
-  Bit64u temp = (product_128.lo >> 52) | ((product_128.hi & BX_CONST64(0x000000ffffffffff)) << 12);
-
-  return dst + temp;
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMADD52LUQ_VdqHdqWdqR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()), dst = BX_READ_AVX_REG(i->dst());
-  unsigned len = i->getVL();
-
-  for (unsigned n=0; n < QWORD_ELEMENTS(len); n++) {
-    dst.vmm64u(n) = pmadd52luq_scalar(dst.vmm64u(n), op1.vmm64u(n), op2.vmm64u(n));
-  }
-
-  BX_WRITE_AVX_REGZ(i->dst(), dst, len);
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMADD52LUQ_MASK_VdqHdqWdqR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()), dst = BX_READ_AVX_REG(i->dst());
-  Bit32u mask = BX_READ_8BIT_OPMASK(i->opmask());
-  unsigned len = i->getVL();
-
-  for (unsigned n=0, tmp_mask = mask; n < QWORD_ELEMENTS(len); n++, tmp_mask >>= 1) {
-    if (tmp_mask & 0x1)
-      dst.vmm64u(n) = pmadd52luq_scalar(dst.vmm64u(n), op1.vmm64u(n), op2.vmm64u(n));
-    else if (i->isZeroMasking())
-      dst.vmm64u(n) = 0;
-  }
-
-  BX_WRITE_AVX_REGZ(i->dst(), dst, len);
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMADD52HUQ_VdqHdqWdqR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()), dst = BX_READ_AVX_REG(i->dst());
-  unsigned len = i->getVL();
-
-  for (unsigned n=0; n < QWORD_ELEMENTS(len); n++) {
-    dst.vmm64u(n) = pmadd52huq_scalar(dst.vmm64u(n), op1.vmm64u(n), op2.vmm64u(n));
-  }
-
-  BX_WRITE_AVX_REGZ(i->dst(), dst, len);
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMADD52HUQ_MASK_VdqHdqWdqR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()), dst = BX_READ_AVX_REG(i->dst());
-  Bit32u mask = BX_READ_8BIT_OPMASK(i->opmask());
-  unsigned len = i->getVL();
-
-  for (unsigned n=0, tmp_mask = mask; n < QWORD_ELEMENTS(len); n++, tmp_mask >>= 1) {
-    if (tmp_mask & 0x1)
-      dst.vmm64u(n) = pmadd52huq_scalar(dst.vmm64u(n), op1.vmm64u(n), op2.vmm64u(n));
-    else if (i->isZeroMasking())
-      dst.vmm64u(n) = 0;
-  }
-
-  BX_WRITE_AVX_REGZ(i->dst(), dst, len);
-  BX_NEXT_INSTR(i);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VP2INTERSECTD_KGqHdqWdqR(bxInstruction_c *i)
 {
   BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2());
