@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: avx512_pfp.cc 13466 2018-02-16 07:57:32Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2013-2018 Stanislav Shwartsman
+//   Copyright (c) 2013-2023 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -29,83 +29,12 @@
 #if BX_SUPPORT_EVEX
 
 extern float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr);
+extern void mxcsr_to_softfloat_status_word_imm_override(float_status_t &status, Bit8u immb);
 
 #include "fpu/softfloat-compare.h"
 #include "simd_int.h"
 #include "simd_pfp.h"
 
-#define EVEX_OP_PACKED_SINGLE(HANDLER, func)                                                \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)                       \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned mask = BX_READ_16BIT_OPMASK(i->opmask());                                      \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                          \
-    softfloat_status_word_rc_override(status, i);                                           \
-                                                                                            \
-    for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 4)                       \
-      (func)(&op1.vmm128(n), &op2.vmm128(n), status, tmp_mask);                             \
-                                                                                            \
-    check_exceptionsSSE(get_exception_flags(status));                                       \
-                                                                                            \
-    if (! i->isZeroMasking()) {                                                             \
-      for (unsigned n=0; n < len; n++, mask >>= 4)                                          \
-        xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &op1.vmm128(n), mask);              \
-                                                                                            \
-      BX_CLEAR_AVX_REGZ(i->dst(), len);                                                     \
-    }                                                                                       \
-    else {                                                                                  \
-      BX_WRITE_AVX_REGZ(i->dst(), op1, len);                                                \
-    }                                                                                       \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-EVEX_OP_PACKED_SINGLE(VADDPS_MASK_VpsHpsWpsR, xmm_addps_mask)
-EVEX_OP_PACKED_SINGLE(VSUBPS_MASK_VpsHpsWpsR, xmm_subps_mask)
-EVEX_OP_PACKED_SINGLE(VMULPS_MASK_VpsHpsWpsR, xmm_mulps_mask)
-EVEX_OP_PACKED_SINGLE(VDIVPS_MASK_VpsHpsWpsR, xmm_divps_mask)
-EVEX_OP_PACKED_SINGLE(VMAXPS_MASK_VpsHpsWpsR, xmm_maxps_mask)
-EVEX_OP_PACKED_SINGLE(VMINPS_MASK_VpsHpsWpsR, xmm_minps_mask)
-EVEX_OP_PACKED_SINGLE(VSCALEFPS_MASK_VpsHpsWpsR, xmm_scalefps_mask)
-
-#define EVEX_OP_PACKED_DOUBLE(HANDLER, func)                                                \
-  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)                       \
-  {                                                                                         \
-    BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2()); \
-    unsigned mask = BX_READ_8BIT_OPMASK(i->opmask());                                       \
-    unsigned len = i->getVL();                                                              \
-                                                                                            \
-    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                          \
-    softfloat_status_word_rc_override(status, i);                                           \
-                                                                                            \
-    for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 2)                       \
-      (func)(&op1.vmm128(n), &op2.vmm128(n), status, tmp_mask);                             \
-                                                                                            \
-    check_exceptionsSSE(get_exception_flags(status));                                       \
-                                                                                            \
-    if (! i->isZeroMasking()) {                                                             \
-      for (unsigned n=0; n < len; n++, mask >>= 2)                                          \
-        xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &op1.vmm128(n), mask);              \
-                                                                                            \
-      BX_CLEAR_AVX_REGZ(i->dst(), len);                                                     \
-    }                                                                                       \
-    else {                                                                                  \
-      BX_WRITE_AVX_REGZ(i->dst(), op1, len);                                                \
-    }                                                                                       \
-                                                                                            \
-    BX_NEXT_INSTR(i);                                                                       \
-  }
-
-EVEX_OP_PACKED_DOUBLE(VADDPD_MASK_VpdHpdWpdR, xmm_addpd_mask)
-EVEX_OP_PACKED_DOUBLE(VSUBPD_MASK_VpdHpdWpdR, xmm_subpd_mask)
-EVEX_OP_PACKED_DOUBLE(VMULPD_MASK_VpdHpdWpdR, xmm_mulpd_mask)
-EVEX_OP_PACKED_DOUBLE(VDIVPD_MASK_VpdHpdWpdR, xmm_divpd_mask)
-EVEX_OP_PACKED_DOUBLE(VMAXPD_MASK_VpdHpdWpdR, xmm_maxpd_mask)
-EVEX_OP_PACKED_DOUBLE(VMINPD_MASK_VpdHpdWpdR, xmm_minpd_mask)
-EVEX_OP_PACKED_DOUBLE(VSCALEFPD_MASK_VpdHpdWpdR, xmm_scalefpd_mask)
-
 #define EVEX_OP_SCALAR_SINGLE(HANDLER, func)                                                \
   void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)                       \
   {                                                                                         \
@@ -170,58 +99,6 @@ EVEX_OP_SCALAR_DOUBLE(VMINSD_MASK_VsdHpdWsdR, float64_min)
 EVEX_OP_SCALAR_DOUBLE(VMAXSD_MASK_VsdHpdWsdR, float64_max)
 EVEX_OP_SCALAR_DOUBLE(VSCALEFSD_MASK_VsdHpdWsdR, float64_scalef)
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSQRTPS_MASK_VpsWpsR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op = BX_READ_AVX_REG(i->src());
-  unsigned mask = BX_READ_16BIT_OPMASK(i->opmask());
-  unsigned len = i->getVL();
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 4)
-    xmm_sqrtps_mask(&op.vmm128(n), status, tmp_mask);
-
-  check_exceptionsSSE(get_exception_flags(status));
-
-  if (! i->isZeroMasking()) {
-    for (unsigned n=0; n < len; n++, mask >>= 4)
-      xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &op.vmm128(n), mask);
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-  else {
-    BX_WRITE_AVX_REGZ(i->dst(), op, len);
-  }
-
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSQRTPD_MASK_VpdWpdR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op = BX_READ_AVX_REG(i->src());
-  unsigned mask = BX_READ_8BIT_OPMASK(i->opmask());
-  unsigned len = i->getVL();
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 2)
-    xmm_sqrtpd_mask(&op.vmm128(n), status, tmp_mask);
-
-  check_exceptionsSSE(get_exception_flags(status));
-
-  if (! i->isZeroMasking()) {
-    for (unsigned n=0; n < len; n++, mask >>= 2)
-      xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &op.vmm128(n), mask);
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-  else {
-    BX_WRITE_AVX_REGZ(i->dst(), op, len);
-  }
-
-  BX_NEXT_INSTR(i);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSQRTSS_MASK_VssHpsWssR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
@@ -732,7 +609,7 @@ static int fpclass(float_class_t op_class, int sign, int selector)
 
 static BX_CPP_INLINE int float32_fpclass(float32 op, int selector, int daz)
 {
-  if (daz) 
+  if (daz)
     op = float32_denormal_to_zero(op);
 
   return fpclass(float32_class(op), float32_sign(op), selector);
@@ -740,7 +617,7 @@ static BX_CPP_INLINE int float32_fpclass(float32 op, int selector, int daz)
 
 static BX_CPP_INLINE int float64_fpclass(float64 op, int selector, int daz)
 {
-  if (daz) 
+  if (daz)
     op = float64_denormal_to_zero(op);
 
   return fpclass(float64_class(op), float64_sign(op), selector);
@@ -810,55 +687,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VFPCLASSSD_MASK_KGbWsdIbR(bxInstruction_c
 
 // getexp
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPPS_MASK_VpsWpsR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPSS_VssHpsWssR(bxInstruction_c *i)
 {
-  BxPackedAvxRegister op = BX_READ_AVX_REG(i->src());
-  Bit32u mask = i->opmask() ? BX_READ_16BIT_OPMASK(i->opmask()) : (Bit32u) -1;
-  unsigned len = i->getVL();
+  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
 
+  float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 4)
-    xmm_getexpps_mask(&op.vmm128(n), status, tmp_mask);
-
+  op1.xmm32u(0) = float32_getexp(op2, status);
   check_exceptionsSSE(get_exception_flags(status));
 
-  if (! i->isZeroMasking()) {
-    for (unsigned n=0; n < len; n++, mask >>= 4)
-      xmm_blendps(&BX_READ_AVX_REG_LANE(i->dst(), n), &op.vmm128(n), mask);
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-  else {
-    BX_WRITE_AVX_REGZ(i->dst(), op, len);
-  }
-
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPPD_MASK_VpdWpdR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op = BX_READ_AVX_REG(i->src());
-  Bit32u mask = i->opmask() ? BX_READ_8BIT_OPMASK(i->opmask()) : (Bit32u) -1;
-  unsigned len = i->getVL();
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0, tmp_mask = mask; n < len; n++, tmp_mask >>= 2)
-    xmm_getexppd_mask(&op.vmm128(n), status, tmp_mask);
-
-  check_exceptionsSSE(get_exception_flags(status));
-
-  if (! i->isZeroMasking()) {
-    for (unsigned n=0; n < len; n++, mask >>= 2)
-      xmm_blendpd(&BX_READ_AVX_REG_LANE(i->dst(), n), &op.vmm128(n), mask);
-    BX_CLEAR_AVX_REGZ(i->dst(), len);
-  }
-  else {
-    BX_WRITE_AVX_REGZ(i->dst(), op, len);
-  }
-
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
   BX_NEXT_INSTR(i);
 }
 
@@ -885,6 +724,20 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPSS_MASK_VssHpsWssR(bxInstruction_c
   BX_NEXT_INSTR(i);
 }
 
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPSD_VsdHpdWsdR(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
+
+  float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());
+  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_word_rc_override(status, i);
+  op1.xmm64u(0) = float64_getexp(op2, status);
+  check_exceptionsSSE(get_exception_flags(status));
+
+  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
+  BX_NEXT_INSTR(i);
+}
+
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VGETEXPSD_MASK_VsdHpdWsdR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
@@ -1035,18 +888,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRNDSCALEPS_MASK_VpsWpsIbR(bxInstruction_c
   BxPackedAvxRegister op = BX_READ_AVX_REG(i->src());
   Bit32u opmask = i->opmask() ? BX_READ_16BIT_OPMASK(i->opmask()) : (Bit32u) -1;
   unsigned len = i->getVL();
+  Bit8u control = i->Ib(), scale = control >> 4;
 
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
   softfloat_status_word_rc_override(status, i);
-
-  Bit8u control = i->Ib(), scale = control >> 4;
-
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  mxcsr_to_softfloat_status_word_imm_override(status, control);
 
   for (unsigned n=0, mask = 0x1; n < DWORD_ELEMENTS(len); n++, mask <<= 1) {
     if (opmask & mask)
@@ -1080,13 +926,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRNDSCALESS_MASK_VssHpsWssIbR(bxInstructio
 
     float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
     softfloat_status_word_rc_override(status, i);
-
-    // override MXCSR rounding mode with control coming from imm8
-    if ((control & 0x4) == 0)
-      status.float_rounding_mode = control & 0x3;
-    // ignore precision exception result
-    if (control & 0x8)
-      status.float_suppress_exception |= float_flag_inexact;
+    mxcsr_to_softfloat_status_word_imm_override(status, control);
 
     op1.xmm32u(0) = float32_round_to_int(op2, scale, status);
 
@@ -1109,17 +949,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRNDSCALEPD_MASK_VpdWpdIbR(bxInstruction_c
   Bit32u opmask = i->opmask() ? BX_READ_8BIT_OPMASK(i->opmask()) : (Bit32u) -1;
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
   Bit8u control = i->Ib(), scale = control >> 4;
 
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_word_rc_override(status, i);
+  mxcsr_to_softfloat_status_word_imm_override(status, control);
 
   for (unsigned n=0, mask = 0x1; n < QWORD_ELEMENTS(len); n++, mask <<= 1) {
     if (opmask & mask)
@@ -1153,13 +987,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRNDSCALESD_MASK_VsdHpdWsdIbR(bxInstructio
 
     float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
     softfloat_status_word_rc_override(status, i);
-
-    // override MXCSR rounding mode with control coming from imm8
-    if ((control & 0x4) == 0)
-      status.float_rounding_mode = control & 0x3;
-    // ignore precision exception result
-    if (control & 0x8)
-      status.float_suppress_exception |= float_flag_inexact;
+    mxcsr_to_softfloat_status_word_imm_override(status, control);
 
     op1.xmm64u(0) = float64_round_to_int(op2, scale, status);
 
@@ -1176,78 +1004,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VRNDSCALESD_MASK_VsdHpdWsdIbR(bxInstructio
   BX_NEXT_INSTR(i);
 }
 
-// scalef
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSCALEFPS_VpsHpsWpsR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2());
-  unsigned len = i->getVL();
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0; n < len; n++) {
-    xmm_scalefps(&op1.vmm128(n), &op2.vmm128(n), status);
-  }
-
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_AVX_REGZ(i->dst(), op1, len);
-
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSCALEFPD_VpdHpdWpdR(bxInstruction_c *i)
-{
-  BxPackedAvxRegister op1 = BX_READ_AVX_REG(i->src1()), op2 = BX_READ_AVX_REG(i->src2());
-  unsigned len = i->getVL();
-  
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  for (unsigned n=0; n < len; n++) {
-    xmm_scalefpd(&op1.vmm128(n), &op2.vmm128(n), status);
-  }
-
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_AVX_REGZ(i->dst(), op1, len);
-
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSCALEFSS_VssHpsWssR(bxInstruction_c *i)
-{
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
-  float32 op2 = BX_READ_XMM_REG_LO_DWORD(i->src2());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  op1.xmm32u(0) = float32_scalef(op1.xmm32u(0), op2, status);
-
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
-
-  BX_NEXT_INSTR(i);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::VSCALEFSD_VsdHpdWsdR(bxInstruction_c *i)
-{
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
-  float64 op2 = BX_READ_XMM_REG_LO_QWORD(i->src2());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
-  op1.xmm64u(0) = float64_scalef(op1.xmm64u(0), op2, status);
-
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_CLEAR_HIGH(i->dst(), op1);
-
-  BX_NEXT_INSTR(i);
-}
-
 // range
 
 static BX_CPP_INLINE float32 float32_range(float32 a, float32 b, int opselect, int sign_ctrl, float_status_t &status)
@@ -1434,17 +1190,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VREDUCEPS_MASK_VpsWpsIbR(bxInstruction_c *
   Bit32u opmask = i->opmask() ? BX_READ_16BIT_OPMASK(i->opmask()) : (Bit32u) -1;
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
   Bit8u control = i->Ib(), scale = control >> 4;
 
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_word_rc_override(status, i);
+  mxcsr_to_softfloat_status_word_imm_override(status, control);
+  status.float_suppress_exception |= float_flag_denormal | float_flag_underflow | float_flag_overflow;
 
   for (unsigned n=0, mask = 0x1; n < DWORD_ELEMENTS(len); n++, mask <<= 1) {
     if (opmask & mask)
@@ -1478,13 +1229,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VREDUCESS_MASK_VssHpsWssIbR(bxInstruction_
 
     float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
     softfloat_status_word_rc_override(status, i);
-
-    // override MXCSR rounding mode with control coming from imm8
-    if ((control & 0x4) == 0)
-      status.float_rounding_mode = control & 0x3;
-    // ignore precision exception result
-    if (control & 0x8)
-      status.float_suppress_exception |= float_flag_inexact;
+    mxcsr_to_softfloat_status_word_imm_override(status, control);
+    status.float_suppress_exception |= float_flag_denormal | float_flag_underflow | float_flag_overflow;
 
     op1.xmm32u(0) = float32_reduce(op2, scale, status);
 
@@ -1507,17 +1253,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VREDUCEPD_MASK_VpdWpdIbR(bxInstruction_c *
   Bit32u opmask = i->opmask() ? BX_READ_8BIT_OPMASK(i->opmask()) : (Bit32u) -1;
   unsigned len = i->getVL();
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  softfloat_status_word_rc_override(status, i);
-
   Bit8u control = i->Ib(), scale = control >> 4;
 
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
+  softfloat_status_word_rc_override(status, i);
+  mxcsr_to_softfloat_status_word_imm_override(status, control);
+  status.float_suppress_exception |= float_flag_denormal | float_flag_underflow | float_flag_overflow;
 
   for (unsigned n=0, mask = 0x1; n < QWORD_ELEMENTS(len); n++, mask <<= 1) {
     if (opmask & mask)
@@ -1551,13 +1292,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VREDUCESD_MASK_VsdHpdWsdIbR(bxInstruction_
 
     float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
     softfloat_status_word_rc_override(status, i);
-
-    // override MXCSR rounding mode with control coming from imm8
-    if ((control & 0x4) == 0)
-      status.float_rounding_mode = control & 0x3;
-    // ignore precision exception result
-    if (control & 0x8)
-      status.float_suppress_exception |= float_flag_inexact;
+    mxcsr_to_softfloat_status_word_imm_override(status, control);
+    status.float_suppress_exception |= float_flag_denormal | float_flag_underflow | float_flag_overflow;
 
     op1.xmm64u(0) = float64_reduce(op2, scale, status);
 
