@@ -1,8 +1,8 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: call_far.cc 13699 2019-12-20 07:42:07Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2005-2019 Stanislav Shwartsman
+//   Copyright (c) 2005-2024 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -62,7 +62,7 @@ BX_CPU_C::call_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
 
 #if BX_SUPPORT_X86_64
     if (long_mode() && cs_descriptor.u.segment.l) {
-      Bit64u temp_rsp = RSP; 
+      Bit64u temp_rsp = RSP;
       // moving to long mode, push return address onto 64-bit stack
       if (i->os64L()) {
         write_new_stack_qword(temp_rsp -  8, cs_descriptor.dpl,
@@ -528,8 +528,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::call_gate64(bx_selector_t *gate_selector)
   parse_descriptor(dword1, dword2, &cs_descriptor);
 
   // find the RIP in the gate_descriptor
-  Bit64u new_RIP = gate_descriptor.u.gate.dest_offset;
-  new_RIP |= ((Bit64u)dword3 << 32);
+  Bit64u new_RIP = GET64_FROM_HI32_LO32(dword3, gate_descriptor.u.gate.dest_offset);
 
   // AR byte of selected descriptor must indicate code segment,
   //   else #GP(code segment selector)
@@ -650,6 +649,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::shadow_stack_switch(bx_address new_SSP)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::call_far_shadow_stack_push(Bit16u cs, bx_address lip, bx_address old_ssp)
 {
+#if BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    BX_CPU_THIS_PTR vmcs.shadow_stack_prematurely_busy = true;
+#endif
+
   if (SSP & 0x7) {
     shadow_stack_write_dword(SSP-4, CPL, 0);
     SSP &= ~BX_CONST64(0x7);
@@ -658,7 +662,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::call_far_shadow_stack_push(Bit16u cs, bx_a
   shadow_stack_push_64(cs);
   shadow_stack_push_64(lip);
   shadow_stack_push_64(old_ssp);
-}
-#endif
 
+#if BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest)
+    BX_CPU_THIS_PTR vmcs.shadow_stack_prematurely_busy = false;
 #endif
+}
+#endif // BX_SUPPORT_CET
+
+#endif // BX_SUPPORT_X86_64
