@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cet.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2019 Stanislav Shwartsman
@@ -40,7 +40,7 @@ const Bit64u BX_CET_WAIT_FOR_ENBRANCH                      = (1 << 11);
 
 bool is_invalid_cet_control(bx_address val)
 {
-  if ((val & (BX_CET_SUPPRESS_INDIRECT_BRANCH_TRACKING | BX_CET_WAIT_FOR_ENBRANCH)) == 
+  if ((val & (BX_CET_SUPPRESS_INDIRECT_BRANCH_TRACKING | BX_CET_WAIT_FOR_ENBRANCH)) ==
              (BX_CET_SUPPRESS_INDIRECT_BRANCH_TRACKING | BX_CET_WAIT_FOR_ENBRANCH)) return true;
 
   if (val & 0x3c0) return true; // reserved bits check
@@ -238,7 +238,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSTORSSP(bxInstruction_c *i)
 
   Bit64u previous_ssp_token = SSP | long64_mode() | 0x02;
 
-// should be done atomically
+// should be done atomically using RMW
   Bit64u SSP_tmp = shadow_stack_read_qword(laddr, CPL); // should be LWSI
   if ((SSP_tmp & 0x03) != long64_mode()) {
     BX_ERROR(("%s: CS.L of shadow stack token doesn't match or bit1 is not 0", i->getIaOpcodeNameShort()));
@@ -256,7 +256,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RSTORSSP(bxInstruction_c *i)
     exception(BX_CP_EXCEPTION, BX_CP_RSTORSSP);
   }
   shadow_stack_write_qword(laddr, CPL, previous_ssp_token);
-// should be done atomically
+// should be done atomically using RMW
 
   SSP = laddr;
 
@@ -275,7 +275,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRSSD(bxInstruction_c *i)
   }
 
   bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
-  bx_address laddr = agen_read_aligned(i->seg(), eaddr, 4);
+  bx_address laddr = agen_write_aligned(i->seg(), eaddr, 4);
   if (laddr & 0x3) {
     BX_ERROR(("%s: must be 4 bytes aligned", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
@@ -293,7 +293,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRSSQ(bxInstruction_c *i)
   }
 
   bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
-  bx_address laddr = agen_read_aligned(i->seg(), eaddr, 8);
+  bx_address laddr = agen_write_aligned(i->seg(), eaddr, 8);
   if (laddr & 0x7) {
     BX_ERROR(("%s: must be 8 bytes aligned", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
@@ -316,7 +316,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRUSSD(bxInstruction_c *i)
   }
 
   bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
-  bx_address laddr = agen_read_aligned(i->seg(), eaddr, 4);
+  bx_address laddr = agen_write_aligned(i->seg(), eaddr, 4);
   if (laddr & 0x3) {
     BX_ERROR(("%s: must be 4 bytes aligned", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
@@ -339,7 +339,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WRUSSQ(bxInstruction_c *i)
   }
 
   bx_address eaddr = BX_CPU_RESOLVE_ADDR(i);
-  bx_address laddr = agen_read_aligned(i->seg(), eaddr, 8);
+  bx_address laddr = agen_write_aligned(i->seg(), eaddr, 8);
   if (laddr & 0x7) {
     BX_ERROR(("%s: must be 8 bytes aligned", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
@@ -371,8 +371,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SETSSBSY(bxInstruction_c *i)
     BX_ERROR(("%s: failed to set SSP busy bit", i->getIaOpcodeNameShort()));
     exception(BX_CP_EXCEPTION, BX_CP_SETSSBSY);
   }
-  
-  SSP = ssp_laddr;  
+
+  SSP = ssp_laddr;
 
   BX_NEXT_INSTR(i);
 }
