@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpu.cc 14141 2021-02-11 15:05:06Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2018  The Bochs Project
@@ -51,6 +51,12 @@ jmp_buf BX_CPU_C::jmp_buf_env;
 
 void BX_CPU_C::cpu_loop(void)
 {
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+  volatile Bit8u stack_anchor = 0;
+
+  BX_CPU_THIS_PTR cpuloop_stack_anchor = &stack_anchor;
+#endif
+
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR break_point = 0;
   BX_CPU_THIS_PTR magic_break = 0;
@@ -81,7 +87,16 @@ void BX_CPU_C::cpu_loop(void)
   // new EIP/ESP, and set up other environmental fields.  This code
   // mirrors similar code below, after the interrupt() call.
   BX_CPU_THIS_PTR prev_rip = RIP; // commit new EIP
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  BX_CPU_THIS_PTR speculative_rsp = false;
+
+#if BX_SUPPORT_CET && BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
+    VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+    if (vm->shadow_stack_prematurely_busy)
+      BX_PANIC(("Shadow stack prematurely busy is left set !"));
+    vm->shadow_stack_prematurely_busy = false; // for safety
+  }
+#endif
 
   while (1) {
 
@@ -249,18 +264,28 @@ bxICacheEntry_c* BX_CPU_C::getICacheEntry(void)
 // The function is called after taken branch instructions and tries to link the branch to the next trace
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::linkTrace(bxInstruction_c *i)
 {
+  volatile Bit8u stack_anchor = 0;
+
 #if BX_SUPPORT_SMP
   if (BX_SMP_PROCESSORS > 1)
     return;
 #endif
 
-#define BX_HANDLERS_CHAINING_MAX_DEPTH 1000
+#define BX_HANDLERS_CHAINING_MAX_LINK_DEPTH 1000
 
   // do not allow extreme trace link depth / avoid host stack overflow
   // (could happen with badly compiled instruction handlers)
   static Bit32u linkDepth = 0;
 
-  if (BX_CPU_THIS_PTR async_event || ++linkDepth > BX_HANDLERS_CHAINING_MAX_DEPTH) {
+  if (BX_CPU_THIS_PTR async_event || ++linkDepth > BX_HANDLERS_CHAINING_MAX_LINK_DEPTH) {
+    linkDepth = 0;
+    return;
+  }
+
+#define BX_HANDLERS_CHAINING_MAX_STACK_DEPTH 0x10000
+
+  size_t stack_depth = BX_CPU_THIS_PTR cpuloop_stack_anchor - &stack_anchor;
+  if (stack_depth > BX_HANDLERS_CHAINING_MAX_STACK_DEPTH) {
     linkDepth = 0;
     return;
   }
@@ -271,6 +296,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::linkTrace(bxInstruction_c *i)
     return;
   }
 
+  BX_SYNC_TIME_IF_SINGLE_PROCESSOR(0);
+
   bxInstruction_c *next = i->getNextTrace(BX_CPU_THIS_PTR iCache.traceLinkTimeStamp);
   if (next) {
     BX_EXECUTE_INSTRUCTION(next);
@@ -308,9 +335,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxRepIterationP
     return;
   }
 
-#if BX_X86_DEBUGGER
-  BX_CPU_THIS_PTR in_repeat = 0;
-#endif
+  BX_CPU_THIS_PTR clear_RF();
 
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
@@ -374,9 +399,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxRepIterationP
     }
   }
 
-#if BX_X86_DEBUGGER
-  BX_CPU_THIS_PTR in_repeat = 1;
-#endif
+  BX_CPU_THIS_PTR assert_RF();
 
   RIP = BX_CPU_THIS_PTR prev_rip; // repeat loop not done, restore RIP
 
@@ -394,9 +417,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxRepIterati
     return;
   }
 
-#if BX_X86_DEBUGGER
-  BX_CPU_THIS_PTR in_repeat = 0;
-#endif
+  BX_CPU_THIS_PTR clear_RF();
 
   if (rep == 3) { /* repeat prefix 0xF3 */
 #if BX_SUPPORT_X86_64
@@ -525,9 +546,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxRepIterati
     }
   }
 
-#if BX_X86_DEBUGGER
-  BX_CPU_THIS_PTR in_repeat = 1;
-#endif
+  BX_CPU_THIS_PTR assert_RF();
 
   RIP = BX_CPU_THIS_PTR prev_rip; // repeat loop not done, restore RIP
 
@@ -552,7 +571,7 @@ void BX_CPU_C::prefetch(void)
 
 #if BX_SUPPORT_X86_64
   if (long64_mode()) {
-    if (! IsCanonical(RIP)) {
+    if (! IsCanonicalAccess(RIP, USER_PL)) {
       BX_ERROR(("prefetch: #GP(0): RIP crossed canonical boundary"));
       exception(BX_GP_EXCEPTION, 0);
     }
@@ -631,7 +650,7 @@ void BX_CPU_C::prefetch(void)
   if ((tlbEntry->lpf == lpf) && (tlbEntry->accessBits & (1 << unsigned(USER_PL))) != 0) {
     BX_CPU_THIS_PTR pAddrFetchPage = tlbEntry->ppf;
     fetchPtr = (Bit8u*) tlbEntry->hostPageAddr;
-  }  
+  }
   else {
     bx_phy_address pAddr = translate_linear(tlbEntry, laddr, USER_PL, BX_EXECUTE);
     BX_CPU_THIS_PTR pAddrFetchPage = PPFOf(pAddr);
@@ -699,13 +718,6 @@ bool BX_CPU_C::dbg_instruction_epilog(void)
     return(1); // on a breakpoint
   }
 
-  // see if debugger requesting icount guard 
-  if (bx_guard.guard_for & BX_DBG_GUARD_ICOUNT) {
-    if (get_icount() >= BX_CPU_THIS_PTR guard_found.icount_max) {
-      return(1);
-    }
-  }
-
   // convenient point to see if user requested debug break or typed Ctrl-C
   if (bx_guard.interrupt_requested) {
     return(1);
@@ -714,8 +726,7 @@ bool BX_CPU_C::dbg_instruction_epilog(void)
   // support for 'show' command in debugger
   extern unsigned dbg_show_mask;
   if(dbg_show_mask) {
-    int rv = bx_dbg_show_symbolic();
-    if (rv) return(rv);
+    bx_dbg_show_symbolic();
   }
 
   // Just committed an instruction, before fetching a new one
@@ -771,11 +782,23 @@ bool BX_CPU_C::dbg_instruction_epilog(void)
     }
 #endif
   }
+
+  // see if debugger requesting icount guard
+  if (bx_guard.guard_for & BX_DBG_GUARD_ICOUNT) {
+    if (get_icount() >= BX_CPU_THIS_PTR guard_found.icount_max) {
+      return(1);
+    }
+  }
 #endif
 
 #if BX_GDBSTUB
   if (bx_dbg.gdbstub_enabled) {
-    unsigned reason = bx_gdbstub_check(EIP);
+    unsigned reason =
+#if BX_SUPPORT_X86_64 == 0
+        bx_gdbstub_check(EIP);
+#else
+        bx_gdbstub_check(RIP);
+#endif
     if (reason != GDBSTUB_STOP_NO_REASON) return(1);
   }
 #endif
