@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: phenomx3_8650_toliman.cc 14100 2021-01-30 19:40:18Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2012-2017 Stanislav Shwartsman
@@ -206,7 +206,7 @@ void phenom_8650_toliman_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [12:12] FMA Instructions support
   // * [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   //   [14:14] xTPR update control
-  //   [15:15] PDCM - Perfon and Debug Capability MSR
+  //   [15:15] PDCM - Perfmon and Debug Capability MSR
   //   [16:16] reserved
   //   [17:17] PCID: Process Context Identifiers
   //   [18:18] DCA - Direct Cache Access
@@ -223,12 +223,7 @@ void phenom_8650_toliman_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] AVX F16C - Float16 conversion support
   //   [30:30] RDRAND instruction
   //   [31:31] reserved
-  leaf->ecx = BX_CPUID_EXT_SSE3 |
-#if BX_SUPPORT_MONITOR_MWAIT
-              BX_CPUID_EXT_MONITOR_MWAIT |
-#endif
-              BX_CPUID_EXT_CMPXCHG16B |
-              BX_CPUID_EXT_POPCNT;
+  leaf->ecx = get_std_cpuid_leaf_1_ecx();
 
   // EDX: Standard Feature Flags
   // * [0:0]   FPU on chip
@@ -263,36 +258,9 @@ void phenom_8650_toliman_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
   //   [31:31] PBE: Pending Break Enable
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD_SYSENTER_SYSEXIT |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-              BX_CPUID_STD_PSE36 |
-              BX_CPUID_STD_CLFLUSH |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD_SSE |
-              BX_CPUID_STD_SSE2
+  leaf->edx = get_std_cpuid_leaf_1_edx();
 #if BX_SUPPORT_SMP
-              | BX_CPUID_STD_HT
-#endif
-              ;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
+  leaf->edx |= BX_CPUID_STD1_EDX_HT;
 #endif
 }
 
@@ -366,19 +334,19 @@ void phenom_8650_toliman_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [23:23] Topology extensions support
   //   [31:24] Reserved
 
-  leaf->ecx = BX_CPUID_EXT2_LAHF_SAHF |
-              BX_CPUID_EXT2_CMP_LEGACY |
+  leaf->ecx = BX_CPUID_EXT1_ECX_LAHF_SAHF |
+              BX_CPUID_EXT1_ECX_CMP_LEGACY |
 #if BX_SUPPORT_SVM
-              BX_CPUID_EXT2_SVM |
+              BX_CPUID_EXT1_ECX_SVM |
 #endif
-              BX_CPUID_EXT2_EXT_APIC_SPACE |
-              BX_CPUID_EXT2_ALT_MOV_CR8 |
-              BX_CPUID_EXT2_LZCNT |
-              BX_CPUID_EXT2_SSE4A |
-              BX_CPUID_EXT2_MISALIGNED_SSE |
-              BX_CPUID_EXT2_PREFETCHW |
-              BX_CPUID_EXT2_OSVW |
-              BX_CPUID_EXT2_IBS;
+              BX_CPUID_EXT1_ECX_EXT_APIC_SPACE |
+              BX_CPUID_EXT1_ECX_ALT_MOV_CR8 |
+              BX_CPUID_EXT1_ECX_LZCNT |
+              BX_CPUID_EXT1_ECX_SSE4A |
+              BX_CPUID_EXT1_ECX_MISALIGNED_SSE |
+              BX_CPUID_EXT1_ECX_PREFETCHW |
+              BX_CPUID_EXT1_ECX_OSVW |
+              BX_CPUID_EXT1_ECX_IBS;
 
   // EDX:
   // Many of the bits in EDX are the same as FN 0x00000001 for AMD
@@ -414,45 +382,14 @@ void phenom_8650_toliman_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] Long Mode
   // * [30:30] AMD 3DNow! Extensions
   // * [31:31] AMD 3DNow! Instructions
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD2_SYSCALL_SYSRET |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-              BX_CPUID_STD_PSE36 |
-              BX_CPUID_STD2_NX |
-              BX_CPUID_STD2_AMD_MMX_EXT |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD2_FFXSR |
-              BX_CPUID_STD2_1G_PAGES |
-              BX_CPUID_STD2_RDTSCP |
-              BX_CPUID_STD2_LONG_MODE |
-              BX_CPUID_STD2_3DNOW_EXT |
-              BX_CPUID_STD2_3DNOW;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
-#endif
+  leaf->edx = get_ext_cpuid_leaf_1_edx_amd();
 }
 
 // leaf 0x80000002 //
 // leaf 0x80000003 //
 // leaf 0x80000004 //
 
-// leaf 0x80000005 //
+// leaf 0x80000005 - L1 Cache and TLB Identifiers //
 void phenom_8650_toliman_t::get_ext_cpuid_leaf_5(cpuid_function_t *leaf) const
 {
   // CPUID function 0x800000005 - L1 Cache and TLB Identifiers
@@ -462,20 +399,20 @@ void phenom_8650_toliman_t::get_ext_cpuid_leaf_5(cpuid_function_t *leaf) const
   leaf->edx = 0x40020140;
 }
 
-// leaf 0x80000006 //
+// leaf 0x80000006 - L2 Cache and TLB Identifiers //
 void phenom_8650_toliman_t::get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const
 {
-  // CPUID function 0x800000006 - L2 Cache and TLB Identifiers
+  // CPUID function 0x80000006 - L2 Cache and TLB Identifiers
   leaf->eax = 0x20800000;
   leaf->ebx = 0x42004200;
   leaf->ecx = 0x02008140;
   leaf->edx = 0x0010A140;
 }
 
-// leaf 0x80000007 //
+// leaf 0x80000007 - Advanced Power Management //
 void phenom_8650_toliman_t::get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const
 {
-  // CPUID function 0x800000007 - Advanced Power Management
+  // CPUID function 0x80000007 - Advanced Power Management
   leaf->eax = 0;
   leaf->ebx = 0;
   leaf->ecx = 0;
@@ -548,7 +485,7 @@ void phenom_8650_toliman_t::get_ext_cpuid_leaf_1A(cpuid_function_t *leaf) const
 
 void phenom_8650_toliman_t::dump_cpuid(void) const
 {
-  bx_cpuid_t::dump_cpuid(BX_SUPPORT_MONITOR_MWAIT ? 0x5 : 0x1, 0x1A);
+  bx_cpuid_t::dump_cpuid(BX_SUPPORT_MONITOR_MWAIT ? 0x5 : 0x1, 0x8000001A);
 }
 
 bx_cpuid_t *create_phenom_8650_toliman_cpuid(BX_CPU_C *cpu) { return new phenom_8650_toliman_t(cpu); }
