@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: zambezi.cc 14100 2021-01-30 19:40:18Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2012-2017 Stanislav Shwartsman
@@ -132,8 +132,8 @@ void zambezi_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_functi
   case 0x80000019:
     get_ext_cpuid_leaf_19(leaf);
     return;
-  case 0x8000001A:
-    get_ext_cpuid_leaf_1A(leaf);
+  case 0x8000001A: // CPUID leaf 0x8000001A - Performance Optimization Identifiers
+    get_leaf(leaf, 0x00000003, 0x00000000, 0x00000000, 0x00000000);
     return;
   case 0x8000001B:
     get_ext_cpuid_leaf_1B(leaf);
@@ -176,7 +176,7 @@ Bit32u zambezi_t::get_svm_extensions_bitmask(void) const
   return BX_CPUID_SVM_NESTED_PAGING |
          BX_CPUID_SVM_LBR_VIRTUALIZATION |
          BX_CPUID_SVM_SVM_LOCK |
-         BX_CPUID_SVM_NRIP_SAVE | 
+         BX_CPUID_SVM_NRIP_SAVE |
 //       BX_CPUID_SVM_TSCRATE | // not implemented yet
 //       BX_CPUID_SVM_VMCB_CLEAN_BITS | // not implemented yet
          BX_CPUID_SVM_FLUSH_BY_ASID |
@@ -237,7 +237,7 @@ void zambezi_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [12:12] FMA Instructions support
   // * [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   //   [14:14] xTPR update control
-  //   [15:15] PDCM - Perfon and Debug Capability MSR
+  //   [15:15] PDCM - Perfmon and Debug Capability MSR
   //   [16:16] reserved
   //   [17:17] PCID: Process Context Identifiers
   //   [18:18] DCA - Direct Cache Access
@@ -254,21 +254,7 @@ void zambezi_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] AVX F16C - Float16 conversion support
   //   [30:30] RDRAND instruction
   //   [31:31] reserved
-  leaf->ecx = BX_CPUID_EXT_SSE3 |
-              BX_CPUID_EXT_PCLMULQDQ |
-#if BX_SUPPORT_MONITOR_MWAIT
-              BX_CPUID_EXT_MONITOR_MWAIT |
-#endif
-              BX_CPUID_EXT_SSSE3 |
-              BX_CPUID_EXT_CMPXCHG16B |
-              BX_CPUID_EXT_SSE4_1 |
-              BX_CPUID_EXT_SSE4_2 |
-              BX_CPUID_EXT_POPCNT |
-              BX_CPUID_EXT_AES |
-              BX_CPUID_EXT_XSAVE |
-              BX_CPUID_EXT_AVX;
-  if (cpu->cr4.get_OSXSAVE())
-    leaf->ecx |= BX_CPUID_EXT_OSXSAVE;
+  leaf->ecx = get_std_cpuid_leaf_1_ecx();
 
   // EDX: Standard Feature Flags
   // * [0:0]   FPU on chip
@@ -303,36 +289,9 @@ void zambezi_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
   //   [31:31] PBE: Pending Break Enable
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD_SYSENTER_SYSEXIT |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-              BX_CPUID_STD_PSE36 |
-              BX_CPUID_STD_CLFLUSH |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD_SSE |
-              BX_CPUID_STD_SSE2
+  leaf->edx = get_std_cpuid_leaf_1_edx();
 #if BX_SUPPORT_SMP
-              | BX_CPUID_STD_HT
-#endif
-              ;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
+  leaf->edx |= BX_CPUID_STD1_EDX_HT;
 #endif
 }
 
@@ -422,26 +381,26 @@ void zambezi_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [24:24] PerfCtrExtNB: NB perf counter extensions support
   //   [31:25] Reserved
 
-  leaf->ecx = BX_CPUID_EXT2_LAHF_SAHF |
-              BX_CPUID_EXT2_CMP_LEGACY |
+  leaf->ecx = BX_CPUID_EXT1_ECX_LAHF_SAHF |
+              BX_CPUID_EXT1_ECX_CMP_LEGACY |
 #if BX_SUPPORT_SVM
-              BX_CPUID_EXT2_SVM |
+              BX_CPUID_EXT1_ECX_SVM |
 #endif
-              BX_CPUID_EXT2_EXT_APIC_SPACE |
-              BX_CPUID_EXT2_ALT_MOV_CR8 |
-              BX_CPUID_EXT2_LZCNT |
-              BX_CPUID_EXT2_SSE4A |
-              BX_CPUID_EXT2_MISALIGNED_SSE |
-              BX_CPUID_EXT2_PREFETCHW |
-              BX_CPUID_EXT2_OSVW |
-              BX_CPUID_EXT2_IBS |
-              BX_CPUID_EXT2_XOP |
-              BX_CPUID_EXT2_WDT |
-              BX_CPUID_EXT2_FMA4 |
-              BX_CPUID_EXT2_NODEID |
-              BX_CPUID_EXT2_TOPOLOGY_EXTENSIONS |
-              BX_CPUID_EXT2_PERFCTR_EXT_CORE |
-              BX_CPUID_EXT2_PERFCTR_EXT_NB;
+              BX_CPUID_EXT1_ECX_EXT_APIC_SPACE |
+              BX_CPUID_EXT1_ECX_ALT_MOV_CR8 |
+              BX_CPUID_EXT1_ECX_LZCNT |
+              BX_CPUID_EXT1_ECX_SSE4A |
+              BX_CPUID_EXT1_ECX_MISALIGNED_SSE |
+              BX_CPUID_EXT1_ECX_PREFETCHW |
+              BX_CPUID_EXT1_ECX_OSVW |
+              BX_CPUID_EXT1_ECX_IBS |
+              BX_CPUID_EXT1_ECX_XOP |
+              BX_CPUID_EXT1_ECX_WDT |
+              BX_CPUID_EXT1_ECX_FMA4 |
+              BX_CPUID_EXT1_ECX_NODEID |
+              BX_CPUID_EXT1_ECX_TOPOLOGY_EXTENSIONS |
+              BX_CPUID_EXT1_ECX_PERFCTR_EXT_CORE |
+              BX_CPUID_EXT1_ECX_PERFCTR_EXT_NB;
 
   // EDX:
   // Many of the bits in EDX are the same as FN 0x00000001 for AMD
@@ -477,36 +436,7 @@ void zambezi_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] Long Mode
   //   [30:30] AMD 3DNow! Extensions
   //   [31:31] AMD 3DNow! Instructions
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD2_SYSCALL_SYSRET |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-              BX_CPUID_STD_PSE36 |
-              BX_CPUID_STD2_NX |
-              BX_CPUID_STD2_AMD_MMX_EXT |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD2_FFXSR |
-              BX_CPUID_STD2_1G_PAGES |
-              BX_CPUID_STD2_RDTSCP |
-              BX_CPUID_STD2_LONG_MODE;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
-#endif
+  leaf->edx = get_ext_cpuid_leaf_1_edx_amd();
 }
 
 // leaf 0x80000002 //
@@ -526,7 +456,7 @@ void zambezi_t::get_ext_cpuid_leaf_5(cpuid_function_t *leaf) const
 // leaf 0x80000006 //
 void zambezi_t::get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const
 {
-  // CPUID function 0x800000006 - L2 Cache and TLB Identifiers
+  // CPUID function 0x80000006 - L2 Cache and TLB Identifiers
   leaf->eax = 0x64000000;
   leaf->ebx = 0x64004200;
   leaf->ecx = 0x08008140;
@@ -536,7 +466,7 @@ void zambezi_t::get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const
 // leaf 0x80000007 //
 void zambezi_t::get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const
 {
-  // CPUID function 0x800000007 - Advanced Power Management
+  // CPUID function 0x80000007 - Advanced Power Management
   leaf->eax = 0;
   leaf->ebx = 0;
   leaf->ecx = 0;
@@ -597,14 +527,7 @@ void zambezi_t::get_ext_cpuid_leaf_19(cpuid_function_t *leaf) const
   leaf->edx = 0;
 }
 
-void zambezi_t::get_ext_cpuid_leaf_1A(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x80000001A - Performance Optimization Identifiers
-  leaf->eax = 0x00000003;
-  leaf->ebx = 0;
-  leaf->ecx = 0;
-  leaf->edx = 0;
-}
+// CPUID leaf 0x80000001A - Performance Optimization Identifiers
 
 void zambezi_t::get_ext_cpuid_leaf_1B(cpuid_function_t *leaf) const
 {
@@ -674,7 +597,7 @@ void zambezi_t::get_ext_cpuid_leaf_1E(cpuid_function_t *leaf) const
 
 void zambezi_t::dump_cpuid(void) const
 {
-  bx_cpuid_t::dump_cpuid(0xD, 0x1E);
+  bx_cpuid_t::dump_cpuid(0xD, 0x8000001E);
 }
 
 bx_cpuid_t *create_zambezi_cpuid(BX_CPU_C *cpu) { return new zambezi_t(cpu); }
