@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: core_duo_t2400_yonah.cc 14100 2021-01-30 19:40:18Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2017 Stanislav Shwartsman
@@ -64,6 +64,9 @@ core_duo_t2400_yonah_t::core_duo_t2400_yonah_t(BX_CPU_C *cpu): bx_cpuid_t(cpu)
   enable_cpu_extension(BX_ISA_PAT);
   enable_cpu_extension(BX_ISA_XAPIC);
   enable_cpu_extension(BX_ISA_NX);
+
+  max_std_leaf = 0xA;
+  max_ext_leaf = 0x80000008;
 }
 
 void core_duo_t2400_yonah_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const
@@ -76,7 +79,7 @@ void core_duo_t2400_yonah_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction,
 
   switch(function) {
   case 0x80000000:
-    get_ext_cpuid_leaf_0(leaf);
+    get_leaf_0(max_ext_leaf, NULL, leaf);
     return;
   case 0x80000001:
     get_ext_cpuid_leaf_1(leaf);
@@ -89,17 +92,17 @@ void core_duo_t2400_yonah_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction,
   case 0x80000005:
     get_reserved_leaf(leaf);
     return;
-  case 0x80000006:
-    get_ext_cpuid_leaf_6(leaf);
+  case 0x80000006: // CPUID leaf 0x80000006 - L2 Cache and TLB Identifiers
+    get_leaf(leaf, 0x00000000, 0x00000000, 0x08006040, 0x00000000);
     return;
-  case 0x80000007:
-    get_ext_cpuid_leaf_7(leaf);
+  case 0x80000007: // CPUID leaf 0x80000007 - Advanced Power Management
+    get_reserved_leaf(leaf);
     return;
   case 0x80000008:
     get_ext_cpuid_leaf_8(leaf);
     return;
   case 0x00000000:
-    get_std_cpuid_leaf_0(leaf);
+    get_leaf_0(max_std_leaf, "GenuineIntel", leaf);
     return;
   case 0x00000001:
     get_std_cpuid_leaf_1(leaf);
@@ -132,14 +135,6 @@ void core_duo_t2400_yonah_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction,
 }
 
 // leaf 0x00000000 //
-void core_duo_t2400_yonah_t::get_std_cpuid_leaf_0(cpuid_function_t *leaf) const
-{
-  // EAX: highest std function understood by CPUID
-  // EBX: vendor ID string
-  // EDX: vendor ID string
-  // ECX: vendor ID string
-  get_leaf_0(0xA, "GenuineIntel", leaf);
-}
 
 // leaf 0x00000001 //
 void core_duo_t2400_yonah_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
@@ -182,7 +177,7 @@ void core_duo_t2400_yonah_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [12:12] FMA Instructions support
   //   [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   // * [14:14] xTPR update control
-  // * [15:15] PDCM - Perfon and Debug Capability MSR
+  // * [15:15] PDCM - Perfmon and Debug Capability MSR
   //   [16:16] reserved
   //   [17:17] PCID: Process Context Identifiers
   //   [18:18] DCA - Direct Cache Access
@@ -199,17 +194,10 @@ void core_duo_t2400_yonah_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] AVX F16C - Float16 conversion support
   //   [30:30] RDRAND instruction
   //   [31:31] reserved
-  leaf->ecx = BX_CPUID_EXT_SSE3 |
-#if BX_SUPPORT_MONITOR_MWAIT
-              BX_CPUID_EXT_MONITOR_MWAIT |
-#endif
-#if BX_SUPPORT_VMX
-              BX_CPUID_EXT_VMX |
-#endif
-              BX_CPUID_EXT_EST |
-              BX_CPUID_EXT_THERMAL_MONITOR2 |
-              BX_CPUID_EXT_xTPR |
-              BX_CPUID_EXT_PDCM;
+  leaf->ecx = get_std_cpuid_leaf_1_ecx(BX_CPUID_STD1_ECX_EST |
+                                       BX_CPUID_STD1_ECX_THERMAL_MONITOR2 |
+                                       BX_CPUID_STD1_ECX_xTPR |
+                                       BX_CPUID_STD1_ECX_PDCM);
 
   // EDX: Standard Feature Flags
   // * [0:0]   FPU on chip
@@ -244,42 +232,13 @@ void core_duo_t2400_yonah_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
   // * [31:31] PBE: Pending Break Enable
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD_SYSENTER_SYSEXIT |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-#if BX_PHY_ADDRESS_LONG
-              BX_CPUID_STD_PSE36 |
-#endif
-              BX_CPUID_STD_CLFLUSH |
-              BX_CPUID_STD_DEBUG_STORE |
-              BX_CPUID_STD_ACPI |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD_SSE |
-              BX_CPUID_STD_SSE2 |
-              BX_CPUID_STD_SELF_SNOOP |
+  leaf->edx = get_std_cpuid_leaf_1_edx(BX_CPUID_STD1_EDX_DEBUG_STORE |
+                                       BX_CPUID_STD1_EDX_ACPI |
+                                       BX_CPUID_STD1_EDX_SELF_SNOOP |
 #if BX_SUPPORT_SMP
-              BX_CPUID_STD_HT |
-#endif
-              BX_CPUID_STD_PBE;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
+                                       BX_CPUID_STD1_EDX_HT |
 #endif
+                                       BX_CPUID_STD1_EDX_PBE);
 }
 
 // leaf 0x00000002 //
@@ -433,14 +392,6 @@ void core_duo_t2400_yonah_t::get_std_cpuid_leaf_A(cpuid_function_t *leaf) const
 }
 
 // leaf 0x80000000 //
-void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_0(cpuid_function_t *leaf) const
-{
-  // EAX: highest extended function understood by CPUID
-  // EBX: reserved
-  // EDX: reserved
-  // ECX: reserved
-  get_leaf_0(0x80000008, NULL, leaf);
-}
 
 // leaf 0x80000001 //
 void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
@@ -469,11 +420,10 @@ void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   leaf->ecx = 0;
 
   // EDX:
-  // Many of the bits in EDX are the same as FN 0x00000001 [*] for AMD
   //    [10:0] Reserved for Intel
   //   [11:11] SYSCALL/SYSRET support
   //   [19:12] Reserved for Intel
-  //   [20:20] No-Execute page protection
+  // * [20:20] No-Execute page protection
   //   [25:21] Reserved
   //   [26:26] 1G paging support
   //   [27:27] Support RDTSCP Instruction
@@ -481,40 +431,20 @@ void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [29:29] Long Mode
   //   [30:30] AMD 3DNow! Extensions
   //   [31:31] AMD 3DNow! Instructions
-  leaf->edx = BX_CPUID_STD2_NX;
+  leaf->edx = get_ext_cpuid_leaf_1_edx_intel();
 }
 
 // leaf 0x80000002 //
 // leaf 0x80000003 //
 // leaf 0x80000004 //
-
 // leaf 0x80000005 - L1 Cache and TLB Identifiers (reserved for Intel)
-
-// leaf 0x80000006 //
-void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x800000006 - L2 Cache and TLB Identifiers
-  leaf->eax = 0x00000000;
-  leaf->ebx = 0x00000000;
-  leaf->ecx = 0x08006040;
-  leaf->edx = 0x00000000;
-}
-
-// leaf 0x80000007 //
-void core_duo_t2400_yonah_t::get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x800000007 - Advanced Power Management
-  leaf->eax = 0;
-  leaf->ebx = 0;
-  leaf->ecx = 0;
-  leaf->edx = 0;
-}
-
+// leaf 0x80000006 - L2 Cache and TLB Identifiers
+// leaf 0x80000007 - Advanced Power Management
 // leaf 0x80000008 //
 
 void core_duo_t2400_yonah_t::dump_cpuid(void) const
 {
-  bx_cpuid_t::dump_cpuid(0xA, 0x8);
+  bx_cpuid_t::dump_cpuid(max_std_leaf, max_ext_leaf);
 }
 
 bx_cpuid_t *create_core_duo_t2400_yonah_cpuid(BX_CPU_C *cpu) { return new core_duo_t2400_yonah_t(cpu); }
