@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: corei7_icelake-u.cc 14149 2021-02-16 18:57:49Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2019 Stanislav Shwartsman
@@ -40,8 +40,11 @@ corei7_icelake_t::corei7_icelake_t(BX_CPU_C *cpu):
     bx_cpuid_t(cpu)
 #endif
 {
-  if (! BX_SUPPORT_X86_64)
-    BX_PANIC(("You must enable x86-64 for Intel Core i7 Icelake configuration"));
+  if (BX_SUPPORT_VMX == 1)
+    BX_INFO(("You must compile with --enable-vmx=2 for Intel Core i7 IceLake VMX configuration"));
+
+  if (! BX_SUPPORT_MONITOR_MWAIT)
+    BX_INFO(("WARNING: MONITOR/MWAIT support is not compiled in !"));
 
   enable_cpu_extension(BX_ISA_X87);
   enable_cpu_extension(BX_ISA_486);
@@ -109,7 +112,6 @@ corei7_icelake_t::corei7_icelake_t(BX_CPU_C *cpu):
   enable_cpu_extension(BX_ISA_GFNI);
   enable_cpu_extension(BX_ISA_VAES_VPCLMULQDQ);
   enable_cpu_extension(BX_ISA_AVX512);
-  enable_cpu_extension(BX_ISA_AVX512_VL);
   enable_cpu_extension(BX_ISA_AVX512_DQ);
   enable_cpu_extension(BX_ISA_AVX512_CD);
   enable_cpu_extension(BX_ISA_AVX512_BW);
@@ -130,11 +132,14 @@ corei7_icelake_t::corei7_icelake_t(BX_CPU_C *cpu):
   enable_cpu_extension(BX_ISA_UMIP);
   enable_cpu_extension(BX_ISA_RDPID);
   enable_cpu_extension(BX_ISA_SCA_MITIGATIONS);
+
+  max_std_leaf = 0x1B;
+  max_ext_leaf = 0x80000008;
 }
 
 void corei7_icelake_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const
 {
-  static const char* brand_string = "QuadCore Intel Core i7-1065G7, 1300 MHz\0\0\0\0\0\0\0";
+  static const char* brand_string = "QuadCore Intel Core i7-1065G7, 1300 MHz\0\0\0\0\0\0\0\0";
 
   static bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
   if (cpuid_limit_winnt)
@@ -142,7 +147,7 @@ void corei7_icelake_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid
 
   switch(function) {
   case 0x80000000:
-    get_ext_cpuid_leaf_0(leaf);
+    get_leaf_0(max_ext_leaf, NULL, leaf); // highest extended function understood by CPUID
     return;
   case 0x80000001:
     get_ext_cpuid_leaf_1(leaf);
@@ -155,23 +160,23 @@ void corei7_icelake_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid
   case 0x80000005:
     get_reserved_leaf(leaf);
     return;
-  case 0x80000006:
-    get_ext_cpuid_leaf_6(leaf);
+  case 0x80000006: // CPUID leaf 0x80000006 - L2 Cache and TLB Identifiers
+    get_leaf(leaf, 0x00000000, 0x00000000, 0x01006040, 0x00000000);
     return;
-  case 0x80000007:
-    get_ext_cpuid_leaf_7(leaf);
+  case 0x80000007: // CPUID leaf 0x80000007 - Advanced Power Management leaf
+    get_leaf(leaf, 0x00000000, 0x00000000, 0x00000000, 0x00000100); // EDX[8] - invariant TSC
     return;
   case 0x80000008:
     get_ext_cpuid_leaf_8(leaf);
     return;
   case 0x00000000:
-    get_std_cpuid_leaf_0(leaf);
+    get_leaf_0(max_std_leaf, "GenuineIntel", leaf);
     return;
   case 0x00000001:
     get_std_cpuid_leaf_1(leaf);
     return;
-  case 0x00000002:
-    get_std_cpuid_leaf_2(leaf);
+  case 0x00000002: // CPUID leaf 0x00000002 - Cache and TLB Descriptors
+    get_leaf(leaf, 0x00feff01, 0x000000f0, 0x00000000, 0x00000000);
     return;
   case 0x00000003:
     get_reserved_leaf(leaf);
@@ -182,8 +187,8 @@ void corei7_icelake_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid
   case 0x00000005:
     get_std_cpuid_leaf_5(leaf);
     return;
-  case 0x00000006:
-    get_std_cpuid_leaf_6(leaf);
+  case 0x00000006: // CPUID leaf 0x00000006 - Thermal and Power Management Leaf
+    get_leaf(leaf, 0x0007aff5, 0x00000002, 0x00000009, 0x00000000);
     return;
   case 0x00000007:
     get_std_cpuid_leaf_7(subfunction, leaf);
@@ -213,11 +218,11 @@ void corei7_icelake_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid
   case 0x00000014: // Processor Trace
     get_reserved_leaf(leaf);
     return;
-  case 0x00000015:
-    get_std_cpuid_leaf_15(leaf);
+  case 0x00000015: // CPUID leaf 0x00000015 - Time Stamp Counter and Core Crystal Clock Information Leaf
+    get_leaf(leaf, 0x00000002, 0x0000004e, 0x0249f000, 0x00000000);
     return;
-  case 0x00000016:
-    get_std_cpuid_leaf_16(leaf);
+  case 0x00000016: // CPUID leaf 0x00000016 - Processor Frequency Information Leaf
+    get_leaf(leaf, 0x000005dc, 0x00000f3c, 0x00000064, 0x00000000);
     return;
   case 0x00000017:
     get_reserved_leaf(leaf);
@@ -260,22 +265,14 @@ Bit32u corei7_icelake_t::get_vmx_extensions_bitmask(void) const
          BX_VMX_EPT_EXCEPTION |
          BX_VMX_PML |
          BX_VMX_SW_INTERRUPT_INJECTION_ILEN_0 |
-      /* BX_VMX_POSTED_INSTERRUPTS - not implemented yet */
-      /* BX_VMX_MBE_CONTROL - not implemeted yet */
+         BX_VMX_POSTED_INTERRUPTS |
+         BX_VMX_MBE_CONTROL |
          BX_VMX_SPP |
          BX_VMX_TSC_SCALING;
 }
 #endif
 
 // leaf 0x00000000 //
-void corei7_icelake_t::get_std_cpuid_leaf_0(cpuid_function_t *leaf) const
-{
-  // EAX: highest std function understood by CPUID
-  // EBX: vendor ID string
-  // EDX: vendor ID string
-  // ECX: vendor ID string
-  get_leaf_0(0x1B, "GenuineIntel", leaf);
-}
 
 // leaf 0x00000001 //
 void corei7_icelake_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
@@ -318,7 +315,7 @@ void corei7_icelake_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [12:12] FMA Instructions support
   // * [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   // * [14:14] xTPR update control
-  // * [15:15] PDCM - Perfon and Debug Capability MSR
+  // * [15:15] PDCM - Perfmon and Debug Capability MSR
   //   [16:16] reserved
   // * [17:17] PCID: Process Context Identifiers
   //   [18:18] DCA - Direct Cache Access
@@ -335,37 +332,12 @@ void corei7_icelake_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] AVX F16C - Float16 conversion support
   // * [30:30] RDRAND instruction
   //   [31:31] reserved
-  leaf->ecx = BX_CPUID_EXT_SSE3 |
-              BX_CPUID_EXT_PCLMULQDQ |
-              BX_CPUID_EXT_DTES64 |
-#if BX_SUPPORT_MONITOR_MWAIT
-              BX_CPUID_EXT_MONITOR_MWAIT |
-#endif
-              BX_CPUID_EXT_DS_CPL |
-#if BX_SUPPORT_VMX >= 2
-              BX_CPUID_EXT_VMX |
-#endif
-              BX_CPUID_EXT_EST |
-              BX_CPUID_EXT_THERMAL_MONITOR2 |
-              BX_CPUID_EXT_SSSE3 |
-              BX_CPUID_EXT_FMA |
-              BX_CPUID_EXT_CMPXCHG16B |
-              BX_CPUID_EXT_xTPR |
-              BX_CPUID_EXT_PDCM |
-              BX_CPUID_EXT_PCID |
-              BX_CPUID_EXT_SSE4_1 |
-              BX_CPUID_EXT_SSE4_2 |
-              BX_CPUID_EXT_X2APIC |
-              BX_CPUID_EXT_MOVBE |
-              BX_CPUID_EXT_POPCNT |
-              BX_CPUID_EXT_TSC_DEADLINE |
-              BX_CPUID_EXT_AES |
-              BX_CPUID_EXT_XSAVE |
-              BX_CPUID_EXT_AVX |
-              BX_CPUID_EXT_AVX_F16C |
-              BX_CPUID_EXT_RDRAND;
-  if (cpu->cr4.get_OSXSAVE())
-    leaf->ecx |= BX_CPUID_EXT_OSXSAVE;
+  leaf->ecx = get_std_cpuid_leaf_1_ecx(BX_CPUID_STD1_ECX_DTES64 |
+                                       BX_CPUID_STD1_ECX_DS_CPL |
+                                       BX_CPUID_STD1_ECX_EST |
+                                       BX_CPUID_STD1_ECX_THERMAL_MONITOR2 |
+                                       BX_CPUID_STD1_ECX_xTPR |
+                                       BX_CPUID_STD1_ECX_PDCM);
 
   // EDX: Standard Feature Flags
   // * [0:0]   FPU on chip
@@ -400,53 +372,17 @@ void corei7_icelake_t::get_std_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] TM: Thermal Monitor
   //   [30:30] Reserved
   // * [31:31] PBE: Pending Break Enable
-  leaf->edx = BX_CPUID_STD_X87 |
-              BX_CPUID_STD_VME |
-              BX_CPUID_STD_DEBUG_EXTENSIONS |
-              BX_CPUID_STD_PSE |
-              BX_CPUID_STD_TSC |
-              BX_CPUID_STD_MSR |
-              BX_CPUID_STD_PAE |
-              BX_CPUID_STD_MCE |
-              BX_CPUID_STD_CMPXCHG8B |
-              BX_CPUID_STD_SYSENTER_SYSEXIT |
-              BX_CPUID_STD_MTRR |
-              BX_CPUID_STD_GLOBAL_PAGES |
-              BX_CPUID_STD_MCA |
-              BX_CPUID_STD_CMOV |
-              BX_CPUID_STD_PAT |
-              BX_CPUID_STD_PSE36 |
-              BX_CPUID_STD_CLFLUSH |
-              BX_CPUID_STD_DEBUG_STORE |
-              BX_CPUID_STD_ACPI |
-              BX_CPUID_STD_MMX |
-              BX_CPUID_STD_FXSAVE_FXRSTOR |
-              BX_CPUID_STD_SSE |
-              BX_CPUID_STD_SSE2 |
-              BX_CPUID_STD_SELF_SNOOP |
+  leaf->edx = get_std_cpuid_leaf_1_edx(BX_CPUID_STD1_EDX_DEBUG_STORE |
+                                       BX_CPUID_STD1_EDX_ACPI |
+                                       BX_CPUID_STD1_EDX_SELF_SNOOP |
 #if BX_SUPPORT_SMP
-              BX_CPUID_STD_HT |
-#endif
-              BX_CPUID_STD_THERMAL_MONITOR |
-              BX_CPUID_STD_PBE;
-#if BX_SUPPORT_APIC
-  // if MSR_APICBASE APIC Global Enable bit has been cleared,
-  // the CPUID feature flag for the APIC is set to 0.
-  if (cpu->msr.apicbase & 0x800)
-    leaf->edx |= BX_CPUID_STD_APIC; // APIC on chip
+                                       BX_CPUID_STD1_EDX_HT |
 #endif
+                                       BX_CPUID_STD1_EDX_THERMAL_MONITOR |
+                                       BX_CPUID_STD1_EDX_PBE);
 }
 
-// leaf 0x00000002 //
-void corei7_icelake_t::get_std_cpuid_leaf_2(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x00000002 - Cache and TLB Descriptors
-  leaf->eax = 0x00feff01;
-  leaf->ebx = 0x000000f0;
-  leaf->ecx = 0x00000000;
-  leaf->edx = 0x00000000;
-}
-
+// leaf 0x00000002 - Cache and TLB Descriptors //
 // leaf 0x00000003 - Processor Serial Number (not supported) //
 
 // leaf 0x00000004 //
@@ -518,14 +454,6 @@ void corei7_icelake_t::get_std_cpuid_leaf_5(cpuid_function_t *leaf) const
 }
 
 // leaf 0x00000006 //
-void corei7_icelake_t::get_std_cpuid_leaf_6(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x00000006 - Thermal and Power Management Leaf
-  leaf->eax = 0x0007aff5;
-  leaf->ebx = 0x00000002;
-  leaf->ecx = 0x00000009;
-  leaf->edx = 0x00000000;
-}
 
 // leaf 0x00000007 //
 void corei7_icelake_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_t *leaf) const
@@ -566,7 +494,7 @@ void corei7_icelake_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_t
     // * [29:29]  SHA instructions support
     // * [30:30]  AVX512BW instructions support
     // * [31:31]  AVX512VL variable vector length support
-    leaf->ebx = get_std_cpuid_leaf_7_ebx(BX_CPUID_EXT3_ENCHANCED_REP_STRINGS);
+    leaf->ebx = get_std_cpuid_leaf_7_ebx(BX_CPUID_STD7_SUBLEAF0_EBX_ENCHANCED_REP_STRINGS);
 
     //     [0:0]    PREFETCHWT1 instruction support
     // *   [1:1]    AVX512 VBMI instructions support
@@ -612,16 +540,15 @@ void corei7_icelake_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_t
     // * [29:29]    Support for the IA32_ARCH_CAPABILITIES MSR.
     //   [30:30]    Support for the IA32_CORE_CAPABILITIES MSR.
     // * [31:31]    SSBD: Speculative Store Bypass Disable
-
-    leaf->edx = BX_CPUID_EXT5_FAST_SHORT_REP_MOV |
-                BX_CPUID_EXT5_AVX512_VPINTERSECT;
+    leaf->edx = get_std_cpuid_leaf_7_edx(BX_CPUID_STD7_SUBLEAF0_EDX_FAST_SHORT_REP_MOV);
 
     if (is_cpu_extension_supported(BX_ISA_SCA_MITIGATIONS))
-      leaf->edx |= BX_CPUID_EXT5_SCA_IBRS_IBPB |
-                   BX_CPUID_EXT5_SCA_STIBP |
-                   BX_CPUID_EXT5_L1D_FLUSH |
-                   BX_CPUID_EXT5_ARCH_CAPABILITIES_MSR |
-                   BX_CPUID_EXT5_SCA_SSBD;
+      leaf->edx |= BX_CPUID_STD7_SUBLEAF0_EDX_SCA_IBRS_IBPB |
+                   BX_CPUID_STD7_SUBLEAF0_EDX_SCA_STIBP |
+                   BX_CPUID_STD7_SUBLEAF0_EDX_L1D_FLUSH |
+                   BX_CPUID_STD7_SUBLEAF0_EDX_ARCH_CAPABILITIES_MSR |
+                   BX_CPUID_STD7_SUBLEAF0_EDX_SCA_SSBD;
+
     break;
 
   default:
@@ -656,25 +583,8 @@ void corei7_icelake_t::get_std_cpuid_leaf_A(cpuid_function_t *leaf) const
 // leaf 0x00000012 - Intel Software Guard Extensions //
 // leaf 0x00000013 - reserved //
 // leaf 0x00000014 - Intel Processor Trace Enumeration //
-
 // leaf 0x00000015 - Time Stamp Counter and Core Crystal Clock Information Leaf //
-void corei7_icelake_t::get_std_cpuid_leaf_15(cpuid_function_t *leaf) const
-{
-  leaf->eax = 0x00000002;
-  leaf->ebx = 0x0000004e;
-  leaf->ecx = 0x0249f000;
-  leaf->edx = 0x00000000;
-}
-
 // leaf 0x00000016 - Processor Frequency Information Leaf
-void corei7_icelake_t::get_std_cpuid_leaf_16(cpuid_function_t *leaf) const
-{
-  leaf->eax = 0x000005dc;
-  leaf->ebx = 0x00000f3c;
-  leaf->ecx = 0x00000064;
-  leaf->edx = 0x00000000;
-}
-
 // leaf 0x00000017 - System-On-Chip Vendor Attribute Enumeration //
 
 // leaf 0x00000018
@@ -739,14 +649,6 @@ void corei7_icelake_t::get_std_cpuid_leaf_18(Bit32u subfunction, cpuid_function_
 }
 
 // leaf 0x80000000 //
-void corei7_icelake_t::get_ext_cpuid_leaf_0(cpuid_function_t *leaf) const
-{
-  // EAX: highest extended function understood by CPUID
-  // EBX: reserved
-  // EDX: reserved
-  // ECX: reserved
-  get_leaf_0(0x80000008, NULL, leaf);
-}
 
 // leaf 0x80000001 //
 void corei7_icelake_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
@@ -772,13 +674,9 @@ void corei7_icelake_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   //   [12:12] SKINIT support
   //   [13:13] WDT: Watchdog timer support
   //   [31:14] reserved
-
-  leaf->ecx = BX_CPUID_EXT2_LAHF_SAHF |
-              BX_CPUID_EXT2_LZCNT |
-              BX_CPUID_EXT2_PREFETCHW;
+  leaf->ecx = get_ext_cpuid_leaf_1_ecx_intel(BX_CPUID_EXT1_ECX_PREFETCHW);
 
   // EDX:
-  // Many of the bits in EDX are the same as EAX [*] for AMD
   //    [10:0] Reserved for Intel
   // * [11:11] SYSCALL/SYSRET support
   //   [19:12] Reserved for Intel
@@ -790,46 +688,20 @@ void corei7_icelake_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const
   // * [29:29] Long Mode
   //   [30:30] AMD 3DNow! Extensions
   //   [31:31] AMD 3DNow! Instructions
-
-  leaf->edx = BX_CPUID_STD2_NX |
-              BX_CPUID_STD2_1G_PAGES |
-              BX_CPUID_STD2_RDTSCP |
-              BX_CPUID_STD2_LONG_MODE;
-  if (cpu->long64_mode())
-    leaf->edx |= BX_CPUID_STD2_SYSCALL_SYSRET;
+  leaf->edx = get_ext_cpuid_leaf_1_edx_intel();
 }
 
 // leaf 0x80000002 //
 // leaf 0x80000003 //
 // leaf 0x80000004 //
-
-// leaf 0x80000005 - L1 Cache and TLB Identifiers (reserved for Intel) //
-
-// leaf 0x80000006 //
-void corei7_icelake_t::get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x800000006 - L2 Cache and TLB Identifiers
-  leaf->eax = 0x00000000;
-  leaf->ebx = 0x00000000;
-  leaf->ecx = 0x01006040;
-  leaf->edx = 0x00000000;
-}
-
-// leaf 0x80000007 //
-void corei7_icelake_t::get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const
-{
-  // CPUID function 0x800000007 - Advanced Power Management
-  leaf->eax = 0;
-  leaf->ebx = 0;
-  leaf->ecx = 0;
-  leaf->edx = 0x00000100; // bit 8 - invariant TSC
-}
-
+// leaf 0x80000005 - L1 Cache and TLB Identifiers (reserved for Intel)
+// leaf 0x80000006 - L2 Cache and TLB Identifiers
+// leaf 0x80000007 - Advanced Power Management
 // leaf 0x80000008 //
 
 void corei7_icelake_t::dump_cpuid(void) const
 {
-  bx_cpuid_t::dump_cpuid(0x18, 0x8);
+  bx_cpuid_t::dump_cpuid(max_std_leaf, max_ext_leaf);
 }
 
 bx_cpuid_t *create_corei7_icelake_u_cpuid(BX_CPU_C *cpu) { return new corei7_icelake_t(cpu); }
