@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: debugstuff.cc 14105 2021-01-30 20:31:03Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2009  The Bochs Project
@@ -243,7 +243,7 @@ void BX_CPU_C::debug(bx_address offset)
     BX_INFO(("| CR0=0x%08x CR2=0x%08x",
       BX_CPU_THIS_PTR cr0.get32(), (unsigned) BX_CPU_THIS_PTR cr2));
     BX_INFO(("| CR3=0x%08x CR4=0x%08x",
-      (unsigned) BX_CPU_THIS_PTR cr3, 
+      (unsigned) BX_CPU_THIS_PTR cr3,
       (unsigned) BX_CPU_THIS_PTR cr4.get32()));
 #endif
   }
@@ -326,11 +326,11 @@ bool BX_CPU_C::dbg_set_sreg(unsigned sreg_no, bx_segment_reg_t *sreg)
       handleAlignmentCheck(/* CPL change */);
 #endif
       invalidate_prefetch_q();
-      return 1;
+      return true;
     }
   }
 
-  return 0;
+  return false;
 }
 
 void BX_CPU_C::dbg_get_tr(bx_dbg_sreg_t *sreg)
