@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: disasm.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2013-2019 Stanislav Shwartsman
@@ -30,7 +30,7 @@
 #include "decoder.h"
 #include "fetchdecode.h"
 
-#if BX_DEBUGGER
+#if BX_DEBUGGER && !defined(BX_STANDALONE_DECODER)
 #include "../../bx_debug/debug.h"
 #define SYMBOLIC_JUMP(fmt)  fmt " %s"
 #define GET_SYMBOL(addr) bx_dbg_disasm_symbolic_address((addr), 0)
@@ -269,7 +269,7 @@ char *resolve_memsize(char *disbufptr, const bxInstruction_c *i, unsigned src_in
     case 4:
       disbufptr = dis_sprintf(disbufptr, "dword ptr ");
       break;
-        
+
     case 8:
       disbufptr = dis_sprintf(disbufptr, "qword ptr ");
       break;
@@ -293,12 +293,10 @@ char *resolve_memsize(char *disbufptr, const bxInstruction_c *i, unsigned src_in
   else if (src_index == BX_SRC_RM) {
     switch(src_type) {
     case BX_GPR8:
-    case BX_GPR32_MEM8:      // 8-bit  memory ref but 32-bit GPR
       disbufptr = dis_sprintf(disbufptr, "byte ptr ");
       break;
 
     case BX_GPR16:
-    case BX_GPR32_MEM16:     // 16-bit memory ref but 32-bit GPR
     case BX_SEGREG:
       disbufptr = dis_sprintf(disbufptr, "word ptr ");
       break;
@@ -329,7 +327,10 @@ char *resolve_memsize(char *disbufptr, const bxInstruction_c *i, unsigned src_in
         disbufptr = dis_sprintf(disbufptr, "xmmword ptr ");
       break;
 
-    default: 
+    case BX_TMM_REG:
+      break;
+
+    default:
       break;
     }
   }
@@ -401,8 +402,6 @@ char *disasm_regref(char *disbufptr, const bxInstruction_c *i, unsigned src_num,
     break;
 
   case BX_GPR32:
-  case BX_GPR32_MEM8:      // 8-bit  memory ref but 32-bit GPR
-  case BX_GPR32_MEM16:     // 16-bit memory ref but 32-bit GPR
     disbufptr = dis_sprintf(disbufptr, "%s", general_32bit_regname[srcreg]);
     break;
 
@@ -451,7 +450,7 @@ char *disasm_regref(char *disbufptr, const bxInstruction_c *i, unsigned src_num,
     break;
 
   case BX_KMASK_REG_PAIR:
-    disbufptr = dis_sprintf(disbufptr, "[%sk%d, %sk%d]", 
+    disbufptr = dis_sprintf(disbufptr, "[%sk%d, %sk%d]",
      (style == BX_DISASM_GAS) ? "%" : "",
       srcreg & ~1,
      (style == BX_DISASM_GAS) ? "%" : "",
@@ -460,6 +459,10 @@ char *disasm_regref(char *disbufptr, const bxInstruction_c *i, unsigned src_num,
     break;
 #endif
 
+  case BX_TMM_REG:
+    disbufptr = dis_sprintf(disbufptr, "tmm%d", srcreg);
+    break;
+
   case BX_SEGREG:
     disbufptr = dis_sprintf(disbufptr, "%s", segment_name[srcreg]);
     break;
@@ -542,7 +545,7 @@ char *disasm_immediate(char *disbufptr, const bxInstruction_c *i, unsigned src_t
       Bit32u imm32 = i->Id();
       Bit16u cs_selector = i->Iw2();
       disbufptr = dis_sprintf(disbufptr, "0x%04x:%08x", cs_selector, imm32);
-#if BX_DEBUGGER
+#if BX_DEBUGGER && !defined(BX_STANDALONE_DECODER)
       bx_address laddr = bx_dbg_get_laddr(cs_selector, imm32);
       // get the symbol
       const char *ptStrSymbol = bx_dbg_disasm_symbolic_address(laddr, 0);
@@ -554,7 +557,7 @@ char *disasm_immediate(char *disbufptr, const bxInstruction_c *i, unsigned src_t
       Bit16u imm16 = i->Iw();
       Bit16u cs_selector = i->Iw2();
       disbufptr = dis_sprintf(disbufptr, "0x%04x:%04x", cs_selector, imm16);
-#if BX_DEBUGGER
+#if BX_DEBUGGER && !defined(BX_STANDALONE_DECODER)
       bx_address laddr = bx_dbg_get_laddr(cs_selector, imm16);
       // get the symbol
       const char *ptStrSymbol = bx_dbg_disasm_symbolic_address(laddr, 0);
@@ -568,7 +571,7 @@ char *disasm_immediate(char *disbufptr, const bxInstruction_c *i, unsigned src_t
   case BX_DIRECT_MEMREF_W:
   case BX_DIRECT_MEMREF_D:
   case BX_DIRECT_MEMREF_Q:
-    disbufptr = dis_sprintf(disbufptr, "%s%s:", 
+    disbufptr = dis_sprintf(disbufptr, "%s%s:",
      (style == BX_DISASM_GAS) ? "%" : "", segment_name[i->seg()]);
 #if BX_SUPPORT_X86_64
     if (i->as64L())
@@ -897,3 +900,11 @@ char* disasm(const Bit8u *opcode, bool is_32, bool is_64, char *disbufptr, bxIns
 
   return disbufptr;
 }
+
+unsigned bx_disasm_wrapper(bool is_32, bool is_64, bx_address cs_base, bx_address ip, const Bit8u *instr, char *disbuf)
+{
+  bxInstruction_c i;
+  disasm(instr, is_32, is_64, disbuf, &i, cs_base, ip, BX_DISASM_INTEL);
+  unsigned ilen = i.ilen();
+  return ilen;
+}
