@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fetchdecode32.cc 14141 2021-02-11 15:05:06Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2019  The Bochs Project
+//  Copyright (C) 2001-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,8 @@
 
 #include "bochs.h"
 #ifndef BX_STANDALONE_DECODER
+#define NEED_CPU_REG_SHORTCUTS 1
+#define NEED_CPU_NEED_TEMPLATE_METHODS 1
 #include "../cpu.h"
 #endif
 
@@ -74,6 +76,14 @@ struct BxOpcodeDecodeDescriptor32 {
   const void *opcode_table;
 };
 
+#ifndef BX_STANDALONE_DECODER
+#include "cpu/simd_int.h"
+#include "cpu/fpu/softfloat.h"
+#include "cpu/simd_pfp.h"
+#include "cpu/simd_compare.h"
+#include "cpu/simd_vnni.h"
+#endif
+
 // table of all Bochs opcodes
 bxIAOpcodeTable BxOpcodesTable[] = {
 #ifndef BX_STANDALONE_DECODER
@@ -882,8 +892,8 @@ static BxOpcodeDecodeDescriptor32 decode32_descriptor[] =
 #endif
    /* 0F 38 F6 */ { &decoder32_modrm, BxOpcodeTable0F38F6 },
    /* 0F 38 F7 */ { &decoder_ud32, NULL },
-   /* 0F 38 F8 */ { &decoder_ud32, NULL },
-   /* 0F 38 F9 */ { &decoder32_modrm, BxOpcodeTable0F38F6 },
+   /* 0F 38 F8 */ { &decoder32_modrm, BxOpcodeTable0F38F8 },
+   /* 0F 38 F9 */ { &decoder32_modrm, BxOpcodeTable0F38F9 },
    /* 0F 38 FA */ { &decoder_ud32, NULL },
    /* 0F 38 FB */ { &decoder_ud32, NULL },
    /* 0F 38 FC */ { &decoder_ud32, NULL },
@@ -1688,9 +1698,7 @@ BxDecodeError assign_srcs(bxInstruction_c *i, unsigned ia_opcode, unsigned nnn,
       }
       else {
         unsigned tmpreg = BX_TMP_REGISTER;
-#if BX_SUPPORT_FPU
         if (type == BX_VMM_REG) tmpreg = BX_VECTOR_TMP_REGISTER;
-#endif
         i->setSrcReg(n, tmpreg);
       }
       break;
@@ -1746,6 +1754,11 @@ BxDecodeError assign_srcs(bxInstruction_c *i, unsigned ia_opcode, bool is_64, un
         if (i->isZeroMasking())
           return BX_EVEX_ILLEGAL_ZERO_MASKING_WITH_KMASK_SRC_OR_DEST;
       }
+#endif
+#if BX_SUPPORT_AMX
+      if (type == BX_TMM_REG) {
+        if (nnn >= 8) return BX_AMX_ILLEGAL_TILE_REGISTER;
+      }
 #endif
       break;
     case BX_SRC_RM:
@@ -1757,6 +1770,11 @@ BxDecodeError assign_srcs(bxInstruction_c *i, unsigned ia_opcode, bool is_64, un
           if (i->isZeroMasking())
             return BX_EVEX_ILLEGAL_ZERO_MASKING_WITH_KMASK_SRC_OR_DEST;
         }
+#endif
+#if BX_SUPPORT_AMX
+        if (type == BX_TMM_REG) {
+          if (rm >= 8) return BX_AMX_ILLEGAL_TILE_REGISTER;
+        }
 #endif
         i->setSrcReg(n, rm);
       }
@@ -1790,6 +1808,11 @@ BxDecodeError assign_srcs(bxInstruction_c *i, unsigned ia_opcode, bool is_64, un
         if (i->isZeroMasking())
           return BX_EVEX_ILLEGAL_ZERO_MASKING_WITH_KMASK_SRC_OR_DEST;
       }
+#endif
+#if BX_SUPPORT_AMX
+      if (type == BX_TMM_REG) {
+        if (vvv >= 8) return BX_AMX_ILLEGAL_TILE_REGISTER;
+      }
 #endif
       break;
     case BX_SRC_VIB:
@@ -1915,7 +1938,7 @@ int decoder_vex32(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i, unsig
     }
     else {
       iptr = decodeModrm32(iptr, remain, i, mod, nnn, rm);
-      if (! iptr) 
+      if (! iptr)
         return(-1);
     }
   }
@@ -2045,13 +2068,13 @@ int decoder_evex32(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i, unsi
 
   if (evex_z && ! opmask)
     return(ia_opcode);
-    
+
   unsigned opcode_byte = (evex >> 24);
   opcode_byte += 256 * (evex_opcext-1);
 
   struct bx_modrm modrm;
   iptr = parseModrm32(iptr, remain, i, &modrm);
-  if (! iptr) 
+  if (! iptr)
     return(-1);
 
   if (modrm.mod == 0x40) { // mod==01b
@@ -2151,7 +2174,7 @@ int decoder_xop32(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i, unsig
 
   struct bx_modrm modrm;
   iptr = parseModrm32(iptr, remain, i, &modrm);
-  if (! iptr) 
+  if (! iptr)
     return(-1);
 
   Bit32u decmask = (i->osize() << OS32_OFFSET) |
@@ -2188,7 +2211,7 @@ int decoder32_fp_escape(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i,
 
   struct bx_modrm modrm;
   iptr = parseModrm32(iptr, remain, i, &modrm);
-  if (! iptr) 
+  if (! iptr)
     return(-1);
 
   i->setFoo((modrm.modrm | (b1 << 8)) & 0x7ff); /* for x87 */
@@ -2221,7 +2244,7 @@ int decoder32_modrm(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i, uns
   // opcode requires modrm byte
   struct bx_modrm modrm;
   iptr = parseModrm32(iptr, remain, i, &modrm);
-  if (! iptr) 
+  if (! iptr)
     return(-1);
 
   Bit32u decmask = (i->osize() << OS32_OFFSET) |
@@ -2305,7 +2328,7 @@ int decoder32_3dnow(const Bit8u *iptr, unsigned &remain, bxInstruction_c *i, uns
   // opcode requires modrm byte
   struct bx_modrm modrm;
   iptr = parseModrm32(iptr, remain, i, &modrm);
-  if (! iptr) 
+  if (! iptr)
     return(-1);
 
   if (remain != 0) {
@@ -2394,7 +2417,7 @@ fetch_b1:
   remain--;
 
 #if BX_SUPPORT_CET
-  // DS prefix is still recorded for CET Endranch suppress hint even if overridden by other prefixes later
+  // DS prefix is still recorded for CET Endbranch suppress hint even if overridden by other prefixes later
   if (b1 == 0x3e)
     seg_override_cet = BX_SEG_REG_DS;
 #endif
@@ -2471,7 +2494,7 @@ fetch_b1:
 #endif
 
   i->modRMForm.Id = 0;
- 
+
   BxOpcodeDecodeDescriptor32 *decode_descriptor = &decode32_descriptor[b1];
   ia_opcode = decode_descriptor->decode_method(iptr, remain, i, b1, sse_prefix, decode_descriptor->opcode_table);
   if (ia_opcode < 0)
@@ -2555,6 +2578,16 @@ int assignHandler(bxInstruction_c *i, Bit32u fetchModeMask)
     }
   }
 #endif
+  if (! (fetchModeMask & BX_FETCH_MODE_FPU_MMX_OK)) {
+     if (op_flags & BX_PREPARE_FPU) {
+        if (i->execute1 != &BX_CPU_C::BxError) i->execute1 = &BX_CPU_C::BxNoFPU;
+        return(1);
+     }
+     if (op_flags & BX_PREPARE_MMX) {
+        if (i->execute1 != &BX_CPU_C::BxError) i->execute1 = &BX_CPU_C::BxNoMMX;
+        return(1);
+     }
+  }
 #if BX_CPU_LEVEL >= 6
   if (! (fetchModeMask & BX_FETCH_MODE_SSE_OK)) {
      if (op_flags & BX_PREPARE_SSE) {
@@ -2582,6 +2615,14 @@ int assignHandler(bxInstruction_c *i, Bit32u fetchModeMask)
        return(1);
     }
   }
+#if BX_SUPPORT_AMX
+  if (! (fetchModeMask & BX_FETCH_MODE_AMX_OK)) {
+    if (op_flags & BX_PREPARE_AMX) {
+       if (i->execute1 != &BX_CPU_C::BxError) i->execute1 = &BX_CPU_C::BxNoAMX;
+       return(1);
+    }
+  }
+#endif
 #endif
 #endif
 #endif
