@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: event.cc 14233 2021-04-27 08:22:04Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011-2013 Stanislav Shwartsman
+//   Copyright (c) 2011-2023 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -25,6 +25,14 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_APIC
+#include "apic.h"
+#endif
+
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #include "iodev/iodev.h"
 
 bool BX_CPU_C::handleWaitForEvent(void)
@@ -41,7 +49,7 @@ bool BX_CPU_C::handleWaitForEvent(void)
   // an interrupt wakes up the CPU.
   while (1)
   {
-    if ((is_pending(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR) && (BX_CPU_THIS_PTR get_IF() || BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_MWAIT_IF)) ||
+    if ((is_pending(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR | BX_EVENT_PENDING_UINTR) && (BX_CPU_THIS_PTR get_IF() || BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_MWAIT_IF)) ||
          is_unmasked_event_pending(BX_EVENT_NMI | BX_EVENT_SMI | BX_EVENT_INIT |
             BX_EVENT_VMX_VTPR_UPDATE |
             BX_EVENT_VMX_VEOI_UPDATE |
@@ -105,10 +113,23 @@ bool BX_CPU_C::handleWaitForEvent(void)
   return 0;
 }
 
-void BX_CPU_C::InterruptAcknowledge(void)
+Bit8u BX_CPU_C::interrupt_acknowledge(void)
 {
   Bit8u vector;
 
+#if BX_SUPPORT_APIC
+  if (is_pending(BX_EVENT_PENDING_LAPIC_INTR))
+    vector = BX_CPU_THIS_PTR lapic->acknowledge_int();
+  else
+#endif
+    // if no local APIC, always acknowledge the PIC.
+    vector = DEV_pic_iac(); // may set INTR with next interrupt
+
+  return vector;
+}
+
+void BX_CPU_C::HandleExtInterrupt(void)
+{
 #if BX_SUPPORT_SVM
   if (BX_CPU_THIS_PTR in_svm_guest) {
     if (SVM_INTERCEPT(SVM_INTERCEPT0_INTR)) Svm_Vmexit(SVM_VMEXIT_INTR);
@@ -129,29 +150,39 @@ void BX_CPU_C::InterruptAcknowledge(void)
   }
 #endif
 
-  // NOTE: similar code in ::take_irq()
-#if BX_SUPPORT_APIC
-  if (is_pending(BX_EVENT_PENDING_LAPIC_INTR))
-    vector = BX_CPU_THIS_PTR lapic.acknowledge_int();
-  else
+  Bit8u vector = interrupt_acknowledge();
+#if BX_SUPPORT_VMX >= 2
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
+    if (VMX_Posted_Interrupt_Processing(vector)) return;
+  }
 #endif
-    // if no local APIC, always acknowledge the PIC.
-    vector = DEV_pic_iac(); // may set INTR with next interrupt
 
   BX_CPU_THIS_PTR EXT = 1; /* external event */
 #if BX_SUPPORT_VMX
   VMexit_Event(BX_EXTERNAL_INTERRUPT, vector, 0, 0);
 #endif
 
-  BX_INSTR_HWINTERRUPT(BX_CPU_ID, vector,
+#if BX_SUPPORT_UINTR
+  if (BX_CPU_THIS_PTR cr4.get_UINTR() && long64_mode() && vector == BX_CPU_THIS_PTR uintr.uinv)
+  {
+#if BX_SUPPORT_APIC
+    BX_CPU_THIS_PTR lapic->receive_EOI();
+#endif
+    Process_UINTR_Notification();
+  }
+  else
+#endif
+  {
+    BX_INSTR_HWINTERRUPT(BX_CPU_ID, vector,
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
-  interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
+    interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
+  }
 
   BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
 }
 
 #if BX_SUPPORT_SVM
-void BX_CPU_C::VirtualInterruptAcknowledge(void)
+void BX_CPU_C::SvmVirtualInterruptAcknowledge(void)
 {
   Bit8u vector = SVM_V_INTR_VECTOR;
 
@@ -161,7 +192,7 @@ void BX_CPU_C::VirtualInterruptAcknowledge(void)
 
   BX_CPU_THIS_PTR EXT = 1; /* external event */
 
-  BX_INSTR_HWINTERRUPT(BX_CPU_ID, vector, 
+  BX_INSTR_HWINTERRUPT(BX_CPU_ID, vector,
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
   interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
 
@@ -233,15 +264,20 @@ bool BX_CPU_C::handleAsyncEvent(void)
   if (is_unmasked_event_pending(BX_EVENT_INIT) && SVM_GIF) {
 #if BX_SUPPORT_SVM
     if (BX_CPU_THIS_PTR in_svm_guest) {
-      if (SVM_INTERCEPT(SVM_INTERCEPT0_INIT)) Svm_Vmexit(SVM_VMEXIT_INIT);
+      if (SVM_INTERCEPT(SVM_INTERCEPT0_INIT)) Svm_Vmexit(SVM_VMEXIT_INIT); // INIT is still pending
+    }
+    if (BX_CPU_THIS_PTR msr.svm_vm_cr & BX_VM_CR_MSR_INIT_REDIRECT_MASK) {
+      clear_event(BX_EVENT_INIT);     // INIT is no longer pending
+      BX_INFO(("SVM INIT Redirect to #SX"));
+      exception(BX_SX_EXCEPTION, 1);  // The only error code is 1, and indicates redirection of INIT
     }
 #endif
+    clear_event(BX_EVENT_INIT);
 #if BX_SUPPORT_VMX
     if (BX_CPU_THIS_PTR in_vmx_guest) {
       VMexit(VMX_VMEXIT_INIT, 0);
     }
 #endif
-    // reset will clear pending INIT
     reset(BX_RESET_SOFTWARE);
 
 #if BX_SUPPORT_SMP
@@ -258,8 +294,13 @@ bool BX_CPU_C::handleAsyncEvent(void)
   }
 
 #if BX_SUPPORT_VMX
-  if (is_unmasked_event_pending(BX_EVENT_VMX_MONITOR_TRAP_FLAG)) {
-    VMexit(VMX_VMEXIT_MONITOR_TRAP_FLAG, 0);
+  if (is_pending(BX_EVENT_VMX_MONITOR_TRAP_FLAG)) {
+    if (is_unmasked_event_pending(BX_EVENT_VMX_MONITOR_TRAP_FLAG)) {
+      VMexit(VMX_VMEXIT_MONITOR_TRAP_FLAG, 0);
+    }
+    else {
+      unmask_event(BX_EVENT_VMX_MONITOR_TRAP_FLAG);
+    }
   }
 #endif
 
@@ -287,6 +328,14 @@ bool BX_CPU_C::handleAsyncEvent(void)
   if (interrupts_inhibited(BX_INHIBIT_INTERRUPTS) || ! SVM_GIF) {
     // Processing external interrupts is inhibited on this
     // boundary because of certain instructions like STI.
+#if BX_SUPPORT_UINTR
+    // execution of STI doesn't block User interrupts delivery, only MOV_SS does
+    if (! interrupts_inhibited(BX_INHIBIT_INTERRUPTS_BY_MOVSS)) {
+      if (is_unmasked_event_pending(BX_EVENT_PENDING_UINTR)) {
+        deliver_UINTR();
+      }
+    }
+#endif
   }
 #if BX_SUPPORT_VMX >= 2
   else if (is_unmasked_event_pending(BX_EVENT_VMX_PREEMPTION_TIMER_EXPIRED)) {
@@ -305,11 +354,11 @@ bool BX_CPU_C::handleAsyncEvent(void)
     }
 #endif
     clear_event(BX_EVENT_NMI);
-     mask_event(BX_EVENT_NMI);
     BX_CPU_THIS_PTR EXT = 1; /* external event */
 #if BX_SUPPORT_VMX
     VMexit_Event(BX_NMI, 2, 0, 0);
 #endif
+    mask_event(BX_EVENT_NMI);
     BX_INSTR_HWINTERRUPT(BX_CPU_ID, 2, BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
     interrupt(2, BX_NMI, 0, 0);
   }
@@ -322,13 +371,18 @@ bool BX_CPU_C::handleAsyncEvent(void)
   else if (is_unmasked_event_pending(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR |
                                      BX_EVENT_PENDING_VMX_VIRTUAL_INTR))
   {
-    InterruptAcknowledge();
+    HandleExtInterrupt();
   }
+#if BX_SUPPORT_UINTR
+  else if (is_unmasked_event_pending(BX_EVENT_PENDING_UINTR))
+  {
+    deliver_UINTR();
+  }
+#endif
 #if BX_SUPPORT_SVM
   else if (is_unmasked_event_pending(BX_EVENT_SVM_VIRQ_PENDING))
   {
-    // virtual interrupt acknowledge
-    VirtualInterruptAcknowledge();
+    SvmVirtualInterruptAcknowledge();
   }
 #endif
   else if (BX_HRQ && BX_DBG_ASYNC_DMA) {
@@ -369,7 +423,7 @@ bool BX_CPU_C::handleAsyncEvent(void)
   //   Alignment check
   // (handled by rest of the code)
 
-  if (!((SVM_GIF && unmasked_events_pending()) || BX_CPU_THIS_PTR debug_trap ||
+  if (!((SVM_GIF && unmasked_events_pending()) || BX_CPU_THIS_PTR debug_trap || is_unmasked_event_pending(BX_EVENT_VMX_MONITOR_TRAP_FLAG) ||
 //      BX_CPU_THIS_PTR get_TF() || // implies debug_trap is set
         BX_HRQ))
   {
