@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: flag_ctrl_pro.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2011  The Bochs Project
@@ -25,6 +25,10 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::setEFlags(Bit32u new_eflags)
 {
   Bit32u eflags = BX_CPU_THIS_PTR eflags;
@@ -129,17 +133,24 @@ void BX_CPU_C::handleInterruptMaskChange(void)
     }
 #endif
 
+#if BX_SUPPORT_UINTR
+    if (!uintr_masked()) unmask_event(BX_EVENT_PENDING_UINTR);
+#endif
+
     return;
   }
- 
+
   // EFLAGS.IF was cleared, some events like INTR would be masked
 
 #if BX_SUPPORT_VMX
-  if (BX_CPU_THIS_PTR in_vmx_guest && PIN_VMEXIT(VMX_VM_EXEC_CTRL1_EXTERNAL_INTERRUPT_VMEXIT)) {
-     // if 'External-interrupt exiting' control is set, the value of EFLAGS.IF
-     // doesn't affect interrupt blocking
-     mask_event(BX_EVENT_VMX_INTERRUPT_WINDOW_EXITING | BX_EVENT_PENDING_VMX_VIRTUAL_INTR);
-     unmask_event(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR);
+  if (BX_CPU_THIS_PTR in_vmx_guest && BX_CPU_THIS_PTR vmcs.pin_vmexec_ctrls.EXTERNAL_INTERRUPT_VMEXIT()) {
+    // if 'External-interrupt exiting' control is set, the value of EFLAGS.IF
+    // doesn't affect interrupt blocking
+    mask_event(BX_EVENT_VMX_INTERRUPT_WINDOW_EXITING | BX_EVENT_PENDING_VMX_VIRTUAL_INTR);
+    unmask_event(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR);
+#if BX_SUPPORT_UINTR
+    if (!uintr_masked()) unmask_event(BX_EVENT_PENDING_UINTR);
+#endif
      return;
   }
 #endif
@@ -147,7 +158,7 @@ void BX_CPU_C::handleInterruptMaskChange(void)
 #if BX_SUPPORT_SVM
   if (BX_CPU_THIS_PTR in_svm_guest && SVM_V_INTR_MASKING) {
      if (! SVM_HOST_IF)
-       mask_event(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR);
+       mask_event(BX_EVENT_PENDING_INTR | BX_EVENT_PENDING_LAPIC_INTR | BX_EVENT_PENDING_UINTR);
 
      mask_event(BX_EVENT_SVM_VIRQ_PENDING);
   }
@@ -157,6 +168,7 @@ void BX_CPU_C::handleInterruptMaskChange(void)
      mask_event(BX_EVENT_VMX_INTERRUPT_WINDOW_EXITING |
                 BX_EVENT_PENDING_INTR |
                 BX_EVENT_PENDING_LAPIC_INTR |
+                BX_EVENT_PENDING_UINTR |
                 BX_EVENT_PENDING_VMX_VIRTUAL_INTR |
                 BX_EVENT_SVM_VIRQ_PENDING);
   }
