@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_arith.cc 13466 2018-02-16 07:57:32Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003-2018 Stanislav Shwartsman
@@ -98,20 +98,20 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float32 b32, int bIsNaN, float_s
     }
 }
 
-int FPU_handle_NaN(floatx80 a, float32 b, floatx80 &r, float_status_t &status)
+bool FPU_handle_NaN(floatx80 a, float32 b, floatx80 &r, float_status_t &status)
 {
   if (floatx80_is_unsupported(a)) {
      float_raise(status, float_flag_invalid);
      r = floatx80_default_nan;
-     return 1;
+     return true;
   }
 
   int aIsNaN = floatx80_is_nan(a), bIsNaN = float32_is_nan(b);
   if (aIsNaN | bIsNaN) {
      r = FPU_handle_NaN(a, aIsNaN, b, bIsNaN, status);
-     return 1;
+     return true;
   }
-  return 0;
+  return false;
 }
 
 floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, float_status_t &status)
@@ -147,25 +147,25 @@ floatx80 FPU_handle_NaN(floatx80 a, int aIsNaN, float64 b64, int bIsNaN, float_s
     }
 }
 
-int FPU_handle_NaN(floatx80 a, float64 b, floatx80 &r, float_status_t &status)
+bool FPU_handle_NaN(floatx80 a, float64 b, floatx80 &r, float_status_t &status)
 {
   if (floatx80_is_unsupported(a)) {
      float_raise(status, float_flag_invalid);
      r = floatx80_default_nan;
-     return 1;
+     return true;
   }
 
   int aIsNaN = floatx80_is_nan(a), bIsNaN = float64_is_nan(b);
   if (aIsNaN | bIsNaN) {
      r = FPU_handle_NaN(a, aIsNaN, b, bIsNaN, status);
-     return 1;
+     return true;
   }
-  return 0;
+  return false;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -179,7 +179,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->src());
 
-  float_status_t status = 
+  float_status_t status =
      i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
@@ -192,7 +192,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -224,7 +224,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -253,7 +253,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -282,7 +282,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -312,7 +312,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -342,7 +342,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -369,7 +369,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -401,7 +401,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -430,7 +430,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -459,7 +459,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -489,7 +489,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -519,7 +519,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -546,7 +546,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -573,7 +573,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -605,7 +605,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -637,7 +637,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -666,7 +666,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -695,7 +695,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -724,7 +724,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -754,7 +754,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -784,7 +784,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -814,7 +814,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -841,7 +841,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -871,7 +871,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -898,7 +898,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -925,7 +925,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -957,7 +957,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int pop_stack = i->b1() & 2;
@@ -989,7 +989,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -1018,7 +1018,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -1047,7 +1047,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -1076,7 +1076,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -1105,7 +1105,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -1135,7 +1135,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -1165,7 +1165,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -1195,7 +1195,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -1225,7 +1225,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSQRT(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
@@ -1249,7 +1249,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSQRT(bxInstruction_c *i)
 /* D9 FC */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRNDINT(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
