@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_load_store.cc 13466 2018-02-16 07:57:32Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003-2018 Stanislav Shwartsman
@@ -38,7 +38,7 @@ extern float_status_t i387cw_to_softfloat_status_word(Bit16u control_word);
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_STi(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   FPU_update_last_instruction(i);
 
   clear_C1();
@@ -55,7 +55,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_STi(bxInstruction_c *i)
   {
     FPU_exception(i, FPU_EX_Stack_Underflow);
 
-    if (! BX_CPU_THIS_PTR the_i387.is_IA_masked()) 
+    if (! BX_CPU_THIS_PTR the_i387.is_IA_masked())
       BX_NEXT_INSTR(i);
   }
   else {
@@ -70,7 +70,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_STi(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
@@ -101,7 +101,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
@@ -132,7 +132,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_DOUBLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_EXTENDED_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   floatx80 result;
 
@@ -158,7 +158,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_EXTENDED_REAL(bxInstruction_c *i)
 /* DF /0 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
@@ -182,7 +182,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_WORD_INTEGER(bxInstruction_c *i)
 /* DB /0 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
@@ -206,7 +206,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_DWORD_INTEGER(bxInstruction_c *i)
 /* DF /5 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_QWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit64s load_reg = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
@@ -230,7 +230,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FILD_QWORD_INTEGER(bxInstruction_c *i)
 /* DF /4 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FBLD_PACKED_BCD(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
   Bit16u hi2 = read_virtual_word(i->seg(), (RMAddr(i) + 8) & i->asize_mask());
@@ -272,17 +272,20 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FBLD_PACKED_BCD(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_STi(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
   FPU_update_last_instruction(i);
 
-  int pop_stack = 0;
-  if (i->getIaOpcode() == BX_IA_FSTP_STi)
-      pop_stack = 1;
+  unsigned opcode = i->getIaOpcode();
+  int pop_stack = (opcode != BX_IA_FST_STi);
 
   clear_C1();
 
   if (IS_TAG_EMPTY(0)) {
-    FPU_stack_underflow(i, i->dst(), pop_stack);
+    // D9D8..D9DF - Behaves the same as FSTP (DDD8..DDDF) but won't cause a stack underflow exception
+    if (opcode != BX_IA_FSTP_SPECIAL_STi)
+        FPU_stack_underflow(i, i->dst(), pop_stack);
+    else
+        BX_CPU_THIS_PTR the_i387.FPU_pop();
   }
   else {
     floatx80 st0_reg = BX_READ_FPU_REG(0);
@@ -297,7 +300,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_STi(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -309,9 +312,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i)
 
   float32 save_reg = float32_default_nan; /* The masked response */
 
-  int pop_stack = 0;
-  if (i->getIaOpcode() == BX_IA_FSTP_SINGLE_REAL)
-      pop_stack = 1;
+  int pop_stack = (i->getIaOpcode() == BX_IA_FSTP_SINGLE_REAL);
 
   if (IS_TAG_EMPTY(0))
   {
@@ -345,7 +346,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -357,9 +358,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i)
 
   float64 save_reg = float64_default_nan; /* The masked response */
 
-  int pop_stack = 0;
-  if (i->getIaOpcode() == BX_IA_FSTP_DOUBLE_REAL)
-      pop_stack = 1;
+  int pop_stack = (i->getIaOpcode() == BX_IA_FSTP_DOUBLE_REAL);
 
   if (IS_TAG_EMPTY(0))
   {
@@ -394,7 +393,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i)
 /* DB /7 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSTP_EXTENDED_REAL(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -426,7 +425,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSTP_EXTENDED_REAL(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -436,9 +435,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c *i)
 
   Bit16s save_reg = int16_indefinite;
 
-  int pop_stack = 0;
-  if (i->getIaOpcode() == BX_IA_FISTP_WORD_INTEGER)
-      pop_stack = 1;
+  int pop_stack = (i->getIaOpcode() == BX_IA_FISTP_WORD_INTEGER);
 
   clear_C1();
 
@@ -474,7 +471,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -484,9 +481,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c *i)
 
   Bit32s save_reg = int32_indefinite; /* The masked response */
 
-  int pop_stack = 0;
-  if (i->getIaOpcode() == BX_IA_FISTP_DWORD_INTEGER)
-      pop_stack = 1;
+  int pop_stack = (i->getIaOpcode() == BX_IA_FISTP_DWORD_INTEGER);
 
   clear_C1();
 
@@ -522,7 +517,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTP_QWORD_INTEGER(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -566,7 +561,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTP_QWORD_INTEGER(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -645,7 +640,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *i)
 /* DF /1 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP16(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -690,7 +685,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP16(bxInstruction_c *i)
 /* DB /1 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP32(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
@@ -735,7 +730,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP32(bxInstruction_c *i)
 /* DD /1 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP64(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_check_pending_exceptions();
 
   RMAddr(i) = BX_CPU_RESOLVE_ADDR(i);
 
