@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: generic_cpuid.h 14149 2021-02-16 18:57:49Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2015 Stanislav Shwartsman
@@ -81,18 +81,14 @@ private:
   void get_ext_cpuid_leaf_5(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const;
-  void get_ext_cpuid_leaf_8(cpuid_function_t *leaf) const;
 #if BX_SUPPORT_SVM
   void get_ext_cpuid_leaf_A(cpuid_function_t *leaf) const;
 #endif
 
-  Bit32u get_std2_cpuid_features(void) const;
   Bit32u get_ext2_cpuid_features(void) const;
-  Bit32u get_extended_cpuid_features(void) const;
 #endif
 
   Bit32u get_cpu_version_information(void) const;
-  Bit32u get_std_cpuid_features(void) const;
 };
 
 extern bx_cpuid_t *create_bx_generic_cpuid(BX_CPU_C *cpu);
