@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: icache.h 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2007-2015 Stanislav Shwartsman
@@ -28,12 +28,12 @@ extern void handleSMC(bx_phy_address pAddr, Bit32u mask);
 
 class bxPageWriteStampTable
 {
-  const Bit32u PHY_MEM_PAGES = 1024*1024;
+  const Bit32u PHY_MEM_PAGES_IN_4G_SPACE;
   Bit32u *fineGranularityMapping;
 
 public:
-  bxPageWriteStampTable() {
-    fineGranularityMapping = new Bit32u[PHY_MEM_PAGES];
+  bxPageWriteStampTable(): PHY_MEM_PAGES_IN_4G_SPACE(1024*1024) {
+    fineGranularityMapping = new Bit32u[PHY_MEM_PAGES_IN_4G_SPACE];
     resetWriteStamps();
   }
  ~bxPageWriteStampTable() { delete [] fineGranularityMapping; }
@@ -85,7 +85,7 @@ public:
           // one of the CPUs might be running trace from this page
           handleSMC(pAddr, mask);
           fineGranularityMapping[index] &= ~mask;
-       }       
+       }
     }
   }
 
@@ -94,7 +94,7 @@ public:
 
 BX_CPP_INLINE void bxPageWriteStampTable::resetWriteStamps(void)
 {
-  for (Bit32u i=0; i<PHY_MEM_PAGES; i++) {
+  for (Bit32u i=0; i<PHY_MEM_PAGES_IN_4G_SPACE; i++) {
     fineGranularityMapping[i] = 0;
   }
 }
@@ -140,7 +140,7 @@ public:
 
 #define BX_ICACHE_PAGE_SPLIT_ENTRIES 8 /* must be power of two */
   struct pageSplitEntryIndex {
-    bx_phy_address ppf; // Physical address of 2nd page of the trace 
+    bx_phy_address ppf; // Physical address of 2nd page of the trace
     bxICacheEntry_c *e; // Pointer to icache entry
   } pageSplitIndex[BX_ICACHE_PAGE_SPLIT_ENTRIES];
   int nextPageSplitIndex;
