@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc 14320 2021-07-25 18:01:28Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2019  The Bochs Project
+//  Copyright (C) 2001-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -29,15 +29,24 @@
 #include "param_names.h"
 #include "cpustats.h"
 
+#if BX_SUPPORT_APIC
+#include "apic.h"
+#endif
+
+#if BX_SUPPORT_AMX
+#include "avx/amx.h"
+#endif
+
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #include <stdlib.h>
 
 BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 #if BX_CPU_LEVEL >= 4
    , cpuid(NULL)
 #endif
-#if BX_SUPPORT_APIC
-   ,lapic (this, id)
-#endif
 {
   // in case of SMF, you cannot reference any member data
   // in the constructor because the only access to it is via
@@ -47,6 +56,10 @@ BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
   sprintf(logname, "cpu%x", bx_cpuid);
   put(logname, name);
 
+#if BX_SUPPORT_APIC
+  lapic = new bx_local_apic_c(this, bx_cpuid);
+#endif
+
   for (unsigned n=0;n<BX_ISA_EXTENSIONS_ARRAY_SIZE;n++)
     ia_extensions_bitmask[n] = 0;
 
@@ -126,6 +139,20 @@ void BX_CPU_C::initialize(void)
   xsave_xrestor_init();
 #endif
 
+#if BX_SUPPORT_AMX
+  amx = NULL;
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AMX)) {
+    amx = new AMX;
+  }
+#endif
+
+#if BX_SUPPORT_SVM
+  vmcb = NULL;
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SVM)) {
+    vmcb = new VMCB_CACHE;
+  }
+#endif
+
 #if BX_CONFIGURE_MSRS
   for (unsigned n=0; n < BX_MSR_MAX_INDEX; n++) {
     BX_CPU_THIS_PTR msrs[n] = 0;
@@ -427,6 +454,18 @@ void BX_CPU_C::register_state(void)
 #if BX_SUPPORT_VMX
   BXRS_HEX_PARAM_FIELD(MSR, ia32_feature_ctrl, msr.ia32_feature_ctrl);
 #endif
+#if BX_SUPPORT_MONITOR_MWAIT
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_WAITPKG)) {
+    BXRS_HEX_PARAM_FIELD(MSR, ia32_umwait_ctrl, msr.ia32_umwait_ctrl);
+  }
+#endif
+
+#if BX_SUPPORT_SVM
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SVM)) {
+    BXRS_HEX_PARAM_FIELD(MSR, svm_hsave_pa, msr.svm_hsave_pa);
+    BXRS_HEX_PARAM_FIELD(MSR, svm_vm_cr, msr.svm_vm_cr);
+  }
+#endif
 
 #if BX_CONFIGURE_MSRS
   bx_list_c *MSRS = new bx_list_c(cpu, "USER_MSR");
@@ -444,6 +483,18 @@ void BX_CPU_C::register_state(void)
 #endif
 #endif
 
+#if BX_SUPPORT_UINTR
+  bx_list_c *UINTR = new bx_list_c(cpu, "UINTR");
+  BXRS_PARAM_BOOL(UINTR, UIF, uintr.UIF);
+  BXRS_HEX_PARAM_FIELD(UINTR, uirr, uintr.uirr);
+  BXRS_HEX_PARAM_FIELD(UINTR, ui_handler, uintr.ui_handler);
+  BXRS_HEX_PARAM_FIELD(UINTR, stack_adjust, uintr.stack_adjust);
+  BXRS_HEX_PARAM_FIELD(UINTR, uinv, uintr.uinv);
+  BXRS_HEX_PARAM_FIELD(UINTR, uitt_size, uintr.uitt_size);
+  BXRS_HEX_PARAM_FIELD(UINTR, uitt_addr, uintr.uitt_addr);
+  BXRS_HEX_PARAM_FIELD(UINTR, upid_addr, uintr.upid_addr);
+#endif
+
 #if BX_SUPPORT_FPU
   bx_list_c *fpu = new bx_list_c(cpu, "FPU");
   BXRS_HEX_PARAM_FIELD(fpu, cwd, the_i387.cwd);
@@ -483,16 +534,37 @@ void BX_CPU_C::register_state(void)
     }
   }
 #endif
+
+#if BX_SUPPORT_AMX
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AMX)) {
+    bx_list_c *amx_list = new bx_list_c(cpu, "AMX");
+    BXRS_DEC_PARAM_FIELD(amx_list, palette, amx->palette_id);
+    BXRS_DEC_PARAM_FIELD(amx_list, start_row, amx->start_row);
+    BXRS_HEX_PARAM_FIELD(amx_list, tile_use_tracker, amx->tile_use_tracker);
+    for (n=0; n<8; n++) {
+      sprintf(name, "tile%d_rows", n);
+      new bx_shadow_num_c(amx_list, name, &(amx->tilecfg[n].rows), BASE_DEC);
+      sprintf(name, "tile%d_colsb", n);
+      new bx_shadow_num_c(amx_list, name, &(amx->tilecfg[n].bytes_per_row), BASE_DEC);
+      for(unsigned row=0;row < 16;row++) {
+        for(unsigned j=0;j < BX_VLMAX*2;j++) {
+          sprintf(name, "tile%d_row%d_%d", n, row, j);
+          new bx_shadow_num_c(amx_list, name, &(amx->tile[n].row[row].vmm64u(j)), BASE_HEX);
+        }
+      }
+    }
+  }
+#endif
 #endif // BX_CPU_LEVEL >= 6
 
 #if BX_SUPPORT_MONITOR_MWAIT
   bx_list_c *monitor_list = new bx_list_c(cpu, "MONITOR");
   BXRS_HEX_PARAM_FIELD(monitor_list, monitor_addr, monitor.monitor_addr);
-  BXRS_PARAM_BOOL(monitor_list, armed, monitor.armed);
+  BXRS_DEC_PARAM_FIELD(monitor_list, armed, monitor.armed_by);
 #endif
 
 #if BX_SUPPORT_APIC
-  lapic.register_state(cpu);
+  lapic->register_state(cpu);
 #endif
 
 #if BX_SUPPORT_VMX
@@ -507,10 +579,6 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_SIMPLE32(cpu, event_mask);
   BXRS_HEX_PARAM_SIMPLE32(cpu, async_event);
 
-#if BX_X86_DEBUGGER
-  BXRS_PARAM_BOOL(cpu, in_repeat, in_repeat);
-#endif
-
   BXRS_PARAM_BOOL(cpu, in_smm, in_smm);
 
 #if BX_DEBUGGER
@@ -651,8 +719,6 @@ void BX_CPU_C::param_restore(bx_param_c *param, Bit64s val)
 
 void BX_CPU_C::after_restore_state(void)
 {
-  handleCpuContextChange();
-
   BX_CPU_THIS_PTR prev_rip = RIP;
 
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_IA32_REAL) CPL = 0;
@@ -672,6 +738,12 @@ void BX_CPU_C::after_restore_state(void)
   set_PKeys(BX_CPU_THIS_PTR pkru, BX_CPU_THIS_PTR pkrs);
 #endif
 
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR linaddr_width = BX_CPU_THIS_PTR cr4.get_LA57() ? 57 : 48;
+#endif
+
+  handleCpuContextChange();
+
   assert_checks();
   debug(RIP);
 }
@@ -683,6 +755,18 @@ BX_CPU_C::~BX_CPU_C()
   delete cpuid;
 #endif
 
+#if BX_SUPPORT_APIC
+  delete lapic;
+#endif
+
+#if BX_SUPPORT_AMX
+  delete amx;
+#endif
+
+#if BX_SUPPORT_SVM
+  delete vmcb;
+#endif
+
 #if InstrumentCPU
   delete stats;
 #endif
@@ -849,10 +933,7 @@ void BX_CPU_C::reset(unsigned source)
 #endif
   BX_CPU_THIS_PTR dr7.val32 = 0x00000400;
 
-#if BX_X86_DEBUGGER
-  BX_CPU_THIS_PTR in_repeat = 0;
-#endif
-  BX_CPU_THIS_PTR in_smm = 0;
+  BX_CPU_THIS_PTR in_smm = false;
 
   BX_CPU_THIS_PTR pending_event = 0;
   BX_CPU_THIS_PTR event_mask = 0;
@@ -876,6 +957,9 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR cr4.set32(0);
   BX_CPU_THIS_PTR cr4_suppmask = get_cr4_allow_mask();
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR linaddr_width = 48;
+#endif
 #endif
 
 #if BX_CPU_LEVEL >= 6
@@ -883,9 +967,19 @@ void BX_CPU_C::reset(unsigned source)
     BX_CPU_THIS_PTR xcr0.set32(0x1);
   }
   BX_CPU_THIS_PTR xcr0_suppmask = get_xcr0_allow_mask();
+  BX_CPU_THIS_PTR ia32_xss_suppmask = get_ia32_xss_allow_mask();
 
   BX_CPU_THIS_PTR msr.ia32_xss = 0;
 
+#if BX_SUPPORT_MONITOR_MWAIT
+  BX_CPU_THIS_PTR msr.ia32_umwait_ctrl = 0;
+#endif
+
+#if BX_SUPPORT_SVM
+  BX_CPU_THIS_PTR msr.svm_hsave_pa = 0;
+  BX_CPU_THIS_PTR msr.svm_vm_cr = 0;     // enable SVME if was disabled, clear LOCK bit
+#endif
+
 #if BX_SUPPORT_CET
   BX_CPU_THIS_PTR msr.ia32_interrupt_ssp_table = 0;
   BX_CPU_THIS_PTR msr.ia32_cet_control[0] = BX_CPU_THIS_PTR msr.ia32_cet_control[1] = 0;
@@ -895,6 +989,10 @@ void BX_CPU_C::reset(unsigned source)
 #endif
 #endif // BX_CPU_LEVEL >= 6
 
+#if BX_SUPPORT_UINTR
+  memset(&BX_CPU_THIS_PTR uintr, 0, sizeof(BX_CPU_THIS_PTR uintr));
+#endif
+
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR msr.ia32_spec_ctrl = 0;
 
@@ -902,32 +1000,17 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_SUPPORT_APIC
   /* APIC Address, APIC enabled and BSP is default, we'll fill in the rest later */
   BX_CPU_THIS_PTR msr.apicbase = BX_LAPIC_BASE_ADDR;
-  BX_CPU_THIS_PTR lapic.reset(source);
+  BX_CPU_THIS_PTR lapic->reset(source);
   BX_CPU_THIS_PTR msr.apicbase |= 0x900;
-  BX_CPU_THIS_PTR lapic.set_base(BX_CPU_THIS_PTR msr.apicbase);
+  BX_CPU_THIS_PTR lapic->set_base(BX_CPU_THIS_PTR msr.apicbase);
 #if BX_CPU_LEVEL >= 6
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XAPIC_EXT))
-    BX_CPU_THIS_PTR lapic.enable_xapic_extensions();
+    BX_CPU_THIS_PTR lapic->enable_xapic_extensions();
 #endif
 #endif
 
   BX_CPU_THIS_PTR efer.set32(0);
-  BX_CPU_THIS_PTR efer_suppmask = 0;
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_NX))
-    BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_NXE_MASK;
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SYSCALL_SYSRET_LEGACY))
-    BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_SCE_MASK;
-#if BX_SUPPORT_X86_64
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_LONG_MODE)) {
-    BX_CPU_THIS_PTR efer_suppmask |= (BX_EFER_SCE_MASK | BX_EFER_LME_MASK | BX_EFER_LMA_MASK);
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_FFXSR))
-      BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_FFXSR_MASK;
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SVM))
-      BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_SVME_MASK;
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_TCE))
-      BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_TCE_MASK;
-  }
-#endif
+  BX_CPU_THIS_PTR efer_suppmask = get_efer_allow_mask();
 
   BX_CPU_THIS_PTR msr.star = 0;
 #if BX_SUPPORT_X86_64
@@ -1027,20 +1110,9 @@ void BX_CPU_C::reset(unsigned source)
   }
 #endif
 
+  BX_CPU_THIS_PTR cpu_state_use_ok = 0;
+
 #if BX_CPU_LEVEL >= 6
-  BX_CPU_THIS_PTR sse_ok = 0;
-#if BX_SUPPORT_AVX
-  BX_CPU_THIS_PTR avx_ok = 0;
-#endif
-
-#if BX_SUPPORT_EVEX
-  BX_CPU_THIS_PTR opmask_ok = BX_CPU_THIS_PTR evex_ok = 0;
-
-  if (source == BX_RESET_HARDWARE) {
-    for (n=0; n<8; n++) BX_WRITE_OPMASK(n, 0);
-  }
-#endif
-
   // Reset XMM state - unchanged on #INIT
   if (source == BX_RESET_HARDWARE) {
     for(n=0; n<BX_XMM_REGISTERS; n++) {
@@ -1053,12 +1125,16 @@ void BX_CPU_C::reset(unsigned source)
       BX_CPU_THIS_PTR mxcsr_mask |= MXCSR_DAZ;
     if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_MISALIGNED_SSE))
       BX_CPU_THIS_PTR mxcsr_mask |= MXCSR_MISALIGNED_EXCEPTION_MASK;
+
+#if BX_SUPPORT_EVEX
+    for (n=0; n<8; n++) BX_WRITE_OPMASK(n, 0);
+#endif
   }
 #endif
 
 #if BX_SUPPORT_VMX
-  BX_CPU_THIS_PTR in_vmx = BX_CPU_THIS_PTR in_vmx_guest = 0;
-  BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_smm_vmx_guest = 0;
+  BX_CPU_THIS_PTR in_vmx = BX_CPU_THIS_PTR in_vmx_guest = false;
+  BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_smm_vmx_guest = false;
   BX_CPU_THIS_PTR vmcsptr = BX_CPU_THIS_PTR vmxonptr = BX_INVALID_VMCSPTR;
   set_VMCSPTR(BX_CPU_THIS_PTR vmcsptr);
   if (source == BX_RESET_HARDWARE) {
@@ -1068,22 +1144,22 @@ void BX_CPU_C::reset(unsigned source)
 
 #if BX_SUPPORT_SVM
   set_VMCBPTR(0);
-  BX_CPU_THIS_PTR in_svm_guest = 0;
-  BX_CPU_THIS_PTR svm_gif = 1;
+  BX_CPU_THIS_PTR in_svm_guest = false;
+  BX_CPU_THIS_PTR svm_gif = true;
 #endif
 
 #if BX_SUPPORT_VMX || BX_SUPPORT_SVM
-  BX_CPU_THIS_PTR in_event = 0;
+  BX_CPU_THIS_PTR in_event = false;
 #endif
 
 #if BX_SUPPORT_VMX
-  BX_CPU_THIS_PTR nmi_unblocking_iret = 0;
+  BX_CPU_THIS_PTR nmi_unblocking_iret = false;
 #endif
 
 #if BX_SUPPORT_SMP
   // notice if I'm the bootstrap processor.  If not, do the equivalent of
   // a HALT instruction.
-  int apic_id = lapic.get_id();
+  int apic_id = lapic->get_id();
   if (BX_BOOTSTRAP_PROCESSOR == apic_id) {
     // boot normally
     BX_CPU_THIS_PTR msr.apicbase |=  0x100; /* set bit 8 BSP */
