@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: io.cc 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2019  The Bochs Project
@@ -25,6 +25,10 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #include "iodev/iodev.h"
 
 //
@@ -219,14 +223,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSB_YbDX(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSB16_YbDX);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 16-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB16_YbDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit8u value8 = read_RMW_virtual_byte_32(BX_SEG_REG_ES, DI);
+  Bit8u value8 = read_RMW_virtual_byte_32(BX_SEG_REG_ES, DI); // no lock
 
   value8 = BX_INP(DX, 1);
 
@@ -242,7 +246,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB16_YbDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB32_YbDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit8u value8 = read_RMW_virtual_byte(BX_SEG_REG_ES, EDI);
+  Bit8u value8 = read_RMW_virtual_byte(BX_SEG_REG_ES, EDI); // no lock
 
   value8 = BX_INP(DX, 1);
 
@@ -263,7 +267,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB32_YbDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSB64_YbDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit8u value8 = read_RMW_linear_byte(BX_SEG_REG_ES, RDI);
+  Bit8u value8 = read_RMW_linear_byte(BX_SEG_REG_ES, RDI); // no lock
 
   value8 = BX_INP(DX, 1);
 
@@ -298,14 +302,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSW_YwDX(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSW16_YwDX);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 16-bit operand size, 16-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW16_YwDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit16u value16 = read_RMW_virtual_word_32(BX_SEG_REG_ES, DI);
+  Bit16u value16 = read_RMW_virtual_word_32(BX_SEG_REG_ES, DI); // no lock
 
   value16 = BX_INP(DX, 2);
 
@@ -344,7 +348,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
     }
     else {
       // trigger any segment or page faults before reading from IO port
-      value16 = read_RMW_virtual_word(BX_SEG_REG_ES, edi);
+      value16 = read_RMW_virtual_word(BX_SEG_REG_ES, edi); // no lock
 
       value16 = BX_INP(DX, 2);
 
@@ -355,7 +359,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
 #endif
   {
     // trigger any segment or page faults before reading from IO port
-    value16 = read_RMW_virtual_word_32(BX_SEG_REG_ES, edi);
+    value16 = read_RMW_virtual_word_32(BX_SEG_REG_ES, edi); // no lock
 
     value16 = BX_INP(DX, 2);
 
@@ -374,7 +378,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW32_YwDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSW64_YwDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit16u value16 = read_RMW_linear_word(BX_SEG_REG_ES, RDI);
+  Bit16u value16 = read_RMW_linear_word(BX_SEG_REG_ES, RDI); // no lock
 
   value16 = BX_INP(DX, 2);
 
@@ -409,14 +413,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_INSD_YdDX(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSD16_YdDX);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 32-bit operand size, 16-bit address size
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD16_YdDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit32u value32 = read_RMW_virtual_dword_32(BX_SEG_REG_ES, DI);
+  Bit32u value32 = read_RMW_virtual_dword_32(BX_SEG_REG_ES, DI); // no lock
 
   value32 = BX_INP(DX, 4);
 
@@ -432,7 +436,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD16_YdDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD32_YdDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit32u value32 = read_RMW_virtual_dword(BX_SEG_REG_ES, EDI);
+  Bit32u value32 = read_RMW_virtual_dword(BX_SEG_REG_ES, EDI); // no lock
 
   value32 = BX_INP(DX, 4);
 
@@ -450,7 +454,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD32_YdDX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INSD64_YdDX(bxInstruction_c *i)
 {
   // trigger any segment or page faults before reading from IO port
-  Bit32u value32 = read_RMW_linear_dword(BX_SEG_REG_ES, RDI);
+  Bit32u value32 = read_RMW_linear_dword(BX_SEG_REG_ES, RDI); // no lock
 
   value32 = BX_INP(DX, 4);
 
@@ -489,7 +493,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSB_DXXb(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSB16_DXXb);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 16-bit address size
@@ -553,7 +557,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSW_DXXw(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSW16_DXXw);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 16-bit operand size, 16-bit address size
@@ -644,7 +648,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::REP_OUTSD_DXXd(bxInstruction_c *i)
     BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSD16_DXXd);
   }
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 // 32-bit operand size, 16-bit address size
@@ -702,7 +706,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALIb(bxInstruction_c *i)
 
   AL = BX_INP(port, 1);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXIb(bxInstruction_c *i)
@@ -716,7 +720,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXIb(bxInstruction_c *i)
 
   AX = BX_INP(port, 2);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXIb(bxInstruction_c *i)
@@ -730,7 +734,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXIb(bxInstruction_c *i)
 
   RAX = BX_INP(port, 4);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAL(bxInstruction_c *i)
@@ -744,7 +748,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAL(bxInstruction_c *i)
 
   BX_OUTP(port, AL, 1);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAX(bxInstruction_c *i)
@@ -758,7 +762,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbAX(bxInstruction_c *i)
 
   BX_OUTP(port, AX, 2);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbEAX(bxInstruction_c *i)
@@ -772,7 +776,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_IbEAX(bxInstruction_c *i)
 
   BX_OUTP(port, EAX, 4);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALDX(bxInstruction_c *i)
@@ -786,7 +790,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_ALDX(bxInstruction_c *i)
 
   AL = BX_INP(port, 1);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXDX(bxInstruction_c *i)
@@ -800,7 +804,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_AXDX(bxInstruction_c *i)
 
   AX = BX_INP(port, 2);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXDX(bxInstruction_c *i)
@@ -814,7 +818,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IN_EAXDX(bxInstruction_c *i)
 
   RAX = BX_INP(port, 4);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAL(bxInstruction_c *i)
@@ -828,7 +832,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAL(bxInstruction_c *i)
 
   BX_OUTP(port, AL, 1);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAX(bxInstruction_c *i)
@@ -842,7 +846,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXAX(bxInstruction_c *i)
 
   BX_OUTP(port, AX, 2);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXEAX(bxInstruction_c *i)
@@ -856,7 +860,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OUT_DXEAX(bxInstruction_c *i)
 
   BX_OUTP(port, EAX, 4);
 
-  BX_NEXT_INSTR(i);
+  BX_NEXT_TRACE(i);
 }
 
 bool BX_CPP_AttrRegparmN(3) BX_CPU_C::allow_io(bxInstruction_c *i, Bit16u port, unsigned len)
@@ -864,6 +868,16 @@ bool BX_CPP_AttrRegparmN(3) BX_CPU_C::allow_io(bxInstruction_c *i, Bit16u port,
   /* If CPL <= IOPL, then all IO portesses are accessible.
    * Otherwise, must check the IO permission map on >286.
    * On the 286, there is no IO permissions map */
+  
+  static bool port_e9_hack_all_rings = SIM->get_param_bool(BXPN_PORT_E9_HACK)->get() && SIM->get_param_bool(BXPN_PORT_E9_HACK_ALL_RINGS)->get();
+  if (0xe9 == port && port_e9_hack_all_rings)
+    return(1); // port e9 hack can be used by unprivileged code
+
+#if BX_SUPPORT_IODEBUG
+  static bool iodebug_all_rings = SIM->get_param_bool(BXPN_IODEBUG_ALL_RINGS)->get();
+  if (0x8A00 == (port & 0xfffe) && iodebug_all_rings)
+    return(1); // iodebug ports (0x8A00 & 0x8A01) can be used by unprivileged code
+#endif /* if BX_SUPPORT_IODEBUG */
 
   if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL > BX_CPU_THIS_PTR get_IOPL())))
   {
