@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: msr.h 13869 2020-05-29 12:35:30Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2019  The Bochs Project
@@ -26,10 +26,13 @@ enum MSR_Register {
   BX_MSR_TSC            = 0x010,
   BX_MSR_APICBASE       = 0x01b,
   BX_MSR_TSC_ADJUST     = 0x03b,
-  BX_MSR_TSC_DEADLINE   = 0x6E0,
+  BX_MSR_TSC_DEADLINE   = 0x6e0,
+
+  BX_MSR_IA32_BARRIER   = 0x02f,
 
   BX_MSR_IA32_SPEC_CTRL         = 0x048,
   BX_MSR_IA32_PRED_CMD          = 0x049,
+  BX_MSR_IA32_UMWAIT_CONTROL    = 0x0e1,
   BX_MSR_IA32_ARCH_CAPABILITIES = 0x10a,
   BX_MSR_IA32_FLUSH_CMD         = 0x10b,
 
@@ -91,7 +94,7 @@ enum MSR_Register {
   BX_MSR_PERFEVTSEL2      = 0x188,
   BX_MSR_PERFEVTSEL3      = 0x189,
   BX_MSR_PERFEVTSEL4      = 0x18a,
-  BX_MSR_PERFEVTSEL5      = 0x18b, 
+  BX_MSR_PERFEVTSEL5      = 0x18b,
   BX_MSR_PERFEVTSEL6      = 0x18c,
   BX_MSR_PERFEVTSEL7      = 0x18d,
   BX_MSR_PERF_FIXED_CTR0  = 0x309,  /* Fixed Performance Counter 0 (R/W): Counts Instr_Retired.Any */
@@ -120,6 +123,8 @@ enum MSR_Register {
   BX_MSR_VMX_TRUE_VMEXIT_CTRLS    = 0x48f,
   BX_MSR_VMX_TRUE_VMENTRY_CTRLS   = 0x490,
   BX_MSR_VMX_VMFUNC               = 0x491,
+  BX_MSR_VMX_PROCBASED_CTRLS3     = 0x492,
+  BX_MSR_VMX_VMEXIT_CTRLS2        = 0x493,
   BX_MSR_IA32_FEATURE_CONTROL     = 0x03A,
   BX_MSR_IA32_SMM_MONITOR_CTL     = 0x09B,
 #endif
@@ -137,7 +142,16 @@ enum MSR_Register {
 
   BX_MSR_IA32_PKRS = 0x6E1,
 
-  BX_MSR_XSS = 0xda0,
+#if BX_SUPPORT_UINTR
+  BX_MSR_IA32_UINTR_RR = 0x985,
+  BX_MSR_IA32_UINTR_HANDLER  = 0x986,
+  BX_MSR_IA32_UINTR_STACKADJUST = 0x987,
+  BX_MSR_IA32_UINTR_MISC = 0x988,
+  BX_MSR_IA32_UINTR_PD = 0x989, // interface to UPID_ADDR
+  BX_MSR_IA32_UINTR_TT = 0x98A, // interface to UITT_ADDR
+#endif
+
+  BX_MSR_XSS = 0xDA0,
 
   /* AMD MSRs */
   BX_MSR_EFER         = 0xc0000080,
