@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: proc_ctrl.cc 14095 2021-01-30 18:47:25Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2019  The Bochs Project
@@ -23,8 +23,13 @@
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
+#include "cpuid.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #include "pc_system.h"
 #include "gui/gui.h"
 
@@ -34,7 +39,7 @@
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxError(bxInstruction_c *i)
 {
   unsigned ia_opcode = i->getIaOpcode();
- 
+
   if (ia_opcode == BX_IA_ERROR) {
     BX_DEBUG(("BxError: Encountered an unknown instruction (signalling #UD)"));
 
@@ -159,7 +164,7 @@ void BX_CPU_C::enter_sleep_state(unsigned state)
   case BX_ACTIVITY_STATE_SHUTDOWN:
     BX_CPU_THIS_PTR clear_IF(); // masking interrupts
     break;
-   
+
   case BX_ACTIVITY_STATE_MWAIT:
   case BX_ACTIVITY_STATE_MWAIT_IF:
     break;
@@ -171,7 +176,7 @@ void BX_CPU_C::enter_sleep_state(unsigned state)
   // artificial trap bit, why use another variable.
   BX_CPU_THIS_PTR activity_state = state;
   BX_CPU_THIS_PTR async_event = 1; // so processor knows to check
-  // Execution completes.  The processor will remain in a sleep 
+  // Execution completes.  The processor will remain in a sleep
   // state until one of the wakeup conditions is met.
 
   BX_INSTR_HLT(BX_CPU_ID);
@@ -200,7 +205,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::HLT(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (VMEXIT(VMX_VM_EXEC_CTRL2_HLT_VMEXIT)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.HLT_VMEXIT()) {
       VMexit(VMX_VMEXIT_HLT, 0);
     }
   }
@@ -264,9 +269,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::WBINVD(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_WBINVD_VMEXIT)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls2.WBINVD_VMEXIT())
       VMexit(VMX_VMEXIT_WBINVD, 0);
-    }
   }
 #endif
 
@@ -296,7 +300,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLFLUSH(bxInstruction_c *i)
 #if BX_SUPPORT_X86_64
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64)
     laddr = get_laddr64(i->seg(), eaddr);
-  else 
+  else
 #endif
     laddr = agen_read_execute32(i->seg(), (Bit32u)eaddr, 1);
 
@@ -322,6 +326,26 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLZERO(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVDIR64B(bxInstruction_c *i)
+{
+#if BX_CPU_LEVEL >= 6
+
+  BxPackedZmmRegister zmm; // zmm is always made available even if EVEX is not compiled in
+  bx_address src_eaddr = BX_CPU_RESOLVE_ADDR(i);
+  read_virtual_zmmword(i->seg(), src_eaddr, &zmm);
+
+#if BX_SUPPORT_X86_64
+  bx_address dst_eaddr = BX_READ_64BIT_REG(i->dst());
+#else
+  bx_address dst_eaddr = BX_READ_32BIT_REG(i->dst());
+#endif
+  write_virtual_zmmword_aligned(BX_SEG_REG_ES, dst_eaddr & i->asize_mask(), &zmm);
+
+#endif
+
+  BX_NEXT_INSTR(i);
+}
+
 void BX_CPU_C::handleCpuModeChange(void)
 {
   unsigned mode = BX_CPU_THIS_PTR cpu_mode;
@@ -410,21 +434,54 @@ void BX_CPU_C::handleAlignmentCheck(void)
 }
 #endif
 
+void BX_CPU_C::handleFpuMmxModeChange(void)
+{
+  if (BX_CPU_THIS_PTR cr0.get_EM() || BX_CPU_THIS_PTR cr0.get_TS())
+    clear_fpu_mmx_ok();
+  else
+    set_fpu_mmx_ok();
+
+  updateFetchModeMask(); /* FPU_MMX_OK changed */
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoFPU(bxInstruction_c *i)
+{
+  if (BX_CPU_THIS_PTR cr0.get_EM() || BX_CPU_THIS_PTR cr0.get_TS())
+    exception(BX_NM_EXCEPTION, 0);
+
+  BX_ASSERT(0);
+
+  BX_NEXT_TRACE(i); // keep compiler happy
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoMMX(bxInstruction_c *i)
+{
+  if(BX_CPU_THIS_PTR cr0.get_EM())
+    exception(BX_UD_EXCEPTION, 0);
+
+  if(BX_CPU_THIS_PTR cr0.get_TS())
+    exception(BX_NM_EXCEPTION, 0);
+
+  BX_ASSERT(0);
+
+  BX_NEXT_TRACE(i); // keep compiler happy
+}
+
 #if BX_CPU_LEVEL >= 6
 void BX_CPU_C::handleSseModeChange(void)
 {
   if(BX_CPU_THIS_PTR cr0.get_TS()) {
-    BX_CPU_THIS_PTR sse_ok = 0;
+    clear_sse_ok();
   }
   else {
     if(BX_CPU_THIS_PTR cr0.get_EM() || !BX_CPU_THIS_PTR cr4.get_OSFXSR())
-      BX_CPU_THIS_PTR sse_ok = 0;
+      clear_sse_ok();
     else
-      BX_CPU_THIS_PTR sse_ok = 1;
+      set_sse_ok();
   }
 
   updateFetchModeMask(); /* SSE_OK changed */
-}  
+}
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoSSE(bxInstruction_c *i)
 {
@@ -442,40 +499,44 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoSSE(bxInstruction_c *i)
 #if BX_SUPPORT_AVX
 void BX_CPU_C::handleAvxModeChange(void)
 {
-  if(BX_CPU_THIS_PTR cr0.get_TS()) {
-    BX_CPU_THIS_PTR avx_ok = 0;
+  if (BX_CPU_THIS_PTR cr0.get_TS()) {
+    clear_avx_ok();
   }
   else {
     if (! protected_mode() || ! BX_CPU_THIS_PTR cr4.get_OSXSAVE() ||
         (~BX_CPU_THIS_PTR xcr0.val32 & (BX_XCR0_SSE_MASK | BX_XCR0_YMM_MASK)) != 0) {
-      BX_CPU_THIS_PTR avx_ok = 0;
+      clear_avx_ok();
     }
     else {
-      BX_CPU_THIS_PTR avx_ok = 1;
+      set_avx_ok();
 
 #if BX_SUPPORT_EVEX
       if ((~BX_CPU_THIS_PTR xcr0.val32 & BX_XCR0_OPMASK_MASK) != 0) {
-        BX_CPU_THIS_PTR opmask_ok = BX_CPU_THIS_PTR evex_ok = 0;
+        clear_opmask_ok();
+        clear_evex_ok();
       }
       else {
-        BX_CPU_THIS_PTR opmask_ok = 1;
+        set_opmask_ok();
 
         if ((~BX_CPU_THIS_PTR xcr0.val32 & (BX_XCR0_ZMM_HI256_MASK | BX_XCR0_HI_ZMM_MASK)) != 0)
-          BX_CPU_THIS_PTR evex_ok = 0;
+          clear_evex_ok();
         else
-          BX_CPU_THIS_PTR evex_ok = 1;
+          set_evex_ok();
       }
 #endif
     }
   }
 
-#if BX_SUPPORT_EVEX
-  if (! BX_CPU_THIS_PTR avx_ok)
-        BX_CPU_THIS_PTR opmask_ok = BX_CPU_THIS_PTR evex_ok = 0;
+#if BX_SUPPORT_AMX
+  if (! long64_mode() || ! BX_CPU_THIS_PTR cr4.get_OSXSAVE() ||
+      (~BX_CPU_THIS_PTR xcr0.val32 & (BX_XCR0_XTILECFG_MASK | BX_XCR0_XTILEDATA_MASK)) != 0)
+    clear_amx_ok();
+  else
+    set_amx_ok();
 #endif
 
   updateFetchModeMask(); /* AVX_OK changed */
-}  
+}
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoAVX(bxInstruction_c *i)
 {
@@ -528,6 +589,21 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoEVEX(bxInstruction_c *i)
 }
 #endif
 
+#if BX_SUPPORT_AMX
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BxNoAMX(bxInstruction_c *i)
+{
+  if (! long64_mode() || ! BX_CPU_THIS_PTR cr4.get_OSXSAVE())
+    exception(BX_UD_EXCEPTION, 0);
+
+  if (~BX_CPU_THIS_PTR xcr0.val32 & (BX_XCR0_XTILECFG_MASK | BX_XCR0_XTILEDATA_MASK))
+    exception(BX_UD_EXCEPTION, 0);
+
+  BX_ASSERT(0);
+
+  BX_NEXT_TRACE(i); // keep compiler happy
+}
+#endif
+
 #endif
 
 void BX_CPU_C::handleCpuContextChange(void)
@@ -545,6 +621,7 @@ void BX_CPU_C::handleCpuContextChange(void)
 
   handleCpuModeChange();
 
+  handleFpuMmxModeChange();
 #if BX_CPU_LEVEL >= 6
   handleSseModeChange();
 #if BX_SUPPORT_AVX
@@ -564,7 +641,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDPMC(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest)  {
-    if (VMEXIT(VMX_VM_EXEC_CTRL2_RDPMC_VMEXIT)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.RDPMC_VMEXIT()) {
       VMexit(VMX_VMEXIT_RDPMC, 0);
     }
   }
@@ -615,21 +692,45 @@ Bit64u BX_CPU_C::get_TSC(void)
   return tsc;
 }
 
-#if BX_SUPPORT_VMX || BX_SUPPORT_SVM
-Bit64u BX_CPU_C::get_TSC_VMXAdjust(Bit64u tsc)
+Bit64u BX_CPU_C::get_Virtual_TSC()
 {
+  Bit64u tsc = BX_CPU_THIS_PTR get_TSC();
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (VMEXIT(VMX_VM_EXEC_CTRL2_TSC_OFFSET) && SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_TSC_SCALING)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.TSC_OFFSET() && BX_CPU_THIS_PTR vmcs.vmexec_ctrls2.TSC_SCALING()) {
+      // RDTSC first computes the product of the value of the IA32_TIME_STAMP_COUNTER MSR and
+      // the value of the TSC multiplier. It then shifts the value of the product right 48 bits and loads 
+      // EAX:EDX with <the sum of that shifted value and the value of the TSC offset>.
       Bit128u product_128;
-      long_mul(&product_128,tsc,BX_CPU_THIS_PTR vmcs.tsc_multiplier);   
+      long_mul(&product_128,tsc,BX_CPU_THIS_PTR vmcs.tsc_multiplier);
       tsc = (product_128.lo >> 48) | (product_128.hi << 16);   // tsc = (uint64) (long128(tsc_value * tsc_multiplier) >> 48);
     }
   }
 #endif
+#if BX_SUPPORT_VMX || BX_SUPPORT_SVM
   tsc += BX_CPU_THIS_PTR tsc_offset;    // BX_CPU_THIS_PTR tsc_offset = 0 if not in VMX or SVM guest
+#endif
   return tsc;
 }
+
+#if BX_SUPPORT_VMX
+Bit64u BX_CPU_C::compute_physical_TSC_delay(Bit64u tsc_delay)
+{
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.TSC_OFFSET() && BX_CPU_THIS_PTR vmcs.vmexec_ctrls2.TSC_SCALING()) {
+      // The virtual delay is multiplied by 2^48 (using a shift) to produce a 128-bit 
+      // integer. That product is then divided by the TSC multiplier to produce a 64-bit integer.
+      // The physical delay is that quotient.
+      Bit128u product128, quotient;
+      product128.hi = tsc_delay >> 16;
+      product128.lo = tsc_delay << 48;
+      long_div(&quotient, &tsc_delay /*just use it as temp to be destroyed*/, &product128, BX_CPU_THIS_PTR vmcs.tsc_multiplier);
+      BX_ASSERT(quotient.hi == 0);
+      tsc_delay = quotient.lo;                // tsc = Bit128(tsc_value << 48) / tsc_multiplier
+    }
+  }
+  return tsc_delay;
+}
 #endif
 
 void BX_CPU_C::set_TSC(Bit64u newval)
@@ -641,7 +742,8 @@ void BX_CPU_C::set_TSC(Bit64u newval)
   // verify
   BX_ASSERT(get_TSC() == newval);
 }
-#endif
+
+#endif // BX_CPU_LEVEL >= 5
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSC(bxInstruction_c *i)
 {
@@ -653,7 +755,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSC(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (VMEXIT(VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.RDTSC_VMEXIT()) {
       VMexit(VMX_VMEXIT_RDTSC, 0);
     }
   }
@@ -664,11 +766,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSC(bxInstruction_c *i)
     if (SVM_INTERCEPT(SVM_INTERCEPT0_RDTSC)) Svm_Vmexit(SVM_VMEXIT_RDTSC);
 #endif
 
-  // return ticks
-  Bit64u ticks = BX_CPU_THIS_PTR get_TSC();
-#if BX_SUPPORT_SVM || BX_SUPPORT_VMX
-  ticks = BX_CPU_THIS_PTR get_TSC_VMXAdjust(ticks);
-#endif
+  Bit64u ticks = BX_CPU_THIS_PTR get_Virtual_TSC();
 
   RAX = GET32L(ticks);
   RDX = GET32H(ticks);
@@ -686,7 +784,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSCP(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   // RDTSCP will always #UD in legacy VMX mode, the #UD takes priority over any other exception the instruction may incur.
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_RDTSCP)) {
+    if (! BX_CPU_THIS_PTR vmcs.vmexec_ctrls2.RDTSCP()) {
        BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
        exception(BX_UD_EXCEPTION, 0);
     }
@@ -700,7 +798,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSCP(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (VMEXIT(VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT)) {
+    if (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1.RDTSC_VMEXIT()) {
       VMexit(VMX_VMEXIT_RDTSCP, 0);
     }
   }
@@ -711,11 +809,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDTSCP(bxInstruction_c *i)
     if (SVM_INTERCEPT(SVM_INTERCEPT1_RDTSCP)) Svm_Vmexit(SVM_VMEXIT_RDTSCP);
 #endif
 
-  // return ticks
-  Bit64u ticks = BX_CPU_THIS_PTR get_TSC();
-#if BX_SUPPORT_SVM || BX_SUPPORT_VMX
-  ticks = BX_CPU_THIS_PTR get_TSC_VMXAdjust(ticks);
-#endif
+  Bit64u ticks = BX_CPU_THIS_PTR get_Virtual_TSC();
 
   RAX = GET32L(ticks);
   RDX = GET32H(ticks);
@@ -733,7 +827,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RDPID_Ed(bxInstruction_c *i)
 #if BX_SUPPORT_VMX
   // RDTSCP will always #UD in legacy VMX mode
   if (BX_CPU_THIS_PTR in_vmx_guest) {
-    if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_RDTSCP)) {
+    if (! BX_CPU_THIS_PTR vmcs.vmexec_ctrls2.RDTSCP()) {
        BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
        exception(BX_UD_EXCEPTION, 0);
     }
@@ -864,6 +958,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SYSEXIT(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
+#if BX_SUPPORT_MONITOR_MWAIT
+  BX_CPU_THIS_PTR monitor.reset_umonitor();
+#endif
+
   BX_INSTR_FAR_BRANCH_ORIGIN();
 
 #if BX_SUPPORT_X86_64
@@ -1119,6 +1217,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SYSRET(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
+#if BX_SUPPORT_MONITOR_MWAIT
+  BX_CPU_THIS_PTR monitor.reset_umonitor();
+#endif
+
   BX_INSTR_FAR_BRANCH_ORIGIN();
 
 #if BX_SUPPORT_X86_64
@@ -1381,7 +1483,7 @@ void BX_CPU_C::set_PKeys(Bit32u pkru_val, Bit32u pkrs_val)
           BX_CPU_THIS_PTR rd_pkey[i] &= ~(TLB_SysReadOK | TLB_SysWriteOK);
           BX_CPU_THIS_PTR wr_pkey[i] &= ~(TLB_SysReadOK | TLB_SysWriteOK);
         }
-    
+
         // writeDisable bit set
         if (pkrs_val & (1<<(i*2+1))) {
           if (BX_CPU_THIS_PTR cr0.get_WP())
