@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: simd_int.h 14133 2021-02-08 13:06:44Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2017 Stanislav Shwartsman
@@ -261,7 +261,7 @@ BX_CPP_INLINE void xmm_punpckhwd(BxPackedXmmRegister *op1, const BxPackedXmmRegi
   op1->xmm16u(6) = op1->xmm16u(7);
   op1->xmm16u(7) = op2->xmm16u(7);
 }
- 
+
 // pack
 
 BX_CPP_INLINE void xmm_packuswb(BxPackedXmmRegister *op1, const BxPackedXmmRegister *op2)
@@ -1009,7 +1009,7 @@ BX_CPP_INLINE void xmm_pmaddwd(BxPackedXmmRegister *op1, const BxPackedXmmRegist
 {
   for(unsigned n=0; n<4; n++)
   {
-    op1->xmm32u(n) = Bit32s(op1->xmm16s(n*2))   * Bit32s(op2->xmm16s(n*2)) + 
+    op1->xmm32u(n) = Bit32s(op1->xmm16s(n*2))   * Bit32s(op2->xmm16s(n*2)) +
                      Bit32s(op1->xmm16s(n*2+1)) * Bit32s(op2->xmm16s(n*2+1));
   }
 }
@@ -1173,7 +1173,7 @@ BX_CPP_INLINE void xmm_psravw(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     unsigned shift = op2->xmm16u(n);
     if(shift > 15)
       op1->xmm16u(n) = (op1->xmm16s(n) < 0) ? 0xffff : 0;
-    else    
+    else
       op1->xmm16u(n) = (Bit16u)(op1->xmm16s(n) >> shift);
   }
 }
@@ -1184,7 +1184,7 @@ BX_CPP_INLINE void xmm_psravd(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit32u shift = op2->xmm32u(n);
     if(shift > 31)
       op1->xmm32u(n) = (op1->xmm32s(n) < 0) ? 0xffffffff : 0;
-    else    
+    else
       op1->xmm32u(n) = (Bit32u)(op1->xmm32s(n) >> shift);
   }
 }
@@ -1195,7 +1195,7 @@ BX_CPP_INLINE void xmm_psravq(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit64u shift = op2->xmm64u(n);
     if(shift > 64)
       op1->xmm64u(n) = (op1->xmm64s(n) < 0) ? BX_CONST64(0xffffffffffffffff) : 0;
-    else    
+    else
       op1->xmm64u(n) = (Bit64u)(op1->xmm64s(n) >> shift);
   }
 }
@@ -1206,7 +1206,7 @@ BX_CPP_INLINE void xmm_psllvw(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     unsigned shift = op2->xmm16u(n);
     if(shift > 15)
       op1->xmm16u(n) = 0;
-    else    
+    else
       op1->xmm16u(n) <<= shift;
   }
 }
@@ -1217,7 +1217,7 @@ BX_CPP_INLINE void xmm_psllvd(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit32u shift = op2->xmm32u(n);
     if(shift > 31)
       op1->xmm32u(n) = 0;
-    else    
+    else
       op1->xmm32u(n) <<= shift;
   }
 }
@@ -1228,7 +1228,7 @@ BX_CPP_INLINE void xmm_psllvq(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit64u shift = op2->xmm64u(n);
     if(shift > 63)
       op1->xmm64u(n) = 0;
-    else    
+    else
       op1->xmm64u(n) <<= shift;
   }
 }
@@ -1239,7 +1239,7 @@ BX_CPP_INLINE void xmm_psrlvw(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     unsigned shift = op2->xmm16u(n);
     if(shift > 15)
       op1->xmm16u(n) = 0;
-    else    
+    else
       op1->xmm16u(n) >>= shift;
   }
 }
@@ -1250,7 +1250,7 @@ BX_CPP_INLINE void xmm_psrlvd(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit32u shift = op2->xmm32u(n);
     if(shift > 31)
       op1->xmm32u(n) = 0;
-    else    
+    else
       op1->xmm32u(n) >>= shift;
   }
 }
@@ -1261,7 +1261,7 @@ BX_CPP_INLINE void xmm_psrlvq(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
     Bit64u shift = op2->xmm64u(n);
     if(shift > 63)
       op1->xmm64u(n) = 0;
-    else    
+    else
       op1->xmm64u(n) >>= shift;
   }
 }
@@ -1383,7 +1383,7 @@ BX_CPP_INLINE void xmm_psllq(BxPackedXmmRegister *op, Bit64u shift_64)
   }
 }
 
-BX_CPP_INLINE void xmm_psrldq(BxPackedXmmRegister *op, Bit8u shift)
+BX_CPP_INLINE void xmm_psrldq(BxPackedXmmRegister *op, Bit64u shift)
 {
   if(shift > 15) op->clear();
   else {
@@ -1402,7 +1402,7 @@ BX_CPP_INLINE void xmm_psrldq(BxPackedXmmRegister *op, Bit8u shift)
   }
 }
 
-BX_CPP_INLINE void xmm_pslldq(BxPackedXmmRegister *op, Bit8u shift)
+BX_CPP_INLINE void xmm_pslldq(BxPackedXmmRegister *op, Bit64u shift)
 {
   if(shift > 15) op->clear();
   else {
@@ -1450,36 +1450,36 @@ BX_CPP_INLINE void xmm_palignr(BxPackedXmmRegister *op2, const BxPackedXmmRegist
 
 // rotate (right)
 
-BX_CPP_INLINE void xmm_prorb(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prorb(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x7;
+  int shift = shift_ctrl & 0x07;
 
   for(unsigned n=0;n<16;n++) {
     op->xmmubyte(n) = (op->xmmubyte(n) >> shift) | (op->xmmubyte(n) << (8 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prorw(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prorw(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0xf;
+  int shift = shift_ctrl & 0x0f;
 
   for(unsigned n=0;n<8;n++) {
     op->xmm16u(n) = (op->xmm16u(n) >> shift) | (op->xmm16u(n) << (16 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prord(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prord(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x1f;
+  int shift = shift_ctrl & 0x1f;
 
   for(unsigned n=0;n<4;n++) {
     op->xmm32u(n) = (op->xmm32u(n) >> shift) | (op->xmm32u(n) << (32 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prorq(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prorq(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x3f;
+  int shift = shift_ctrl & 0x3f;
 
   for(unsigned n=0;n<2;n++) {
     op->xmm64u(n) = (op->xmm64u(n) >> shift) | (op->xmm64u(n) << (64 - shift));
@@ -1504,36 +1504,36 @@ BX_CPP_INLINE void xmm_prorvq(BxPackedXmmRegister *op1, const BxPackedXmmRegiste
 
 // rotate (left)
 
-BX_CPP_INLINE void xmm_prolb(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prolb(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x7;
+  int shift = shift_ctrl & 0x07;
 
   for(unsigned n=0;n<16;n++) {
     op->xmmubyte(n) = (op->xmmubyte(n) << shift) | (op->xmmubyte(n) >> (8 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prolw(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prolw(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0xf;
+  int shift = shift_ctrl & 0x0f;
 
   for(unsigned n=0;n<8;n++) {
     op->xmm16u(n) = (op->xmm16u(n) << shift) | (op->xmm16u(n) >> (16 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prold(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prold(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x1f;
+  int shift = shift_ctrl & 0x1f;
 
   for(unsigned n=0;n<4;n++) {
     op->xmm32u(n) = (op->xmm32u(n) << shift) | (op->xmm32u(n) >> (32 - shift));
   }
 }
 
-BX_CPP_INLINE void xmm_prolq(BxPackedXmmRegister *op, int shift)
+BX_CPP_INLINE void xmm_prolq(BxPackedXmmRegister *op, Bit64u shift_ctrl)
 {
-  shift &= 0x3f;
+  int shift = shift_ctrl & 0x3f;
 
   for(unsigned n=0;n<2;n++) {
     op->xmm64u(n) = (op->xmm64u(n) << shift) | (op->xmm64u(n) >> (64 - shift));
@@ -1746,56 +1746,4 @@ BX_CPP_INLINE void xmm_pshlq(BxPackedXmmRegister *op1, const BxPackedXmmRegister
   }
 }
 
-// VNNI
-
-BX_CPP_INLINE void xmm_pdpbusd(BxPackedXmmRegister *dst, BxPackedXmmRegister *op1, const BxPackedXmmRegister *op2)
-{
-  for(unsigned n=0; n<4; n++)
-  {
-    Bit32s p1word = (Bit32u) op1->xmmubyte(n*4)   * (Bit32s) op2->xmmsbyte(n*4);
-    Bit32s p2word = (Bit32u) op1->xmmubyte(n*4+1) * (Bit32s) op2->xmmsbyte(n*4+1);
-    Bit32s p3word = (Bit32u) op1->xmmubyte(n*4+2) * (Bit32s) op2->xmmsbyte(n*4+2);
-    Bit32s p4word = (Bit32u) op1->xmmubyte(n*4+3) * (Bit32s) op2->xmmsbyte(n*4+3);
-
-    dst->xmm32s(n) += (p1word + p2word + p3word + p4word);
-  }
-}
-
-BX_CPP_INLINE void xmm_pdpbusds(BxPackedXmmRegister *dst, BxPackedXmmRegister *op1, const BxPackedXmmRegister *op2)
-{
-  for(unsigned n=0; n<4; n++)
-  {
-    Bit32s p1word = (Bit32u) op1->xmmubyte(n*4)   * (Bit32s) op2->xmmsbyte(n*4);
-    Bit32s p2word = (Bit32u) op1->xmmubyte(n*4+1) * (Bit32s) op2->xmmsbyte(n*4+1);
-    Bit32s p3word = (Bit32u) op1->xmmubyte(n*4+2) * (Bit32s) op2->xmmsbyte(n*4+2);
-    Bit32s p4word = (Bit32u) op1->xmmubyte(n*4+3) * (Bit32s) op2->xmmsbyte(n*4+3);
-
-    Bit64s result = (Bit64s) dst->xmm32s(n) + (p1word + p2word + p3word + p4word);
-    dst->xmm32s(n) = SaturateQwordSToDwordS(result);
-  }
-}
-
-BX_CPP_INLINE void xmm_pdpwssd(BxPackedXmmRegister *dst, BxPackedXmmRegister *op1, const BxPackedXmmRegister *op2)
-{
-  for(unsigned n=0; n<4; n++)
-  {
-    Bit32s p1_dword = (Bit32s) op1->xmm16s(n*2)   * (Bit32s) op2->xmm16s(n*2);
-    Bit32s p2_dword = (Bit32s) op1->xmm16s(n*2+1) * (Bit32s) op2->xmm16s(n*2+1);
-
-    dst->xmm32s(n) += (p1_dword + p2_dword);
-  }
-}
-
-BX_CPP_INLINE void xmm_pdpwssds(BxPackedXmmRegister *dst, BxPackedXmmRegister *op1, const BxPackedXmmRegister *op2)
-{
-  for(unsigned n=0; n<4; n++)
-  {
-    Bit32s p1_dword = (Bit32s) op1->xmm16s(n*2)   * (Bit32s) op2->xmm16s(n*2);
-    Bit32s p2_dword = (Bit32s) op1->xmm16s(n*2+1) * (Bit32s) op2->xmm16s(n*2+1);
-
-    Bit64s result = (Bit64s) dst->xmm32s(n) + (p1_dword + p2_dword);
-    dst->xmm32s(n) = SaturateQwordSToDwordS(result);
-  }
-}
-
 #endif
