@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_pfp.cc 13466 2018-02-16 07:57:32Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003-2018 Stanislav Shwartsman
+//   Copyright (c) 2003-2023 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -67,6 +67,16 @@ float_status_t mxcsr_to_softfloat_status_word(bx_mxcsr_t mxcsr)
   return status;
 }
 
+void mxcsr_to_softfloat_status_word_imm_override(float_status_t &status, Bit8u control)
+{
+  // override MXCSR rounding mode with control coming from imm8
+  if ((control & 0x4) == 0)
+    status.float_rounding_mode = control & 0x3;
+  // ignore precision exception result
+  if (control & 0x8)
+    status.float_suppress_exception |= float_flag_inexact;
+}
+
 /* Comparison predicate for CMPSS/CMPPS instructions */
 static float32_compare_method compare32[8] = {
   float32_eq_ordered_quiet,
@@ -233,9 +243,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CVTSI2SS_VssEdR(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CVTSI2SS_VssEqR(bxInstruction_c *i)
 {
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-
   float32 result = int64_to_float32(BX_READ_64BIT_REG(i->src()), status);
-
   check_exceptionsSSE(get_exception_flags(status));
   BX_WRITE_XMM_REG_LO_DWORD(i->dst(), result);
 
@@ -818,7 +826,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::UCOMISS_VssWssR(bxInstruction_c *i)
  * Compare two double precision FP numbers and set EFLAGS accordintly.
  * Possible floating point exceptions: #I, #D
  */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::UCOMISD_VsdWsdR(bxInstruction_c *i)            	
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::UCOMISD_VsdWsdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
@@ -873,42 +881,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::COMISD_VsdWsdR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-/*
- * Opcode: 0F 51
- * Square Root packed single precision.
- * Possible floating point exceptions: #I, #D, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SQRTPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_sqrtps(&op, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG(i->dst(), op);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 51
- * Square Root packed double precision.
- * Possible floating point exceptions: #I, #D, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SQRTPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_sqrtpd(&op, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG(i->dst(), op);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
 /*
  * Opcode: F2 0F 51
  * Square Root scalar double precision.
@@ -947,553 +919,71 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SQRTSS_VssWssR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-/*
- * Opcode: 0F 58
- * Add packed single precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDPS_VpsWpsR(bxInstruction_c *i)
-{
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_addps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
+#define SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(HANDLER, func)                                          \
+  /* SSE packed shift instruction */                                                            \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)                           \
+  {                                                                                             \
+    float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src()); \
+                                                                                                \
+    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                              \
+    op1 = (func)(op1, op2, status);                                                             \
+    check_exceptionsSSE(get_exception_flags(status));                                           \
+    BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);                                                   \
+    BX_NEXT_INSTR(i);                                                                           \
+  }                                                                                             \
+
+#else
+
+#define SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(HANDLER, func)                                          \
+  /* SSE instruction with two src operands */                                                   \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                          \
+  {                                                                                             \
+    BX_NEXT_INSTR(i);                                                                           \
+  }
 
-  BX_WRITE_XMM_REG(i->dst(), op1);
 #endif
 
-  BX_NEXT_INSTR(i);
-}
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(ADDSS_VssWssR, float32_add);
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(SUBSS_VssWssR, float32_sub);
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(MULSS_VssWssR, float32_mul);
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(DIVSS_VssWssR, float32_div);
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(MINSS_VssWssR, float32_min);
+SSE_SCALAR_SINGLE_FP_CPU_LEVEL6(MAXSS_VssWssR, float32_max);
 
-/*
- * Opcode: 66 0F 58
- * Add packed double precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDPD_VpdWpdR(bxInstruction_c *i)
-{
 #if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
 
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_addpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
+#define SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(HANDLER, func)                                          \
+  /* SSE packed shift instruction */                                                            \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C:: HANDLER (bxInstruction_c *i)                           \
+  {                                                                                             \
+    float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src()); \
+                                                                                                \
+    float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);                              \
+    op1 = (func)(op1, op2, status);                                                             \
+    check_exceptionsSSE(get_exception_flags(status));                                           \
+    BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);                                                   \
+    BX_NEXT_INSTR(i);                                                                           \
+  }                                                                                             \
+
+#else
+
+#define SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(HANDLER, func)                                          \
+  /* SSE instruction with two src operands */                                                   \
+  void BX_CPP_AttrRegparmN(1) BX_CPU_C :: HANDLER (bxInstruction_c *i)                          \
+  {                                                                                             \
+    BX_NEXT_INSTR(i);                                                                           \
+  }
 
-  BX_WRITE_XMM_REG(i->dst(), op1);
 #endif
 
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 58
- * Add the lower double precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_add(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 58
- * Add the lower single precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_add(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 0F 59
- * Multiply packed single precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MULPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_mulps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 59
- * Multiply packed double precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MULPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_mulpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 59
- * Multiply the lower double precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MULSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_mul(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 59
- * Multiply the lower single precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MULSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_mul(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 0F 5C
- * Subtract packed single precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUBPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_subps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 5C
- * Subtract packed double precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUBPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_subpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 5C
- * Subtract the lower double precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUBSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_sub(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 5C
- * Subtract the lower single precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUBSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_sub(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 0F 5D
- * Calculate the minimum single precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MINPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_minps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 5D
- * Calculate the minimum double precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MINPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_minpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 5D
- * Calculate the minimum scalar double precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MINSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_min(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 5D
- * Calculate the minimum scalar single precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MINSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_min(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 0F 5E
- * Divide packed single precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #Z, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIVPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_divps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 5E
- * Divide packed double precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #Z, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIVPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_divpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 5E
- * Divide the lower double precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #Z, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIVSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_div(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 5E
- * Divide the lower single precision FP number from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #Z, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIVSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_div(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 0F 5F
- * Calculate the maximum single precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MAXPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_maxps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 5F
- * Calculate the maximum double precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MAXPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_maxpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 5F
- * Calculate the maximum scalar double precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MAXSD_VsdWsdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float64 op1 = BX_READ_XMM_REG_LO_QWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_QWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float64_max(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F3 0F 5F
- * Calculate the maxumim scalar single precision FP between XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MAXSS_VssWssR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  float32 op1 = BX_READ_XMM_REG_LO_DWORD(i->dst()), op2 = BX_READ_XMM_REG_LO_DWORD(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  op1 = float32_max(op1, op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-  BX_WRITE_XMM_REG_LO_DWORD(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 7C
- * Add horizontally packed double precision FP in XMM2/MEM from XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::HADDPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_haddpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 7C
- * Add horizontally packed single precision FP in XMM2/MEM from XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::HADDPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_haddps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: 66 0F 7D
- * Subtract horizontally packed double precision FP in XMM2/MEM from XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::HSUBPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_hsubpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F 7D
- * Subtract horizontally packed single precision FP in XMM2/MEM from XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::HSUBPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_hsubps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(ADDSD_VsdWsdR, float64_add);
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(SUBSD_VsdWsdR, float64_sub);
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(MULSD_VsdWsdR, float64_mul);
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(DIVSD_VsdWsdR, float64_div);
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(MINSD_VsdWsdR, float64_min);
+SSE_SCALAR_DOUBLE_FP_CPU_LEVEL6(MAXSD_VsdWsdR, float64_max);
 
 /*
  * Opcode: 0F C2
@@ -1593,46 +1083,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPSS_VssWssIbR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-/*
- * Opcode: 66 0F D0
- * Add/Subtract packed double precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDSUBPD_VpdWpdR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_addsubpd(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
-/*
- * Opcode: F2 0F D0
- * Add/Substract packed single precision FP numbers from XMM2/MEM to XMM1.
- * Possible floating point exceptions: #I, #D, #O, #U, #P
- */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADDSUBPS_VpsWpsR(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-
-  float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  xmm_addsubps(&op1, &op2, status);
-  check_exceptionsSSE(get_exception_flags(status));
-
-  BX_WRITE_XMM_REG(i->dst(), op1);
-#endif
-
-  BX_NEXT_INSTR(i);
-}
-
 #if BX_CPU_LEVEL >= 6
 
 /* 66 0F 3A 08 */
@@ -1641,14 +1091,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROUNDPS_VpsWpsIbR(bxInstruction_c *i)
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
 
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  Bit8u control = i->Ib();
-
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   op.xmm32u(0) = float32_round_to_int(op.xmm32u(0), status);
   op.xmm32u(1) = float32_round_to_int(op.xmm32u(1), status);
@@ -1667,14 +1110,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROUNDPD_VpdWpdIbR(bxInstruction_c *i)
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
 
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  Bit8u control = i->Ib();
-
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   op.xmm64u(0) = float64_round_to_int(op.xmm64u(0), status);
   op.xmm64u(1) = float64_round_to_int(op.xmm64u(1), status);
@@ -1691,14 +1127,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROUNDSS_VssWssIbR(bxInstruction_c *i)
   float32 op = BX_READ_XMM_REG_LO_DWORD(i->src());
 
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  Bit8u control = i->Ib();
-
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   op = float32_round_to_int(op, status);
 
@@ -1714,14 +1143,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROUNDSD_VsdWsdIbR(bxInstruction_c *i)
   float64 op = BX_READ_XMM_REG_LO_QWORD(i->src());
 
   float_status_t status = mxcsr_to_softfloat_status_word(MXCSR);
-  Bit8u control = i->Ib();
-
-  // override MXCSR rounding mode with control coming from imm8
-  if ((control & 0x4) == 0)
-    status.float_rounding_mode = control & 0x3;
-  // ignore precision exception result
-  if (control & 0x8)
-    status.float_suppress_exception |= float_flag_inexact;
+  mxcsr_to_softfloat_status_word_imm_override(status, i->Ib());
 
   op = float64_round_to_int(op, status);
 
