@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: svm.cc 14318 2021-07-23 09:30:17Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2018 Stanislav Shwartsman
@@ -28,6 +28,10 @@
 
 #if BX_SUPPORT_SVM
 
+#include "svm.h"
+#include "cpuid.h"
+
+#include "gui/paramtree.h"
 #include "decoder/ia_opcodes.h"
 
 extern const char *segname[];
@@ -71,12 +75,12 @@ BX_CPP_INLINE Bit8u BX_CPU_C::vmcb_read8(unsigned offset)
   if (BX_CPU_THIS_PTR vmcbhostptr) {
     Bit8u *hostAddr = (Bit8u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     val_8 = *hostAddr;
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_8));
   }
   else {
-    access_read_physical(pAddr, 1, (Bit8u*)(&val_8));
+    val_8 = read_physical_byte(pAddr, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
 
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_8));
   return val_8;
 }
 
@@ -88,12 +92,12 @@ BX_CPP_INLINE Bit16u BX_CPU_C::vmcb_read16(unsigned offset)
   if (BX_CPU_THIS_PTR vmcbhostptr) {
     Bit16u *hostAddr = (Bit16u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     val_16 = ReadHostWordFromLittleEndian(hostAddr);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 2, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_16));
   }
   else {
-    access_read_physical(pAddr, 2, (Bit8u*)(&val_16));
+    val_16 = read_physical_word(pAddr, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
 
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 2, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_16));
   return val_16;
 }
 
@@ -105,12 +109,12 @@ BX_CPP_INLINE Bit32u BX_CPU_C::vmcb_read32(unsigned offset)
   if (BX_CPU_THIS_PTR vmcbhostptr) {
     Bit32u *hostAddr = (Bit32u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     val_32 = ReadHostDWordFromLittleEndian(hostAddr);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_32));
   }
   else {
-    access_read_physical(pAddr, 4, (Bit8u*)(&val_32));
+    val_32 = read_physical_dword(pAddr, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
 
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_32));
   return val_32;
 }
 
@@ -122,12 +126,12 @@ BX_CPP_INLINE Bit64u BX_CPU_C::vmcb_read64(unsigned offset)
   if (BX_CPU_THIS_PTR vmcbhostptr) {
     Bit64u *hostAddr = (Bit64u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     val_64 = ReadHostQWordFromLittleEndian(hostAddr);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 8, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_64));
   }
   else {
-    access_read_physical(pAddr, 8, (Bit8u*)(&val_64));
+    val_64 = read_physical_qword(pAddr, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
 
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 8, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_READ, BX_VMCS_ACCESS, (Bit8u*)(&val_64));
   return val_64;
 }
 
@@ -139,12 +143,11 @@ BX_CPP_INLINE void BX_CPU_C::vmcb_write8(unsigned offset, Bit8u val_8)
     Bit8u *hostAddr = (Bit8u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     pageWriteStampTable.decWriteStamp(pAddr, 1);
     *hostAddr = val_8;
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_8));
   }
   else {
-    access_write_physical(pAddr, 1, (Bit8u*)(&val_8));
+    write_physical_byte(pAddr, val_8, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
-
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_8));
 }
 
 BX_CPP_INLINE void BX_CPU_C::vmcb_write16(unsigned offset, Bit16u val_16)
@@ -155,12 +158,11 @@ BX_CPP_INLINE void BX_CPU_C::vmcb_write16(unsigned offset, Bit16u val_16)
     Bit16u *hostAddr = (Bit16u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     pageWriteStampTable.decWriteStamp(pAddr, 2);
     WriteHostWordToLittleEndian(hostAddr, val_16);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 2, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_16));
   }
   else {
-    access_write_physical(pAddr, 2, (Bit8u*)(&val_16));
+    write_physical_word(pAddr, val_16, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
-
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 2, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_16));
 }
 
 BX_CPP_INLINE void BX_CPU_C::vmcb_write32(unsigned offset, Bit32u val_32)
@@ -171,12 +173,11 @@ BX_CPP_INLINE void BX_CPU_C::vmcb_write32(unsigned offset, Bit32u val_32)
     Bit32u *hostAddr = (Bit32u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     pageWriteStampTable.decWriteStamp(pAddr, 4);
     WriteHostDWordToLittleEndian(hostAddr, val_32);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_32));
   }
   else {
-    access_write_physical(pAddr, 4, (Bit8u*)(&val_32));
+    write_physical_dword(pAddr, val_32, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
-
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 4, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_32));
 }
 
 BX_CPP_INLINE void BX_CPU_C::vmcb_write64(unsigned offset, Bit64u val_64)
@@ -187,12 +188,11 @@ BX_CPP_INLINE void BX_CPU_C::vmcb_write64(unsigned offset, Bit64u val_64)
     Bit64u *hostAddr = (Bit64u*) (BX_CPU_THIS_PTR vmcbhostptr | offset);
     pageWriteStampTable.decWriteStamp(pAddr, 8);
     WriteHostQWordToLittleEndian(hostAddr, val_64);
+    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 8, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_64));
   }
   else {
-    access_write_physical(pAddr, 8, (Bit8u*)(&val_64));
+    write_physical_qword(pAddr, val_64, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_VMCS_ACCESS);
   }
-
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 8, MEMTYPE(BX_CPU_THIS_PTR vmcb_memtype), BX_WRITE, BX_VMCS_ACCESS, (Bit8u*)(&val_64));
 }
 
 BX_CPP_INLINE void BX_CPU_C::svm_segment_read(bx_segment_reg_t *seg, unsigned offset)
@@ -212,7 +212,7 @@ BX_CPP_INLINE void BX_CPU_C::svm_segment_write(bx_segment_reg_t *seg, unsigned o
   Bit32u selector = seg->selector.value;
   bx_address base = seg->cache.u.segment.base;
   Bit32u limit = seg->cache.u.segment.limit_scaled;
-  Bit32u attr = (seg->cache.valid) ? 
+  Bit32u attr = (seg->cache.valid) ?
      (get_descriptor_h(&seg->cache) & 0x00f0ff00) : 0;
 
   vmcb_write16(offset, selector);
@@ -317,7 +317,7 @@ void BX_CPU_C::SvmExitSaveGuestState(void)
 
   vmcb_write8(SVM_CONTROL_INTERRUPT_SHADOW, interrupts_inhibited(BX_INHIBIT_INTERRUPTS));
 
-  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
 
   if (ctrls->nested_paging) {
     vmcb_write64(SVM_GUEST_PAT, BX_CPU_THIS_PTR msr.pat.u64);
@@ -554,7 +554,7 @@ bool BX_CPU_C::SvmEnterLoadCheckGuestState(void)
   if (paged_real_mode)
     BX_CPU_THIS_PTR cr0.val32 |= BX_CR0_PG_MASK;
 
-  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
   if (! ctrls->nested_paging) {
     if (BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr4.get_PAE() && !long_mode()) {
       if (! CheckPDPTR(BX_CPU_THIS_PTR cr3)) {
@@ -577,7 +577,7 @@ bool BX_CPU_C::SvmEnterLoadCheckGuestState(void)
 
   BX_CPU_THIS_PTR gdtr = guest.gdtr;
   BX_CPU_THIS_PTR idtr = guest.idtr;
- 
+
   RIP = BX_CPU_THIS_PTR prev_rip = guest.rip;
   RSP = guest.rsp;
   RAX = guest.rax;
@@ -625,7 +625,7 @@ void BX_CPU_C::Svm_Vmexit(int reason, Bit64u exitinfo1, Bit64u exitinfo2)
   RIP = BX_CPU_THIS_PTR prev_rip;
   if (BX_CPU_THIS_PTR speculative_rsp)
     RSP = BX_CPU_THIS_PTR prev_rsp;
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  BX_CPU_THIS_PTR speculative_rsp = false;
 
   if (BX_SUPPORT_SVM_EXTENSION(BX_CPUID_SVM_DECODE_ASSIST)) {
     //
@@ -651,14 +651,14 @@ void BX_CPU_C::Svm_Vmexit(int reason, Bit64u exitinfo1, Bit64u exitinfo2)
 
   mask_event(BX_EVENT_SVM_VIRQ_PENDING);
 
-  BX_CPU_THIS_PTR in_svm_guest = 0;
-  BX_CPU_THIS_PTR svm_gif = 0;
+  BX_CPU_THIS_PTR in_svm_guest = false;
+  BX_CPU_THIS_PTR svm_gif = false;
 
   //
   // STEP 0: Update exit reason
   //
 
-  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
 
   vmcb_write64(SVM_CONTROL64_EXITCODE, (Bit64u) ((Bit64s) reason));
   vmcb_write64(SVM_CONTROL64_EXITINFO1, exitinfo1);
@@ -670,7 +670,7 @@ void BX_CPU_C::Svm_Vmexit(int reason, Bit64u exitinfo1, Bit64u exitinfo2)
   if (BX_CPU_THIS_PTR in_event) {
     vmcb_write32(SVM_CONTROL32_EXITINTINFO, ctrls->exitintinfo | 0x80000000);
     vmcb_write32(SVM_CONTROL32_EXITINTINFO_ERROR_CODE, ctrls->exitintinfo_error_code);
-    BX_CPU_THIS_PTR in_event = 0;
+    BX_CPU_THIS_PTR in_event = false;
   }
   else {
     vmcb_write32(SVM_CONTROL32_EXITINTINFO, 0);
@@ -684,7 +684,7 @@ void BX_CPU_C::Svm_Vmexit(int reason, Bit64u exitinfo1, Bit64u exitinfo2)
   //
   // Step 2:
   //
-  SvmExitLoadHostState(&BX_CPU_THIS_PTR vmcb.host_state);
+  SvmExitLoadHostState(&BX_CPU_THIS_PTR vmcb->host_state);
 
   //
   // STEP 3: Go back to SVM host
@@ -703,14 +703,14 @@ void BX_CPU_C::Svm_Vmexit(int reason, Bit64u exitinfo1, Bit64u exitinfo2)
   longjmp(BX_CPU_THIS_PTR jmp_buf_env, 1); // go back to main decode loop
 }
 
-extern struct BxExceptionInfo exceptions_info[];
+extern int get_exception_type(unsigned vector);
 
 bool BX_CPU_C::SvmInjectEvents(void)
 {
-  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
 
   ctrls->eventinj = vmcb_read32(SVM_CONTROL32_EVENT_INJECTION);
-  if ((ctrls->eventinj & 0x80000000) == 0) return 1;
+  if ((ctrls->eventinj & 0x80000000) == 0) return true;
 
   /* the VMENTRY injecting event to the guest */
   unsigned vector = ctrls->eventinj & 0xff;
@@ -732,7 +732,7 @@ bool BX_CPU_C::SvmInjectEvents(void)
     case BX_HARDWARE_EXCEPTION:
       if (vector == 2 || vector > 31) {
         BX_ERROR(("SvmInjectEvents: invalid vector %d for HW exception", vector));
-        return 0;
+        return false;
       }
       if (vector == BX_BP_EXCEPTION || vector == BX_OF_EXCEPTION) {
         type = BX_SOFTWARE_EXCEPTION;
@@ -745,7 +745,7 @@ bool BX_CPU_C::SvmInjectEvents(void)
 
     default:
       BX_ERROR(("SvmInjectEvents: unsupported event injection type %d !", type));
-      return 0;
+      return false;
   }
 
   BX_DEBUG(("SvmInjectEvents: Injecting vector 0x%02x (error_code 0x%04x)", vector, error_code));
@@ -753,7 +753,7 @@ bool BX_CPU_C::SvmInjectEvents(void)
   if (type == BX_HARDWARE_EXCEPTION) {
     // record exception the same way as BX_CPU_C::exception does
     BX_ASSERT(vector < BX_CPU_HANDLED_EXCEPTIONS);
-    BX_CPU_THIS_PTR last_exception_type = exceptions_info[vector].exception_type;
+    BX_CPU_THIS_PTR last_exception_type = get_exception_type(vector);
   }
 
   ctrls->exitintinfo = ctrls->eventinj & ~0x80000000;
@@ -763,7 +763,7 @@ bool BX_CPU_C::SvmInjectEvents(void)
 
   BX_CPU_THIS_PTR last_exception_type = 0; // error resolved
 
-  return 1;
+  return true;
 }
 
 void BX_CPU_C::SvmInterceptException(unsigned type, unsigned vector, Bit16u errcode, bool errcode_valid, Bit64u qualification)
@@ -772,7 +772,7 @@ void BX_CPU_C::SvmInterceptException(unsigned type, unsigned vector, Bit16u errc
 
   BX_ASSERT(vector < 32);
 
-  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+  SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
 
   BX_ASSERT(type == BX_HARDWARE_EXCEPTION || type == BX_SOFTWARE_EXCEPTION);
 
@@ -788,11 +788,11 @@ void BX_CPU_C::SvmInterceptException(unsigned type, unsigned vector, Bit16u errc
     // [31:31] | interruption info valid
     //
 
-    // record IDT vectoring information 
+    // record IDT vectoring information
     ctrls->exitintinfo_error_code = errcode;
     ctrls->exitintinfo = vector | (BX_HARDWARE_EXCEPTION << 8);
     if (errcode_valid)
-      BX_CPU_THIS_PTR vmcb.ctrls.exitintinfo |= (1 << 11); // error code delivered
+      BX_CPU_THIS_PTR vmcb->ctrls.exitintinfo |= (1 << 11); // error code delivered
     return;
   }
 
@@ -801,7 +801,7 @@ void BX_CPU_C::SvmInterceptException(unsigned type, unsigned vector, Bit16u errc
   // VMEXIT is not considered to occur during event delivery if it results
   // in a double fault exception that causes VMEXIT directly
   if (vector == BX_DF_EXCEPTION)
-    BX_CPU_THIS_PTR in_event = 0; // clear in_event indication on #DF
+    BX_CPU_THIS_PTR in_event = false; // clear in_event indication on #DF
 
   BX_CPU_THIS_PTR debug_trap = 0; // clear debug_trap field
   BX_CPU_THIS_PTR inhibit_mask = 0;
@@ -809,15 +809,17 @@ void BX_CPU_C::SvmInterceptException(unsigned type, unsigned vector, Bit16u errc
   Svm_Vmexit(SVM_VMEXIT_EXCEPTION + vector, (errcode_valid ? errcode : 0), qualification);
 }
 
-#define SVM_VMEXIT_IO_PORTIN        (1 << 0)
-#define SVM_VMEXIT_IO_INSTR_STRING  (1 << 2)
-#define SVM_VMEXIT_IO_INSTR_REP     (1 << 3)
-#define SVM_VMEXIT_IO_INSTR_LEN8    (1 << 4)
-#define SVM_VMEXIT_IO_INSTR_LEN16   (1 << 5)
-#define SVM_VMEXIT_IO_INSTR_LEN32   (1 << 6)
-#define SVM_VMEXIT_IO_INSTR_ASIZE16 (1 << 7)
-#define SVM_VMEXIT_IO_INSTR_ASIZE32 (1 << 8)
-#define SVM_VMEXIT_IO_INSTR_ASIZE64 (1 << 9)
+enum {
+  SVM_VMEXIT_IO_PORTIN        = (1 << 0),
+  SVM_VMEXIT_IO_INSTR_STRING  = (1 << 2),
+  SVM_VMEXIT_IO_INSTR_REP     = (1 << 3),
+  SVM_VMEXIT_IO_INSTR_LEN8    = (1 << 4),
+  SVM_VMEXIT_IO_INSTR_LEN16   = (1 << 5),
+  SVM_VMEXIT_IO_INSTR_LEN32   = (1 << 6),
+  SVM_VMEXIT_IO_INSTR_ASIZE16 = (1 << 7),
+  SVM_VMEXIT_IO_INSTR_ASIZE32 = (1 << 8),
+  SVM_VMEXIT_IO_INSTR_ASIZE64 = (1 << 9)
+};
 
 void BX_CPU_C::SvmInterceptIO(bxInstruction_c *i, unsigned port, unsigned len)
 {
@@ -829,13 +831,11 @@ void BX_CPU_C::SvmInterceptIO(bxInstruction_c *i, unsigned port, unsigned len)
   bx_phy_address pAddr;
 
   // access_read_physical cannot read 2 bytes cross 4K boundary :(
-  pAddr = BX_CPU_THIS_PTR vmcb.ctrls.iopm_base + (port / 8);
-  access_read_physical(pAddr, 1, &bitmap[0]);
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[0]);
+  pAddr = BX_CPU_THIS_PTR vmcb->ctrls.iopm_base + (port / 8);
+  bitmap[0] = read_physical_byte(pAddr, MEMTYPE(resolve_memtype(pAddr)), BX_IO_BITMAP_ACCESS);
 
   pAddr++;
-  access_read_physical(pAddr, 1, &bitmap[1]);
-  BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_IO_BITMAP_ACCESS, &bitmap[1]);
+  bitmap[1] = read_physical_byte(pAddr, MEMTYPE(resolve_memtype(pAddr)), BX_IO_BITMAP_ACCESS);
 
   Bit16u combined_bitmap = bitmap[1];
   combined_bitmap = (combined_bitmap << 8) | bitmap[0];
@@ -897,7 +897,7 @@ void BX_CPU_C::SvmInterceptIO(bxInstruction_c *i, unsigned port, unsigned len)
       qualification |= SVM_VMEXIT_IO_INSTR_ASIZE64;
     else if (i->as32L())
       qualification |= SVM_VMEXIT_IO_INSTR_ASIZE32;
-    else 
+    else
       qualification |= SVM_VMEXIT_IO_INSTR_ASIZE16;
 
     Svm_Vmexit(SVM_VMEXIT_IO, qualification, RIP);
@@ -920,13 +920,11 @@ void BX_CPU_C::SvmInterceptMSR(unsigned op, Bit32u msr)
   else if (msr >= 0xc0010000 && msr <= 0xc0011fff) msr_map_offset = 4096;
 
   if (msr_map_offset >= 0) {
-    bx_phy_address msr_bitmap_addr = BX_CPU_THIS_PTR vmcb.ctrls.msrpm_base + msr_map_offset;
+    bx_phy_address msr_bitmap_addr = BX_CPU_THIS_PTR vmcb->ctrls.msrpm_base + msr_map_offset;
     Bit32u msr_offset = (msr & 0x1fff) * 2 + op;
 
-    Bit8u msr_bitmap;
     bx_phy_address pAddr = msr_bitmap_addr + (msr_offset / 8);
-    access_read_physical(pAddr, 1, (Bit8u*)(&msr_bitmap));
-    BX_NOTIFY_PHY_MEMORY_ACCESS(pAddr, 1, MEMTYPE(resolve_memtype(pAddr)), BX_READ, BX_MSR_BITMAP_ACCESS, &msr_bitmap);
+    Bit8u msr_bitmap = read_physical_byte(pAddr, MEMTYPE(resolve_memtype(pAddr)), BX_MSR_BITMAP_ACCESS);
 
     vmexit = (msr_bitmap >> (msr_offset & 7)) & 0x1;
   }
@@ -971,7 +969,7 @@ void BX_CPU_C::SvmInterceptTaskSwitch(Bit16u tss_selector, unsigned source, bool
 void BX_CPU_C::SvmInterceptPAUSE(void)
 {
   if (BX_SUPPORT_SVM_EXTENSION(BX_CPUID_SVM_PAUSE_FILTER)) {
-    SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb.ctrls;
+    SVM_CONTROLS *ctrls = &BX_CPU_THIS_PTR vmcb->ctrls;
     if (ctrls->pause_filter_count) {
       ctrls->pause_filter_count--;
       return;
@@ -981,11 +979,38 @@ void BX_CPU_C::SvmInterceptPAUSE(void)
   Svm_Vmexit(SVM_VMEXIT_PAUSE);
 }
 
-#endif
+void BX_CPU_C::Svm_Update_VM_CR_MSR(Bit64u val_64)
+{
+  // VM_CR_MSR 0xc0010114
+
+  // [0]: DPD - If set, disables the external hardware debug port and certain internal debug features
+  // [1]: R_INIT - If set, non-intercepted INIT signals are converted into an #SX exception
+  // [2]: DIS_A20M - If set, disables A20 masking
+  // [3]: LOCK - When this bit is set, writes to LOCK and SVMDIS are silently ignored
+  // [4]: SVMDIS - When this bit is set, writes to EFER treat the SVME bit as MBZ
+  //      Setting SVMDIS while EFER.SVME is 1 generates a #GP fault, regardless of the current state of VM_CR.LOCK
+  //      It is cleared by INIT when LOCK is cleared to 0; otherwise, it is not affected.
+  if (val_64 >> 5) {
+    BX_ERROR(("VM_CR_MSR: attempt to set reserved bits"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+
+  if (BX_CPU_THIS_PTR msr.svm_vm_cr & BX_VM_CR_MSR_SVMDIS_MASK) {
+    if (BX_CPU_THIS_PTR efer.get_SVME()) {
+      BX_ERROR(("VM_CR_MSR: attempt to set SVMDIS when EFER.SVME=1"));
+      exception(BX_GP_EXCEPTION, 0);
+    }
+  }
+
+  if (BX_CPU_THIS_PTR msr.svm_vm_cr & BX_VM_CR_MSR_LOCK_MASK) {
+    val_64 = (val_64 & 0x7) | (BX_CPU_THIS_PTR msr.svm_vm_cr & (BX_VM_CR_MSR_LOCK_MASK | BX_VM_CR_MSR_SVMDIS_MASK));
+  }
+
+  BX_CPU_THIS_PTR msr.svm_vm_cr = GET32L(val_64);
+}
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMRUN(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1011,12 +1036,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMRUN(bxInstruction_c *i)
   //
   // Step 1: Save host state to physical memory indicated in SVM_HSAVE_PHY_ADDR_MSR
   //
-  SvmEnterSaveHostState(&BX_CPU_THIS_PTR vmcb.host_state);
+  SvmEnterSaveHostState(&BX_CPU_THIS_PTR vmcb->host_state);
 
   //
   // Step 2: Load control information from the VMCB
   //
-  if (!SvmEnterLoadCheckControls(&BX_CPU_THIS_PTR vmcb.ctrls))
+  if (!SvmEnterLoadCheckControls(&BX_CPU_THIS_PTR vmcb->ctrls))
     Svm_Vmexit(SVM_VMEXIT_INVALID);
 
   //
@@ -1025,8 +1050,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMRUN(bxInstruction_c *i)
   if (!SvmEnterLoadCheckGuestState())
     Svm_Vmexit(SVM_VMEXIT_INVALID);
 
-  BX_CPU_THIS_PTR in_svm_guest = 1;
-  BX_CPU_THIS_PTR svm_gif = 1;
+  BX_CPU_THIS_PTR in_svm_guest = true;
+  BX_CPU_THIS_PTR svm_gif = true;
   BX_CPU_THIS_PTR async_event = 1;
 
   //
@@ -1034,14 +1059,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMRUN(bxInstruction_c *i)
   //
   if (!SvmInjectEvents())
     Svm_Vmexit(SVM_VMEXIT_INVALID);
-#endif
 
   BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMMCALL(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (BX_CPU_THIS_PTR efer.get_SVME()) {
     if (BX_CPU_THIS_PTR in_svm_guest) {
       if (SVM_INTERCEPT(SVM_INTERCEPT1_VMMCALL)) Svm_Vmexit(SVM_VMEXIT_VMMCALL);
@@ -1049,14 +1072,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMMCALL(bxInstruction_c *i)
   }
 
   exception(BX_UD_EXCEPTION, 0);
-#endif
 
   BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMLOAD(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1074,6 +1095,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMLOAD(bxInstruction_c *i)
     BX_ERROR(("VMLOAD: invalid or not page aligned VMCB physical address !"));
     exception(BX_GP_EXCEPTION, 0);
   }
+  bx_phy_address vmcbPtr = BX_CPU_THIS_PTR vmcbptr;
   set_VMCBPTR(pAddr);
 
   BX_DEBUG(("VMLOAD VMCB ptr: 0x" FMT_ADDRX64, BX_CPU_THIS_PTR vmcbptr));
@@ -1099,14 +1121,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMLOAD(bxInstruction_c *i)
   BX_CPU_THIS_PTR msr.sysenter_cs_msr = vmcb_read64(SVM_GUEST_SYSENTER_CS_MSR);
   BX_CPU_THIS_PTR msr.sysenter_eip_msr = CanonicalizeAddress(vmcb_read64(SVM_GUEST_SYSENTER_EIP_MSR));
   BX_CPU_THIS_PTR msr.sysenter_esp_msr = CanonicalizeAddress(vmcb_read64(SVM_GUEST_SYSENTER_ESP_MSR));
-#endif
+
+  set_VMCBPTR(vmcbPtr);
 
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMSAVE(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1124,6 +1146,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMSAVE(bxInstruction_c *i)
     BX_ERROR(("VMSAVE: invalid or not page aligned VMCB physical address !"));
     exception(BX_GP_EXCEPTION, 0);
   }
+  bx_phy_address vmcbPtr = BX_CPU_THIS_PTR vmcbptr;
   set_VMCBPTR(pAddr);
 
   BX_DEBUG(("VMSAVE VMCB ptr: 0x" FMT_ADDRX64, BX_CPU_THIS_PTR vmcbptr));
@@ -1142,14 +1165,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VMSAVE(bxInstruction_c *i)
   vmcb_write64(SVM_GUEST_SYSENTER_CS_MSR, BX_CPU_THIS_PTR msr.sysenter_cs_msr);
   vmcb_write64(SVM_GUEST_SYSENTER_ESP_MSR, BX_CPU_THIS_PTR msr.sysenter_esp_msr);
   vmcb_write64(SVM_GUEST_SYSENTER_EIP_MSR, BX_CPU_THIS_PTR msr.sysenter_eip_msr);
-#endif
+
+  set_VMCBPTR(vmcbPtr);
 
   BX_NEXT_INSTR(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SKINIT(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1163,14 +1186,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SKINIT(bxInstruction_c *i)
   }
 
   BX_PANIC(("SVM: SKINIT is not implemented yet"));
-#endif
 
   BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLGI(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1183,15 +1204,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLGI(bxInstruction_c *i)
     if (SVM_INTERCEPT(SVM_INTERCEPT1_CLGI)) Svm_Vmexit(SVM_VMEXIT_CLGI);
   }
 
-  BX_CPU_THIS_PTR svm_gif = 0;
-#endif
+  BX_CPU_THIS_PTR svm_gif = false;
 
   BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::STGI(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1204,16 +1223,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STGI(bxInstruction_c *i)
     if (SVM_INTERCEPT(SVM_INTERCEPT1_STGI)) Svm_Vmexit(SVM_VMEXIT_STGI);
   }
 
-  BX_CPU_THIS_PTR svm_gif = 1;
+  BX_CPU_THIS_PTR svm_gif = true;
   BX_CPU_THIS_PTR async_event = 1;
-#endif
 
   BX_NEXT_TRACE(i);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::INVLPGA(bxInstruction_c *i)
 {
-#if BX_SUPPORT_SVM
   if (! protected_mode() || ! BX_CPU_THIS_PTR efer.get_SVME())
     exception(BX_UD_EXCEPTION, 0);
 
@@ -1222,17 +1239,18 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INVLPGA(bxInstruction_c *i)
     exception(BX_GP_EXCEPTION, 0);
   }
 
+  bx_address laddr = RAX & i->asize_mask();
+
   if (BX_CPU_THIS_PTR in_svm_guest) {
-    if (SVM_INTERCEPT(SVM_INTERCEPT0_INVLPGA)) Svm_Vmexit(SVM_VMEXIT_INVLPGA);
+    if (SVM_INTERCEPT(SVM_INTERCEPT0_INVLPGA))
+      Svm_Vmexit(SVM_VMEXIT_INVLPGA, BX_SUPPORT_SVM_EXTENSION(BX_CPUID_SVM_DECODE_ASSIST) ? laddr : 0);
   }
 
-  TLB_flush(); // FIXME: flush all entries for now
-#endif
+  TLB_invlpg(laddr); // FIXME: flush all ASID entries for now
 
   BX_NEXT_TRACE(i);
 }
 
-#if BX_SUPPORT_SVM
 void BX_CPU_C::register_svm_state(bx_param_c *parent)
 {
   if (! is_cpu_extension_supported(BX_ISA_SVM)) return;
@@ -1250,26 +1268,26 @@ void BX_CPU_C::register_svm_state(bx_param_c *parent)
 
   bx_list_c *vmcb_ctrls = new bx_list_c(svm, "VMCB_CTRLS");
 
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, cr_rd_ctrl, BX_CPU_THIS_PTR vmcb.ctrls.cr_rd_ctrl);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, cr_wr_ctrl, BX_CPU_THIS_PTR vmcb.ctrls.cr_wr_ctrl);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, dr_rd_ctrl, BX_CPU_THIS_PTR vmcb.ctrls.dr_rd_ctrl);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, dr_wr_ctrl, BX_CPU_THIS_PTR vmcb.ctrls.dr_wr_ctrl);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exceptions_intercept, BX_CPU_THIS_PTR vmcb.ctrls.exceptions_intercept);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, intercept_vector0, BX_CPU_THIS_PTR vmcb.ctrls.intercept_vector[0]);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, intercept_vector1, BX_CPU_THIS_PTR vmcb.ctrls.intercept_vector[1]);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, iopm_base, BX_CPU_THIS_PTR vmcb.ctrls.iopm_base);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, msrpm_base, BX_CPU_THIS_PTR vmcb.ctrls.msrpm_base);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exitintinfo, BX_CPU_THIS_PTR vmcb.ctrls.exitintinfo);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exitintinfo_errcode, BX_CPU_THIS_PTR vmcb.ctrls.exitintinfo_error_code);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, eventinj, BX_CPU_THIS_PTR vmcb.ctrls.eventinj);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, cr_rd_ctrl, BX_CPU_THIS_PTR vmcb->ctrls.cr_rd_ctrl);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, cr_wr_ctrl, BX_CPU_THIS_PTR vmcb->ctrls.cr_wr_ctrl);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, dr_rd_ctrl, BX_CPU_THIS_PTR vmcb->ctrls.dr_rd_ctrl);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, dr_wr_ctrl, BX_CPU_THIS_PTR vmcb->ctrls.dr_wr_ctrl);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exceptions_intercept, BX_CPU_THIS_PTR vmcb->ctrls.exceptions_intercept);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, intercept_vector0, BX_CPU_THIS_PTR vmcb->ctrls.intercept_vector[0]);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, intercept_vector1, BX_CPU_THIS_PTR vmcb->ctrls.intercept_vector[1]);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, iopm_base, BX_CPU_THIS_PTR vmcb->ctrls.iopm_base);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, msrpm_base, BX_CPU_THIS_PTR vmcb->ctrls.msrpm_base);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exitintinfo, BX_CPU_THIS_PTR vmcb->ctrls.exitintinfo);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, exitintinfo_errcode, BX_CPU_THIS_PTR vmcb->ctrls.exitintinfo_error_code);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, eventinj, BX_CPU_THIS_PTR vmcb->ctrls.eventinj);
 
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_tpr, BX_CPU_THIS_PTR vmcb.ctrls.v_tpr);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_intr_prio, BX_CPU_THIS_PTR vmcb.ctrls.v_intr_prio);
-  BXRS_PARAM_BOOL(vmcb_ctrls, v_ignore_tpr, BX_CPU_THIS_PTR vmcb.ctrls.v_ignore_tpr);
-  BXRS_PARAM_BOOL(vmcb_ctrls, v_intr_masking, BX_CPU_THIS_PTR vmcb.ctrls.v_intr_masking);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_intr_vector, BX_CPU_THIS_PTR vmcb.ctrls.v_intr_vector);
-  BXRS_PARAM_BOOL(vmcb_ctrls, nested_paging, BX_CPU_THIS_PTR vmcb.ctrls.nested_paging);
-  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, ncr3, BX_CPU_THIS_PTR vmcb.ctrls.ncr3);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_tpr, BX_CPU_THIS_PTR vmcb->ctrls.v_tpr);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_intr_prio, BX_CPU_THIS_PTR vmcb->ctrls.v_intr_prio);
+  BXRS_PARAM_BOOL(vmcb_ctrls, v_ignore_tpr, BX_CPU_THIS_PTR vmcb->ctrls.v_ignore_tpr);
+  BXRS_PARAM_BOOL(vmcb_ctrls, v_intr_masking, BX_CPU_THIS_PTR vmcb->ctrls.v_intr_masking);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, v_intr_vector, BX_CPU_THIS_PTR vmcb->ctrls.v_intr_vector);
+  BXRS_PARAM_BOOL(vmcb_ctrls, nested_paging, BX_CPU_THIS_PTR vmcb->ctrls.nested_paging);
+  BXRS_HEX_PARAM_FIELD(vmcb_ctrls, ncr3, BX_CPU_THIS_PTR vmcb->ctrls.ncr3);
 
   //
   // VMCB Host State
@@ -1278,7 +1296,7 @@ void BX_CPU_C::register_svm_state(bx_param_c *parent)
   bx_list_c *host = new bx_list_c(svm, "VMCB_HOST_STATE");
 
   for(unsigned n=0; n<4; n++) {
-    bx_segment_reg_t *segment = &BX_CPU_THIS_PTR vmcb.host_state.sregs[n];
+    bx_segment_reg_t *segment = &BX_CPU_THIS_PTR vmcb->host_state.sregs[n];
     bx_list_c *sreg = new bx_list_c(host, segname[n]);
     BXRS_HEX_PARAM_FIELD(sreg, selector, segment->selector.value);
     BXRS_HEX_PARAM_FIELD(sreg, valid, segment->cache.valid);
@@ -1297,20 +1315,21 @@ void BX_CPU_C::register_svm_state(bx_param_c *parent)
   }
 
   bx_list_c *GDTR = new bx_list_c(host, "GDTR");
-  BXRS_HEX_PARAM_FIELD(GDTR, base, BX_CPU_THIS_PTR vmcb.host_state.gdtr.base);
-  BXRS_HEX_PARAM_FIELD(GDTR, limit, BX_CPU_THIS_PTR vmcb.host_state.gdtr.limit);
+  BXRS_HEX_PARAM_FIELD(GDTR, base, BX_CPU_THIS_PTR vmcb->host_state.gdtr.base);
+  BXRS_HEX_PARAM_FIELD(GDTR, limit, BX_CPU_THIS_PTR vmcb->host_state.gdtr.limit);
 
   bx_list_c *IDTR = new bx_list_c(host, "IDTR");
-  BXRS_HEX_PARAM_FIELD(IDTR, base, BX_CPU_THIS_PTR vmcb.host_state.idtr.base);
-  BXRS_HEX_PARAM_FIELD(IDTR, limit, BX_CPU_THIS_PTR vmcb.host_state.idtr.limit);
+  BXRS_HEX_PARAM_FIELD(IDTR, base, BX_CPU_THIS_PTR vmcb->host_state.idtr.base);
+  BXRS_HEX_PARAM_FIELD(IDTR, limit, BX_CPU_THIS_PTR vmcb->host_state.idtr.limit);
 
-  BXRS_HEX_PARAM_FIELD(host, efer, BX_CPU_THIS_PTR vmcb.host_state.efer.val32);
-  BXRS_HEX_PARAM_FIELD(host, cr0, BX_CPU_THIS_PTR vmcb.host_state.cr0.val32);
-  BXRS_HEX_PARAM_FIELD(host, cr3, BX_CPU_THIS_PTR vmcb.host_state.cr3);
-  BXRS_HEX_PARAM_FIELD(host, cr4, BX_CPU_THIS_PTR vmcb.host_state.cr4.val32);
-  BXRS_HEX_PARAM_FIELD(host, eflags, BX_CPU_THIS_PTR vmcb.host_state.eflags);
-  BXRS_HEX_PARAM_FIELD(host, rip, BX_CPU_THIS_PTR vmcb.host_state.rip);
-  BXRS_HEX_PARAM_FIELD(host, rsp, BX_CPU_THIS_PTR vmcb.host_state.rsp);
-  BXRS_HEX_PARAM_FIELD(host, rax, BX_CPU_THIS_PTR vmcb.host_state.rax);
+  BXRS_HEX_PARAM_FIELD(host, efer, BX_CPU_THIS_PTR vmcb->host_state.efer.val32);
+  BXRS_HEX_PARAM_FIELD(host, cr0, BX_CPU_THIS_PTR vmcb->host_state.cr0.val32);
+  BXRS_HEX_PARAM_FIELD(host, cr3, BX_CPU_THIS_PTR vmcb->host_state.cr3);
+  BXRS_HEX_PARAM_FIELD(host, cr4, BX_CPU_THIS_PTR vmcb->host_state.cr4.val32);
+  BXRS_HEX_PARAM_FIELD(host, eflags, BX_CPU_THIS_PTR vmcb->host_state.eflags);
+  BXRS_HEX_PARAM_FIELD(host, rip, BX_CPU_THIS_PTR vmcb->host_state.rip);
+  BXRS_HEX_PARAM_FIELD(host, rsp, BX_CPU_THIS_PTR vmcb->host_state.rsp);
+  BXRS_HEX_PARAM_FIELD(host, rax, BX_CPU_THIS_PTR vmcb->host_state.rax);
 }
-#endif
+
+#endif // BX_SUPPORT_SVM
