@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vmx.h 14086 2021-01-30 08:35:35Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2009-2019 Stanislav Shwartsman
+//   Copyright (c) 2009-2024 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -134,22 +134,30 @@ enum VMX_vmexit_reason {
    VMX_VMEXIT_PML_LOGFULL = 62,
    VMX_VMEXIT_XSAVES = 63,
    VMX_VMEXIT_XRSTORS = 64,
-   VMX_VMEXIT_RESERVED65 = 65,
+   VMX_VMEXIT_PCONFIG = 65,
    VMX_VMEXIT_SPP = 66,
    VMX_VMEXIT_UMWAIT = 67,
    VMX_VMEXIT_TPAUSE = 68,
-   VMX_VMEXIT_RESERVED69 = 69,
-   VMX_VMEXIT_RESERVED70 = 70,
+   VMX_VMEXIT_LOADIWKEY = 69,
+   VMX_VMEXIT_ENCLV = 70,
    VMX_VMEXIT_RESERVED71 = 71,
    VMX_VMEXIT_ENQCMD_PASID = 72,
    VMX_VMEXIT_ENQCMDS_PASID = 73,
+   VMX_VMEXIT_BUS_LOCK = 74,
+   VMX_VMEXIT_NOTIFY_WINDOW = 75,
+   VMX_VMEXIT_SEAMCALL = 76,
+   VMX_VMEXIT_TDCALL = 77,
+   VMX_VMEXIT_RDMSRLIST = 78,
+   VMX_VMEXIT_WRMSRLIST = 79,
    VMX_VMEXIT_LAST_REASON
 };
 
+// note, MONITOR_TRAP_FLAG should not be here because it taken between instructions
 #define IS_TRAP_LIKE_VMEXIT(reason) \
       (reason == VMX_VMEXIT_TPR_THRESHOLD || \
        reason == VMX_VMEXIT_VIRTUALIZED_EOI || \
-       reason == VMX_VMEXIT_APIC_WRITE)
+       reason == VMX_VMEXIT_APIC_WRITE || \
+       reason == VMX_VMEXIT_BUS_LOCK)
 
 // VMexit on CR register access
 enum {
@@ -169,7 +177,7 @@ enum VMX_vmentry_error {
 
 // VMABORT error code
 enum VMX_vmabort_code {
-   VMABORT_SAVING_GUEST_MSRS_FAILURE,
+   VMABORT_SAVING_GUEST_MSRS_FAILURE = 0,
    VMABORT_HOST_PDPTR_CORRUPTED,
    VMABORT_VMEXIT_VMCS_CORRUPTED,
    VMABORT_LOADING_HOST_MSRS,
@@ -196,8 +204,10 @@ const Bit64u VMX_VMFUNC_EPTP_SWITCHING_MASK = (BX_CONST64(1) << VMX_VMFUNC_EPTP_
 /* VMCS 16-bit control fields */
 /* binary 0000_00xx_xxxx_xxx0 */
 #define VMCS_16BIT_CONTROL_VPID                            0x00000000 /* VPID */
-#define VMCS_16BIT_CONTROL_POSTED_INTERRUPT_VECTOR         0x00000002 /* Posted Interrupts - not implememted yet */
+#define VMCS_16BIT_CONTROL_POSTED_INTERRUPT_VECTOR         0x00000002 /* Posted Interrupts */
 #define VMCS_16BIT_CONTROL_EPTP_INDEX                      0x00000004 /* #VE Exception */
+#define VMCS_16BIT_CONTROL_HLAT_PREFIX                     0x00000006 /* HLAT */
+#define VMCS_16BIT_CONTROL_LAST_PID_POINTER_INDEX          0x00000008 /* IPI Virtualization */
 
 /* VMCS 16-bit guest-state fields */
 /* binary 0000_10xx_xxxx_xxx0 */
@@ -211,6 +221,7 @@ const Bit64u VMX_VMFUNC_EPTP_SWITCHING_MASK = (BX_CONST64(1) << VMX_VMFUNC_EPTP_
 #define VMCS_16BIT_GUEST_TR_SELECTOR                       0x0000080E
 #define VMCS_16BIT_GUEST_INTERRUPT_STATUS                  0x00000810 /* Virtual Interrupt Delivery */
 #define VMCS_16BIT_GUEST_PML_INDEX                         0x00000812 /* Page Modification Logging */
+#define VMCS_16BIT_GUEST_UINV                              0x00000814 /* UINTR */
 
 /* VMCS 16-bit host-state fields */
 /* binary 0000_11xx_xxxx_xxx0 */
@@ -224,63 +235,87 @@ const Bit64u VMX_VMFUNC_EPTP_SWITCHING_MASK = (BX_CONST64(1) << VMX_VMFUNC_EPTP_
 
 /* VMCS 64-bit control fields */
 /* binary 0010_00xx_xxxx_xxx0 */
-#define VMCS_64BIT_CONTROL_IO_BITMAP_A                     0x00002000
-#define VMCS_64BIT_CONTROL_IO_BITMAP_A_HI                  0x00002001
-#define VMCS_64BIT_CONTROL_IO_BITMAP_B                     0x00002002
-#define VMCS_64BIT_CONTROL_IO_BITMAP_B_HI                  0x00002003
-#define VMCS_64BIT_CONTROL_MSR_BITMAPS                     0x00002004
-#define VMCS_64BIT_CONTROL_MSR_BITMAPS_HI                  0x00002005
-#define VMCS_64BIT_CONTROL_VMEXIT_MSR_STORE_ADDR           0x00002006
-#define VMCS_64BIT_CONTROL_VMEXIT_MSR_STORE_ADDR_HI        0x00002007
-#define VMCS_64BIT_CONTROL_VMEXIT_MSR_LOAD_ADDR            0x00002008
-#define VMCS_64BIT_CONTROL_VMEXIT_MSR_LOAD_ADDR_HI         0x00002009
-#define VMCS_64BIT_CONTROL_VMENTRY_MSR_LOAD_ADDR           0x0000200A
-#define VMCS_64BIT_CONTROL_VMENTRY_MSR_LOAD_ADDR_HI        0x0000200B
-#define VMCS_64BIT_CONTROL_EXECUTIVE_VMCS_PTR              0x0000200C
-#define VMCS_64BIT_CONTROL_EXECUTIVE_VMCS_PTR_HI           0x0000200D
-#define VMCS_64BIT_CONTROL_PML_ADDRESS                     0x0000200E /* Page Modification Logging */
-#define VMCS_64BIT_CONTROL_PML_ADDRESS_HI                  0x0000200F
-#define VMCS_64BIT_CONTROL_TSC_OFFSET                      0x00002010
-#define VMCS_64BIT_CONTROL_TSC_OFFSET_HI                   0x00002011
-#define VMCS_64BIT_CONTROL_VIRTUAL_APIC_PAGE_ADDR          0x00002012 /* TPR shadow */
-#define VMCS_64BIT_CONTROL_VIRTUAL_APIC_PAGE_ADDR_HI       0x00002013
-#define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR                0x00002014 /* APIC virtualization */
-#define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR_HI             0x00002015
-#define VMCS_64BIT_CONTROL_POSTED_INTERRUPT_DESC_ADDR      0x00002016 /* Posted Interrupts - not implemented yet */
-#define VMCS_64BIT_CONTROL_POSTED_INTERRUPT_DESC_ADDR_HI   0x00002017
-#define VMCS_64BIT_CONTROL_VMFUNC_CTRLS                    0x00002018 /* VM Functions */
-#define VMCS_64BIT_CONTROL_VMFUNC_CTRLS_HI                 0x00002019
-#define VMCS_64BIT_CONTROL_EPTPTR                          0x0000201A /* EPT */
-#define VMCS_64BIT_CONTROL_EPTPTR_HI                       0x0000201B
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP0                0x0000201C /* Virtual Interrupt Delivery */
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP0_HI             0x0000201D
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP1                0x0000201E
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP1_HI             0x0000201F
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP2                0x00002020
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP2_HI             0x00002021
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP3                0x00002022
-#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP3_HI             0x00002023
-#define VMCS_64BIT_CONTROL_EPTP_LIST_ADDRESS               0x00002024 /* VM Functions - EPTP switching */
-#define VMCS_64BIT_CONTROL_EPTP_LIST_ADDRESS_HI            0x00002025
-#define VMCS_64BIT_CONTROL_VMREAD_BITMAP_ADDR              0x00002026 /* VMCS Shadowing */
-#define VMCS_64BIT_CONTROL_VMREAD_BITMAP_ADDR_HI           0x00002027
-#define VMCS_64BIT_CONTROL_VMWRITE_BITMAP_ADDR             0x00002028 /* VMCS Shadowing */
-#define VMCS_64BIT_CONTROL_VMWRITE_BITMAP_ADDR_HI          0x00002029
-#define VMCS_64BIT_CONTROL_VE_EXCEPTION_INFO_ADDR          0x0000202A /* #VE Exception */
-#define VMCS_64BIT_CONTROL_VE_EXCEPTION_INFO_ADDR_HI       0x0000202B
-#define VMCS_64BIT_CONTROL_XSS_EXITING_BITMAP              0x0000202C /* XSAVES */
-#define VMCS_64BIT_CONTROL_XSS_EXITING_BITMAP_HI           0x0000202D
-#define VMCS_64BIT_CONTROL_ENCLS_EXITING_BITMAP            0x0000202E /* ENCLS/SGX */
-#define VMCS_64BIT_CONTROL_ENCLS_EXITING_BITMAP_HI         0x0000202F
-#define VMCS_64BIT_CONTROL_SPPTP                           0x00002030 /* Sup-Page Write Protection */
-#define VMCS_64BIT_CONTROL_SPPTP_HI                        0x00002031
-#define VMCS_64BIT_CONTROL_TSC_MULTIPLIER                  0x00002032 /* TSC Scaling */
-#define VMCS_64BIT_CONTROL_TSC_MULTIPLIER_HI               0x00002033
+#define VMCS_64BIT_CONTROL_IO_BITMAP_A                        0x00002000
+#define VMCS_64BIT_CONTROL_IO_BITMAP_A_HI                     0x00002001
+#define VMCS_64BIT_CONTROL_IO_BITMAP_B                        0x00002002
+#define VMCS_64BIT_CONTROL_IO_BITMAP_B_HI                     0x00002003
+#define VMCS_64BIT_CONTROL_MSR_BITMAPS                        0x00002004
+#define VMCS_64BIT_CONTROL_MSR_BITMAPS_HI                     0x00002005
+#define VMCS_64BIT_CONTROL_VMEXIT_MSR_STORE_ADDR              0x00002006
+#define VMCS_64BIT_CONTROL_VMEXIT_MSR_STORE_ADDR_HI           0x00002007
+#define VMCS_64BIT_CONTROL_VMEXIT_MSR_LOAD_ADDR               0x00002008
+#define VMCS_64BIT_CONTROL_VMEXIT_MSR_LOAD_ADDR_HI            0x00002009
+#define VMCS_64BIT_CONTROL_VMENTRY_MSR_LOAD_ADDR              0x0000200A
+#define VMCS_64BIT_CONTROL_VMENTRY_MSR_LOAD_ADDR_HI           0x0000200B
+#define VMCS_64BIT_CONTROL_EXECUTIVE_VMCS_PTR                 0x0000200C
+#define VMCS_64BIT_CONTROL_EXECUTIVE_VMCS_PTR_HI              0x0000200D
+#define VMCS_64BIT_CONTROL_PML_ADDRESS                        0x0000200E /* Page Modification Logging */
+#define VMCS_64BIT_CONTROL_PML_ADDRESS_HI                     0x0000200F
+#define VMCS_64BIT_CONTROL_TSC_OFFSET                         0x00002010
+#define VMCS_64BIT_CONTROL_TSC_OFFSET_HI                      0x00002011
+#define VMCS_64BIT_CONTROL_VIRTUAL_APIC_PAGE_ADDR             0x00002012 /* TPR shadow */
+#define VMCS_64BIT_CONTROL_VIRTUAL_APIC_PAGE_ADDR_HI          0x00002013
+#define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR                   0x00002014 /* APIC virtualization */
+#define VMCS_64BIT_CONTROL_APIC_ACCESS_ADDR_HI                0x00002015
+#define VMCS_64BIT_CONTROL_POSTED_INTERRUPT_DESC_ADDR         0x00002016 /* Posted Interrupts */
+#define VMCS_64BIT_CONTROL_POSTED_INTERRUPT_DESC_ADDR_HI      0x00002017
+#define VMCS_64BIT_CONTROL_VMFUNC_CTRLS                       0x00002018 /* VM Functions */
+#define VMCS_64BIT_CONTROL_VMFUNC_CTRLS_HI                    0x00002019
+#define VMCS_64BIT_CONTROL_EPTPTR                             0x0000201A /* EPT */
+#define VMCS_64BIT_CONTROL_EPTPTR_HI                          0x0000201B
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP0                   0x0000201C /* Virtual Interrupt Delivery */
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP0_HI                0x0000201D
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP1                   0x0000201E
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP1_HI                0x0000201F
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP2                   0x00002020
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP2_HI                0x00002021
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP3                   0x00002022
+#define VMCS_64BIT_CONTROL_EOI_EXIT_BITMAP3_HI                0x00002023
+#define VMCS_64BIT_CONTROL_EPTP_LIST_ADDRESS                  0x00002024 /* VM Functions - EPTP switching */
+#define VMCS_64BIT_CONTROL_EPTP_LIST_ADDRESS_HI               0x00002025
+#define VMCS_64BIT_CONTROL_VMREAD_BITMAP_ADDR                 0x00002026 /* VMCS Shadowing */
+#define VMCS_64BIT_CONTROL_VMREAD_BITMAP_ADDR_HI              0x00002027
+#define VMCS_64BIT_CONTROL_VMWRITE_BITMAP_ADDR                0x00002028 /* VMCS Shadowing */
+#define VMCS_64BIT_CONTROL_VMWRITE_BITMAP_ADDR_HI             0x00002029
+#define VMCS_64BIT_CONTROL_VE_EXCEPTION_INFO_ADDR             0x0000202A /* #VE Exception */
+#define VMCS_64BIT_CONTROL_VE_EXCEPTION_INFO_ADDR_HI          0x0000202B
+#define VMCS_64BIT_CONTROL_XSS_EXITING_BITMAP                 0x0000202C /* XSAVES */
+#define VMCS_64BIT_CONTROL_XSS_EXITING_BITMAP_HI              0x0000202D
+#define VMCS_64BIT_CONTROL_ENCLS_EXITING_BITMAP               0x0000202E /* ENCLS/SGX (not implemented) */
+#define VMCS_64BIT_CONTROL_ENCLS_EXITING_BITMAP_HI            0x0000202F
+#define VMCS_64BIT_CONTROL_SPPTP                              0x00002030 /* Sup-Page Write Protection */
+#define VMCS_64BIT_CONTROL_SPPTP_HI                           0x00002031
+#define VMCS_64BIT_CONTROL_TSC_MULTIPLIER                     0x00002032 /* TSC Scaling */
+#define VMCS_64BIT_CONTROL_TSC_MULTIPLIER_HI                  0x00002033
+#define VMCS_64BIT_CONTROL_TERTIARY_VMEXEC_CONTROLS           0x00002034
+#define VMCS_64BIT_CONTROL_TERTIARY_VMEXEC_CONTROLS_HI        0x00002035
+#define VMCS_64BIT_CONTROL_ENCLV_EXITING_BITMAP               0x00002036 /* ENCLV/SGX (not implemented) */
+#define VMCS_64BIT_CONTROL_ENCLV_EXITING_BITMAP_HI            0x00002037
+#define VMCS_64BIT_CONTROL_LO_PASID_DIRECTORY_ADDR            0x00002038
+#define VMCS_64BIT_CONTROL_LO_PASID_DIRECTORY_ADDR_HI         0x00002039
+#define VMCS_64BIT_CONTROL_HI_PASID_DIRECTORY_ADDR            0x0000203A
+#define VMCS_64BIT_CONTROL_HI_PASID_DIRECTORY_ADDR_HI         0x0000203B
+#define VMCS_64BIT_CONTROL_SHARED_EPT_POINTER                 0x0000203C
+#define VMCS_64BIT_CONTROL_SHARED_EPT_POINTER_HI              0x0000203D
+#define VMCS_64BIT_CONTROL_PCONFIG_EXITING_BITMAP             0x0000203E
+#define VMCS_64BIT_CONTROL_PCONFIG_EXITING_BITMAP_HI          0x0000203F
+#define VMCS_64BIT_CONTROL_HLAT_POINTER                       0x00002040 /* HLAT (not implemented) */
+#define VMCS_64BIT_CONTROL_HLAT_POINTER_HI                    0x00002041
+#define VMCS_64BIT_CONTROL_PID_POINTER_TABLE_ADDRESS          0x00002042 /* IPI Virtualization (not implemented) */
+#define VMCS_64BIT_CONTROL_PID_POINTER_TABLE_ADDRESS_HI       0x00002043
+#define VMCS_64BIT_CONTROL_SECONDARY_VMEXIT_CONTROLS          0x00002044
+#define VMCS_64BIT_CONTROL_SECONDARY_VMEXIT_CONTROLS_HI       0x00002045
+#define VMCS_64BIT_CONTROL_IA32_SPEC_CTRL_MASK                0x0000204A
+#define VMCS_64BIT_CONTROL_IA32_SPEC_CTRL_MASK_HI             0x0000204B
+#define VMCS_64BIT_CONTROL_IA32_SPEC_CTRL_SHADOW              0x0000204C
+#define VMCS_64BIT_CONTROL_IA32_SPEC_CTRL_SHADOW_HI           0x0000204D
 
 /* VMCS 64-bit read only data fields */
 /* binary 0010_01xx_xxxx_xxx0 */
 #define VMCS_64BIT_GUEST_PHYSICAL_ADDR                     0x00002400 /* EPT */
 #define VMCS_64BIT_GUEST_PHYSICAL_ADDR_HI                  0x00002401
+#define VMCS_64BIT_MSR_DATA                                0x00002402 /* MSRLIST */
+#define VMCS_64BIT_MSR_DATA_HI                             0x00002403
 
 /* VMCS 64-bit guest state fields */
 /* binary 0010_10xx_xxxx_xxx0 */
@@ -392,7 +427,7 @@ const Bit64u VMX_VMFUNC_EPTP_SWITCHING_MASK = (BX_CONST64(1) << VMX_VMFUNC_EPTP_
 #define VMCS_CR3_TARGET1                                   0x0000600A
 #define VMCS_CR3_TARGET2                                   0x0000600C
 #define VMCS_CR3_TARGET3                                   0x0000600E
-                                                           
+
 /* VMCS natural width read only data fields */
 /* binary 0110_01xx_xxxx_xxx0 */
 #define VMCS_VMEXIT_QUALIFICATION                          0x00006400
@@ -446,8 +481,6 @@ const Bit64u VMX_VMFUNC_EPTP_SWITCHING_MASK = (BX_CONST64(1) << VMX_VMFUNC_EPTP_
 #define VMCS_HOST_SSP                                      0x00006C1A
 #define VMCS_HOST_INTERRUPT_SSP_TABLE_ADDR                 0x00006C1C
 
-#define VMX_HIGHEST_VMCS_ENCODING   (0x34)
-
 // ===============================
 //  VMCS fields encoding/decoding
 // ===============================
@@ -481,6 +514,13 @@ enum {
 #define VMCS_FIELD_INDEX(encoding) \
     ((VMCS_FIELD_WIDTH(encoding) << 2) + VMCS_FIELD_TYPE(encoding))
 
+#define VMX_HIGHEST_16BIT_VMCS_ENCODING          (0x20)
+#define VMX_HIGHEST_64BIT_VMCS_ENCODING          (0x50)
+#define VMX_HIGHEST_32BIT_VMCS_ENCODING          (0x30)
+#define VMX_HIGHEST_NATURAL_WIDTH_VMCS_ENCODING  (0x30)
+
+#define VMX_HIGHEST_VMCS_ENCODING VMX_HIGHEST_64BIT_VMCS_ENCODING
+
 const Bit32u VMCS_ENCODING_RESERVED_BITS = 0xffff9000;
 
 // =============
@@ -498,6 +538,16 @@ enum VMCS_Access_Rights_Format {
 #define VMCS_VMX_ABORT_FIELD_ENCODING            (0xfffffffc)
 #define VMCS_REVISION_ID_FIELD_ENCODING          (0xfffffffa)
 
+#define VMCS_DATA_OFFSET                         (0x10)           /* 16-bytes allocated for above fields */
+
+// 16-bit encodings: 4 types, 32 encodings (0x20), 4 bytes each field =>  512 bytes
+// 64-bit encodings: 4 types, 80 encodings (0x50), 4 bytes each field => 1280 bytes
+// 32-bit encodings: 4 types, 48 encodings (0x30), 4 bytes each field =>  768 bytes
+// Natural-width encodings: 4 types, 48 encodings (0x30), 4 bytes each field => 768 bytes
+#if (VMCS_DATA_OFFSET + 4*4*(VMX_HIGHEST_16BIT_VMCS_ENCODING + VMX_HIGHEST_32BIT_VMCS_ENCODING + VMX_HIGHEST_64BIT_VMCS_ENCODING + VMX_HIGHEST_NATURAL_WIDTH_VMCS_ENCODING) >= VMX_VMCS_AREA_SIZE)
+  #error "VMCS area size exceeded !"
+#endif
+
 class VMCS_Mapping {
 private:
    Bit32u revision_id;
@@ -531,7 +581,7 @@ public:
    void set_vmcs_recision_id_offset(unsigned offset) { vmcs_revision_id_field_offset = offset; }
    void set_vmx_abort_field_offset(unsigned offset) { vmx_abort_field_offset = offset; }
    void set_vmcs_launch_state_field_offset(unsigned offset) { vmcs_launch_state_field_offset = offset; }
-   
+
    unsigned vmcs_field_offset(Bit32u encoding) const;
 
    bool is_reserved(Bit32u encoding) const {
@@ -543,12 +593,6 @@ public:
    }
 };
 
-#define VMCS_DATA_OFFSET                         (0x0010)
-
-#if ((VMCS_DATA_OFFSET + 4*(64*15 + VMX_HIGHEST_VMCS_ENCODING)) > VMX_VMCS_AREA_SIZE)
-  #error "VMCS area size exceeded !"
-#endif
-
 // =============
 //  VMCS state
 // =============
@@ -562,7 +606,7 @@ enum VMX_state {
 //  VMCS structure
 // ================
 
-typedef struct bx_VMCS_GUEST_STATE 
+typedef struct bx_VMCS_GUEST_STATE
 {
    bx_address cr0;
    bx_address cr3;
@@ -604,6 +648,10 @@ typedef struct bx_VMCS_GUEST_STATE
    bx_address interrupt_ssp_table_address;
 #endif
 
+#if BX_SUPPORT_UINTR
+   Bit16u uintr_uinv;
+#endif
+
 #if BX_SUPPORT_PKEYS
    Bit32u pkrs;
 #endif
@@ -660,7 +708,9 @@ typedef struct bx_VMX_Cap
   Bit32u vmx_pin_vmexec_ctrl_supported_bits;
   Bit32u vmx_proc_vmexec_ctrl_supported_bits;
   Bit32u vmx_vmexec_ctrl2_supported_bits;
-  Bit32u vmx_vmexit_ctrl_supported_bits;
+  Bit64u vmx_vmexec_ctrl3_supported_bits;
+  Bit32u vmx_vmexit_ctrl1_supported_bits;
+  Bit32u vmx_vmexit_ctrl2_supported_bits;
   Bit32u vmx_vmentry_ctrl_supported_bits;
 #if BX_SUPPORT_VMX >= 2
   Bit64u vmx_ept_vpid_cap_supported_bits;
@@ -680,79 +730,33 @@ struct VMX_PLE {
 
 #endif
 
+#include "vmx_ctrls.h"
+
 typedef struct bx_VMCS
 {
   //
   // VM-Execution Control Fields
   //
 
-#define VMX_VM_EXEC_CTRL1_EXTERNAL_INTERRUPT_VMEXIT   (1 << 0)
-#define VMX_VM_EXEC_CTRL1_NMI_EXITING                 (1 << 3)
-#define VMX_VM_EXEC_CTRL1_VIRTUAL_NMI                 (1 << 5) /* Virtual NMI */
-#define VMX_VM_EXEC_CTRL1_VMX_PREEMPTION_TIMER_VMEXIT (1 << 6) /* VMX preemption timer */
-#define VMX_VM_EXEC_CTRL1_PROCESS_POSTED_INTERRUPTS   (1 << 7) /* Posted Interrupts (not implemented) */
-
-#define VMX_VM_EXEC_CTRL1_SUPPORTED_BITS \
+#define VMX_PIN_BASED_VMEXEC_CTRL_SUPPORTED_BITS \
     (BX_CPU_THIS_PTR vmx_cap.vmx_pin_vmexec_ctrl_supported_bits)
 
-   Bit32u vmexec_ctrls1;
+   VmxPinBasedVmexecControls pin_vmexec_ctrls;
 
-#define VMX_VM_EXEC_CTRL2_INTERRUPT_WINDOW_VMEXIT   (1 << 2)
-#define VMX_VM_EXEC_CTRL2_TSC_OFFSET                (1 << 3)
-#define VMX_VM_EXEC_CTRL2_HLT_VMEXIT                (1 << 7)
-#define VMX_VM_EXEC_CTRL2_INVLPG_VMEXIT             (1 << 9)
-#define VMX_VM_EXEC_CTRL2_MWAIT_VMEXIT              (1 << 10)
-#define VMX_VM_EXEC_CTRL2_RDPMC_VMEXIT              (1 << 11)
-#define VMX_VM_EXEC_CTRL2_RDTSC_VMEXIT              (1 << 12)
-#define VMX_VM_EXEC_CTRL2_CR3_WRITE_VMEXIT          (1 << 15) /* legacy must be '1 */
-#define VMX_VM_EXEC_CTRL2_CR3_READ_VMEXIT           (1 << 16) /* legacy must be '1 */
-#define VMX_VM_EXEC_CTRL2_CR8_WRITE_VMEXIT          (1 << 19) /* TPR shadow */
-#define VMX_VM_EXEC_CTRL2_CR8_READ_VMEXIT           (1 << 20) /* TPR shadow */
-#define VMX_VM_EXEC_CTRL2_TPR_SHADOW                (1 << 21) /* TPR shadow */
-#define VMX_VM_EXEC_CTRL2_NMI_WINDOW_EXITING        (1 << 22) /* Virtual NMI */
-#define VMX_VM_EXEC_CTRL2_DRx_ACCESS_VMEXIT         (1 << 23)
-#define VMX_VM_EXEC_CTRL2_IO_VMEXIT                 (1 << 24)
-#define VMX_VM_EXEC_CTRL2_IO_BITMAPS                (1 << 25)
-#define VMX_VM_EXEC_CTRL2_MONITOR_TRAP_FLAG         (1 << 27) /* Monitor Trap Flag */
-#define VMX_VM_EXEC_CTRL2_MSR_BITMAPS               (1 << 28)
-#define VMX_VM_EXEC_CTRL2_MONITOR_VMEXIT            (1 << 29)
-#define VMX_VM_EXEC_CTRL2_PAUSE_VMEXIT              (1 << 30)
-#define VMX_VM_EXEC_CTRL2_SECONDARY_CONTROLS        (1 << 31)
-
-#define VMX_VM_EXEC_CTRL2_SUPPORTED_BITS \
+#define VMX_VM_EXEC_CTRL1_SUPPORTED_BITS \
     (BX_CPU_THIS_PTR vmx_cap.vmx_proc_vmexec_ctrl_supported_bits)
 
-   Bit32u vmexec_ctrls2;
+   VmxVmexec1Controls vmexec_ctrls1;
 
-#define VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_ACCESSES  (1 <<  0) /* APIC virtualization */
-#define VMX_VM_EXEC_CTRL3_EPT_ENABLE                (1 <<  1) /* EPT */
-#define VMX_VM_EXEC_CTRL3_DESCRIPTOR_TABLE_VMEXIT   (1 <<  2) /* Descriptor Table VMEXIT */
-#define VMX_VM_EXEC_CTRL3_RDTSCP                    (1 <<  3)
-#define VMX_VM_EXEC_CTRL3_VIRTUALIZE_X2APIC_MODE    (1 <<  4) /* Virtualize X2APIC */
-#define VMX_VM_EXEC_CTRL3_VPID_ENABLE               (1 <<  5) /* VPID */
-#define VMX_VM_EXEC_CTRL3_WBINVD_VMEXIT             (1 <<  6) /* WBINVD VMEXIT */
-#define VMX_VM_EXEC_CTRL3_UNRESTRICTED_GUEST        (1 <<  7) /* Unrestricted Guest */
-#define VMX_VM_EXEC_CTRL3_VIRTUALIZE_APIC_REGISTERS (1 <<  8)
-#define VMX_VM_EXEC_CTRL3_VIRTUAL_INT_DELIVERY      (1 <<  9)
-#define VMX_VM_EXEC_CTRL3_PAUSE_LOOP_VMEXIT         (1 << 10) /* PAUSE loop exiting */
-#define VMX_VM_EXEC_CTRL3_RDRAND_VMEXIT             (1 << 11)
-#define VMX_VM_EXEC_CTRL3_INVPCID                   (1 << 12)
-#define VMX_VM_EXEC_CTRL3_VMFUNC_ENABLE             (1 << 13) /* VM Functions */
-#define VMX_VM_EXEC_CTRL3_VMCS_SHADOWING            (1 << 14) /* VMCS Shadowing */
-#define VMX_VM_EXEC_CTRL3_SGX_ENCLS_VMEXIT          (1 << 15) /* ENCLS/SGX */
-#define VMX_VM_EXEC_CTRL3_RDSEED_VMEXIT             (1 << 16)
-#define VMX_VM_EXEC_CTRL3_PML_ENABLE                (1 << 17) /* Page Modification Logging */
-#define VMX_VM_EXEC_CTRL3_EPT_VIOLATION_EXCEPTION   (1 << 18) /* #VE Exception */
-#define VMX_VM_EXEC_CTRL3_SUPPRESS_GUEST_VMX_TRACE  (1 << 19) /* Processor Trace (not implemented) */
-#define VMX_VM_EXEC_CTRL3_XSAVES_XRSTORS            (1 << 20) /* XSAVES */
-#define VMX_VM_EXEC_CTRL3_MBE_CTRL                  (1 << 22) /* Mode Based Execution Control (not implemented yet) */
-#define VMX_VM_EXEC_CTRL3_SUBPAGE_WR_PROTECT_CTRL   (1 << 23) /* Sub-Page Write Protection Control */
-#define VMX_VM_EXEC_CTRL3_TSC_SCALING               (1 << 25) /* TSC Scaling */
-
-#define VMX_VM_EXEC_CTRL3_SUPPORTED_BITS \
+#define VMX_VM_EXEC_CTRL2_SUPPORTED_BITS \
     (BX_CPU_THIS_PTR vmx_cap.vmx_vmexec_ctrl2_supported_bits)
 
-   Bit32u vmexec_ctrls3;
+   VmxVmexec2Controls vmexec_ctrls2;
+
+#define VMX_VM_EXEC_CTRL3_SUPPORTED_BITS \
+    (BX_CPU_THIS_PTR vmx_cap.vmx_vmexec_ctrl3_supported_bits)
+
+   VmxVmexec3Controls vmexec_ctrls3;
 
    Bit64u vmcs_linkptr;
 
@@ -763,6 +767,10 @@ typedef struct bx_VMCS
    Bit32u vm_pf_match;
    Bit64u io_bitmap_addr[2];
    bx_phy_address msr_bitmap_addr;
+   Bit64u msr_data;
+
+   Bit64u ia32_spec_ctrl_shadow;
+   Bit64u ia32_spec_ctrl_mask;
 
    bx_address vm_cr0_mask;
    bx_address vm_cr0_read_shadow;
@@ -770,7 +778,6 @@ typedef struct bx_VMCS
    bx_address vm_cr4_read_shadow;
 
 #define VMX_CR3_TARGET_MAX_CNT 4
-
    Bit32u vm_cr3_target_cnt;
    bx_address vm_cr3_target_value[VMX_CR3_TARGET_MAX_CNT];
 
@@ -781,6 +788,9 @@ typedef struct bx_VMCS
    unsigned apic_access;
 #endif
 
+   unsigned posted_intr_notification_vector;
+   bx_phy_address pid_addr; /* posted interrupt descriptor address */
+
 #if BX_SUPPORT_VMX >= 2
    Bit64u eptptr;
    Bit16u vpid;
@@ -814,28 +824,23 @@ typedef struct bx_VMCS
    Bit64u xss_exiting_bitmap;
 #endif
 
+#if BX_SUPPORT_CET
+   bool shadow_stack_prematurely_busy;
+#endif
+
    //
    // VM-Exit Control Fields
    //
 
-#define VMX_VMEXIT_CTRL1_SAVE_DBG_CTRLS             (1 <<  2) /* legacy must be '1 */
-#define VMX_VMEXIT_CTRL1_HOST_ADDR_SPACE_SIZE       (1 <<  9)
-#define VMX_VMEXIT_CTRL1_LOAD_PERF_GLOBAL_CTRL_MSR  (1 << 12) /* Perf Global Control */
-#define VMX_VMEXIT_CTRL1_INTA_ON_VMEXIT             (1 << 15)
-#define VMX_VMEXIT_CTRL1_STORE_PAT_MSR              (1 << 18) /* PAT */
-#define VMX_VMEXIT_CTRL1_LOAD_PAT_MSR               (1 << 19) /* PAT */
-#define VMX_VMEXIT_CTRL1_STORE_EFER_MSR             (1 << 20) /* EFER */
-#define VMX_VMEXIT_CTRL1_LOAD_EFER_MSR              (1 << 21) /* EFER */
-#define VMX_VMEXIT_CTRL1_STORE_VMX_PREEMPTION_TIMER (1 << 22) /* VMX preemption timer */
-#define VMX_VMEXIT_CTRL1_CLEAR_BNDCFGS              (1 << 23) /* MPX (not implemented) */
-#define VMX_VMEXIT_CTRL1_SUPPRESS_HOST_VMX_TRACE    (1 << 24) /* Processor Trace (not implemented) */
-#define VMX_VMEXIT_CTRL1_LOAD_HOST_CET_STATE        (1 << 28) /* CET */
-#define VMX_VMEXIT_CTRL1_LOAD_HOST_PKRS             (1 << 29) /* Supervisor-Mode Protection Keys */
-
 #define VMX_VMEXIT_CTRL1_SUPPORTED_BITS \
-    (BX_CPU_THIS_PTR vmx_cap.vmx_vmexit_ctrl_supported_bits)
+    (BX_CPU_THIS_PTR vmx_cap.vmx_vmexit_ctrl1_supported_bits)
 
-   Bit32u vmexit_ctrls;
+   BxVmexit1Controls vmexit_ctrls1;
+
+#define VMX_VMEXIT_CTRL2_SUPPORTED_BITS \
+    (BX_CPU_THIS_PTR vmx_cap.vmx_vmexit_ctrl2_supported_bits)
+
+   BxVmexit2Controls vmexit_ctrls2;
 
    Bit32u vmexit_msr_store_cnt;
    bx_phy_address vmexit_msr_store_addr;
@@ -846,22 +851,10 @@ typedef struct bx_VMCS
    // VM-Entry Control Fields
    //
 
-#define VMX_VMENTRY_CTRL1_LOAD_DBG_CTRLS                    (1 <<  2) /* legacy must be '1 */
-#define VMX_VMENTRY_CTRL1_X86_64_GUEST                      (1 <<  9)
-#define VMX_VMENTRY_CTRL1_SMM_ENTER                         (1 << 10)
-#define VMX_VMENTRY_CTRL1_DEACTIVATE_DUAL_MONITOR_TREATMENT (1 << 11)
-#define VMX_VMENTRY_CTRL1_LOAD_PERF_GLOBAL_CTRL_MSR         (1 << 13) /* Perf Global Ctrl */
-#define VMX_VMENTRY_CTRL1_LOAD_PAT_MSR                      (1 << 14) /* PAT */
-#define VMX_VMENTRY_CTRL1_LOAD_EFER_MSR                     (1 << 15) /* EFER */
-#define VMX_VMENTRY_CTRL1_LOAD_BNDCFGS                      (1 << 16) /* MPX (not implemented) */
-#define VMX_VMENTRY_CTRL1_SUPPRESS_VMX_PACKETS              (1 << 17) /* Processor Trace (not implemented) */
-#define VMX_VMENTRY_CTRL1_LOAD_GUEST_CET_STATE              (1 << 20) /* CET */
-#define VMX_VMENTRY_CTRL1_LOAD_GUEST_PKRS                   (1 << 22) /* Supervisor-Mode Protection Keys */
-
 #define VMX_VMENTRY_CTRL1_SUPPORTED_BITS \
     (BX_CPU_THIS_PTR vmx_cap.vmx_vmentry_ctrl_supported_bits)
-   
-   Bit32u vmentry_ctrls;
+
+   VmxVmentryControls vmentry_ctrls;
 
    Bit32u vmentry_msr_load_cnt;
    bx_phy_address vmentry_msr_load_addr;
@@ -899,11 +892,6 @@ typedef struct bx_VMCS
 
 } VMCS_CACHE;
 
-#define PIN_VMEXIT(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls1 & (ctrl))
-#define     VMEXIT(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls2 & (ctrl))
-
-#define SECONDARY_VMEXEC_CONTROL(ctrl) (BX_CPU_THIS_PTR vmcs.vmexec_ctrls3 & (ctrl))
-
 const Bit32u BX_VMX_INTERRUPTS_BLOCKED_BY_STI      = (1 << 0);
 const Bit32u BX_VMX_INTERRUPTS_BLOCKED_BY_MOV_SS   = (1 << 1);
 const Bit32u BX_VMX_INTERRUPTS_BLOCKED_SMI_BLOCKED = (1 << 2);
@@ -926,7 +914,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 // 48:48 use 32-bit physical address, set when x86_64 disabled
 // 49:49 support of dual-monitor treatment of SMI and SMM
 // 53:50 memory type used for VMCS access
-// 54:54 logical processor reports information in the VM-exit 
+// 54:54 logical processor reports information in the VM-exit
 //       instruction-information field on VM exits due to
 //       execution of INS/OUTS
 // 55:55 set if any VMX controls that default to `1 may be
@@ -945,7 +933,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
      (is_cpu_extension_supported(BX_ISA_CET) ? (1<<24) : 0)
 
 #define VMX_MSR_VMX_BASIC \
-   ((((Bit64u) VMX_MSR_VMX_BASIC_HI) << 32) | VMX_MSR_VMX_BASIC_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_BASIC_HI, VMX_MSR_VMX_BASIC_LO)
 
 
 // ------------------------------------------------------------------------
@@ -965,18 +953,18 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 // IA32_MSR_VMX_PINBASED_CTRLS MSR (0x481)
 // ---------------------------
 
-// Bits 1, 2 and 4 must be '1 
+// Bits 1, 2 and 4 must be '1
 
 // Allowed 0-settings: VMentry fail if a bit is '0 in pin-based vmexec controls
 // but set to '1 in this MSR
-#define VMX_MSR_VMX_PINBASED_CTRLS_LO (0x00000016)
+const Bit32u VMX_MSR_VMX_PINBASED_CTRLS_LO = 0x00000016;
 // Allowed 1-settings: VMentry fail if a bit is '1 in pin-based vmexec controls
 // but set to '0 in this MSR.
 #define VMX_MSR_VMX_PINBASED_CTRLS_HI \
-       (VMX_VM_EXEC_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_PINBASED_CTRLS_LO)
+       (VMX_PIN_BASED_VMEXEC_CTRL_SUPPORTED_BITS | VMX_MSR_VMX_PINBASED_CTRLS_LO)
 
 #define VMX_MSR_VMX_PINBASED_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_PINBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_PINBASED_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_PINBASED_CTRLS_HI, VMX_MSR_VMX_PINBASED_CTRLS_LO)
 
 // IA32_MSR_VMX_TRUE_PINBASED_CTRLS MSR (0x48d)
 // --------------------------------
@@ -986,7 +974,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 #define VMX_MSR_VMX_TRUE_PINBASED_CTRLS_HI (VMX_MSR_VMX_PINBASED_CTRLS_HI)
 
 #define VMX_MSR_VMX_TRUE_PINBASED_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_TRUE_PINBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_PINBASED_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_TRUE_PINBASED_CTRLS_HI, VMX_MSR_VMX_TRUE_PINBASED_CTRLS_LO)
 
 
 // IA32_MSR_VMX_PROCBASED_CTRLS MSR (0x482)
@@ -997,13 +985,13 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 // Bits 19-21 also must be '0 when x86-64 is not supported
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_PROCBASED_CTRLS_LO (0x0401E172)
+const Bit32u VMX_MSR_VMX_PROCBASED_CTRLS_LO = 0x0401E172;
 // Allowed 1-settings
 #define VMX_MSR_VMX_PROCBASED_CTRLS_HI \
-       (VMX_VM_EXEC_CTRL2_SUPPORTED_BITS | VMX_MSR_VMX_PROCBASED_CTRLS_LO)
+       (VMX_VM_EXEC_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_PROCBASED_CTRLS_LO)
 
 #define VMX_MSR_VMX_PROCBASED_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_PROCBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_PROCBASED_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_PROCBASED_CTRLS_HI, VMX_MSR_VMX_PROCBASED_CTRLS_LO)
 
 // IA32_MSR_VMX_TRUE_PROCBASED_CTRLS MSR (0x48e)
 // ---------------------------------
@@ -1013,7 +1001,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 #define VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_HI (VMX_MSR_VMX_PROCBASED_CTRLS_HI)
 
 #define VMX_MSR_VMX_TRUE_PROCBASED_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_HI, VMX_MSR_VMX_TRUE_PROCBASED_CTRLS_LO)
 
 
 // IA32_MSR_VMX_VMEXIT_CTRLS MSR (0x483)
@@ -1022,13 +1010,13 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 // Bits 0-8, 10, 11, 13, 14, 16, 17 must be '1
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_VMEXIT_CTRLS_LO (0x00036DFF)
+const Bit32u VMX_MSR_VMX_VMEXIT_CTRLS_LO = 0x00036DFF;
 // Allowed 1-settings
 #define VMX_MSR_VMX_VMEXIT_CTRLS_HI \
        (VMX_VMEXIT_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_VMEXIT_CTRLS_LO)
 
 #define VMX_MSR_VMX_VMEXIT_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_VMEXIT_CTRLS_HI) << 32) | VMX_MSR_VMX_VMEXIT_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_VMEXIT_CTRLS_HI, VMX_MSR_VMX_VMEXIT_CTRLS_LO)
 
 // IA32_MSR_VMX_TRUE_VMEXIT_CTRLS MSR (0x48f)
 // ------------------------------
@@ -1038,7 +1026,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 #define VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_HI (VMX_MSR_VMX_VMEXIT_CTRLS_HI)
 
 #define VMX_MSR_VMX_TRUE_VMEXIT_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_HI, VMX_MSR_VMX_TRUE_VMEXIT_CTRLS_LO)
 
 
 // IA32_MSR_VMX_VMENTRY_CTRLS MSR (0x484)
@@ -1047,13 +1035,13 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 // Bits 0-8, 12 must be '1
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_VMENTRY_CTRLS_LO (0x000011FF)
+const Bit32u VMX_MSR_VMX_VMENTRY_CTRLS_LO = 0x000011FF;
 // Allowed 1-settings
 #define VMX_MSR_VMX_VMENTRY_CTRLS_HI \
        (VMX_VMENTRY_CTRL1_SUPPORTED_BITS | VMX_MSR_VMX_VMENTRY_CTRLS_LO)
 
 #define VMX_MSR_VMX_VMENTRY_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_VMENTRY_CTRLS_HI) << 32) | VMX_MSR_VMX_VMENTRY_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_VMENTRY_CTRLS_HI, VMX_MSR_VMX_VMENTRY_CTRLS_LO)
 
 // IA32_MSR_VMX_TRUE_VMENTRY_CTRLS MSR (0x490)
 // -------------------------------
@@ -1063,7 +1051,7 @@ const Bit32u BX_VMCS_SHADOW_BIT_MASK = 0x80000000;
 #define VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_HI (VMX_MSR_VMX_VMENTRY_CTRLS_HI)
 
 #define VMX_MSR_VMX_TRUE_VMENTRY_CTRLS \
-   ((((Bit64u) VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_HI) << 32) | VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_HI, VMX_MSR_VMX_TRUE_VMENTRY_CTRLS_LO)
 
 
 // IA32_MSR_VMX_MISC MSR (0x485)
@@ -1101,12 +1089,13 @@ const Bit32u VMX_MISC_ALLOW_INJECTION_OF_SW_INTERRUPT_WITH_ILEN_0 = (1<<30);
 //Rate to increase VMX preemtion timer
 const Bit32u VMX_MISC_PREEMPTION_TIMER_RATE = 0;
 
-#define VMX_MSR_MISC (VMX_MISC_PREEMPTION_TIMER_RATE | \
-                      VMX_MISC_STORE_LMA_TO_X86_64_GUEST_VMENTRY_CONTROL | \
-                      VMX_SUPPORT_VMENTER_TO_NON_ACTIVE_STATE | \
-                     (VMX_CR3_TARGET_MAX_CNT << 16) | \
-                     (BX_SUPPORT_VMX_EXTENSION(BX_VMX_VMCS_SHADOWING) ? VMX_MISC_SUPPORT_VMWRITE_READ_ONLY_FIELDS : 0) | \
-                     (BX_SUPPORT_VMX_EXTENSION(BX_VMX_SW_INTERRUPT_INJECTION_ILEN_0) ? VMX_MISC_ALLOW_INJECTION_OF_SW_INTERRUPT_WITH_ILEN_0 : 0))
+#define VMX_MSR_MISC \
+     (VMX_MISC_PREEMPTION_TIMER_RATE |                     \
+      VMX_MISC_STORE_LMA_TO_X86_64_GUEST_VMENTRY_CONTROL | \
+      VMX_SUPPORT_VMENTER_TO_NON_ACTIVE_STATE |            \
+     (VMX_CR3_TARGET_MAX_CNT << 16) |                      \
+     (BX_SUPPORT_VMX_EXTENSION(BX_VMX_VMCS_SHADOWING) ? VMX_MISC_SUPPORT_VMWRITE_READ_ONLY_FIELDS : 0) | \
+     (BX_SUPPORT_VMX_EXTENSION(BX_VMX_SW_INTERRUPT_INJECTION_ILEN_0) ? VMX_MISC_ALLOW_INJECTION_OF_SW_INTERRUPT_WITH_ILEN_0 : 0))
 
 //
 // IA32_VMX_CR0_FIXED0 MSR (0x486)   IA32_VMX_CR0_FIXED1 MSR (0x487)
@@ -1117,15 +1106,15 @@ const Bit32u VMX_MISC_PREEMPTION_TIMER_RATE = 0;
 const Bit32u VMX_MSR_CR0_FIXED0_LO = 0x80000021;
 const Bit32u VMX_MSR_CR0_FIXED0_HI = 0x00000000;
 
-const Bit64u VMX_MSR_CR0_FIXED0 = 
-   ((((Bit64u) VMX_MSR_CR0_FIXED0_HI) << 32) | VMX_MSR_CR0_FIXED0_LO);
+const Bit64u VMX_MSR_CR0_FIXED0 =
+   GET64_FROM_HI32_LO32(VMX_MSR_CR0_FIXED0_HI, VMX_MSR_CR0_FIXED0_LO);
 
 // allowed 1-setting in CR0 in VMX mode
 const Bit32u VMX_MSR_CR0_FIXED1_LO = 0xFFFFFFFF;
 const Bit32u VMX_MSR_CR0_FIXED1_HI = 0x00000000;
 
-const Bit64u VMX_MSR_CR0_FIXED1 = 
-   ((((Bit64u) VMX_MSR_CR0_FIXED1_HI) << 32) | VMX_MSR_CR0_FIXED1_LO);
+const Bit64u VMX_MSR_CR0_FIXED1 =
+   GET64_FROM_HI32_LO32(VMX_MSR_CR0_FIXED1_HI, VMX_MSR_CR0_FIXED1_LO);
 
 //
 // IA32_VMX_CR4_FIXED0 MSR (0x488)   IA32_VMX_CR4_FIXED1 MSR (0x489)
@@ -1136,15 +1125,15 @@ const Bit64u VMX_MSR_CR0_FIXED1 =
 const Bit32u VMX_MSR_CR4_FIXED0_LO = 0x00002000;
 const Bit32u VMX_MSR_CR4_FIXED0_HI = 0x00000000;
 
-const Bit64u VMX_MSR_CR4_FIXED0 = 
-   ((((Bit64u) VMX_MSR_CR4_FIXED0_HI) << 32) | VMX_MSR_CR4_FIXED0_LO);
+const Bit64u VMX_MSR_CR4_FIXED0 =
+   GET64_FROM_HI32_LO32(VMX_MSR_CR4_FIXED0_HI, VMX_MSR_CR4_FIXED0_LO);
 
 // allowed 1-setting in CR0 in VMX mode
 #define VMX_MSR_CR4_FIXED1_LO (BX_CPU_THIS_PTR cr4_suppmask)
 #define VMX_MSR_CR4_FIXED1_HI (0)
 
 #define VMX_MSR_CR4_FIXED1 \
-   ((((Bit64u) VMX_MSR_CR4_FIXED1_HI) << 32) | VMX_MSR_CR4_FIXED1_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_CR4_FIXED1_HI, VMX_MSR_CR4_FIXED1_LO)
 
 
 //
@@ -1156,24 +1145,39 @@ const Bit64u VMX_MSR_CR4_FIXED0 =
 // 63:10 reserved, must be zero
 //
 
-#define VMX_MSR_VMCS_ENUM_LO (VMX_HIGHEST_VMCS_ENCODING)
-#define VMX_MSR_VMCS_ENUM_HI (0x00000000)
+const Bit32u VMX_MSR_VMCS_ENUM_LO = VMX_HIGHEST_VMCS_ENCODING;
+const Bit32u VMX_MSR_VMCS_ENUM_HI = 0x00000000;
 
-#define VMX_MSR_VMCS_ENUM \
-   ((((Bit64u) VMX_MSR_VMCS_ENUM_HI) << 32) | VMX_MSR_VMCS_ENUM_LO)
+const Bit64u VMX_MSR_VMCS_ENUM =
+   GET64_FROM_HI32_LO32(VMX_MSR_VMCS_ENUM_HI, VMX_MSR_VMCS_ENUM_LO);
 
 
 // IA32_VMX_MSR_PROCBASED_CTRLS2 MSR (0x48b)
 // -----------------------------
 
 // Allowed 0-settings (must be '1 bits)
-#define VMX_MSR_VMX_PROCBASED_CTRLS2_LO (0x00000000)
+const Bit32u VMX_MSR_VMX_PROCBASED_CTRLS2_LO = 0x00000000;
 // Allowed 1-settings
 #define VMX_MSR_VMX_PROCBASED_CTRLS2_HI \
-       (VMX_VM_EXEC_CTRL3_SUPPORTED_BITS | VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
+       (VMX_VM_EXEC_CTRL2_SUPPORTED_BITS | VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
 
 #define VMX_MSR_VMX_PROCBASED_CTRLS2 \
-   ((((Bit64u) VMX_MSR_VMX_PROCBASED_CTRLS2_HI) << 32) | VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
+   GET64_FROM_HI32_LO32(VMX_MSR_VMX_PROCBASED_CTRLS2_HI, VMX_MSR_VMX_PROCBASED_CTRLS2_LO)
+
+
+// IA32_VMX_MSR_PROCBASED_CTRLS3 MSR (0x492)
+// -----------------------------
+
+// Allowed 1-settings
+#define VMX_MSR_VMX_PROCBASED_CTRLS3 (VMX_VM_EXEC_CTRL3_SUPPORTED_BITS)
+
+
+// IA32_VMX_EXIT_CTLS2 MSR (0x493)
+// ---------------------
+
+// Allowed 1-settings
+#define VMX_MSR_VMX_VMEXIT_CTRLS2 (VMX_VMEXIT_CTRL2_SUPPORTED_BITS)
+
 
 #if BX_SUPPORT_VMX >= 2
 
