@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: xmm.h 13963 2020-10-03 09:23:28Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003-2018 Stanislav Shwartsman
@@ -400,6 +400,7 @@ BX_CPP_INLINE void softfloat_status_word_rc_override(float_status_t &status, bxI
   #define softfloat_status_word_rc_override(status, i)
 #endif
 
+#if BX_SUPPORT_FPU
 /* convert float32 NaN number to QNaN */
 BX_CPP_INLINE float32 convert_to_QNaN(float32 op)
 {
@@ -411,6 +412,7 @@ BX_CPP_INLINE float64 convert_to_QNaN(float64 op)
 {
   return op | BX_CONST64(0x7FF8000000000000);
 }
+#endif
 
 /* MXCSR REGISTER */
 
@@ -452,19 +454,19 @@ const Bit32u MXCSR_ROUNDING_CONTROL          = 0x00006000;
 const Bit32u MXCSR_FLUSH_MASKED_UNDERFLOW    = 0x00008000;
 const Bit32u MXCSR_MISALIGNED_EXCEPTION_MASK = 0x00020000;
 
-#define MXCSR_IE 0x00000001
-#define MXCSR_DE 0x00000002
-#define MXCSR_ZE 0x00000004
-#define MXCSR_OE 0x00000008
-#define MXCSR_UE 0x00000010
-#define MXCSR_PE 0x00000020
+const Bit32u MXCSR_IE = 0x00000001;
+const Bit32u MXCSR_DE = 0x00000002;
+const Bit32u MXCSR_ZE = 0x00000004;
+const Bit32u MXCSR_OE = 0x00000008;
+const Bit32u MXCSR_UE = 0x00000010;
+const Bit32u MXCSR_PE = 0x00000020;
 
-#define MXCSR_IM 0x00000080
-#define MXCSR_DM 0x00000100
-#define MXCSR_ZM 0x00000200
-#define MXCSR_OM 0x00000400
-#define MXCSR_UM 0x00000800
-#define MXCSR_PM 0x00001000
+const Bit32u MXCSR_IM = 0x00000080;
+const Bit32u MXCSR_DM = 0x00000100;
+const Bit32u MXCSR_ZM = 0x00000200;
+const Bit32u MXCSR_OM = 0x00000400;
+const Bit32u MXCSR_UM = 0x00000800;
+const Bit32u MXCSR_PM = 0x00001000;
 
 const Bit32u MXCSR_RESET = 0x00001F80;  /* reset value of the MXCSR register */
 
@@ -520,8 +522,8 @@ struct BOCHSAPI bx_mxcsr_t
 
 /*
  * SaturateWordSToByteS  converts  a  signed  16-bit  value  to a signed
- * 8-bit value.  If  the signed 16-bit value is less than -128, it is 
- * represented by the saturated value  -128  (0x80).  If it is  greater 
+ * 8-bit value.  If  the signed 16-bit value is less than -128, it is
+ * represented by the saturated value  -128  (0x80).  If it is  greater
  * than 127, it is represented by the saturated value 127 (0x7F).
 */
 BX_CPP_INLINE Bit8s BX_CPP_AttrRegparmN(1) SaturateWordSToByteS(Bit16s value)
@@ -534,7 +536,7 @@ BX_CPP_INLINE Bit8s BX_CPP_AttrRegparmN(1) SaturateWordSToByteS(Bit16s value)
 /*
  * SaturateQwordSToByteS converts a signed 32-bit value to a signed
  * 8-bit value. If the signed 32-bit value is less than -128, it is
- * represented by the saturated value -128 (0x80). If it is greater 
+ * represented by the saturated value -128 (0x80). If it is greater
  * than 127, it is represented by the saturated value 127 (0x7F).
 */
 BX_CPP_INLINE Bit8s BX_CPP_AttrRegparmN(1) SaturateDwordSToByteS(Bit32s value)
@@ -547,7 +549,7 @@ BX_CPP_INLINE Bit8s BX_CPP_AttrRegparmN(1) SaturateDwordSToByteS(Bit32s value)
 /*
  * SaturateQwordSToByteS converts a signed 64-bit value to a signed
  * 8-bit value. If the signed 64-bit value is less than -128, it is
- * represented by the saturated value -128 (0x80). If it is greater 
+ * represented by the saturated value -128 (0x80). If it is greater
  * than 127, it is represented by the saturated value 127 (0x7F).
 */
 BX_CPP_INLINE Bit8s BX_CPP_AttrRegparmN(1) SaturateQwordSToByteS(Bit64s value)
