@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: xsave.cc 14326 2021-07-27 15:36:11Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008-2019 Stanislav Shwartsman
+//   Copyright (c) 2008-2024 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -27,6 +27,10 @@
 #include "msr.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_SVM
+#include "svm.h"
+#endif
+
 #include "decoder/ia_opcodes.h"
 
 const Bit64u XSAVEC_COMPACTION_ENABLED = BX_CONST64(0x8000000000000000);
@@ -141,13 +145,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX >= 2
     if (BX_CPU_THIS_PTR in_vmx_guest) {
-      if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_XSAVES_XRSTORS)) {
+      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+      if (! vm->vmexec_ctrls2.XSAVES_XRSTORS()) {
         BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
         exception(BX_UD_EXCEPTION, 0);
       }
 
-      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
-      Bit64u requested_features = (((Bit64u) EDX) << 32) | EAX;
+      Bit64u requested_features = GET64_FROM_HI32_LO32(EDX, EAX);
       if (requested_features & BX_CPU_THIS_PTR msr.ia32_xss & vm->xss_exiting_bitmap)
         VMexit_Instruction(i, VMX_VMEXIT_XSAVES);
     }
@@ -184,24 +189,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XSAVEC(bxInstruction_c *i)
   Bit64u xstate_bv = requested_feature_bitmap & xinuse;
   Bit64u xcomp_bv = requested_feature_bitmap | XSAVEC_COMPACTION_ENABLED;
 
-  if ((requested_feature_bitmap & BX_XCR0_FPU_MASK) != 0)
+  if ((xstate_bv & BX_XCR0_FPU_MASK) != 0)
   {
-    if (xinuse & BX_XCR0_FPU_MASK) {
-      xsave_x87_state(i, eaddr);
-    }
+    xsave_x87_state(i, eaddr);
   }
 
-  if ((requested_feature_bitmap & BX_XCR0_SSE_MASK) != 0)
+  if ((xstate_bv & (BX_XCR0_SSE_MASK|BX_XCR0_YMM_MASK)) != 0)
   {
     // write cannot cause any boundary cross because XSAVE image is 64-byte aligned
     write_virtual_dword(i->seg(), eaddr + 24, BX_MXCSR_REGISTER);
     write_virtual_dword(i->seg(), eaddr + 28, MXCSR_MASK);
   }
 
-  Bit32u offset = XSAVE_SSE_STATE_OFFSET;
+  if ((xstate_bv & BX_XCR0_SSE_MASK) != 0)
+  {
+    xsave_sse_state(i, eaddr);
+  }
+
+  Bit32u offset = XSAVE_YMM_STATE_OFFSET;
 
   /////////////////////////////////////////////////////////////////////////////
-  for (unsigned feature = xcr0_t::BX_XCR0_SSE_BIT; feature < xcr0_t::BX_XCR0_LAST; feature++)
+  for (unsigned feature = xcr0_t::BX_XCR0_YMM_BIT; feature < xcr0_t::BX_XCR0_LAST; feature++)
   {
     Bit32u feature_mask = (1 << feature);
 
@@ -244,13 +252,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
 
 #if BX_SUPPORT_VMX >= 2
     if (BX_CPU_THIS_PTR in_vmx_guest) {
-      if (! SECONDARY_VMEXEC_CONTROL(VMX_VM_EXEC_CTRL3_XSAVES_XRSTORS)) {
+      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
+
+      if (! vm->vmexec_ctrls2.XSAVES_XRSTORS()) {
         BX_ERROR(("%s in VMX guest: not allowed to use instruction !", i->getIaOpcodeNameShort()));
         exception(BX_UD_EXCEPTION, 0);
       }
 
-      VMCS_CACHE *vm = &BX_CPU_THIS_PTR vmcs;
-      Bit64u requested_features = (((Bit64u) EDX) << 32) | EAX;
+      Bit64u requested_features = GET64_FROM_HI32_LO32(EDX, EAX);
       if (requested_features & BX_CPU_THIS_PTR msr.ia32_xss & vm->xss_exiting_bitmap)
         VMexit_Instruction(i, VMX_VMEXIT_XRSTORS);
     }
@@ -316,7 +325,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
       BX_ERROR(("%s: Invalid xcomp_bv state", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
-    
+
     if (xstate_bv & ~xcomp_bv) {
       BX_ERROR(("%s: xstate_bv set a bit which is not in xcomp_bv state", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
@@ -335,19 +344,23 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
   }
 
   Bit32u requested_feature_bitmap = xcr0 & EAX;
+  Bit64u format = (compaction) ? (xcomp_bv & ~XSAVEC_COMPACTION_ENABLED) : (~XSAVEC_COMPACTION_ENABLED);
+  Bit32u restore_mask = xstate_bv & format;
 
   /////////////////////////////////////////////////////////////////////////////
   if ((requested_feature_bitmap & BX_XCR0_FPU_MASK) != 0)
   {
-    if (xstate_bv & BX_XCR0_FPU_MASK)
+    if (restore_mask & BX_XCR0_FPU_MASK)
       xrstor_x87_state(i, eaddr);
     else
       xrstor_init_x87_state();
   }
 
   /////////////////////////////////////////////////////////////////////////////
-  if ((requested_feature_bitmap & BX_XCR0_SSE_MASK) != 0 || 
-     ((requested_feature_bitmap & BX_XCR0_YMM_MASK) != 0 && ! compaction))
+  // Legacy form of XRSTOR loads the MXCSR register from memory whenever the
+  // RFBM[1](SSE) or RFBM[2](AVX) is set, regardless of the values of XSTATE_BV[1] and XSTATE_BV[2]
+  if (((requested_feature_bitmap & (BX_XCR0_SSE_MASK|BX_XCR0_YMM_MASK)) != 0 && ! compaction) ||
+      ((requested_feature_bitmap & restore_mask & BX_XCR0_SSE_MASK) != 0))
   {
     // read cannot cause any boundary cross because XSAVE image is 64-byte aligned
     Bit32u new_mxcsr = read_virtual_dword(i->seg(), eaddr + 24);
@@ -359,7 +372,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
   /////////////////////////////////////////////////////////////////////////////
   if ((requested_feature_bitmap & BX_XCR0_SSE_MASK) != 0)
   {
-    if (xstate_bv & BX_XCR0_SSE_MASK)
+    if (restore_mask & BX_XCR0_SSE_MASK)
       xrstor_sse_state(i, eaddr+XSAVE_SSE_STATE_OFFSET);
     else
       xrstor_init_sse_state();
@@ -372,7 +385,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
     for (unsigned feature = xcr0_t::BX_XCR0_YMM_BIT; feature < xcr0_t::BX_XCR0_LAST; feature++)
     {
       Bit32u feature_mask = (1 << feature);
-    
+
       if ((requested_feature_bitmap & feature_mask) != 0)
       {
         if (! xsave_restore[feature].len) {
@@ -380,7 +393,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
           continue;
         }
 
-        if (xstate_bv & feature_mask) {
+        if (restore_mask & feature_mask) {
           BX_ASSERT(xsave_restore[feature].xrstor_method);
           CALL_XSAVE_FN(xsave_restore[feature].xrstor_method)(i, eaddr+offset);
         }
@@ -389,7 +402,8 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
           CALL_XSAVE_FN(xsave_restore[feature].xrstor_init_method)();
         }
 
-        offset += xsave_restore[feature].len;
+        if (format & feature_mask)
+          offset += xsave_restore[feature].len;
       }
     }
   }
@@ -399,7 +413,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XRSTOR(bxInstruction_c *i)
     for (unsigned feature = xcr0_t::BX_XCR0_YMM_BIT; feature < xcr0_t::BX_XCR0_LAST; feature++)
     {
       Bit32u feature_mask = (1 << feature);
-    
+
       if ((requested_feature_bitmap & feature_mask) != 0)
       {
         if (! xsave_restore[feature].len) {
@@ -617,12 +631,12 @@ void BX_CPU_C::xsave_sse_state(bxInstruction_c *i, bx_address offset)
 {
   bx_address asize_mask = i->asize_mask();
 
+  // save XMM8-XMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   /* store XMM register file */
-  for(unsigned index=0; index < 16; index++) {
-    // save XMM8-XMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      write_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_XMM_REG(index));
-    }
+  for(unsigned index=0; index < num_regs; index++) {
+    write_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_XMM_REG(index));
   }
 }
 
@@ -630,32 +644,34 @@ void BX_CPU_C::xrstor_sse_state(bxInstruction_c *i, bx_address offset)
 {
   bx_address asize_mask = i->asize_mask();
 
+  // restore XMM8-XMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   // load SSE state from XSAVE area
-  for(unsigned index=0; index < 16; index++) {
-    // restore XMM8-XMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      read_virtual_xmmword(i->seg(), (offset+index*16) & asize_mask, &BX_READ_XMM_REG(index));
-    }
+  for(unsigned index=0; index < num_regs; index++) {
+    read_virtual_xmmword(i->seg(), (offset+index*16) & asize_mask, &BX_READ_XMM_REG(index));
   }
 }
 
 void BX_CPU_C::xrstor_init_sse_state(void)
 {
+  // set XMM8-XMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   // initialize SSE with reset values
-  for(unsigned index=0; index < 16; index++) {
-    // set XMM8-XMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) BX_CLEAR_XMM_REG(index);
+  for(unsigned index=0; index < num_regs; index++) {
+    BX_CLEAR_XMM_REG(index);
   }
 }
 
 bool BX_CPU_C::xsave_sse_state_xinuse(void)
 {
-  for(unsigned index=0; index < 16; index++) {
-    // set XMM8-XMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      const BxPackedXmmRegister *reg = &BX_XMM_REG(index);
-      if (! is_clear(reg)) return true;
-    }
+  // set XMM8-XMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
+  for(unsigned index=0; index < num_regs; index++) {
+    const BxPackedXmmRegister *reg = &BX_XMM_REG(index);
+    if (! is_clear(reg)) return true;
   }
 
   return false;
@@ -669,12 +685,12 @@ void BX_CPU_C::xsave_ymm_state(bxInstruction_c *i, bx_address offset)
 {
   bx_address asize_mask = i->asize_mask();
 
+  // save YMM8-YMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   /* store AVX state */
-  for(unsigned index=0; index < 16; index++) {
-    // save YMM8-YMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      write_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_AVX_REG_LANE(index, 1));
-    }
+  for(unsigned index=0; index < num_regs; index++) {
+    write_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_AVX_REG_LANE(index, 1));
   }
 }
 
@@ -682,32 +698,34 @@ void BX_CPU_C::xrstor_ymm_state(bxInstruction_c *i, bx_address offset)
 {
   bx_address asize_mask = i->asize_mask();
 
+  // restore YMM8-YMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   // load AVX state from XSAVE area
-  for(unsigned index=0; index < 16; index++) {
-    // restore YMM8-YMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      read_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_AVX_REG_LANE(index, 1));
-    }
+  for(unsigned index=0; index < num_regs; index++) {
+    read_virtual_xmmword(i->seg(), (offset + index*16) & asize_mask, &BX_READ_AVX_REG_LANE(index, 1));
   }
 }
 
 void BX_CPU_C::xrstor_init_ymm_state(void)
 {
+  // set YMM8-YMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
   // initialize upper part of AVX registers with reset values
-  for(unsigned index=0; index < 16; index++) {
-    // set YMM8-YMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) BX_CLEAR_AVX_HIGH128(index);
+  for(unsigned index=0; index < num_regs; index++) {
+    BX_CLEAR_AVX_HIGH128(index);
   }
 }
 
 bool BX_CPU_C::xsave_ymm_state_xinuse(void)
 {
-  for(unsigned index=0; index < 16; index++) {
-    // set YMM8-YMM15 only in 64-bit mode
-    if (index < 8 || long64_mode()) {
-      const BxPackedXmmRegister *reg = &BX_READ_AVX_REG_LANE(index, 1);
-      if (! is_clear(reg)) return true;
-    }
+  // set YMM8-YMM15 only in 64-bit mode
+  unsigned num_regs = long64_mode() ? 16 : 8;
+
+  for(unsigned index=0; index < num_regs; index++) {
+    const BxPackedXmmRegister *reg = &BX_READ_AVX_REG_LANE(index, 1);
+    if (! is_clear(reg)) return true;
   }
 
   return false;
@@ -923,8 +941,8 @@ void BX_CPU_C::xrstor_init_cet_u_state(void)
 
 bool BX_CPU_C::xsave_cet_u_state_xinuse(void)
 {
-  return BX_CPU_THIS_PTR msr.ia32_cet_control[1] == 0 &&
-         BX_CPU_THIS_PTR msr.ia32_pl_ssp[3] == 0;
+  return BX_CPU_THIS_PTR msr.ia32_cet_control[1] != 0 ||
+         BX_CPU_THIS_PTR msr.ia32_pl_ssp[3] != 0;
 }
 
 // CET S state management //
@@ -960,12 +978,171 @@ void BX_CPU_C::xrstor_init_cet_s_state(void)
 bool BX_CPU_C::xsave_cet_s_state_xinuse(void)
 {
   for (unsigned n=0;n<3;n++)
-    return BX_CPU_THIS_PTR msr.ia32_pl_ssp[n] != 0;
+    if (BX_CPU_THIS_PTR msr.ia32_pl_ssp[n])
+        return true;
 
   return false;
 }
 #endif
 
+#if BX_SUPPORT_UINTR
+void BX_CPU_C::xsave_uintr_state(bxInstruction_c *i, bx_address offset)
+{
+  bx_address asize_mask = i->asize_mask();
+
+  write_virtual_qword(i->seg(),  offset,                    BX_CPU_THIS_PTR uintr.ui_handler);
+  write_virtual_qword(i->seg(), (offset +  8) & asize_mask, BX_CPU_THIS_PTR uintr.stack_adjust);
+
+  Bit64u misc = GET64_FROM_HI32_LO32(BX_CPU_THIS_PTR uintr.uinv, BX_CPU_THIS_PTR uintr.uitt_size);
+  if (BX_CPU_THIS_PTR uintr.UIF)
+    misc |= BX_CONST64(0x8000000000000000);
+  write_virtual_qword(i->seg(), (offset + 16) & asize_mask, misc);
+
+  write_virtual_qword(i->seg(), (offset + 24) & asize_mask, BX_CPU_THIS_PTR uintr.upid_addr);
+  write_virtual_qword(i->seg(), (offset + 32) & asize_mask, BX_CPU_THIS_PTR uintr.uirr);
+  write_virtual_qword(i->seg(), (offset + 40) & asize_mask, BX_CPU_THIS_PTR uintr.uitt_addr);
+
+  BX_CPU_THIS_PTR uintr.uinv = 0;
+}
+
+void BX_CPU_C::xrstor_uintr_state(bxInstruction_c *i, bx_address offset)
+{
+  if (BX_CPU_THIS_PTR uintr.uinv) {
+    BX_ERROR(("Attempting to restore UINTR state when UINTR.UINV is set: #GP(0)"));
+    exception(BX_GP_EXCEPTION, 0);
+  }
+
+  BX_CPU_THIS_PTR uintr.uinv = 0; // will be restored and overwritten, should stay '0 in case of any fault
+
+  bx_address asize_mask = i->asize_mask();
+
+  Bit64u ui_handler   = read_virtual_qword(i->seg(),  offset);
+  Bit64u stack_adjust = read_virtual_qword(i->seg(), (offset +  8) & asize_mask);
+  Bit64u misc         = read_virtual_qword(i->seg(), (offset + 16) & asize_mask);
+  bool UIF = (misc >> 63) != 0;
+  misc &= ~BX_CONST64(0x8000000000000000); // clear UIF
+  Bit64u upid_addr    = read_virtual_qword(i->seg(), (offset + 24) & asize_mask);
+  Bit64u uirr         = read_virtual_qword(i->seg(), (offset + 32) & asize_mask);
+  Bit64u uitt_addr    = read_virtual_qword(i->seg(), (offset + 40) & asize_mask);
+
+  // XRSTOR on UINTR state does all reserved bits and canonicality check like WRMSR would do
+  wrmsr(BX_MSR_IA32_UINTR_HANDLER, ui_handler);
+  wrmsr(BX_MSR_IA32_UINTR_STACKADJUST, stack_adjust);
+  wrmsr(BX_MSR_IA32_UINTR_PD, upid_addr);
+  wrmsr(BX_MSR_IA32_UINTR_TT, uitt_addr);
+  wrmsr(BX_MSR_IA32_UINTR_MISC, misc);   // make sure all faults happen before
+  wrmsr(BX_MSR_IA32_UINTR_RR, uirr);
+
+  BX_CPU_THIS_PTR uintr.UIF = UIF;
+  uintr_control(); // potentially enable user interrupt delivery
+}
+
+void BX_CPU_C::xrstor_init_uintr_state(void)
+{
+  BX_CPU_THIS_PTR uintr.ui_handler = 0;
+  BX_CPU_THIS_PTR uintr.stack_adjust = 0;
+  BX_CPU_THIS_PTR uintr.uitt_size = 0;
+  BX_CPU_THIS_PTR uintr.uinv = 0;
+  BX_CPU_THIS_PTR uintr.UIF = 0;
+  BX_CPU_THIS_PTR uintr.upid_addr = 0;
+  BX_CPU_THIS_PTR uintr.uitt_addr = 0;
+  BX_CPU_THIS_PTR uintr.uirr = 0;
+}
+
+bool BX_CPU_C::xsave_uintr_state_xinuse(void)
+{
+  return BX_CPU_THIS_PTR uintr.ui_handler != 0 ||
+         BX_CPU_THIS_PTR uintr.stack_adjust != 0 ||
+         BX_CPU_THIS_PTR uintr.uitt_size != 0 ||
+         BX_CPU_THIS_PTR uintr.uinv != 0 ||
+         BX_CPU_THIS_PTR uintr.UIF != 0 ||
+         BX_CPU_THIS_PTR uintr.upid_addr != 0 ||
+         BX_CPU_THIS_PTR uintr.uitt_addr != 0 ||
+         BX_CPU_THIS_PTR uintr.uirr != 0;
+}
+#endif
+
+#if BX_SUPPORT_AMX
+
+#include "avx/amx.h"
+
+// TILECFG state management //
+void BX_CPU_C::xsave_tilecfg_state(bxInstruction_c *i, bx_address offset)
+{
+  BxPackedAvxRegister tilecfg;
+  tilecfg.clear();
+
+  if (BX_CPU_THIS_PTR amx->tiles_configured()) {
+    tilecfg.vmmubyte(0) = BX_CPU_THIS_PTR amx->palette_id;
+    tilecfg.vmmubyte(1) = BX_CPU_THIS_PTR amx->start_row;
+
+    for (unsigned n=0; n < 8; n++) {
+      tilecfg.vmm16u(8+n)    = BX_CPU_THIS_PTR amx->tilecfg[n].rows;
+      tilecfg.vmmubyte(48+n) = BX_CPU_THIS_PTR amx->tilecfg[n].bytes_per_row;
+    }
+  }
+
+  write_virtual_zmmword(i->seg(), offset, &tilecfg);
+}
+
+void BX_CPU_C::xrstor_tilecfg_state(bxInstruction_c *i, bx_address offset)
+{
+  BxPackedAvxRegister tilecfg;
+  read_virtual_zmmword(i->seg(), offset, &tilecfg);
+
+  if (!configure_tiles(i, tilecfg))
+    BX_CPU_THIS_PTR amx->clear();
+}
+
+void BX_CPU_C::xrstor_init_tilecfg_state(void)
+{
+  BX_CPU_THIS_PTR amx->clear();
+}
+
+bool BX_CPU_C::xsave_tilecfg_state_xinuse(void)
+{
+  return BX_CPU_THIS_PTR amx->tiles_configured();
+}
+
+// TILEDATA state management //
+void BX_CPU_C::xsave_tiledata_state(bxInstruction_c *i, bx_address offset)
+{
+  bx_address asize_mask = i->asize_mask();
+
+  for (unsigned tile=0; tile < BX_TILE_REGISTERS; tile++) {
+    for (unsigned row=0; row < BX_TILE_REGISTERS; row++) {
+      write_virtual_zmmword(i->seg(), (offset+(tile*BX_TILE_MAX_ROWS+row)*64) & asize_mask, &(BX_CPU_THIS_PTR amx->tile[tile].row[row]));
+    }
+  }
+}
+
+void BX_CPU_C::xrstor_tiledata_state(bxInstruction_c *i, bx_address offset)
+{
+  bx_address asize_mask = i->asize_mask();
+
+  for (unsigned tile=0; tile < BX_TILE_REGISTERS; tile++) {
+    for (unsigned row=0; row < BX_TILE_REGISTERS; row++) {
+      read_virtual_zmmword(i->seg(), (offset+(tile*BX_TILE_MAX_ROWS+row)*64) & asize_mask, &(BX_CPU_THIS_PTR amx->tile[tile].row[row]));
+    }
+    BX_CPU_THIS_PTR amx->set_tile_used(tile);
+  }
+}
+
+void BX_CPU_C::xrstor_init_tiledata_state(void)
+{
+  for (unsigned tile=0; tile < BX_TILE_REGISTERS; tile++) {
+    BX_CPU_THIS_PTR amx->tile[tile].clear();
+    BX_CPU_THIS_PTR amx->clear_tile_used(tile);
+  }
+}
+
+bool BX_CPU_C::xsave_tiledata_state_xinuse(void)
+{
+  return (BX_CPU_THIS_PTR amx->tile_use_tracker == 0);  // all tiles are zero
+}
+
+#endif
+
 Bit32u BX_CPU_C::get_xinuse_vector(Bit32u requested_feature_bitmap)
 {
   Bit32u xinuse = 0;
@@ -974,7 +1151,7 @@ Bit32u BX_CPU_C::get_xinuse_vector(Bit32u requested_feature_bitmap)
   for (unsigned feature = xcr0_t::BX_XCR0_FPU_BIT; feature < xcr0_t::BX_XCR0_LAST; feature++)
   {
     Bit32u feature_mask = (1 << feature);
-  
+
     if ((requested_feature_bitmap & feature_mask) != 0)
     {
       if (! xsave_restore[feature].len) {
@@ -1089,6 +1266,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XSETBV(bxInstruction_c *i)
   }
 #endif
 
+#if BX_SUPPORT_AMX
+  if (EAX & BX_XCR0_XTILE_BITS_MASK) {
+    if ((EAX & BX_XCR0_XTILE_BITS_MASK) != BX_XCR0_XTILE_BITS_MASK) {
+      BX_ERROR(("XSETBV: Illegal attempt to enable AMX state"));
+      exception(BX_GP_EXCEPTION, 0);
+    }
+  }
+#endif
+
   BX_CPU_THIS_PTR xcr0.set32(EAX);
 
 #if BX_SUPPORT_AVX
