@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gui.cc 14274 2021-06-07 11:30:08Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2021  The Bochs Project
+//  Copyright (C) 2002-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -150,7 +150,8 @@ bx_gui_c::bx_gui_c(void): disp_mode(DISP_MODE_SIM)
   command_mode.active = 0;
   marker_count = 0;
   memset(palette, 0, sizeof(palette));
-  memset(vga_charmap, 0, 0x2000);
+  memset(vga_charmap[0], 0, 0x2000);
+  memset(vga_charmap[1], 0, 0x2000);
 }
 
 bx_gui_c::~bx_gui_c()
@@ -420,6 +421,7 @@ void bx_gui_c::cdrom1_handler(void)
 void bx_gui_c::reset_handler(void)
 {
   BX_INFO(("system RESET callback"));
+  BX_GUI_THIS statusbar_setall(0);
   bx_pc_system.Reset(BX_RESET_HARDWARE);
 }
 
@@ -698,6 +700,9 @@ bool bx_gui_c::mouse_toggle_check(Bit32u key, bool pressed)
   if (pressed) {
     newstate |= key;
     if (newstate == toggle_keystate) return 0;
+    if ((key == BX_MT_KEY_CTRL) || (key == BX_MT_KEY_ALT)) {
+      newstate &= (BX_MT_KEY_CTRL | BX_MT_KEY_ALT);
+    }
     switch (toggle_method) {
       case BX_MOUSE_TOGGLE_CTRL_MB:
         toggle = (newstate & BX_GUI_MT_CTRL_MB) == BX_GUI_MT_CTRL_MB;
@@ -819,17 +824,19 @@ void bx_gui_c::userbutton_handler(void)
 
 void bx_gui_c::save_restore_handler(void)
 {
-  int ret;
   char sr_path[BX_PATHNAME_LEN];
 
   if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_SAVE_RESTORE) {
     BX_GUI_THIS set_display_mode(DISP_MODE_CONFIG);
     sr_path[0] = 0;
-    ret = SIM->ask_filename(sr_path, sizeof(sr_path),
+    int ret = SIM->ask_filename(sr_path, sizeof(sr_path),
                             "Save Bochs state to folder...", "none",
                             bx_param_string_c::SELECT_FOLDER_DLG);
     if ((ret >= 0) && (strcmp(sr_path, "none"))) {
+      void *hwnd = SIM->ml_message_box("Please wait.", "Waiting for the 'save state' to finish.\n"
+              "Depending on the size of your emulated disk images, this may be a little while.");
       if (SIM->save_state(sr_path)) {
+        SIM->ml_message_box_kill(hwnd);
         if (!SIM->ask_yes_no("WARNING",
               "The state of cpu, memory, devices and hard drive images is saved now.\n"
               "It is possible to continue, but when using the restore function in a\n"
@@ -838,6 +845,7 @@ void bx_gui_c::save_restore_handler(void)
           power_handler();
         }
       } else {
+        SIM->ml_message_box_kill(hwnd);
         SIM->message_box("ERROR", "Failed to save state");
       }
     }
@@ -899,17 +907,10 @@ void bx_gui_c::init_signal_handlers()
 #endif
 }
 
-void bx_gui_c::set_text_charmap(Bit8u *fbuffer)
+void bx_gui_c::set_text_charmap(Bit8u map, Bit8u *fbuffer)
 {
-  memcpy(& BX_GUI_THIS vga_charmap, fbuffer, 0x2000);
-  for (unsigned i=0; i<256; i++) BX_GUI_THIS char_changed[i] = 1;
-  BX_GUI_THIS charmap_updated = 1;
-}
-
-void bx_gui_c::set_text_charbyte(Bit16u address, Bit8u data)
-{
-  BX_GUI_THIS vga_charmap[address] = data;
-  BX_GUI_THIS char_changed[address >> 5] = 1;
+  memcpy(& BX_GUI_THIS vga_charmap[map], fbuffer, 0x2000);
+  for (unsigned i=0; i<256; i++) BX_GUI_THIS char_changed[map][i] = 1;
   BX_GUI_THIS charmap_updated = 1;
 }
 
@@ -983,6 +984,12 @@ void bx_gui_c::statusbar_setitem(int element, bool active, bool w)
   }
 }
 
+void bx_gui_c::statusbar_setall(bool active)
+{
+  for (int i=0; i<BX_MAX_STATUSITEMS; i++)
+    statusbar_setitem(i, active);
+}
+
 void bx_gui_c::led_timer_handler(void *this_ptr)
 {
   bx_gui_c *class_ptr = (bx_gui_c *) this_ptr;
@@ -1146,7 +1153,7 @@ void bx_gui_c::graphics_tile_update_in_place(unsigned x0, unsigned y0,
 void bx_gui_c::draw_char_common(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc,
                                 Bit16u yc, Bit8u fw, Bit8u fh, Bit8u fx,
                                 Bit8u fy, bool gfxcharw9, Bit8u cs, Bit8u ce,
-                                bool curs)
+                                bool curs, bool font2)
 {
   Bit8u *buf, *font_ptr, fontpixels;
   Bit16u font_row, mask;
@@ -1154,7 +1161,11 @@ void bx_gui_c::draw_char_common(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc,
 
   buf = BX_GUI_THIS snapshot_buffer + yc * BX_GUI_THIS guest_xres + xc;
   dwidth = (BX_GUI_THIS guest_fwidth > 9);
-  font_ptr = &vga_charmap[(ch << 5) + fy];
+  if (font2) {
+    font_ptr = &vga_charmap[1][(ch << 5) + fy];
+  } else {
+    font_ptr = &vga_charmap[0][(ch << 5) + fy];
+  }
   do {
     font_row = *font_ptr++;
     if (gfxcharw9) {
@@ -1192,7 +1203,7 @@ void bx_gui_c::text_update_common(Bit8u *old_text, Bit8u *new_text,
   Bit8u cfheight, cfwidth, cfrow, cfcol, fgcolor, bgcolor;
   Bit8u split_textrow, split_fontrows, x, y;
   Bit8u *new_line, *old_line, *text_base;
-  bool cursor_visible, gfxcharw9, split_screen;
+  bool cursor_visible, gfxcharw9, split_screen, font2;
   bool forceUpdate = 0, blink_mode = 0, blink_state = 0;
 
   if (BX_GUI_THIS snapshot_mode || BX_GUI_THIS new_text_api) {
@@ -1226,7 +1237,7 @@ void bx_gui_c::text_update_common(Bit8u *old_text, Bit8u *new_text,
         old_text[BX_GUI_THIS cursor_address] = ~new_text[BX_GUI_THIS cursor_address];
         BX_GUI_THIS cursor_address = cursor_address;
       }
-      if (cursor_address < 0xffff) {
+      if (cursor_address < 0x7fff) {
         old_text[cursor_address] = ~new_text[cursor_address];
       }
     }
@@ -1300,17 +1311,18 @@ void bx_gui_c::text_update_common(Bit8u *old_text, Bit8u *new_text,
           } else {
             bgcolor = tm_info->actl_palette[(new_text[1] >> 4) & 0x0F];
           }
+          font2 = (new_text[1] & 0x08) > 0;
           gfxcharw9 = ((tm_info->line_graphics) && ((new_text[0] & 0xE0) == 0xC0));
           if (BX_GUI_THIS snapshot_mode) {
             BX_GUI_THIS draw_char_common(new_text[0], fgcolor, bgcolor, xc, yc,
                                          cfwidth, cfheight, cfcol, cfrow,
                                          gfxcharw9, tm_info->cs_start,
-                                         tm_info->cs_end, (offset == curs));
+                                         tm_info->cs_end, (offset == curs), font2);
           } else {
             BX_GUI_THIS draw_char(new_text[0], fgcolor, bgcolor, xc, yc,
                                   cfwidth, cfheight, cfcol, cfrow,
                                   gfxcharw9, tm_info->cs_start,
-                                  tm_info->cs_end, (offset == curs));
+                                  tm_info->cs_end, (offset == curs), font2);
           }
         }
         new_text += 2;
@@ -1419,10 +1431,10 @@ void bx_gui_c::console_init(void)
   console.saved_bpp = guest_bpp;
   console.saved_fwidth = guest_fwidth;
   console.saved_fheight = guest_fheight;
-  memcpy(console.saved_charmap, BX_GUI_THIS vga_charmap, 0x2000);
+  memcpy(console.saved_charmap, BX_GUI_THIS vga_charmap[0], 0x2000);
   for (i = 0; i < 256; i++) {
-    memcpy(&BX_GUI_THIS vga_charmap[0]+i*32, &sdl_font8x16[i], 16);
-    BX_GUI_THIS char_changed[i] = 1;
+    memcpy(&BX_GUI_THIS vga_charmap[0][0]+i*32, &sdl_font8x16[i], 16);
+    BX_GUI_THIS char_changed[0][i] = 1;
   }
   BX_GUI_THIS charmap_updated = 1;
   console.cursor_x = 0;
@@ -1452,7 +1464,7 @@ void bx_gui_c::console_cleanup(void)
                         console.saved_palette[28]);
   unsigned fheight = (console.saved_fheight);
   unsigned fwidth = (console.saved_fwidth);
-  set_text_charmap(console.saved_charmap);
+  set_text_charmap(0, console.saved_charmap);
   dimension_update(console.saved_xres, console.saved_yres, fheight, fwidth,
                    console.saved_bpp);
   DEV_vga_refresh(1);
