@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.cc 14293 2021-06-27 14:50:26Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2021  The Bochs Project
+//  Copyright (C) 2002-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
-  
+
 // See siminterface.h for description of the siminterface concept.
 // Basically, the siminterface is visible from both the simulator and
 // the configuration user interface, and allows them to talk to each other.
@@ -72,6 +72,7 @@ class bx_real_sim_c : public bx_simulator_interface_c {
   rt_conf_entry_t *rt_conf_entries;
   addon_option_t *addon_options;
   bool init_done;
+  bool ci_started;
   bool enabled;
   // save context to jump to if we must quit unexpectedly
   jmp_buf *quit_context;
@@ -86,6 +87,7 @@ public:
   virtual void set_quit_context(jmp_buf *context) { quit_context = context; }
   virtual bool get_init_done() { return init_done; }
   virtual int set_init_done(bool n);
+  virtual bool get_ci_started() { return ci_started; }
   virtual void reset_all_param();
   // new param methods
   virtual bx_param_c *get_param(const char *pname, bx_param_c *base=NULL);
@@ -136,6 +138,10 @@ public:
   virtual int ask_yes_no(const char *title, const char *prompt, bool the_default);
   // simple message box
   virtual void message_box(const char *title, const char *message);
+  // simple modeless message box
+  virtual void *ml_message_box(const char *title, const char *message);
+  // kill modeless message box
+  virtual void ml_message_box_kill(void *ptr);
   // called at a regular interval, currently by the keyboard handler.
   virtual void periodic();
   virtual int create_disk_image(const char *filename, int sectors, bool overwrite);
@@ -217,7 +223,7 @@ public:
   virtual void init_std_nic_options(const char *name, bx_list_c *menu);
 #endif
 #if BX_SUPPORT_PCIUSB
-  virtual void init_usb_options(const char *usb_name, const char *pname, int maxports);
+  virtual void init_usb_options(const char *usb_name, const char *pname, int maxports, int param0);
 #endif
   virtual int  parse_param_from_list(const char *context, const char *param, bx_list_c *base);
   virtual int  parse_nic_params(const char *context, const char *param, bx_list_c *base);
@@ -341,18 +347,35 @@ bx_param_enum_c *bx_real_sim_c::get_param_enum(const char *pname, bx_param_c *ba
 void bx_init_siminterface()
 {
   if (SIM == NULL) {
+    SIM = new bx_real_sim_c();
+  }
+  if (siminterface_log == NULL) {
     siminterface_log = new logfunctions();
     siminterface_log->put("siminterface", "SIM");
-    SIM = new bx_real_sim_c();
   }
   if (root_param == NULL) {
     root_param = new bx_list_c(NULL,
       "bochs",
-      "list of top level bochs parameters"
-      );
+      "list of top level bochs parameters");
   }
 }
 
+int bx_cleanup_siminterface()
+{
+  if (siminterface_log) {
+    delete siminterface_log;
+  }
+  if (root_param) {
+    root_param->clear();
+    delete root_param;
+    root_param = NULL;
+  }
+  io->exit_log2();
+  int exit_code = SIM->get_exit_code();
+  delete SIM;
+  return exit_code;
+}
+
 bx_real_sim_c::bx_real_sim_c()
 {
   bxevent_callback = NULL;
@@ -366,6 +389,7 @@ bx_real_sim_c::bx_real_sim_c()
 
   enabled = 1;
   init_done = 0;
+  ci_started = 0;
   quit_context = NULL;
   exit_code = 0;
   param_id = BXP_NEW_PARAM_ID;
@@ -662,6 +686,27 @@ void bx_real_sim_c::message_box(const char *title, const char *message)
   sim_to_ci_event(&event);
 }
 
+void *bx_real_sim_c::ml_message_box(const char *title, const char *message)
+{
+  BxEvent event;
+
+  event.type = BX_SYNC_EVT_ML_MSG_BOX;
+  event.param0 = NULL;
+  event.u.logmsg.prefix = title;
+  event.u.logmsg.msg = message;
+  sim_to_ci_event(&event);
+  return event.param0;
+}
+
+void bx_real_sim_c::ml_message_box_kill(void *ptr)
+{
+  BxEvent event;
+
+  event.type = BX_SYNC_EVT_ML_MSG_BOX_KILL;
+  event.param0 = ptr;
+  sim_to_ci_event(&event);
+}
+
 void bx_real_sim_c::periodic()
 {
   // give the GUI a chance to do periodic things on the bochs thread. in
@@ -875,6 +920,11 @@ int bx_real_sim_c::configuration_interface(const char *ignore, ci_command_t comm
   else
     wxsel = 0;
   bx_debug_gui = wxsel;
+  if (command == CI_START) {
+    ci_started = 1;
+  } else if (command == CI_SHUTDOWN) {
+    ci_started = 0;
+  }
   // enter configuration mode, just while running the configuration interface
   set_display_mode(DISP_MODE_CONFIG);
   int retval = (*ci_callback)(ci_callback_data, command);
@@ -1048,9 +1098,9 @@ void bx_real_sim_c::init_statistics()
 
 void bx_real_sim_c::cleanup_statistics()
 {
-  bx_list_c *list;
+  bx_list_c *list = get_statistics_root();
 
-  if ((list = get_statistics_root()) != NULL) {
+  if (list != NULL) {
     list->clear();
   }
 }
@@ -1121,7 +1171,12 @@ bool bx_real_sim_c::restore_config()
   char config[BX_PATHNAME_LEN];
   sprintf(config, "%s/config", get_param_string(BXPN_RESTORE_PATH)->getptr());
   BX_INFO(("restoring '%s'", config));
-  return (read_rc(config) >= 0);
+  if (read_rc(config) >= 0) {
+    return 1;
+  } else {
+     get_param_bool(BXPN_RESTORE_FLAG)->set(0);
+    return 0;
+  }
 }
 
 bool bx_real_sim_c::restore_logopts()
@@ -1345,8 +1400,8 @@ bool bx_real_sim_c::save_sr_param(FILE *fp, bx_param_c *node, const char *sr_pat
   for (i=0; i<level; i++)
     fprintf(fp, "  ");
   if (node == NULL) {
-      BX_ERROR(("NULL pointer"));
-      return 0;
+    BX_ERROR(("NULL pointer"));
+    return 0;
   }
   fprintf(fp, "%s = ", node->get_name());
   switch (node->get_type()) {
@@ -1490,9 +1545,9 @@ void bx_real_sim_c::init_std_nic_options(const char *name, bx_list_c *menu)
 #endif
 
 #if BX_SUPPORT_PCIUSB
-void bx_real_sim_c::init_usb_options(const char *usb_name, const char *pname, int maxports)
+void bx_real_sim_c::init_usb_options(const char *usb_name, const char *pname, int maxports, int param0)
 {
-  bx_init_usb_options(usb_name, pname, maxports);
+  bx_init_usb_options(usb_name, pname, maxports, param0);
 }
 #endif
 
