@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.h 14189 2021-03-18 19:50:12Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2021  The Bochs Project
+//  Copyright (C) 2001-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -140,7 +140,7 @@ enum {
   BX_BOOT_NONE,
   BX_BOOT_FLOPPYA,
   BX_BOOT_DISKC,
-  BX_BOOT_CDROM, 
+  BX_BOOT_CDROM,
   BX_BOOT_NETWORK
 };
 
@@ -205,6 +205,8 @@ enum {
 
 ///// types and definitions used in event structures
 
+#include "paramtree.h"
+
 #define BX_EVT_IS_ASYNC(type) ((type) > __ALL_EVENTS_BELOW_ARE_ASYNC__)
 
 typedef enum {
@@ -215,6 +217,8 @@ typedef enum {
   BX_SYNC_EVT_LOG_DLG,            // simulator -> CI, wait for response.
   BX_SYNC_EVT_GET_DBG_COMMAND,    // simulator -> CI, wait for response.
   BX_SYNC_EVT_MSG_BOX,            // simulator -> CI, wait for response.
+  BX_SYNC_EVT_ML_MSG_BOX,         // simulator -> CI, do not wait for response.
+  BX_SYNC_EVT_ML_MSG_BOX_KILL,    // simulator -> CI, do not wait for response.
   __ALL_EVENTS_BELOW_ARE_ASYNC__,
   BX_ASYNC_EVT_KEY,               // vga window -> simulator
   BX_ASYNC_EVT_MOUSE,             // vga window -> simulator
@@ -405,6 +409,7 @@ typedef struct {
 typedef struct {
   BxEventType type; // what kind is this?
   Bit32s retcode;   // success or failure. only used for synchronous events.
+  void *param0;     // misc parameter
   union {
     BxKeyEvent key;
     BxMouseEvent mouse;
@@ -587,6 +592,7 @@ public:
   virtual void set_quit_context(jmp_buf *context) {}
   virtual bool get_init_done() { return 0; }
   virtual int set_init_done(bool n) {return 0;}
+  virtual bool get_ci_started() { return 0; }
   virtual void reset_all_param() {}
   // new param methods
   virtual bx_param_c *get_param(const char *pname, bx_param_c *base=NULL) {return NULL;}
@@ -665,6 +671,10 @@ public:
   virtual int ask_yes_no(const char *title, const char *prompt, bool the_default) {return -1;}
   // simple message box
   virtual void message_box(const char *title, const char *message) {}
+  // modeless message box
+  virtual void *ml_message_box(const char *title, const char *message) {return NULL;}
+  // kill modeless message box
+  virtual void ml_message_box_kill(void *ptr) {}
   // called at a regular interval, currently by the bx_devices_c::timer()
   virtual void periodic() {}
   virtual int create_disk_image(const char *filename, int sectors, bool overwrite) {return -3;}
@@ -681,9 +691,9 @@ public:
   // as a toolbar click will be processed.
   virtual void handle_events() {}
   // return first hard disk in ATA interface
-  virtual bx_param_c *get_first_cdrom() {return NULL;}
-  // return first cdrom in ATA interface
   virtual bx_param_c *get_first_hd() {return NULL;}
+  // return first cdrom in ATA interface
+  virtual bx_param_c *get_first_cdrom() {return NULL;}
   // return 1 if device is connected to a PCI slot
   virtual bool is_pci_device(const char *name) {return 0;}
   // return 1 if device is connected to the AGP slot
@@ -729,7 +739,7 @@ public:
   virtual void init_statistics() {}
   virtual void cleanup_statistics() {}
   virtual bx_list_c *get_statistics_root() {return NULL;}
- 
+
   // save/restore support
   virtual void init_save_restore() {}
   virtual void cleanup_save_restore() {}
@@ -743,7 +753,7 @@ public:
   // special config parameter and options functions for plugins
   virtual bool opt_plugin_ctrl(const char *plugname, bool load) {return 0;}
   virtual void init_std_nic_options(const char *name, bx_list_c *menu) {}
-  virtual void init_usb_options(const char *usb_name, const char *pname, int maxports) {}
+  virtual void init_usb_options(const char *usb_name, const char *pname, int maxports, int param0) {}
   virtual int  parse_param_from_list(const char *context, const char *param, bx_list_c *base) {return 0;}
   virtual int  parse_nic_params(const char *context, const char *param, bx_list_c *base) {return 0;}
   virtual int  parse_usb_port_params(const char *context, const char *param,
@@ -761,6 +771,7 @@ public:
 BOCHSAPI extern bx_simulator_interface_c *SIM;
 
 extern void bx_init_siminterface();
+extern int bx_cleanup_siminterface();
 
 #if defined(__WXMSW__) || defined(WIN32)
 // Just to provide HINSTANCE, etc. in files that have not included bochs.h.
