@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: win32.cc 14094 2021-01-30 18:32:52Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2021  The Bochs Project
+//  Copyright (C) 2002-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -51,7 +51,7 @@ public:
   virtual void set_font(bool lg);
   virtual void draw_char(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc, Bit16u yc,
                          Bit8u fw, Bit8u fh, Bit8u fx, Bit8u fy,
-                         bool gfxcharw9, Bit8u cs, Bit8u ce, bool curs);
+                         bool gfxcharw9, Bit8u cs, Bit8u ce, bool curs, bool font2);
   virtual void statusbar_setitem_specific(int element, bool active, bool w);
   virtual void get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp);
   virtual void set_tooltip(unsigned hbar_id, const char *tip);
@@ -131,7 +131,7 @@ static BOOL toolbarVisible, statusVisible;
 static BOOL fullscreenMode, inFullscreenToggle;
 
 // Text mode screen stuff
-static HBITMAP vgafont[256];
+static HBITMAP vgafont[2][256];
 static int xChar = 8, yChar = 16;
 
 // Headerbar stuff
@@ -159,7 +159,7 @@ static BOOL  hideIPS = FALSE;
 static char ipsText[20];
 #endif
 #define BX_SB_MAX_TEXT_ELEMENTS    2
-#define SIZE_OF_SB_ELEMENT        40
+#define SIZE_OF_SB_ELEMENT        50
 #define SIZE_OF_SB_MOUSE_MESSAGE 170
 #define SIZE_OF_SB_IPS_MESSAGE    90
 Bit32u SB_Led_Colors[3] = {0x0000FF00, 0x000040FF, 0x0000FFFF};
@@ -487,7 +487,7 @@ void gen_key_event(Bit32u key, Bit32u press_release)
 LRESULT CALLBACK LowLevelKeyboardProc( int nCode, WPARAM wParam, LPARAM lParam )
 {
   if (nCode < 0 || nCode != HC_ACTION || (!mouseCaptureMode && !fullscreenMode))
-    return CallNextHookEx(hKeyboardHook, nCode, wParam, lParam); 
+    return CallNextHookEx(hKeyboardHook, nCode, wParam, lParam);
 
   KBDLLHOOKSTRUCT* p = (KBDLLHOOKSTRUCT*)lParam;
   Bit32u press_release = (p->flags & LLKHF_UP) ? BX_KEY_RELEASED : BX_KEY_PRESSED;
@@ -588,24 +588,26 @@ void terminateEmul(int reason)
 
   for (unsigned b=0; b<bx_bitmap_entries; b++)
     if (bx_bitmaps[b].bmap) DeleteObject(bx_bitmaps[b].bmap);
-  for (unsigned c=0; c<256; c++)
-    if (vgafont[c]) DeleteObject(vgafont[c]);
+  for (unsigned c=0; c<256; c++) {
+    if (vgafont[0][c]) DeleteObject(vgafont[0][c]);
+    if (vgafont[1][c]) DeleteObject(vgafont[1][c]);
+  }
 
   switch (reason) {
-  case EXIT_GUI_SHUTDOWN:
-    BX_FATAL(("Window closed, exiting!"));
-    break;
-  case EXIT_GMH_FAILURE:
-    BX_FATAL(("GetModuleHandle failure!"));
-    break;
-  case EXIT_FONT_BITMAP_ERROR:
-    BX_FATAL(("Font bitmap creation failure!"));
-    break;
-  case EXIT_HEADER_BITMAP_ERROR:
-    BX_FATAL(("Header bitmap creation failure!"));
-    break;
-  case EXIT_NORMAL:
-    break;
+    case EXIT_GUI_SHUTDOWN:
+      BX_INFO(("Window closed, exiting!"));
+      break;
+    case EXIT_GMH_FAILURE:
+      BX_FATAL(("GetModuleHandle failure!"));
+      break;
+    case EXIT_FONT_BITMAP_ERROR:
+      BX_FATAL(("Font bitmap creation failure!"));
+      break;
+    case EXIT_HEADER_BITMAP_ERROR:
+      BX_FATAL(("Header bitmap creation failure!"));
+      break;
+    case EXIT_NORMAL:
+      break;
   }
 }
 
@@ -665,14 +667,16 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
       } else if (!strcmp(argv[i], "traphotkeys")) {
         BX_INFO(("trap system hotkeys for Bochs window"));
         win32_traphotkeys = 1;
-#if BX_DEBUGGER && BX_DEBUGGER_GUI
       } else if (!strcmp(argv[i], "gui_debug")) {
+#if BX_DEBUGGER && BX_DEBUGGER_GUI
         if (gui_ci) {
           gui_debug = TRUE;
           SIM->set_debug_gui(1);
         } else {
           BX_PANIC(("Config interface 'win32config' is required for gui debugger"));
         }
+#else
+        SIM->message_box("ERROR", "Bochs debugger not available - ignoring 'gui_debug' option");
 #endif
 #if BX_SHOW_IPS
       } else if (!strcmp(argv[i], "hideIPS")) {
@@ -712,7 +716,10 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
   stretched_y = dimension_y;
   stretch_factor = 1;
 
-  for(unsigned c=0; c<256; c++) vgafont[c] = NULL;
+  for (unsigned c = 0; c < 256; c++) {
+    vgafont[0][c] = NULL;
+    vgafont[1][c] = NULL;
+  }
   create_vga_font();
 
   bitmap_info=(BITMAPINFO*)new char[sizeof(BITMAPINFOHEADER)+259*sizeof(RGBQUAD)]; // 256 + 3 entries for 16 bpp mode
@@ -818,6 +825,8 @@ void set_fullscreen_mode(BOOL enable)
     if (saveParent) {
       BX_DEBUG(("Restoring parent window"));
       SetParent(stInfo.mainWnd, saveParent);
+      SetWindowPos(stInfo.mainWnd, HWND_NOTOPMOST,
+        0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
       saveParent = NULL;
     }
     // put back the title bar, border, etc...
@@ -1112,7 +1121,7 @@ LRESULT CALLBACK mainWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
     break;
 
   case WM_DESTROY:
-    PostQuitMessage (0);
+    PostQuitMessage(0);
     return 0;
 
   case WM_SIZE:
@@ -1725,25 +1734,27 @@ void bx_win32_gui_c::set_font(bool lg)
 {
   Bit8u data[64], i;
 
-  for (unsigned c = 0; c<256; c++) {
-    if (char_changed[c]) {
-      memset(data, 0, sizeof(data));
-      BOOL gfxchar = lg && ((c & 0xE0) == 0xC0);
-      for (i=0; i<(unsigned)yChar; i++) {
-        data[i*2] = vga_charmap[c*32+i];
-        if (gfxchar) {
-          data[i*2+1] = (data[i*2] << 7);
+  for (unsigned m = 0; m < 2; m++) {
+    for (unsigned c = 0; c < 256; c++) {
+      if (char_changed[m][c]) {
+        memset(data, 0, sizeof(data));
+        BOOL gfxchar = lg && ((c & 0xE0) == 0xC0);
+        for (i=0; i<(unsigned)yChar; i++) {
+          data[i*2] = vga_charmap[m][c*32+i];
+          if (gfxchar) {
+            data[i*2+1] = (data[i*2] << 7);
+          }
         }
+        SetBitmapBits(vgafont[m][c], 64, data);
+        char_changed[m][c] = 0;
       }
-      SetBitmapBits(vgafont[c], 64, data);
-      char_changed[c] = 0;
     }
   }
 }
 
 void bx_win32_gui_c::draw_char(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc, Bit16u yc,
                                Bit8u fw, Bit8u fh, Bit8u fx, Bit8u fy,
-                               bool gfxcharw9, Bit8u cs, Bit8u ce, bool curs)
+                               bool gfxcharw9, Bit8u cs, Bit8u ce, bool curs, bool font2)
 {
   HDC hdc;
 
@@ -1751,7 +1762,8 @@ void bx_win32_gui_c::draw_char(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc, Bit16u y
 
   EnterCriticalSection(&stInfo.drawCS);
   hdc = GetDC(stInfo.simWnd);
-  DrawBitmap(hdc, vgafont[ch], xc, yc, fw, fh, fx, fy, fc, bc);
+  Bit8u map = (font2) ? 1 : 0;
+  DrawBitmap(hdc, vgafont[map][ch], xc, yc, fw, fh, fx, fy, fc, bc);
   if (curs && (ce >= fy) && (cs < (fh + fy))) {
     if (cs > fy) {
       yc += (cs - fy);
@@ -1760,7 +1772,7 @@ void bx_win32_gui_c::draw_char(Bit8u ch, Bit8u fc, Bit8u bc, Bit16u xc, Bit16u y
     if ((ce - cs + 1) < fh) {
       fh = ce - cs + 1;
     }
-    DrawBitmap(hdc, vgafont[ch], xc, yc, fw, fh, fx, fy, bc, fc);
+    DrawBitmap(hdc, vgafont[map][ch], xc, yc, fw, fh, fx, fy, bc, fc);
   }
   ReleaseDC(stInfo.simWnd, hdc);
   LeaveCriticalSection(&stInfo.drawCS);
@@ -1778,11 +1790,11 @@ int bx_win32_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
   if (OpenClipboard(stInfo.simWnd)) {
     HGLOBAL hg = GetClipboardData(CF_TEXT);
     char *data = (char *)GlobalLock(hg);
-    *nbytes = strlen(data);
+    *nbytes = (Bit32s)strlen(data);
     *bytes = new Bit8u[1 + *nbytes];
-    BX_INFO (("found %d bytes on the clipboard", *nbytes));
+    BX_INFO(("found %d bytes on the clipboard", *nbytes));
     memcpy (*bytes, data, *nbytes+1);
-    BX_INFO (("first byte is 0x%02x", *bytes[0]));
+    BX_INFO(("first byte is 0x%02x", *bytes[0]));
     GlobalUnlock(hg);
     CloseClipboard();
     return 1;
@@ -1879,26 +1891,28 @@ void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
   }
 
   bitmap_info->bmiHeader.biBitCount = bpp;
-  if (bpp == 16) {
-    bitmap_info->bmiHeader.biCompression = BI_BITFIELDS;
-    static RGBQUAD red_mask   = {0x00, 0xF8, 0x00, 0x00};
-    static RGBQUAD green_mask = {0xE0, 0x07, 0x00, 0x00};
-    static RGBQUAD blue_mask  = {0x1F, 0x00, 0x00, 0x00};
-    bitmap_info->bmiColors[256] = bitmap_info->bmiColors[0];
-    bitmap_info->bmiColors[257] = bitmap_info->bmiColors[1];
-    bitmap_info->bmiColors[258] = bitmap_info->bmiColors[2];
-    bitmap_info->bmiColors[0] = red_mask;
-    bitmap_info->bmiColors[1] = green_mask;
-    bitmap_info->bmiColors[2] = blue_mask;
-  } else {
-    if (current_bpp == 16) {
-      bitmap_info->bmiColors[0] = bitmap_info->bmiColors[256];
-      bitmap_info->bmiColors[1] = bitmap_info->bmiColors[257];
-      bitmap_info->bmiColors[2] = bitmap_info->bmiColors[258];
-    }
-    bitmap_info->bmiHeader.biCompression = BI_RGB;
-    if (bpp == 15) {
-      bitmap_info->bmiHeader.biBitCount = 16;
+  if (bpp != current_bpp) {
+    if (bpp == 16) {
+      bitmap_info->bmiHeader.biCompression = BI_BITFIELDS;
+      static RGBQUAD red_mask   = {0x00, 0xF8, 0x00, 0x00};
+      static RGBQUAD green_mask = {0xE0, 0x07, 0x00, 0x00};
+      static RGBQUAD blue_mask  = {0x1F, 0x00, 0x00, 0x00};
+      bitmap_info->bmiColors[256] = bitmap_info->bmiColors[0];
+      bitmap_info->bmiColors[257] = bitmap_info->bmiColors[1];
+      bitmap_info->bmiColors[258] = bitmap_info->bmiColors[2];
+      bitmap_info->bmiColors[0] = red_mask;
+      bitmap_info->bmiColors[1] = green_mask;
+      bitmap_info->bmiColors[2] = blue_mask;
+    } else {
+      if (current_bpp == 16) {
+        bitmap_info->bmiColors[0] = bitmap_info->bmiColors[256];
+        bitmap_info->bmiColors[1] = bitmap_info->bmiColors[257];
+        bitmap_info->bmiColors[2] = bitmap_info->bmiColors[258];
+      }
+      bitmap_info->bmiHeader.biCompression = BI_RGB;
+      if (bpp == 15) {
+        bitmap_info->bmiHeader.biBitCount = 16;
+      }
     }
   }
   current_bpp = guest_bpp = bpp;
@@ -2024,7 +2038,11 @@ void bx_win32_gui_c::exit(void)
   // Wait until it dies
   while ((stInfo.kill == 0) && (workerThreadID != 0)) Sleep(500);
 
-  if (!stInfo.kill) terminateEmul(EXIT_NORMAL);
+  if (bx_user_quit) {
+    terminateEmul(EXIT_NORMAL);
+  } else {
+    terminateEmul(stInfo.kill);
+  }
 }
 
 
@@ -2033,15 +2051,16 @@ void create_vga_font(void)
   unsigned char data[64];
 
   // VGA font is 8 or 9 wide and up to 32 high
-  for (unsigned c = 0; c<256; c++) {
-    vgafont[c] = CreateBitmap(9,32,1,1,NULL);
-    if (!vgafont[c]) terminateEmul(EXIT_FONT_BITMAP_ERROR);
-    memset(data, 0, sizeof(data));
-    for (unsigned i=0; i<16; i++)
-      data[i*2] = reverse_bitorder(bx_vgafont[c].data[i]);
-    SetBitmapBits(vgafont[c], 64, data);
-  }
-}
+  for (unsigned m = 0; m < 2; m++) {
+    for (unsigned c = 0; c<256; c++) {
+      vgafont[m][c] = CreateBitmap(9,32,1,1,NULL);
+      if (!vgafont[m][c]) terminateEmul(EXIT_FONT_BITMAP_ERROR);
+      memset(data, 0, sizeof(data));
+      for (unsigned i=0; i<16; i++)
+        data[i*2] = reverse_bitorder(bx_vgafont[c].data[i]);
+      SetBitmapBits(vgafont[m][c], 64, data);
+    }
+  }}
 
 
 COLORREF GetColorRef(Bit8u pal_idx)
