@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: hpet.cc 14229 2021-04-18 17:20:41Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  High Precision Event Timer emulation ported from Qemu
@@ -9,7 +9,7 @@
 //
 //  Authors: Beth Kon <bkon@us.ibm.com>
 //
-//  Copyright (C) 2017-2021  The Bochs Project
+//  Copyright (C) 2017-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -37,6 +37,23 @@
 
 #include "hpet.h"
 
+/* HPET will set up timers to fire after a certain period of time.
+ * These values can be used to clamp this period to reasonable/supported values.
+ * The values are in ticks.
+ */
+const Bit64u HPET_MAX_ALLOWED_PERIOD = BX_CONST64(0x0400000000000000);  // Must not overflow when multiplied by HPET_CLK_PERIOD
+const Bit64u HPET_MIN_ALLOWED_PERIOD = BX_CONST64(1);
+
+#define HPET_BASE 0xfed00000
+#define HPET_LEN  0x400
+
+#define RTC_ISA_IRQ 8
+
+#define HPET_CLK_PERIOD         10 // 10 ns
+#define HPET_ROUTING_CAP        BX_CONST64(0xffffff) // allowed irq routing
+
+#define FS_PER_NS 1000000       /* 1000000 femtosectons == 1 ns */
+
 #define LOG_THIS theHPET->
 
 bx_hpet_c *theHPET = NULL;
@@ -73,40 +90,40 @@ static Bit32u hpet_time_between(Bit64u start, Bit64u end, Bit64u value)
 /* Returns earliest 64-bit tick value that is after reference
  * and has same lower 32 bits as value
  */
-static Bit64u hpet_cmp32_to_cmp64(Bit64u reference, Bit32u value)
+static BX_CPP_INLINE Bit64u hpet_cmp32_to_cmp64(Bit64u reference, Bit32u value)
 {
   if ((Bit32u)reference <= value) {
-    return (reference & 0xFFFFFFFF00000000ull) | (Bit64u)value;
+    return (reference & BX_CONST64(0xFFFFFFFF00000000)) | (Bit64u)value;
   } else {
-    return ((reference + 0x100000000ull) & 0xFFFFFFFF00000000ull) | (Bit64u)value;
+    return ((reference + BX_CONST64(0x100000000)) & BX_CONST64(0xFFFFFFFF00000000)) | (Bit64u)value;
   }
 }
 
-static Bit64u ticks_to_ns(Bit64u value)
+static BX_CPP_INLINE Bit64u ticks_to_ns(Bit64u value)
 {
-    return value * HPET_CLK_PERIOD;
+  return value * HPET_CLK_PERIOD;
 }
 
-static Bit64u ns_to_ticks(Bit64u value)
+static BX_CPP_INLINE Bit64u ns_to_ticks(Bit64u value)
 {
-    return value / HPET_CLK_PERIOD;
+  return value / HPET_CLK_PERIOD;
 }
 
-static Bit64u hpet_fixup_reg(Bit64u _new, Bit64u old, Bit64u mask)
+static BX_CPP_INLINE Bit64u hpet_fixup_reg(Bit64u _new, Bit64u old, Bit64u mask)
 {
-    _new &= mask;
-    _new |= old & ~mask;
-    return _new;
+  _new &= mask;
+  _new |= old & ~mask;
+  return _new;
 }
 
-static int activating_bit(Bit64u old, Bit64u _new, Bit64u mask)
+static BX_CPP_INLINE int activating_bit(Bit64u old, Bit64u _new, Bit64u mask)
 {
-    return (!(old & mask) && (_new & mask));
+  return (!(old & mask) && (_new & mask));
 }
 
-static int deactivating_bit(Bit64u old, Bit64u _new, Bit64u mask)
+static BX_CPP_INLINE int deactivating_bit(Bit64u old, Bit64u _new, Bit64u mask)
 {
-    return ((old & mask) && !(_new & mask));
+  return ((old & mask) && !(_new & mask));
 }
 
 // static memory read/write functions
@@ -119,24 +136,24 @@ static bool hpet_read(bx_phy_address a20addr, unsigned len, void *data, void *pa
   if (len == 4) { // must be 32-bit aligned
     if ((a20addr & 0x3) != 0) {
       BX_PANIC(("Unaligned HPET read at address 0x" FMT_PHY_ADDRX, a20addr));
-      return 1;
+      return true;
     }
     value1 = theHPET->read_aligned(a20addr);
     *((Bit32u *)data) = value1;
-    return 1;
+    return true;
   } else if (len == 8) { // must be 64-bit aligned
     if ((a20addr & 0x7) != 0) {
       BX_PANIC(("Unaligned HPET read at address 0x" FMT_PHY_ADDRX, a20addr));
-      return 1;
+      return true;
     }
     value1 = theHPET->read_aligned(a20addr);
     value2 = theHPET->read_aligned(a20addr + 4);
     *((Bit64u *)data) = (value1 | (value2 << 32));
-    return 1;
+    return true;
   } else {
     BX_PANIC(("Unsupported HPET read at address 0x" FMT_PHY_ADDRX, a20addr));
   }
-  return 1;
+  return true;
 }
 
 static bool hpet_write(bx_phy_address a20addr, unsigned len, void *data, void *param)
@@ -144,22 +161,22 @@ static bool hpet_write(bx_phy_address a20addr, unsigned len, void *data, void *p
   if (len == 4) { // must be 32-bit aligned
     if ((a20addr & 0x3) != 0) {
       BX_PANIC(("Unaligned HPET write at address 0x" FMT_PHY_ADDRX, a20addr));
-      return 1;
+      return true;
     }
-    theHPET->write_aligned(a20addr, *((Bit32u*) data));
-    return 1;
+    theHPET->write_aligned(a20addr, *((Bit32u*) data), true);
+    return true;
   } else if (len == 8) { // must be 64-bit aligned
     if ((a20addr & 0x7) != 0) {
       BX_PANIC(("Unaligned HPET write at address 0x" FMT_PHY_ADDRX, a20addr));
-      return 1;
+      return true;
     }
     Bit64u val64 = *((Bit64u*) data);
-    theHPET->write_aligned(a20addr, (Bit32u)val64);
-    theHPET->write_aligned(a20addr + 4, (Bit32u)(val64 >> 32));
+    theHPET->write_aligned(a20addr, (Bit32u)val64, false);
+    theHPET->write_aligned(a20addr + 4, (Bit32u)(val64 >> 32), true);
   } else {
     BX_PANIC(("Unsupported HPET write at address 0x" FMT_PHY_ADDRX, a20addr));
   }
-  return 1;
+  return true;
 }
 
 // the device object
@@ -212,6 +229,8 @@ void bx_hpet_c::reset(unsigned type)
   s.hpet_reference_value = BX_CONST64(0);
   s.hpet_reference_time = BX_CONST64(0);
   s.config = BX_CONST64(0);
+  DEV_pit_enable_irq(1);
+  DEV_cmos_enable_irq(1);
 }
 
 void bx_hpet_c::register_state(void)
@@ -241,19 +260,15 @@ Bit64u bx_hpet_c::hpet_get_ticks(void)
 /*
  * calculate diff between comparator value and current ticks
  */
-Bit64u bx_hpet_c::hpet_calculate_diff(HPETTimer *t, Bit64u current)
+Bit64u bx_hpet_c::hpet_calculate_diff(const HPETTimer *t, Bit64u current) const
 {
   if (t->config & HPET_TN_32BIT) {
-    Bit32u diff, cmp;
-
-    cmp = (Bit32u)t->cmp;
-    diff = cmp - (Bit32u)current;
+    Bit32u cmp = (Bit32u)t->cmp;
+    Bit32u diff = cmp - (Bit32u)current;
     return (Bit64u)diff;
   } else {
-    Bit64u diff2, cmp2;
-
-    cmp2 = t->cmp;
-    diff2 = cmp2 - current;
+    Bit64u cmp2 = t->cmp;
+    Bit64u diff2 = cmp2 - current;
     return diff2;
   }
 }
@@ -346,7 +361,7 @@ void bx_hpet_c::hpet_timer()
   }
   hpet_set_timer(t);
   t->last_checked = cur_tick;
-  
+
   Bit64u ticks_passed = ns_to_ticks(cur_time - s.hpet_reference_time);
   if (ticks_passed != 0) {
     s.hpet_reference_time += ticks_to_ns(ticks_passed);
@@ -360,7 +375,7 @@ void bx_hpet_c::hpet_set_timer(HPETTimer *t)
   Bit64u diff = hpet_calculate_diff(t, cur_tick);
   if (diff == 0) {
     if (t->config & HPET_TN_32BIT) {
-      diff = 0x100000000ull;
+      diff = BX_CONST64(0x100000000);
     } else {
       diff = HPET_MAX_ALLOWED_PERIOD;
     }
@@ -370,7 +385,7 @@ void bx_hpet_c::hpet_set_timer(HPETTimer *t)
    */
   if (!timer_is_periodic(t)) {
     if (t->config & HPET_TN_32BIT) {
-      Bit64u wrap_diff = 0x100000000ull - (Bit64u)(Bit32u)cur_tick;
+      Bit64u wrap_diff = BX_CONST64(0x100000000) - (Bit64u)(Bit32u)cur_tick;
       if (wrap_diff < diff) diff = wrap_diff;
     }
   }
@@ -436,7 +451,7 @@ Bit32u bx_hpet_c::read_aligned(bx_phy_address address)
       BX_ERROR(("read: timer id out of range"));
       return 0;
     }
-    HPETTimer *timer = &s.timer[id];
+    const HPETTimer *timer = &s.timer[id];
     switch (index & 0x1f) {
       case HPET_TN_CFG:
         value = (Bit32u)timer->config;
@@ -463,7 +478,7 @@ Bit32u bx_hpet_c::read_aligned(bx_phy_address address)
   return value;
 }
 
-void bx_hpet_c::write_aligned(bx_phy_address address, Bit32u value)
+void bx_hpet_c::write_aligned(bx_phy_address address, Bit32u value, bool trailing_write)
 {
   int i;
   Bit16u index = (Bit16u)(address & 0x3ff);
@@ -582,7 +597,9 @@ void bx_hpet_c::write_aligned(bx_phy_address address, Bit32u value)
           timer->cmp = (timer->cmp & BX_CONST64(0xffffffff00000000)) | new_val;
         }
         timer->period = (timer->period & BX_CONST64(0xffffffff00000000)) | new_val;
-        timer->config &= ~HPET_TN_SETVAL;
+        if (trailing_write) {
+          timer->config &= ~HPET_TN_SETVAL;
+        }
         if (hpet_enabled()) {
           hpet_set_timer(timer);
         }
@@ -593,7 +610,9 @@ void bx_hpet_c::write_aligned(bx_phy_address address, Bit32u value)
           timer->cmp = (timer->cmp & BX_CONST64(0xffffffff)) | (new_val << 32);
         }
         timer->period = (timer->period & BX_CONST64(0xffffffff)) | (new_val << 32);
-        timer->config &= ~HPET_TN_SETVAL;
+        if (trailing_write) {
+          timer->config &= ~HPET_TN_SETVAL;
+        }
         if (hpet_enabled()) {
           hpet_set_timer(timer);
         }
