@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.cc 14229 2021-04-18 17:20:41Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2021  The Bochs Project
@@ -351,11 +351,10 @@ void bx_ioapic_c::register_state(void)
 #if BX_DEBUGGER
 void bx_ioapic_c::debug_dump(int argc, char **argv)
 {
-  int i;
   char buf[1024];
 
   dbg_printf("82093AA I/O APIC\n\n");
-  for (i = 0; i < BX_IOAPIC_NUM_PINS; i++) {
+  for (int i = 0; i < BX_IOAPIC_NUM_PINS; i++) {
     bx_io_redirect_entry_t *entry = ioredtbl + i;
     entry->sprintf_self(buf);
     dbg_printf("entry[%d]: %s\n", i, buf);
