@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: iodev.h 14293 2021-06-27 14:50:26Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2021  The Bochs Project
@@ -120,9 +120,11 @@ class cdrom_base_c;
   else if (io_len == 4) \
     BX_DEBUG(("write PCI register 0x%02X value 0x%08X (len=4)", addr, value));
 
-#define BX_PCI_BAR_TYPE_NONE 0
-#define BX_PCI_BAR_TYPE_MEM  1
-#define BX_PCI_BAR_TYPE_IO   2
+enum {
+  BX_PCI_BAR_TYPE_NONE = 0,
+  BX_PCI_BAR_TYPE_MEM  = 1,
+  BX_PCI_BAR_TYPE_IO   = 2
+};
 
 #define BX_PCI_ADVOPT_NOACPI 0x01
 #define BX_PCI_ADVOPT_NOHPET 0x02
@@ -233,27 +235,27 @@ public:
 
 class BOCHSAPI bx_dma_stub_c : public bx_devmodel_c {
 public:
-  virtual unsigned registerDMA8Channel(
+  virtual bool registerDMA8Channel(
     unsigned channel,
     Bit16u (* dmaRead)(Bit8u *data_byte, Bit16u maxlen),
     Bit16u (* dmaWrite)(Bit8u *data_byte, Bit16u maxlen),
     const char *name)
   {
-    STUBFUNC(dma, registerDMA8Channel); return 0;
+    STUBFUNC(dma, registerDMA8Channel); return false;
   }
-  virtual unsigned registerDMA16Channel(
+  virtual bool registerDMA16Channel(
     unsigned channel,
     Bit16u (* dmaRead)(Bit16u *data_word, Bit16u maxlen),
     Bit16u (* dmaWrite)(Bit16u *data_word, Bit16u maxlen),
     const char *name)
   {
-    STUBFUNC(dma, registerDMA16Channel); return 0;
+    STUBFUNC(dma, registerDMA16Channel); return false;
   }
-  virtual unsigned unregisterDMAChannel(unsigned channel) {
-    STUBFUNC(dma, unregisterDMAChannel); return 0;
+  virtual bool unregisterDMAChannel(unsigned channel) {
+    STUBFUNC(dma, unregisterDMAChannel); return false;
   }
-  virtual unsigned get_TC(void) {
-    STUBFUNC(dma, get_TC); return 0;
+  virtual bool get_TC(void) {
+    STUBFUNC(dma, get_TC); return false;
   }
   virtual void set_DRQ(unsigned channel, bool val) {
     STUBFUNC(dma, set_DRQ);
@@ -629,7 +631,7 @@ BX_CPP_INLINE void DEV_MEM_READ_PHYSICAL(bx_phy_address phy_addr, unsigned len,
 
 BX_CPP_INLINE void DEV_MEM_READ_PHYSICAL_DMA(bx_phy_address phy_addr, unsigned len, Bit8u *ptr)
 {
-  while(len > 0) { 
+  while(len > 0) {
     unsigned remainingInPage = 0x1000 - (phy_addr & 0xfff);
     if (len < remainingInPage) remainingInPage = len;
     BX_MEM(0)->dmaReadPhysicalPage(phy_addr, remainingInPage, ptr);
@@ -657,7 +659,7 @@ BX_CPP_INLINE void DEV_MEM_WRITE_PHYSICAL(bx_phy_address phy_addr, unsigned len,
 
 BX_CPP_INLINE void DEV_MEM_WRITE_PHYSICAL_DMA(bx_phy_address phy_addr, unsigned len, Bit8u *ptr)
 {
-  while(len > 0) { 
+  while(len > 0) {
     unsigned remainingInPage = 0x1000 - (phy_addr & 0xfff);
     if (len < remainingInPage) remainingInPage = len;
     BX_MEM(0)->dmaWritePhysicalPage(phy_addr, remainingInPage, ptr);
