@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: keyboard.cc 14163 2021-02-26 20:37:49Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2021  The Bochs Project
+//  Copyright (C) 2002-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -106,7 +106,6 @@ void bx_keyb_c::resetinternals(bool powerup)
 
 void bx_keyb_c::init(void)
 {
-  BX_DEBUG(("Init $Id: keyboard.cc 14163 2021-02-26 20:37:49Z vruppert $"));
   Bit32u   i;
 
   DEV_register_irq(1, "8042 Keyboard controller");
@@ -190,7 +189,7 @@ void bx_keyb_c::init(void)
 
 void bx_keyb_c::reset(unsigned type)
 {
-  // Nothing here yet
+  BX_KEY_THIS s.kbd_internal_buffer.led_status = 0;
 }
 
 void bx_keyb_c::register_state(void)
@@ -314,7 +313,7 @@ Bit32u bx_keyb_c::read(Bit32u address, unsigned io_len)
 
       DEV_pic_lower_irq(12);
       activate_timer();
-      BX_DEBUG(("[mouse] read from 0x%02x returns 0x%02x", address, val));
+      BX_DEBUG(("[mouse] read from 0x%04x returns 0x%02x", (Bit16u)address, val));
       return val;
     } else if (BX_KEY_THIS s.kbd_controller.outb) { /* kbd byte available */
       val = BX_KEY_THIS s.kbd_controller.kbd_output_buffer;
@@ -334,13 +333,13 @@ Bit32u bx_keyb_c::read(Bit32u address, unsigned io_len)
           // move Q elements towards head of queue by one
           BX_KEY_THIS s.controller_Q[i] = BX_KEY_THIS s.controller_Q[i+1];
         }
-        BX_DEBUG(("s.controller_Qsize: %02X",BX_KEY_THIS s.controller_Qsize));
+        BX_DEBUG(("s.controller_Qsize: %u",BX_KEY_THIS s.controller_Qsize));
         BX_KEY_THIS s.controller_Qsize--;
       }
 
       DEV_pic_lower_irq(1);
       activate_timer();
-      BX_DEBUG(("READ(%02x) = %02x", (unsigned) address, (unsigned) val));
+      BX_DEBUG(("READ(0x%04x) = %02x", (Bit16u)address, val));
       return val;
     } else {
       BX_DEBUG(("num_elements = %d", BX_KEY_THIS s.kbd_internal_buffer.num_elements));
@@ -360,8 +359,7 @@ Bit32u bx_keyb_c::read(Bit32u address, unsigned io_len)
     return val;
   }
 
-  BX_PANIC(("unknown address in io read to keyboard port %x",
-      (unsigned) address));
+  BX_PANIC(("unknown address in io read to keyboard port 0x%x", address));
   return 0; /* keep compiler happy */
 }
 
@@ -383,7 +381,7 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
   Bit8u   command_byte;
   static int kbd_initialized=0;
 
-  BX_DEBUG(("keyboard: 8-bit write to %04x = %02x", (unsigned)address, (unsigned)value));
+  BX_DEBUG(("keyboard: 8-bit write to 0x%04x = 0x%02x", (Bit16u)address, (Bit8u)value));
 
   switch (address) {
     case 0x60: // input buffer
@@ -414,8 +412,7 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
             else if (BX_KEY_THIS s.kbd_controller.allow_irq1  && BX_KEY_THIS s.kbd_controller.outb)
               BX_KEY_THIS s.kbd_controller.irq1_requested = 1;
 
-            BX_DEBUG((" allow_irq12 set to %u",
-              (unsigned) BX_KEY_THIS s.kbd_controller.allow_irq12));
+            BX_DEBUG((" allow_irq12 set to %u", (unsigned) BX_KEY_THIS s.kbd_controller.allow_irq12));
             if (!scan_convert)
               BX_INFO(("keyboard: scan convert turned off"));
 
@@ -424,10 +421,10 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
             }
             break;
           case 0xcb: // write keyboard controller mode
-            BX_DEBUG(("write keyboard controller mode with value %02xh", (unsigned) value));
+            BX_DEBUG(("write keyboard controller mode with value 0x%02x", (Bit8u) value));
             break;
           case 0xd1: // write output port
-            BX_DEBUG(("write output port with value %02xh", (unsigned) value));
+            BX_DEBUG(("write output port with value 0x%02x", (Bit8u)value));
             BX_DEBUG(("write output port : %sable A20",(value & 0x02)?"en":"dis"));
             BX_SET_ENABLE_A20((value & 0x02) != 0);
             if (!(value & 0x01)) {
@@ -454,7 +451,7 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
           default:
             BX_PANIC(("=== unsupported write to port 60h(lastcomm=%02x): %02x",
-              (unsigned) BX_KEY_THIS s.kbd_controller.last_comm, (unsigned) value));
+              (unsigned) BX_KEY_THIS s.kbd_controller.last_comm, (Bit8u) value));
           }
       } else {
         // data byte written last to 0x60
@@ -622,7 +619,7 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
         case 0xc1: // Continuous Input Port Poll, Low
         case 0xc2: // Continuous Input Port Poll, High
         case 0xe0: // Read Test Inputs
-          BX_PANIC(("io write 0x64: command = %02xh", (unsigned) value));
+          BX_PANIC(("io write 0x64: command = 0x%02x", (Bit8u) value));
           break;
 
         case 0xfe: // System (cpu?) Reset, transition to real mode
@@ -633,12 +630,11 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
         default:
           if (value==0xff || (value>=0xf0 && value<=0xfd)) {
             /* useless pulse output bit commands ??? */
-            BX_DEBUG(("io write to port 64h, useless command %02x",
-                (unsigned) value));
+            BX_DEBUG(("io write to port 64h, useless command 0x%02x", (Bit8u) value));
             return;
           }
           BX_ERROR(("unsupported io write to keyboard port %x, value = %x",
-            (unsigned) address, (unsigned) value));
+            address, value));
           break;
       }
       break;
@@ -651,7 +647,7 @@ void bx_keyb_c::write(Bit32u address, Bit32u value, unsigned io_len)
 bool bx_keyb_c::gen_scancode_static(void *dev, Bit32u key)
 {
   ((bx_keyb_c*)dev)->gen_scancode(key);
-  return 1;
+  return true;
 }
 
 void bx_keyb_c::gen_scancode(Bit32u key)
@@ -733,7 +729,7 @@ void bx_keyb_c::set_aux_clock_enable(Bit8u value)
 {
   bool prev_aux_clock_enabled;
 
-  BX_DEBUG(("set_aux_clock_enable(%u)", (unsigned) value));
+  BX_DEBUG(("set_aux_clock_enable(%u)", value));
   if (value==0) {
     BX_KEY_THIS s.kbd_controller.aux_clock_enabled = 0;
   } else {
@@ -748,7 +744,7 @@ void bx_keyb_c::set_aux_clock_enable(Bit8u value)
 Bit8u bx_keyb_c::get_kbd_enable(void)
 {
   BX_DEBUG(("get_kbd_enable(): getting kbd_clock_enabled of: %02x",
-      (unsigned) BX_KEY_THIS s.kbd_controller.kbd_clock_enabled));
+      BX_KEY_THIS s.kbd_controller.kbd_clock_enabled));
 
   return(BX_KEY_THIS s.kbd_controller.kbd_clock_enabled);
 }
@@ -757,7 +753,7 @@ void bx_keyb_c::controller_enQ(Bit8u data, unsigned source)
 {
   // source is 0 for keyboard, 1 for mouse
 
-  BX_DEBUG(("controller_enQ(%02x) source=%02x", (unsigned) data,source));
+  BX_DEBUG(("controller_enQ(%02x) source=%02x", (unsigned) data, source));
 
   // see if we need to Q this byte from the controller
   // remember this includes mouse bytes.
@@ -793,12 +789,6 @@ void bx_keyb_c::kbd_enQ_imm(Bit8u val)
     BX_PANIC(("internal keyboard buffer full (imm)"));
     return;
   }
-
-  /* enqueue scancode in multibyte internal keyboard buffer */
-/*
-  int tail = (BX_KEY_THIS s.kbd_internal_buffer.head + BX_KEY_THIS s.kbd_internal_buffer.num_elements) %
-    BX_KBD_ELEMENTS;
-*/
   BX_KEY_THIS s.kbd_controller.kbd_output_buffer = val;
   BX_KEY_THIS s.kbd_controller.outb = 1;
 
@@ -810,17 +800,15 @@ void bx_keyb_c::kbd_enQ(Bit8u scancode)
 {
   int tail;
 
-  BX_DEBUG(("kbd_enQ(0x%02x)", (unsigned) scancode));
+  BX_DEBUG(("kbd_enQ(0x%02x)", scancode));
 
   if (BX_KEY_THIS s.kbd_internal_buffer.num_elements >= BX_KBD_ELEMENTS) {
-    BX_INFO(("internal keyboard buffer full, ignoring scancode.(%02x)",
-      (unsigned) scancode));
+    BX_INFO(("internal keyboard buffer full, ignoring scancode 0x%02x", scancode));
     return;
   }
 
   /* enqueue scancode in multibyte internal keyboard buffer */
-  BX_DEBUG(("kbd_enQ: putting scancode 0x%02x in internal buffer",
-      (unsigned) scancode));
+  BX_DEBUG(("kbd_enQ: putting scancode 0x%02x in internal buffer", scancode));
   tail = (BX_KEY_THIS s.kbd_internal_buffer.head + BX_KEY_THIS s.kbd_internal_buffer.num_elements) %
    BX_KBD_ELEMENTS;
   BX_KEY_THIS s.kbd_internal_buffer.buffer[tail] = scancode;
@@ -854,11 +842,11 @@ void bx_keyb_c::mouse_enQ(Bit8u mouse_data)
 {
   int tail;
 
-  BX_DEBUG(("mouse_enQ(%02x)", (unsigned) mouse_data));
+  BX_DEBUG(("mouse_enQ(0x%02x)", mouse_data));
 
   if (BX_KEY_THIS s.mouse_internal_buffer.num_elements >= BX_MOUSE_BUFF_SIZE) {
-    BX_ERROR(("[mouse] internal mouse buffer full, ignoring mouse data.(%02x)",
-      (unsigned) mouse_data));
+    BX_ERROR(("[mouse] internal mouse buffer full, ignoring mouse data.(0x%02x)",
+      mouse_data));
     return;
   }
 
@@ -935,7 +923,7 @@ void bx_keyb_c::kbd_ctrl_to_kbd(Bit8u value)
 
   switch (value) {
     case 0x00: // ??? ignore and let OS timeout with no response
-      kbd_enQ(0xFA); // send ACK %%%
+      kbd_enQ(0xFA); // send ACK
       break;
 
     case 0x05: // ???
@@ -946,7 +934,7 @@ void bx_keyb_c::kbd_ctrl_to_kbd(Bit8u value)
 
     case 0xed: // LED Write
       BX_KEY_THIS s.kbd_internal_buffer.expecting_led_write = 1;
-      kbd_enQ_imm(0xFA); // send ACK %%%
+      kbd_enQ_imm(0xFA); // send ACK
       break;
 
     case 0xee: // echo
@@ -1040,9 +1028,9 @@ void bx_keyb_c::timer_handler(void *this_ptr)
 
   retval=class_ptr->periodic(1);
 
-  if(retval&0x01)
+  if (retval & 0x01)
     DEV_pic_raise_irq(1);
-  if(retval&0x02)
+  if (retval & 0x02)
     DEV_pic_raise_irq(12);
 }
 
@@ -1050,8 +1038,6 @@ unsigned bx_keyb_c::periodic(Bit32u usec_delta)
 {
   Bit8u  retval;
 
-  UNUSED(usec_delta);
-
   retval = (Bit8u)BX_KEY_THIS s.kbd_controller.irq1_requested |
            (BX_KEY_THIS s.kbd_controller.irq12_requested << 1);
   BX_KEY_THIS s.kbd_controller.irq1_requested = 0;
