@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: e1000.h 14117 2021-02-01 12:42:12Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Intel(R) 82540EM Gigabit Ethernet support (ported from QEMU)
@@ -12,7 +12,7 @@
 //  Copyright (c) 2007 Dan Aloni
 //  Copyright (c) 2004 Antony T Curtis
 //
-//  Copyright (C) 2011-2021  The Bochs Project
+//  Copyright (C) 2011-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -107,6 +107,8 @@ typedef struct {
   Bit8u devfunc;
   char devname[16];
   char ldevname[32];
+
+  Bit32u io_memaddr;
 } bx_e1000_t;
 
 
@@ -114,7 +116,7 @@ class bx_e1000_c : public bx_pci_device_c {
 public:
   bx_e1000_c();
   virtual ~bx_e1000_c();
-  virtual void init(Bit8u card);
+  virtual void init_card(Bit8u card);
   virtual void reset(unsigned type);
   void         e1000_register_state(bx_list_c *parent, Bit8u card);
   virtual void after_restore_state(void);
