@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit82c54.cc 14109 2021-01-30 23:55:24Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2021  The Bochs Project
+//  Copyright (C) 2001-2024  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -724,6 +724,9 @@ void pit_82C54::write(Bit8u address, Bit8u data)
           thisctr.rw_mode=RW;
           thisctr.bcd_mode=(BCD > 0);
           thisctr.mode=M;
+          // mode 6 is the same as mode 2, mode 7 is the same as mode 3
+          if (thisctr.mode > 5)
+            thisctr.mode &= 0x3;
           switch(RW) {
           case 0x1:
             BX_DEBUG(("Setting read_state to LSB"));
@@ -761,6 +764,7 @@ void pit_82C54::write(Bit8u address, Bit8u data)
       case LSByte_multiple:
         thisctr.inlatch = data;
         thisctr.write_state = MSByte_multiple;
+        thisctr.count_written = 0;
         break;
       case LSByte:
         thisctr.inlatch = data;
@@ -779,13 +783,13 @@ void pit_82C54::write(Bit8u address, Bit8u data)
         BX_ERROR(("write counter in invalid write state."));
         break;
       }
-      if (thisctr.count_written && thisctr.write_state != MSByte_multiple) { 
+      if (thisctr.count_written) {
         thisctr.null_count = 1;
         set_count(thisctr, thisctr.inlatch);
       }
       switch(thisctr.mode) {
       case 0:
-        if (thisctr.write_state==MSByte_multiple) {
+        if (thisctr.count_written) {
           set_OUT(thisctr,0);
         }
         thisctr.next_change_time=1;
@@ -918,7 +922,7 @@ bool pit_82C54::read_OUT(Bit8u cnum)
 {
   if (cnum>MAX_COUNTER) {
     BX_ERROR(("Counter number incorrect in 82C54 read_OUT"));
-    return 0;
+    return false;
   }
 
   return counter[cnum].OUTpin;
@@ -928,7 +932,7 @@ bool pit_82C54::read_GATE(Bit8u cnum)
 {
   if (cnum>MAX_COUNTER) {
     BX_ERROR(("Counter number incorrect in 82C54 read_GATE"));
-    return 0;
+    return false;
   }
 
   return counter[cnum].GATE;
@@ -938,7 +942,7 @@ Bit32u pit_82C54::get_clock_event_time(Bit8u cnum)
 {
   if (cnum>MAX_COUNTER) {
     BX_ERROR(("Counter number incorrect in 82C54 read_GATE"));
-    return 0;
+    return false;
   }
 
   return counter[cnum].next_change_time;
@@ -958,21 +962,11 @@ Bit32u pit_82C54::get_next_event_time(void)
   return out;
 }
 
-Bit16u pit_82C54::get_inlatch(int counternum)
-{
-  return counter[counternum].inlatch;
-}
-
-bool pit_82C54::new_count_ready(int countnum)
+bool pit_82C54::new_count_ready(int countnum) const
 {
   return (counter[countnum].write_state != MSByte_multiple);
 }
 
-Bit8u pit_82C54::get_mode(int countnum)
-{
-  return counter[countnum].mode;
-}
-
 void pit_82C54::set_OUT_handler(Bit8u counternum, out_handler_t outh)
 {
   counter[counternum].out_handler = outh;
