@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: scancodes.h 10209 2011-02-24 22:05:47Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2009  The Bochs Project
@@ -22,7 +22,7 @@
 #define BX_SCANCODES_H
 
 // Translation table of the 8042
-extern unsigned char translation8042[256];
+extern const unsigned char translation8042[256];
 
 typedef struct {
   const char *make;
