@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: serial.cc 14312 2021-07-12 19:05:25Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2021  The Bochs Project
@@ -50,7 +50,7 @@ typedef int SOCKET;
 #define FILE_FLAG_FIRST_PIPE_INSTANCE 0
 #endif
 
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
 #include "serial_raw.h"
 #endif
 
@@ -99,7 +99,7 @@ void serial_init_options(void)
       serial_mode_list, BX_SER_MODE_NULL, BX_SER_MODE_NULL);
     mode->set_ask_format("Choose I/O mode of the serial device [%s] ");
     sprintf(label, "Pathname of the serial device for COM%d", i+1);
-    bx_param_filename_c *path = new bx_param_filename_c(menu, "dev", label, 
+    bx_param_filename_c *path = new bx_param_filename_c(menu, "dev", label,
       "The path can be a real serial device or a pty (X/Unix only)",
       "", BX_PATHNAME_LEN);
     bx_list_c *deplist = new bx_list_c(NULL);
@@ -216,7 +216,7 @@ bx_serial_c::~bx_serial_c(void)
 #endif
           break;
         case BX_SER_MODE_RAW:
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
           delete [] BX_SER_THIS s[i].raw;
 #endif
           break;
@@ -281,15 +281,15 @@ bx_serial_c::init(void)
         DEV_register_irq(BX_SER_THIS s[i].IRQ, name);
       }
       /* internal state */
-      BX_SER_THIS s[i].ls_ipending = 0;
-      BX_SER_THIS s[i].ms_ipending = 0;
-      BX_SER_THIS s[i].rx_ipending = 0;
-      BX_SER_THIS s[i].fifo_ipending = 0;
-      BX_SER_THIS s[i].ls_interrupt = 0;
-      BX_SER_THIS s[i].ms_interrupt = 0;
-      BX_SER_THIS s[i].rx_interrupt = 0;
-      BX_SER_THIS s[i].tx_interrupt = 0;
-      BX_SER_THIS s[i].fifo_interrupt = 0;
+      BX_SER_THIS s[i].ls_ipending = false;
+      BX_SER_THIS s[i].ms_ipending = false;
+      BX_SER_THIS s[i].rx_ipending = false;
+      BX_SER_THIS s[i].fifo_ipending = false;
+      BX_SER_THIS s[i].ls_interrupt = false;
+      BX_SER_THIS s[i].ms_interrupt = false;
+      BX_SER_THIS s[i].rx_interrupt = false;
+      BX_SER_THIS s[i].tx_interrupt = false;
+      BX_SER_THIS s[i].fifo_interrupt = false;
 
       if (BX_SER_THIS s[i].tx_timer_index == BX_NULL_TIMER_HANDLE) {
         BX_SER_THIS s[i].tx_timer_index =
@@ -438,7 +438,7 @@ bx_serial_c::init(void)
         BX_PANIC(("serial terminal support not available"));
 #endif   /* def SERIAL_ENABLE */
       } else if (mode == BX_SER_MODE_RAW) {
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
         BX_SER_THIS s[i].raw = new serial_raw(dev);
         BX_SER_THIS s[i].io_mode = BX_SER_MODE_RAW;
 #else
@@ -563,7 +563,7 @@ bx_serial_c::init(void)
           } else {
             // client mode
             pipe = CreateFile( dev,
-               GENERIC_READ | GENERIC_WRITE, 
+               GENERIC_READ | GENERIC_WRITE,
                0, NULL, OPEN_EXISTING, 0, NULL);
 
             if (pipe == INVALID_HANDLE_VALUE) {
@@ -697,59 +697,59 @@ void bx_serial_c::register_state(void)
 void bx_serial_c::lower_interrupt(Bit8u port)
 {
   /* If there are no more ints pending, clear the irq */
-  if ((BX_SER_THIS s[port].rx_interrupt == 0) &&
-      (BX_SER_THIS s[port].tx_interrupt == 0) &&
-      (BX_SER_THIS s[port].ls_interrupt == 0) &&
-      (BX_SER_THIS s[port].ms_interrupt == 0) &&
-      (BX_SER_THIS s[port].fifo_interrupt == 0)) {
+  if ((BX_SER_THIS s[port].rx_interrupt == false) &&
+      (BX_SER_THIS s[port].tx_interrupt == false) &&
+      (BX_SER_THIS s[port].ls_interrupt == false) &&
+      (BX_SER_THIS s[port].ms_interrupt == false) &&
+      (BX_SER_THIS s[port].fifo_interrupt == false)) {
     DEV_pic_lower_irq(BX_SER_THIS s[port].IRQ);
   }
 }
 
 void bx_serial_c::raise_interrupt(Bit8u port, int type)
 {
-  bool gen_int = 0;
+  bool gen_int = false;
 
   switch (type) {
     case BX_SER_INT_IER: /* IER has changed */
-      gen_int = 1;
+      gen_int = true;
       break;
     case BX_SER_INT_RXDATA:
       if (BX_SER_THIS s[port].int_enable.rxdata_enable) {
-        BX_SER_THIS s[port].rx_interrupt = 1;
-        gen_int = 1;
+        BX_SER_THIS s[port].rx_interrupt = true;
+        gen_int = true;
       } else {
-        BX_SER_THIS s[port].rx_ipending = 1;
+        BX_SER_THIS s[port].rx_ipending = true;
       }
       break;
     case BX_SER_INT_TXHOLD:
       if (BX_SER_THIS s[port].int_enable.txhold_enable) {
-        BX_SER_THIS s[port].tx_interrupt = 1;
-        gen_int = 1;
+        BX_SER_THIS s[port].tx_interrupt = true;
+        gen_int = true;
       }
       break;
     case BX_SER_INT_RXLSTAT:
       if (BX_SER_THIS s[port].int_enable.rxlstat_enable) {
-        BX_SER_THIS s[port].ls_interrupt = 1;
-        gen_int = 1;
+        BX_SER_THIS s[port].ls_interrupt = true;
+        gen_int = true;
       } else {
-        BX_SER_THIS s[port].ls_ipending = 1;
+        BX_SER_THIS s[port].ls_ipending = true;
       }
       break;
     case BX_SER_INT_MODSTAT:
-      if ((BX_SER_THIS s[port].ms_ipending == 1) &&
+      if ((BX_SER_THIS s[port].ms_ipending) &&
           (BX_SER_THIS s[port].int_enable.modstat_enable == 1)) {
-        BX_SER_THIS s[port].ms_interrupt = 1;
-        BX_SER_THIS s[port].ms_ipending = 0;
-        gen_int = 1;
+        BX_SER_THIS s[port].ms_interrupt = true;
+        BX_SER_THIS s[port].ms_ipending = false;
+        gen_int = true;
       }
       break;
     case BX_SER_INT_FIFO:
       if (BX_SER_THIS s[port].int_enable.rxdata_enable) {
-        BX_SER_THIS s[port].fifo_interrupt = 1;
-        gen_int = 1;
+        BX_SER_THIS s[port].fifo_interrupt = true;
+        gen_int = true;
       } else {
-        BX_SER_THIS s[port].fifo_ipending = 1;
+        BX_SER_THIS s[port].fifo_ipending = true;
       }
       break;
   }
@@ -805,17 +805,17 @@ Bit32u bx_serial_c::read(Bit32u address, unsigned io_len)
           }
           if (BX_SER_THIS s[port].rx_fifo_end == 0) {
             BX_SER_THIS s[port].line_status.rxdata_ready = 0;
-            BX_SER_THIS s[port].rx_interrupt = 0;
-            BX_SER_THIS s[port].rx_ipending = 0;
-            BX_SER_THIS s[port].fifo_interrupt = 0;
-            BX_SER_THIS s[port].fifo_ipending = 0;
+            BX_SER_THIS s[port].rx_interrupt = false;
+            BX_SER_THIS s[port].rx_ipending = false;
+            BX_SER_THIS s[port].fifo_interrupt = false;
+            BX_SER_THIS s[port].fifo_ipending = false;
             lower_interrupt(port);
           }
         } else {
           val = BX_SER_THIS s[port].rxbuffer;
           BX_SER_THIS s[port].line_status.rxdata_ready = 0;
-          BX_SER_THIS s[port].rx_interrupt = 0;
-          BX_SER_THIS s[port].rx_ipending = 0;
+          BX_SER_THIS s[port].rx_interrupt = false;
+          BX_SER_THIS s[port].rx_ipending = false;
           lower_interrupt(port);
         }
       }
@@ -855,7 +855,7 @@ Bit32u bx_serial_c::read(Bit32u address, unsigned io_len)
         BX_SER_THIS s[port].int_ident.int_ID = 0x0;
         BX_SER_THIS s[port].int_ident.ipending = 1;
       }
-      BX_SER_THIS s[port].tx_interrupt = 0;
+      BX_SER_THIS s[port].tx_interrupt = false;
       lower_interrupt(port);
 
       val = (Bit8u)BX_SER_THIS s[port].int_ident.ipending  |
@@ -893,13 +893,13 @@ Bit32u bx_serial_c::read(Bit32u address, unsigned io_len)
       BX_SER_THIS s[port].line_status.overrun_error = 0;
       BX_SER_THIS s[port].line_status.framing_error = 0;
       BX_SER_THIS s[port].line_status.break_int = 0;
-      BX_SER_THIS s[port].ls_interrupt = 0;
-      BX_SER_THIS s[port].ls_ipending = 0;
+      BX_SER_THIS s[port].ls_interrupt = false;
+      BX_SER_THIS s[port].ls_ipending = false;
       lower_interrupt(port);
       break;
 
     case BX_SER_MSR: /* MODEM status register */
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
       if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
         bool prev_cts = BX_SER_THIS s[port].modem_status.cts;
         bool prev_dsr = BX_SER_THIS s[port].modem_status.dsr;
@@ -936,8 +936,8 @@ Bit32u bx_serial_c::read(Bit32u address, unsigned io_len)
       BX_SER_THIS s[port].modem_status.delta_dsr = 0;
       BX_SER_THIS s[port].modem_status.ri_trailedge = 0;
       BX_SER_THIS s[port].modem_status.delta_dcd = 0;
-      BX_SER_THIS s[port].ms_interrupt = 0;
-      BX_SER_THIS s[port].ms_ipending = 0;
+      BX_SER_THIS s[port].ms_interrupt = false;
+      BX_SER_THIS s[port].ms_ipending = false;
       lower_interrupt(port);
       break;
 
@@ -972,9 +972,9 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
 #else
   UNUSED(this_ptr);
 #endif  // !BX_USE_SER_SMF
-  bool gen_int = 0;
+  bool gen_int = false;
   Bit8u offset, new_wordlen;
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
   bool mcr_changed = 0;
   Bit8u p_mode;
 #endif
@@ -1045,7 +1045,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
                                           0); /* not continuous */
             }
           } else {
-            BX_SER_THIS s[port].tx_interrupt = 0;
+            BX_SER_THIS s[port].tx_interrupt = false;
             lower_interrupt(port);
           }
         } else {
@@ -1069,15 +1069,15 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
         if (new_b3 != BX_SER_THIS s[port].int_enable.modstat_enable) {
           BX_SER_THIS s[port].int_enable.modstat_enable  = new_b3;
           if (BX_SER_THIS s[port].int_enable.modstat_enable == 1) {
-            if (BX_SER_THIS s[port].ms_ipending == 1) {
-              BX_SER_THIS s[port].ms_interrupt = 1;
-              BX_SER_THIS s[port].ms_ipending = 0;
-              gen_int = 1;
+            if (BX_SER_THIS s[port].ms_ipending) {
+              BX_SER_THIS s[port].ms_interrupt = true;
+              BX_SER_THIS s[port].ms_ipending = false;
+              gen_int = true;
             }
           } else {
-            if (BX_SER_THIS s[port].ms_interrupt == 1) {
-              BX_SER_THIS s[port].ms_interrupt = 0;
-              BX_SER_THIS s[port].ms_ipending = 1;
+            if (BX_SER_THIS s[port].ms_interrupt) {
+              BX_SER_THIS s[port].ms_interrupt = false;
+              BX_SER_THIS s[port].ms_ipending = true;
               lower_interrupt(port);
             }
           }
@@ -1086,34 +1086,34 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
           BX_SER_THIS s[port].int_enable.txhold_enable  = new_b1;
           if (BX_SER_THIS s[port].int_enable.txhold_enable == 1) {
             BX_SER_THIS s[port].tx_interrupt = BX_SER_THIS s[port].line_status.thr_empty;
-            if (BX_SER_THIS s[port].tx_interrupt) gen_int = 1;
+            if (BX_SER_THIS s[port].tx_interrupt) gen_int = true;
           } else {
-            BX_SER_THIS s[port].tx_interrupt = 0;
+            BX_SER_THIS s[port].tx_interrupt = false;
             lower_interrupt(port);
           }
         }
         if (new_b0 != BX_SER_THIS s[port].int_enable.rxdata_enable) {
           BX_SER_THIS s[port].int_enable.rxdata_enable  = new_b0;
           if (BX_SER_THIS s[port].int_enable.rxdata_enable == 1) {
-            if (BX_SER_THIS s[port].fifo_ipending == 1) {
-              BX_SER_THIS s[port].fifo_interrupt = 1;
-              BX_SER_THIS s[port].fifo_ipending = 0;
-              gen_int = 1;
+            if (BX_SER_THIS s[port].fifo_ipending) {
+              BX_SER_THIS s[port].fifo_interrupt = true;
+              BX_SER_THIS s[port].fifo_ipending = false;
+              gen_int = true;
             }
-            if (BX_SER_THIS s[port].rx_ipending == 1) {
-              BX_SER_THIS s[port].rx_interrupt = 1;
-              BX_SER_THIS s[port].rx_ipending = 0;
-              gen_int = 1;
+            if (BX_SER_THIS s[port].rx_ipending) {
+              BX_SER_THIS s[port].rx_interrupt = true;
+              BX_SER_THIS s[port].rx_ipending = false;
+              gen_int = true;
             }
           } else {
-            if (BX_SER_THIS s[port].rx_interrupt == 1) {
-              BX_SER_THIS s[port].rx_interrupt = 0;
-              BX_SER_THIS s[port].rx_ipending = 1;
+            if (BX_SER_THIS s[port].rx_interrupt) {
+              BX_SER_THIS s[port].rx_interrupt = false;
+              BX_SER_THIS s[port].rx_ipending = true;
               lower_interrupt(port);
             }
-            if (BX_SER_THIS s[port].fifo_interrupt == 1) {
-              BX_SER_THIS s[port].fifo_interrupt = 0;
-              BX_SER_THIS s[port].fifo_ipending = 1;
+            if (BX_SER_THIS s[port].fifo_interrupt) {
+              BX_SER_THIS s[port].fifo_interrupt = false;
+              BX_SER_THIS s[port].fifo_ipending = true;
               lower_interrupt(port);
             }
           }
@@ -1121,15 +1121,15 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
         if (new_b2 != BX_SER_THIS s[port].int_enable.rxlstat_enable) {
           BX_SER_THIS s[port].int_enable.rxlstat_enable  = new_b2;
           if (BX_SER_THIS s[port].int_enable.rxlstat_enable == 1) {
-            if (BX_SER_THIS s[port].ls_ipending == 1) {
-              BX_SER_THIS s[port].ls_interrupt = 1;
-              BX_SER_THIS s[port].ls_ipending = 0;
-              gen_int = 1;
+            if (BX_SER_THIS s[port].ls_ipending) {
+              BX_SER_THIS s[port].ls_interrupt = true;
+              BX_SER_THIS s[port].ls_ipending = false;
+              gen_int = true;
             }
           } else {
-            if (BX_SER_THIS s[port].ls_interrupt == 1) {
-              BX_SER_THIS s[port].ls_interrupt = 0;
-              BX_SER_THIS s[port].ls_ipending = 1;
+            if (BX_SER_THIS s[port].ls_interrupt) {
+              BX_SER_THIS s[port].ls_interrupt = false;
+              BX_SER_THIS s[port].ls_ipending = true;
               lower_interrupt(port);
             }
           }
@@ -1156,7 +1156,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
     case BX_SER_LCR: /* Line control register */
       new_wordlen = value & 0x03;
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
       if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
         if (BX_SER_THIS s[port].line_cntl.wordlen_sel != new_wordlen) {
           BX_SER_THIS s[port].raw->set_data_bits(new_wordlen + 5);
@@ -1179,7 +1179,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
           BX_SER_THIS s[port].raw->set_break(new_b6);
         }
       }
-#endif // USE_RAW_SERIAL
+#endif // BX_USE_RAW_SERIAL
       /* These are ignored, but set them up so they can be read back */
       BX_SER_THIS s[port].line_cntl.stopbits = new_b2;
       BX_SER_THIS s[port].line_cntl.parity_enable = new_b3;
@@ -1201,7 +1201,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
             BX_SER_THIS s[port].baudrate = new_baudrate;
             restart_timer = 1;
             BX_DEBUG(("com%d: baud rate set to %d", port+1, BX_SER_THIS s[port].baudrate));
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
             if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
               BX_SER_THIS s[port].raw->set_baudrate(BX_SER_THIS s[port].baudrate);
             }
@@ -1238,7 +1238,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
           BX_SER_THIS detect_mouse = 2;
         }
       }
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
       if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
         mcr_changed = (BX_SER_THIS s[port].modem_cntl.dtr != new_b0) |
                       (BX_SER_THIS s[port].modem_cntl.rts != new_b1);
@@ -1253,7 +1253,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
         BX_SER_THIS s[port].modem_cntl.local_loopback = new_b4;
         if (BX_SER_THIS s[port].modem_cntl.local_loopback) {
           /* transition to loopback mode */
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
           if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
             if (BX_SER_THIS s[port].modem_cntl.dtr ||
                 BX_SER_THIS s[port].modem_cntl.rts) {
@@ -1262,7 +1262,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
           }
 #endif
           if (BX_SER_THIS s[port].line_cntl.break_cntl) {
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
             if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
               BX_SER_THIS s[port].raw->set_break(0);
             }
@@ -1273,7 +1273,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
           }
         } else {
           /* transition to normal mode */
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
           if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
             mcr_changed = 1;
             if (BX_SER_THIS s[port].line_cntl.break_cntl) {
@@ -1295,19 +1295,19 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
         BX_SER_THIS s[port].modem_status.dcd = BX_SER_THIS s[port].modem_cntl.out2;
         if (BX_SER_THIS s[port].modem_status.cts != prev_cts) {
           BX_SER_THIS s[port].modem_status.delta_cts = 1;
-          BX_SER_THIS s[port].ms_ipending = 1;
+          BX_SER_THIS s[port].ms_ipending = true;
         }
         if (BX_SER_THIS s[port].modem_status.dsr != prev_dsr) {
           BX_SER_THIS s[port].modem_status.delta_dsr = 1;
-          BX_SER_THIS s[port].ms_ipending = 1;
+          BX_SER_THIS s[port].ms_ipending = true;
         }
         if (BX_SER_THIS s[port].modem_status.ri != prev_ri)
-          BX_SER_THIS s[port].ms_ipending = 1;
+          BX_SER_THIS s[port].ms_ipending = true;
         if ((BX_SER_THIS s[port].modem_status.ri == 0) && (prev_ri == 1))
           BX_SER_THIS s[port].modem_status.ri_trailedge = 1;
         if (BX_SER_THIS s[port].modem_status.dcd != prev_dcd) {
           BX_SER_THIS s[port].modem_status.delta_dcd = 1;
-          BX_SER_THIS s[port].ms_ipending = 1;
+          BX_SER_THIS s[port].ms_ipending = true;
         }
         raise_interrupt(port, BX_SER_INT_MODSTAT);
       } else {
@@ -1337,7 +1337,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
         }
 
         if (BX_SER_THIS s[port].io_mode == BX_SER_MODE_RAW) {
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
           if (mcr_changed) {
             BX_SER_THIS s[port].raw->set_modem_control(value & 0x03);
           }
@@ -1373,7 +1373,7 @@ void bx_serial_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
 void bx_serial_c::rx_fifo_enq(Bit8u port, Bit8u data)
 {
-  bool gen_int = 0;
+  bool gen_int = false;
 
   if (BX_SER_THIS s[port].fifo_cntl.enable) {
     if (BX_SER_THIS s[port].rx_fifo_end == 16) {
@@ -1386,16 +1386,16 @@ void bx_serial_c::rx_fifo_enq(Bit8u port, Bit8u data)
       BX_SER_THIS s[port].rx_fifo[BX_SER_THIS s[port].rx_fifo_end++] = data;
       switch (BX_SER_THIS s[port].fifo_cntl.rxtrigger) {
         case 1:
-          if (BX_SER_THIS s[port].rx_fifo_end == 4) gen_int = 1;
+          if (BX_SER_THIS s[port].rx_fifo_end == 4) gen_int = true;
           break;
         case 2:
-          if (BX_SER_THIS s[port].rx_fifo_end == 8) gen_int = 1;
+          if (BX_SER_THIS s[port].rx_fifo_end == 8) gen_int = true;
           break;
         case 3:
-          if (BX_SER_THIS s[port].rx_fifo_end == 14) gen_int = 1;
+          if (BX_SER_THIS s[port].rx_fifo_end == 14) gen_int = true;
           break;
         default:
-          gen_int = 1;
+          gen_int = true;
       }
       if (gen_int) {
         bx_pc_system.deactivate_timer(BX_SER_THIS s[port].fifo_timer_index);
@@ -1428,7 +1428,7 @@ void bx_serial_c::tx_timer_handler(void *this_ptr)
 
 void bx_serial_c::tx_timer(void)
 {
-  bool gen_int = 0;
+  bool gen_int = false;
   Bit8u port = (Bit8u)bx_pc_system.triggeredTimerParam();
 
   switch (BX_SER_THIS s[port].io_mode) {
@@ -1456,7 +1456,7 @@ void bx_serial_c::tx_timer(void)
 #endif
       break;
     case BX_SER_MODE_RAW:
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
       if (!BX_SER_THIS s[port].raw->ready_transmit())
         BX_PANIC(("com%d: not ready to transmit", port+1));
       BX_SER_THIS s[port].raw->transmit(BX_SER_THIS s[port].tsrbuffer);
@@ -1493,7 +1493,7 @@ void bx_serial_c::tx_timer(void)
   } else if (!BX_SER_THIS s[port].line_status.thr_empty) {
     BX_SER_THIS s[port].tsrbuffer = BX_SER_THIS s[port].thrbuffer;
     BX_SER_THIS s[port].line_status.tsr_empty = 0;
-    gen_int = 1;
+    gen_int = true;
   }
   if (!BX_SER_THIS s[port].line_status.tsr_empty) {
     if (gen_int) {
@@ -1564,7 +1564,7 @@ void bx_serial_c::rx_timer(void)
 #endif
         break;
       case BX_SER_MODE_RAW:
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
         int data;
         if ((data_ready = BX_SER_THIS s[port].raw->ready_receive())) {
           data = BX_SER_THIS s[port].raw->receive();
@@ -1654,8 +1654,7 @@ void bx_serial_c::rx_timer(void)
     db_usec *= 4;
   }
 
-  bx_pc_system.activate_timer(BX_SER_THIS s[port].rx_timer_index,
-                              db_usec, 0); /* not continuous */
+  bx_pc_system.activate_timer(BX_SER_THIS s[port].rx_timer_index, db_usec, 0); /* not continuous */
 }
 
 void bx_serial_c::fifo_timer_handler(void *this_ptr)
@@ -1713,7 +1712,6 @@ void bx_serial_c::update_mouse_data()
   Bit8u b1, b2, b3, button_state, mouse_data[5];
   int bytes, tail;
 
-
   if (BX_SER_THIS mouse_delayed_dx > 127) {
     delta_x = 127;
     BX_SER_THIS mouse_delayed_dx -= 127;
