@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: serial_raw.cc 14112 2021-01-31 10:50:53Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004-2021  The Bochs Project
@@ -26,7 +26,7 @@
 
 #include "iodev.h"
 
-#if USE_RAW_SERIAL
+#if BX_USE_RAW_SERIAL
 
 #include "serial_raw.h"
 
