@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: virt_timer.cc 14109 2021-01-30 23:55:24Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2021  The Bochs Project
@@ -62,9 +62,10 @@
 #include "param_names.h"
 #include "virt_timer.h"
 
-//Important constant #defines:
-#define USEC_PER_SECOND (1000000)
+const Bit64u BX_MAX_VIRTUAL_TIME = BX_CONST64(0x7fffffff);
 
+//Important constant #defines:
+const Bit64u USEC_PER_SECOND = BX_CONST64(1000000);
 
 // define a macro to convert floating point numbers into 64-bit integers.
 // In MSVC++ you can convert a 64-bit float into a 64-bit signed integer,
@@ -84,7 +85,7 @@
 //Minimum number of emulated useconds per second.
 //  Now calculated using BX_MIN_IPS, the minimum number of
 //   instructions per second.
-#define MIN_USEC_PER_SECOND (((((Bit64u)USEC_PER_SECOND)*((Bit64u)BX_MIN_IPS))/((Bit64u)ips))+(Bit64u)1)
+#define MIN_USEC_PER_SECOND (((USEC_PER_SECOND*((Bit64u)BX_MIN_IPS))/((Bit64u)ips))+(Bit64u)1)
 
 
 //DEBUG configuration:
@@ -172,7 +173,7 @@ void bx_virt_timer_c::periodic(Bit64u time_passed, bool mode)
   s[mode].timers_next_event_time = s[mode].current_timers_time + BX_MAX_VIRTUAL_TIME;
   for (i=0;i<numTimers;i++) {
     if (timer[i].inUse && timer[i].active && (timer[i].realtime == mode) &&
-        ((timer[i].timeToFire)<s[mode].timers_next_event_time)) {
+        (timer[i].timeToFire < s[mode].timers_next_event_time)) {
       s[mode].timers_next_event_time = timer[i].timeToFire;
     }
   }
@@ -221,12 +222,12 @@ int bx_virt_timer_c::register_timer(void *this_ptr, bx_timer_handler_t handler,
                                     const char *id)
 {
   //We don't like starting with a zero period timer.
-  BX_ASSERT((!active) || (useconds>0));
+  BX_ASSERT(!active || useconds > 0);
 
   //Search for an unused timer.
   unsigned int i;
   for (i=0; i < numTimers; i++) {
-    if ((timer[i].inUse == 0) || (i == numTimers))
+    if (!timer[i].inUse || (i == numTimers))
       break;
   }
   // If we didn't find a free slot, increment the bound, numTimers.
@@ -265,13 +266,13 @@ bool bx_virt_timer_c::unregisterTimer(unsigned timerID)
 
   if (timer[timerID].active) {
     BX_PANIC(("unregisterTimer: timer '%s' is still active!", timer[timerID].id));
-    return(0); // Fail.
+    return false; // Fail
   }
 
   //No need to prevent doing this to unused timers.
   timer[timerID].inUse = 0;
   if (timerID == (numTimers-1)) numTimers--;
-  return 1;
+  return true;
 }
 
 void bx_virt_timer_c::start_timers(void)
@@ -280,8 +281,7 @@ void bx_virt_timer_c::start_timers(void)
 }
 
 //activate a deactivated but registered timer.
-void bx_virt_timer_c::activate_timer(unsigned timer_index, Bit32u useconds,
-                                     bool continuous)
+void bx_virt_timer_c::activate_timer(unsigned timer_index, Bit32u useconds, bool continuous)
 {
   BX_ASSERT(timer_index < BX_MAX_VIRTUAL_TIMERS);
 
@@ -307,7 +307,7 @@ void bx_virt_timer_c::deactivate_timer(unsigned timer_index)
   BX_ASSERT(timer_index < BX_MAX_VIRTUAL_TIMERS);
 
   //No need to prevent doing this to unused/inactive timers.
-  timer[timer_index].active = 0;
+  timer[timer_index].active = false;
 }
 
 void bx_virt_timer_c::advance_virtual_time(Bit64u time_passed, bool mode)
@@ -427,15 +427,12 @@ void bx_virt_timer_c::timer_handler(bool mode)
 {
   if (!mode) {
     Bit64u temp_final_time = bx_pc_system.time_usec();
+    BX_ASSERT(temp_final_time >= s[0].current_virtual_time);
     temp_final_time -= s[0].current_virtual_time;
     while (temp_final_time) {
-      if ((temp_final_time)>(s[0].virtual_next_event_time)) {
-        temp_final_time -= s[0].virtual_next_event_time;
-        advance_virtual_time(s[0].virtual_next_event_time, 0);
-      } else {
-        advance_virtual_time(temp_final_time, 0);
-        temp_final_time -= temp_final_time;
-      }
+      Bit64u step = (temp_final_time < s[0].virtual_next_event_time) ? temp_final_time : s[0].virtual_next_event_time;
+      advance_virtual_time(step, 0);
+      temp_final_time -= step;
     }
     bx_pc_system.activate_timer(s[0].system_timer_id,
                                 (Bit32u)BX_MIN(0x7FFFFFFF,(s[0].virtual_next_event_time>2)?(s[0].virtual_next_event_time-2):1),
@@ -443,7 +440,7 @@ void bx_virt_timer_c::timer_handler(bool mode)
     return;
   }
 
-  Bit64u usec_delta = bx_pc_system.time_usec()-last_usec;
+  Bit64u usec_delta = bx_pc_system.time_usec() - last_usec;
 
   if (usec_delta) {
 #if BX_HAVE_REALTIME_USEC
@@ -465,7 +462,7 @@ void bx_virt_timer_c::timer_handler(bool mode)
       //  probably only an issue on startup, but it solves some problems.
       ticks_delta = 0;
     }
-    if (ticks_delta + total_ticks - last_realtime_ticks > (F2I(MAX_MULT * I2F(last_realtime_delta)))) {
+    if (ticks_delta + total_ticks - last_realtime_ticks > F2I(MAX_MULT * I2F(last_realtime_delta))) {
       //This keeps us from going too fast in relation to real time.
 #if 0
       ticks_delta = (F2I(MAX_MULT * I2F(last_realtime_delta))) + last_realtime_ticks - total_ticks;
@@ -483,7 +480,7 @@ void bx_virt_timer_c::timer_handler(bool mode)
 
     if (ticks_delta) {
 
-#  if DEBUG_REALTIME_WITH_PRINTF
+#if DEBUG_REALTIME_WITH_PRINTF
       //Every second print some info.
       if (((last_real_time + real_time_delta) / USEC_PER_SECOND) > (last_real_time / USEC_PER_SECOND)) {
         Bit64u temp1, temp2, temp3, temp4;
@@ -496,7 +493,7 @@ void bx_virt_timer_c::timer_handler(bool mode)
         printf("ticks: " FMT_LL "u, ", temp3);
         printf("diff: " FMT_LL "u\n", temp4);
       }
-#  endif
+#endif
 
       last_real_time += real_time_delta;
       total_real_usec += real_time_delta;
@@ -511,10 +508,10 @@ void bx_virt_timer_c::timer_handler(bool mode)
     if (real_time_delta) {
       //FIXME
       Bit64u em_realtime_delta = last_system_usec + stored_delta - em_last_realtime;
-      b=((Bit64u)USEC_PER_SECOND * em_realtime_delta / real_time_delta);
+      b = (USEC_PER_SECOND * em_realtime_delta / real_time_delta);
       em_last_realtime = last_system_usec + stored_delta;
     } else {
-      b=a;
+      b = a;
     }
     usec_per_second = ALPHA_LOWER(a,b);
 #else
@@ -525,7 +522,7 @@ void bx_virt_timer_c::timer_handler(bool mode)
 #endif
   }
 
-  last_usec=last_usec + usec_delta;
+  last_usec += usec_delta;
   bx_pc_system.deactivate_timer(s[1].system_timer_id);
   BX_ASSERT(s[1].virtual_next_event_time);
   bx_pc_system.activate_timer(s[1].system_timer_id,
