@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logio.cc 14100 2021-01-30 19:40:18Z sshwarts $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2020  The Bochs Project
+//  Copyright (C) 2001-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -171,6 +171,7 @@ void iofunctions::init_log(int fd)
   init_log(tmpfd);
 };
 
+// called at simulation exit
 void iofunctions::exit_log()
 {
   flush();
@@ -182,6 +183,13 @@ void iofunctions::exit_log()
   }
 }
 
+// called at application exit
+void iofunctions::exit_log2()
+{
+  delete log;
+  delete genlog;
+}
+
 // all other functions may use genlog safely.
 #define LOG_THIS genlog->
 
