@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: main.cc 14204 2021-03-27 17:23:31Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2021  The Bochs Project
+//  Copyright (C) 2001-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -68,6 +68,7 @@ int  bx_init_main(int argc, char *argv[]);
 void bx_init_hardware(void);
 void bx_plugin_ctrl_reset(bool init_done);
 void bx_init_options(void);
+void bx_cleanup_options(void);
 void bx_init_bx_dbg(void);
 
 static const char *divider = "========================================================================";
@@ -104,7 +105,7 @@ char *bochsrc_filename = NULL;
 
 size_t bx_get_timestamp(char *buffer)
 {
-#if VER_SVNFLAG == 1
+#if VER_DEVFLAG == 1
 #ifdef __DATE__
 #ifdef __TIME__
   sprintf(buffer, "Compiled on %s at %s", __DATE__, __TIME__);
@@ -352,7 +353,9 @@ int bxmain(void)
 #endif
   plugin_cleanup();
   BX_INSTR_EXIT_ENV();
-  return SIM->get_exit_code();
+  int exit_code = bx_cleanup_siminterface();
+  bx_cleanup_options();
+  return exit_code;
 }
 
 #if defined(__WXMSW__)
@@ -911,11 +914,13 @@ int bx_init_main(int argc, char *argv[])
   }
 
   if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    if (!SIM->restore_config()) {
+      BX_PANIC(("cannot restore configuration"));
+    }
     if (arg < argc) {
       BX_ERROR(("WARNING: bochsrc options are ignored in restore mode!"));
     }
-  }
-  else {
+  } else {
     // parse the rest of the command line.  This is done after reading the
     // configuration file so that the command line arguments can override
     // the settings from the file.
@@ -974,12 +979,7 @@ bool load_and_init_display_lib(void)
 int bx_begin_simulation(int argc, char *argv[])
 {
   bx_user_quit = 0;
-  if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
-    if (!SIM->restore_config()) {
-      BX_PANIC(("cannot restore configuration"));
-      SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
-    }
-  } else {
+  if (!SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
     // make sure all optional plugins have been loaded
     SIM->opt_plugin_ctrl("*", 1);
   }
@@ -1327,10 +1327,13 @@ void bx_init_hardware()
   bx_param_num_c *bxp_host_memsize = SIM->get_param_num(BXPN_HOST_MEM_SIZE);
   Bit64u hostMemSize = bxp_host_memsize->get64() * BX_CONST64(1024*1024);
 
-  // do not allocate more host memory than needed for emulation of guest RAM 
+  // do not allocate more host memory than needed for emulation of guest RAM
   if (memSize < hostMemSize) hostMemSize = memSize;
 
-  BX_MEM(0)->init_memory(memSize, hostMemSize);
+  bx_param_num_c *bxp_memblock_size = SIM->get_param_num(BXPN_MEM_BLOCK_SIZE);
+  Bit32u memBlockSize = bxp_memblock_size->get64() * 1024;
+
+  BX_MEM(0)->init_memory(memSize, hostMemSize, memBlockSize);
 
   // First load the system BIOS (VGABIOS loading moved to the vga code)
   BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_ROM_PATH)->getptr(),
@@ -1384,16 +1387,21 @@ void bx_init_hardware()
   bx_pc_system.Reset(BX_RESET_HARDWARE);
 
   if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
+    void *hwnd = SIM->ml_message_box("Please wait.", "Waiting for the 'restore state' to finish.\n"
+                 "Depending on the size of your emulated disk images, this may be a little while.");
     if (SIM->restore_hardware()) {
       if (!SIM->restore_logopts()) {
+        SIM->ml_message_box_kill(hwnd);
         BX_PANIC(("cannot restore log options"));
         SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
       }
       bx_sr_after_restore_state();
     } else {
+      SIM->ml_message_box_kill(hwnd);
       BX_PANIC(("cannot restore hardware state"));
       SIM->get_param_bool(BXPN_RESTORE_FLAG)->set(0);
     }
+    SIM->ml_message_box_kill(hwnd);
   }
 
   bx_gui->init_signal_handlers();
