@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: param_names.h 14017 2020-12-04 17:04:40Z vruppert $
+// $Id$
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009-2020  The Bochs Project
+//  Copyright (C) 2009-2023  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -79,12 +79,15 @@
 #define BXPN_CPUID_FSGSBASE              "cpuid.fsgsbase"
 #define BXPN_CPUID_SMEP                  "cpuid.smep"
 #define BXPN_CPUID_SMAP                  "cpuid.smap"
-#define BXPN_MEM_SIZE                    "memory.standard.ram.size"
-#define BXPN_HOST_MEM_SIZE               "memory.standard.ram.host_size"
+#define BXPN_MEMORY                      "memory.standard.ram"
+#define BXPN_MEM_SIZE                    "memory.standard.ram.guest"
+#define BXPN_HOST_MEM_SIZE               "memory.standard.ram.host"
+#define BXPN_MEM_BLOCK_SIZE              "memory.standard.ram.block_size"
 #define BXPN_ROMIMAGE                    "memory.standard.rom"
 #define BXPN_ROM_PATH                    "memory.standard.rom.file"
 #define BXPN_ROM_ADDRESS                 "memory.standard.rom.address"
 #define BXPN_ROM_OPTIONS                 "memory.standard.rom.options"
+#define BXPN_ROM_FLASH_DATA              "memory.standard.rom.flash_data"
 #define BXPN_VGA_ROMIMAGE                "memory.standard.vgarom"
 #define BXPN_VGA_ROM_PATH                "memory.standard.vgarom.file"
 #define BXPN_OPTROM_BASE                 "memory.optrom"
@@ -156,8 +159,11 @@
 #define BXPN_OHCI_ENABLED                "ports.usb.ohci.enabled"
 #define BXPN_USB_EHCI                    "ports.usb.ehci"
 #define BXPN_EHCI_ENABLED                "ports.usb.ehci.enabled"
+#define BXPN_EHCI_COMPANION              "ports.usb.ehci.companion"
 #define BXPN_USB_XHCI                    "ports.usb.xhci"
 #define BXPN_XHCI_ENABLED                "ports.usb.xhci.enabled"
+#define BXPN_XHCI_MODEL                  "ports.usb.xhci.model"
+#define BXPN_XHCI_N_PORTS                "ports.usb.xhci.n_ports"
 #define BXPN_NE2K                        "network.ne2k"
 #define BXPN_PNIC                        "network.pcipnic"
 #define BXPN_E1000                       "network.e1000"
@@ -171,7 +177,10 @@
 #define BXPN_SOUND_SPEAKER               "sound.speaker"
 #define BXPN_SOUND_SB16                  "sound.sb16"
 #define BXPN_SOUND_ES1370                "sound.es1370"
-#define BXPN_PORT_E9_HACK                "misc.port_e9_hack"
+#define BXPN_PORT_E9_HACK_ROOT           "misc.port_e9_hack"
+#define BXPN_PORT_E9_HACK                "misc.port_e9_hack.enabled"
+#define BXPN_PORT_E9_HACK_ALL_RINGS      "misc.port_e9_hack.all_rings"
+#define BXPN_IODEBUG_ALL_RINGS           "misc.iodebug_all_rings"
 #define BXPN_GDBSTUB                     "misc.gdbstub"
 #define BXPN_LOG_FILENAME                "log.filename"
 #define BXPN_LOG_PREFIX                  "log.prefix"
